package com.boco.nbd.wios.export.impl;

import com.boco.nbd.wios.export.Export;
import com.boco.nbd.wios.manage.entity.bo.InvoiceQueryBo;
import com.boco.nbd.wios.manage.entity.settlement.vo.InvoiceDTO;
import com.boco.nbd.wios.manage.service.impl.InvoiceService;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class InvoiceExport implements Export<Map<String, Object>> {
    
    @Autowired
    private InvoiceService invoiceService;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        InvoiceQueryBo invoiceQueryBo = mapper.readValue(json, InvoiceQueryBo.class);
        List<InvoiceDTO> list = invoiceService.getList(invoiceQueryBo);
        List<Map<String, Object>> returnList = new ArrayList<>();
        for (InvoiceDTO dto : list) {
            Map<String, Object> map = mapper.readValue(mapper.writeValueAsString(dto), Map.class);
            if (dto.getStatus() != null) {
                map.put("statusStr", dto.getStatus().intValue() == 0 ? "待开票" : (dto.getStatus().intValue() == 1 ? "已开票" : "已完成"));
            }
            if (dto.getInvoiceType() != null) {
                String im = dto.getInvoiceMaterial() == 2?"电⼦发票-":"纸质发票-";
                map.put("invoiceTypeStr", im + (dto.getInvoiceType().intValue() == 1 ? "增值税普通发票" : "增值税专⽤发票"));
            }
            if (dto.getBusinessType() != null) {
                map.put("businessTypeStr", dto.getBusinessType().intValue()==1 ? "个⼈" : "企业");
            }
            returnList.add(map);
        }
        return returnList;
    }
    
}
