package com.boco.nbd.wios.export.impl;

import com.boco.nbd.wios.export.Export;
import com.boco.nbd.wios.manage.entity.dto.MaterialsDTO;
import com.boco.nbd.wios.manage.entity.dto.MaterialsQuery;
import com.boco.nbd.wios.manage.service.impl.WarehouseService;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

/**
 * 物料库存导出
 * 
 * @author
 * @version
 */
@Service
public class MaterialInventoryExport implements Export<MaterialsDTO> {
    
    @Autowired
    private WarehouseService warehouseService;
    
    @Override
    public List<MaterialsDTO> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        MaterialsQuery materialsQuery = mapper.readValue(json, MaterialsQuery.class);
        
        List<MaterialsDTO> list = new ArrayList<>();
        int page = 1;
        int pagecount = 10;
        while (true) {
            materialsQuery.setPage(page);
            materialsQuery.setPagecount(pagecount);
            List<MaterialsDTO> list1 = warehouseService.qryStockMaterials(materialsQuery);
            if (list1 == null || list1.isEmpty()) {
                break;
            }
            list.addAll(list1);
            if (list1.size() < pagecount) {
                break;
            }
            page++;
        }
        list.stream().forEach(item -> {
            if (item.getTotalNumber() != null) {
                item.setTotalNumber(item.getTotalNumber().setScale(0, RoundingMode.HALF_DOWN));
            }
            if (item.getHasNumber() != null) {
                item.setHasNumber(item.getHasNumber().setScale(0, RoundingMode.HALF_DOWN));
            }
            if (item.getRemainingNumber() != null) {
                item.setRemainingNumber(item.getRemainingNumber().setScale(0, RoundingMode.HALF_DOWN));
            }
            if (item.getStayWithNumber() != null) {
                item.setStayWithNumber(item.getStayWithNumber().setScale(0, RoundingMode.HALF_DOWN));
            }
        });
        
        return list;
    }
    
}
