package com.boco.nbd.wios.export.impl;

import com.boco.nbd.wios.export.Export;
import com.boco.nbd.wios.manage.entity.bo.OperateLog;
import com.boco.nbd.wios.manage.entity.bo.OperateLogBo;
import com.boco.nbd.wios.manage.entity.cams.enums.OperateLogType;
import com.boco.nbd.wios.manage.service.impl.OperateLogService;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class OperateLogExport implements Export<Map<String, Object>> {
    
    @Autowired
    private OperateLogService operateLogService;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        OperateLogBo condition = mapper.readValue(json, OperateLogBo.class);
        List<Map<String, Object>> returnList = new ArrayList<>();
        if (StringUtils.isEmpty(condition.getTypes())) {
            condition.setTypes(OperateLogType.ORDER_STATUS.getType() + "," + OperateLogType.STAFF_CHANGE.getType() + ","
                + OperateLogType.SUPPLIER_CHANGE.getType());
        }
        List<OperateLog> operateLogList = operateLogService.getList(condition);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        int no = 1;
        for (OperateLog log : operateLogList) {
            Map<String, Object> map = mapper.readValue(mapper.writeValueAsString(log), Map.class);
            map.put("no", no++);
            if (log.getCreateTime() != null)
                map.put("createTimeStr", sdf.format(log.getCreateTime()));
            returnList.add(map);
        }
        return returnList;
    }
    
}
