package com.boco.nbd.wios.export.impl;

import cn.hutool.core.date.DateUtil;
import com.boco.nbd.wios.export.Export;
import com.boco.nbd.wios.flow.enums.LevelEnum;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.entity.dto.MaterialsDTO;
import com.boco.nbd.wios.manage.entity.settlement.vo.InvoiceVo;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.boco.nbd.wios.manage.service.impl.InvoiceService;
import com.boco.nbd.wios.manage.service.impl.OrderService;
import com.boco.nbd.wios.manage.service.impl.UserPayBillService;
import com.boco.nbd.wios.manage.service.impl.WarehouseService;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 订单明细导出
 */
@Service
public class OrderDetailExport implements Export<Map<String, Object>> {
    private static final Logger logger = LoggerFactory.getLogger(OrderDetailExport.class);
    @Autowired
    private OrderService orderService;
    @Autowired
    private SupplierDao supplierDao;
    @Autowired
    private OrderInstallItemDao orderInstallItemDao;
    @Autowired
    private WarehouseService warehouseService;
    @Autowired
    private UserPayBillService userPayBillService;
    @Autowired
    private InvoiceService invoiceService;
    @Autowired
    private OrderInstallDao orderInstallDao;
    @Autowired
    private RegionDao regionDao;
    @Autowired
    private OemContractDao oemContractDao;
    @Autowired
    private OemContractPriceDao oemContractPriceDao;

    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        OrderBo condition = mapper.readValue(json, OrderBo.class);
        List<OrderVo> orderList = orderService.getList(condition);
        List<Map<String, Object>> returnList = new ArrayList<>(8);

        for (OrderVo order : orderList) {
            if(order.getStatus() >= OrderStatus.INSTALL_FINISH.getType() && order.getStatus() != OrderStatus.CLOSED.getType()){
                Map<String, Object> map = new HashMap<>(8);
                map.put("id",order.getId());
                if(order.getInstallSupplierId() == null){
                    continue;
                }
                Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
                if(supplier == null){
                    continue;
                }
                map.put("installType",supplier.getName());

                //套包内
                map.put("orderType","套包内");
                map.put("itemName","安装服务费");
                map.put("itemBrand",order.getOemName());

                try{
                    OemContract contract = oemContractDao.selectById(order.getContractId());
                    if(contract != null){
                        List<OemContractPrice> prices = oemContractPriceDao.selectPureByContractId(contract.getId());
                        Region region = regionDao.selectById(Integer.valueOf(order.getRegionId()));
                        if(region != null){
                            String idTree = region.getIdTree();
                            String[] idTreeArr = idTree.split("_");
                            OemContractPrice contractPrice = null;
                            for (int i = idTreeArr.length - 1; i >= 0; i--) {
                                final int iFinal = i;
                                List<OemContractPrice> filterResult =
                                        prices.stream().filter(item -> item.getRegionId().equals(idTreeArr[iFinal]))
                                                .collect(Collectors.toList());
                                if (!filterResult.isEmpty()) {
                                    contractPrice = filterResult.get(0);
                                    break;
                                }
                            }
                            if (contractPrice != null) {
                                map.put("fee",contractPrice.getInstallationRatePrice().add(contractPrice.getSurveyRatePrice()));
                            }
                        }
                    }
                }catch(Exception e){
                    logger.error(e.getMessage(), e);
                }

                returnList.add(map);
                //增项费用
                if(LevelEnum.A.getKey().equals(supplier.getLevel())){
                    List<OrderInstallItemVo> oiiList = orderInstallItemDao.selectByOrderIdAndType(order.getId(), 2);
                    for (OrderInstallItemVo installItemVo : oiiList) {
                        Map<String, Object> zxMap = new HashMap<>(8);
                        zxMap.put("id",order.getId());
                        zxMap.put("installType",supplier.getName());
                        zxMap.put("orderType","增项费用");
                        zxMap.put("itemBrand",order.getOemName());
                        zxMap.put("itemName",installItemVo.getItemName());
                        zxMap.put("itemModel",installItemVo.getItemModel());

                        MaterialsDTO material = warehouseService.getMaterialById(installItemVo.getItemId());
                        if(material != null){
                            zxMap.put("materialsCode",material.getMaterialsCode());
                            zxMap.put("unit",material.getUnit());
                        }

                        zxMap.put("qunatity",installItemVo.getQunatity());
                        zxMap.put("ratePrice",installItemVo.getRatePrice());
                        zxMap.put("fee",installItemVo.getFee());

                        UserPayBill payBill = userPayBillService.getByRefId(order.getId());
                        if(payBill != null && payBill.getStatus() == 1){
                            zxMap.put("hasPay","是");
                            zxMap.put("payName",payBill.getPayAccount());
                            zxMap.put("payType",(payBill.getPayType()!=null && payBill.getPayType() == 3)?"微信":((payBill.getPayType()!=null && payBill.getPayType() == 4)?"支付宝":"其它"));
                            zxMap.put("payTime",DateUtil.formatDateTime(payBill.getPayTime()));

                            InvoiceVo invoiceVo = null;
                            try {
                                invoiceVo = invoiceService.getInvoiceByBillId(payBill.getId());
                            }catch (Exception e){
                            }
                            if(invoiceVo != null && invoiceVo.getStatus() > 0){
                                zxMap.put("invoiceType",invoiceVo.getInvoiceMaterial() == 1?"线下":"线上");
                                zxMap.put("invoiceName",invoiceVo.getInvoiceTitle());
                                zxMap.put("invoiceTime", DateUtil.formatDateTime(invoiceVo.getInvoiceTime()));
                                zxMap.put("invoiceNum",invoiceVo.getInvoiceNumber());
                            }else{
                                zxMap.put("invoiceType","");
                                zxMap.put("invoiceName","");
                                zxMap.put("invoiceTime","");
                                zxMap.put("invoiceNum","");
                            }
                        }else{
                            zxMap.put("hasPay","否");
                            zxMap.put("payName","");
                            zxMap.put("payType","");
                            zxMap.put("payTime","");
                        }
                        returnList.add(zxMap);
                    }
                }
            }
        }
        return returnList;
    }
}
