package com.boco.nbd.wios.export.impl;

import com.boco.nbd.wios.export.Export;
import com.boco.nbd.wios.manage.entity.bo.WallboxApply;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.entity.vo.WallboxApplyInstallListVO;
import com.boco.nbd.wios.manage.entity.vo.WallboxApplyReqVO;
import com.boco.nbd.wios.manage.mapper.def.WallboxApplyMapper;
import com.boco.nbd.wios.manage.service.impl.WallboxApplyService;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;

/**
 * 辅材导出WallboxApplyInstallListVO
 *
 * @author
 */
@Service
public class WallBoxExport implements Export<WallboxApplyInstallListVO> {

    @Autowired
    private WallboxApplyMapper wallboxApplyMapper;
    @Autowired
    private WallboxApplyService wallboxApplyService;

    @Override
    public List<WallboxApplyInstallListVO> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        WallboxApplyReqVO condition = mapper.readValue(json, WallboxApplyReqVO.class);
        WallboxApply wallboxApply = new WallboxApply();
        BeanUtils.copyProperties(condition, wallboxApply);
        List<WallboxApplyInstallListVO> list = wallboxApplyService.selectWallboxApplyListFromOrder(wallboxApply);
        LocalDateTime currentTime = LocalDateTime.now();
        Long no = 1L;
        for (WallboxApplyInstallListVO apply : list) {
            Date createdDate = apply.getCreatedTime();
            LocalDateTime createdTime = LocalDateTime.ofInstant(createdDate.toInstant(), ZoneId.systemDefault());
            LocalDateTime endTime = createdTime.plusHours(24);
            long remainingHours = currentTime.until(endTime, ChronoUnit.HOURS);
            remainingHours = Math.max(remainingHours, 0);  // 将剩余小时数限制在0-24之间
            if (remainingHours == 0) {
                apply.setIfTimeout(1L);
                apply.setIfTimeoutName("已超时");
            } else {
                apply.setIfTimeout(0L);
                apply.setIfTimeoutName("未超时");
            }
            apply.setTimeRemaining(Long.toString(remainingHours));
            apply.setId(no++);
            apply.setNumber("1");
            if (!StringUtils.isEmpty(apply.getInstallStatus())) {
                apply.setInstallStatusName(OrderStatus.getText(Integer.parseInt(apply.getInstallStatus())));
            }
        }
        return list;
    }

}
