package com.boco.nbd.wios.flow.controller;


import cn.hutool.core.collection.CollUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.PageData;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.parent.microservice.exception.ErrorInfoException;
import com.boco.nbd.wios.flow.entity.po.HolidayPO;
import com.boco.nbd.wios.flow.service.IHolidayService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import java.util.List;
import java.util.UUID;

/**
 * @Description: 节假日配置
 * @Author: ZQY
 * @Date: 2022/8/25
 **/
@RestController
@Slf4j
@RequestMapping("holidayConfig")
@Api(tags = "节假日配置")
public class HolidayController {

    @Autowired
    private IHolidayService holidayService;


    @ApiOperation(value = "查询节假日配置")
    @GetMapping("/getList")
    public ResponseMessage2<PageData<HolidayPO>> getList(int pageIndex, int pageSize){

        Page<HolidayPO> result = PageHelper.startPage(pageIndex, pageSize);
        holidayService.getList();
        return  ResponseMessage2.Success2(PageData.create2(result.getTotal(),result.getResult()));
    }


    @ApiOperation(value = "新增节假日配置")
    @PostMapping("/add")
    public ResponseMessage2<Boolean> add(HolidayPO po){
        Example example = new Example(HolidayPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(HolidayPO::getHolidayDate), po.getHolidayDate());
        List<HolidayPO> holidayPOList= holidayService.selectByExample(example);
        if(CollUtil.isNotEmpty(holidayPOList)){
            throw new ErrorInfoException("存在重复数据！");
        }

        po.setId(UUID.randomUUID().toString());
        holidayService.insert(po);
        return  ResponseMessage2.Success();
    }


    @ApiOperation(value = "删除节假日配置")
    @PostMapping("/delete")
    public ResponseMessage2<Boolean> delete(HolidayPO po){

        holidayService.delete(po);
        return  ResponseMessage2.Success();
    }


    @ApiOperation(value = "修改节假日配置")
    @PostMapping("/update")
    public ResponseMessage2<Boolean> update(HolidayPO po){
        Example example = new Example(HolidayPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(HolidayPO::getHolidayDate), po.getHolidayDate());
        List<HolidayPO> holidayPOList= holidayService.selectByExample(example);
        if(CollUtil.isNotEmpty(holidayPOList)){
            throw new ErrorInfoException("存在重复数据！");
        }
        holidayService.updateByPrimaryKey(po);
        return  ResponseMessage2.Success();
    }


    @ApiOperation(value = "导入节假日配置")
    @PostMapping("/importExcel")
    public ResponseMessage2<Boolean> importExcel(@RequestParam("file") MultipartFile excel) {
        return holidayService.importExcel(excel);
    }



}
