package com.boco.nbd.wios.flow.controller;

import com.boco.nbd.framework.common.PageData;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.wios.flow.entity.bo.OrderTempBO;
import com.boco.nbd.wios.flow.service.IImportOrderService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @Description: TODO
 * @Author: ZQY
 * @Date: 2022/7/19
 **/
@RestController
@Slf4j
@RequestMapping("importOrder")
@Api(tags = "订单导入")
public class ImportOrderController {


    @Autowired
    private IImportOrderService importOrderService;

    /**
     * 导入订单
     * @param excel
     * @return
     */
    @PostMapping("order/importExcel")
    @ApiOperation(value = "订单导入")
    public ResponseMessage2 importExcel(@RequestParam("file") MultipartFile excel,@RequestParam("type") Integer type) {
        return importOrderService.importExcel(excel,type);
    }
    /**
     * 获取导入订单列表
     *
     * @param
     * @return
     */
    @ApiOperation(value = "获取导入订单列表")
    @GetMapping(value = "/getList")
    public ResponseMessage2<PageData<OrderTempBO>> getList(Integer type, int pageIndex, int pageSize) {

        Page<OrderTempBO> result = PageHelper.startPage(pageIndex, pageSize);
        importOrderService.getImportOrderAll(type);
        return ResponseMessage2.Success2(PageData.create2(result.getTotal(),result.getResult()));
    }

    /**
     * 批量删除导入订单
     * @param ids
     * @return
     */
    @ApiOperation(value = "批量删除导入订单")
    @PostMapping(value = "/deleteImportOrder")
    public ResponseMessage2 deleteImportOrder(@RequestParam("ids")List<String> ids) {

        return  importOrderService.deleteImportOrder(ids);
    }


    /**
     * 批量生成新订单
     * @param ids
     * @return
     */
    @ApiOperation(value = "批量生成新订单")
    @PostMapping(value = "/addOrder")
    @ApiImplicitParams({@ApiImplicitParam(name = "ids", value = "导入订单id", paramType = "query", required = false),
            @ApiImplicitParam(name = "stateAll", value = "全选状态：0-非全选，1-全选", dataType = "String", paramType = "query", required = false)})
    public ResponseMessage2 addOrder(@RequestParam("ids")List<String> ids,String stateAll) {
        importOrderService.addOrder(ids,stateAll);
        return  ResponseMessage2.Success();
    }





}
