package com.boco.nbd.wios.flow.controller;

import cn.hutool.core.util.ObjectUtil;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.parent.microservice.exception.ErrorInfoException;
import com.boco.nbd.wios.manage.client.DepponClient;
import com.boco.nbd.wios.manage.entity.bo.Order;
import com.boco.nbd.wios.manage.entity.bo.Supplier;
import com.boco.nbd.wios.manage.entity.bo.SupplierStaff;
import com.boco.nbd.wios.manage.entity.bo.WallboxApply;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.entity.vo.WallboxApplyExcelVO;
import com.boco.nbd.wios.manage.mapper.def.OrderDao;
import com.boco.nbd.wios.manage.service.impl.*;
import com.boco.nbd.wios.utils.ExcelUtils;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@RestController
public class ImportWallboxApplyController {

    @Autowired
    private Validator validator;
    @Autowired
    private WallboxApplyService wallboxApplyService;
    @Autowired
    private OrderDao orderDao;
    @Autowired
    private OrderService orderService;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private SupplierStaffService supplierStaffService;
    @Autowired
    private SupplierService supplierService;
    @Resource
    private DepponClient depponClient;

    @PostMapping("/wallboxApply/importExcel")
    @ApiOperation("导入墙盒申请")
    public ResponseMessage2 importExcel(@RequestParam(value = "file") MultipartFile file
    ) throws Exception {
        List<WallboxApplyExcelVO> list = ExcelUtils.read(file, WallboxApplyExcelVO.class);

        for (int i = 0; i < list.size(); i++) {
            String orderId = list.get(i).getOrderId();
            Integer ifEmergency = list.get(i).getIfEmergency();
            Order order = orderService.get(orderId);
            if (ObjectUtil.isNull(order)) {
                throw new ErrorInfoException("第" + (i + 1) + "行订单不存在！");
            }
            Integer status = order.getStatus();
            // 检查订单状态是否在1-43之间
            boolean allowed = status >= OrderStatus.DISPATCHING.getType() && status <= OrderStatus.INSTALLING.getType();
            if (!allowed) {
                throw new ErrorInfoException("第" + (i + 1) + "行订单派单之后才能进行墙盒申请！");
            }

            SessionInfo loginUser = tokenService.getUser();
            String userId = loginUser.getUserId();
            SupplierStaff infoByAccount = supplierStaffService.getByAccountId(Integer.valueOf(userId));
            Supplier supplier = supplierService.getById(infoByAccount.getSupplierId());
            if (supplier == null) {
                throw new ErrorInfoException("第" + (i + 1) + "行供应商不存在！");
            }
            WallboxApply wa = new WallboxApply();
            wa.setSupplierId(Long.valueOf(supplier.getId()));
            wa.setSupplierTeam(supplier.getName());
            wa.setOrderId(orderId);
            wa.setIfEmergency(ifEmergency);
            wa.setCreateAccount(tokenService.getUser().getUserId());
            //TODO 默认仓库
//            wa.setReceiverWarehouseId(receiverWarehouseId);
            wallboxApplyService.insertWallboxApply(wa);

        }
        return ResponseMessage2.Success();
    }

}
