package com.boco.nbd.wios.flow.controller;

import com.boco.nbd.framework.common.PageData;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.wios.flow.entity.bo.OrderAbnormalBO;
import com.boco.nbd.wios.flow.entity.po.OrderAbnormalPO;
import com.boco.nbd.wios.flow.service.IOrderAbnormalService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


/**
 * @Description: OrderAbnormalController
 * @Author: ZQY
 * @Date: 2022/7/12
 **/
@RestController
@Slf4j
@RequestMapping("orderAbnormal")
@Api(tags = "异常订单")
public class OrderAbnormalController {

    @Autowired
    private IOrderAbnormalService orderAbnormalService;


    /**
     * 获取异常订单列表
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "获取异常订单列表")
    @GetMapping(value = "/getList")
    public ResponseMessage2<PageData<OrderAbnormalBO>> getList(int pageIndex, int pageSize) {

        Page<OrderAbnormalBO> result = PageHelper.startPage(pageIndex, pageSize);
        orderAbnormalService.getOrderAbnormalAll();
        return ResponseMessage2.Success2(PageData.create2(result.getTotal(),result.getResult()));
    }


    /**
     * 获取异常订单详情
     * @param id
     * @return
     */
    @ApiOperation(value = "获取异常订单详情")
    @GetMapping(value = "/getOrderAbnormal")
    public ResponseMessage2<OrderAbnormalPO> getOrderAbnormal(String id) {
        return ResponseMessage2.Success2(orderAbnormalService.selectByPrimaryKey(id));
    }

    /**
     * 修改异常订单
     * @param po
     * @return
     */
    @ApiOperation(value = "修改异常订单")
    @PostMapping(value = "/update")
    public ResponseMessage2<Boolean> updateOrderAbnormal(OrderAbnormalPO po){
        try{
            orderAbnormalService.updateByPrimaryKeySelective(po);
            return ResponseMessage2.Success();
        }catch (Exception e) {
            e.printStackTrace();
            log.error("修改异常订单",e.getMessage());
            return ResponseMessage2.Failed(e.getMessage());
        }
    }

    /**
     * 新增异常订单
     * @param po
     * @return
     */
    @ApiOperation(value = "新增异常订单")
    @PostMapping(value = "/add")
    public ResponseMessage2 saveOrderAbnormal(OrderAbnormalPO po) {
        try{
            orderAbnormalService.insertSelective(po);
            return ResponseMessage2.Success();
        }catch (Exception e) {
                e.printStackTrace();
                log.error("新增异常订单",e.getMessage());
                return ResponseMessage2.Failed(e.getMessage());
        }
    }



}
