package com.boco.nbd.wios.flow.controller;

import com.boco.nbd.framework.common.PageData;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.wios.flow.entity.bo.InstallationCompletionRateBO;
import com.boco.nbd.wios.flow.entity.bo.InstallationDataStatisticsBO;
import com.boco.nbd.wios.flow.entity.bo.InstallationDurationStatisticsBO;
import com.boco.nbd.wios.flow.entity.bo.ReportStatisticsBO;
import com.boco.nbd.wios.flow.service.IReportStatisticsService;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.util.List;

/**
 * @Description: TODO
 * @Author: ZQY
 * @Date: 2022/8/29
 **/
@RestController
@Slf4j
@RequestMapping("reportStatistics")
@Api(tags = "新版本报表统计接口")
public class ReportStatisticsController {

    @Autowired
    private IReportStatisticsService reportStatisticsService;


    /**
     * 充电桩安装数据统计
     * @param bo
     * @return
     */
    @ApiOperation(value = "充电桩安装数据统计")
    @GetMapping(value = "/getInstallationDataStatistics")
    public ResponseMessage2<PageData<InstallationDataStatisticsBO>> getInstallationDataStatistics(ReportStatisticsBO bo,int pageIndex, int pageSize){
        List<InstallationDataStatisticsBO> list= ProcessUtil.getPageInfo(pageIndex,pageSize,reportStatisticsService.getInstallationDataStatistics(bo));
        return  ResponseMessage2.Success2(PageData.create2((long)list.size() ,list));
    }



    /**
     * 充电桩安装订单时长统计
     * @param bo
     * @return
     */
    @ApiOperation(value = "充电桩安装订单时长统计")
    @GetMapping(value = "/getInstallationDurationStatistics")
    public  ResponseMessage2<PageData<InstallationDurationStatisticsBO>> getInstallationDurationStatistics(ReportStatisticsBO bo, int pageIndex, int pageSize){
        List<InstallationDurationStatisticsBO> list= ProcessUtil.getPageInfo(pageIndex,pageSize,reportStatisticsService.getInstallationDurationStatistics(bo));
        return  ResponseMessage2.Success2(PageData.create2((long)list.size() ,list));
    }



    /**
     * 首联时效统计
     * @param bo
     * @return
     */
    @ApiOperation(value = "首联时效统计")
    @GetMapping(value = "/getFirstTimeStatistics")
    public  ResponseMessage2 getFirstTimeStatistics(ReportStatisticsBO bo) {
        return  ResponseMessage2.Success2(reportStatisticsService.getTimeStatistics(bo,"getFirstTimeStatistics"));
    }


    /**
     * 勘测时效统计
     * @param bo
     * @return
     */
    @ApiOperation(value = "勘测时效统计")
    @GetMapping(value = "/getSurveyTimeStatistics")
    public  ResponseMessage2 getSurveyTimeStatistics(ReportStatisticsBO bo)  {
        return  ResponseMessage2.Success2(reportStatisticsService.getTimeStatistics(bo,"getSurveyTimeStatistics"));
    }


    /**
     * 勘测审核时效统计
     * @param bo
     * @return
     */
    @ApiOperation(value = "勘测审核时效统计")
    @GetMapping(value = "/getSurveyExamineTimeStatistics")
    public  ResponseMessage2 getSurveyxamineTimeStatistics(ReportStatisticsBO bo) {
        return  ResponseMessage2.Success2(reportStatisticsService.getTimeStatistics(bo,"getSurveyExamineTimeStatistics"));
    }


    /**
     * 平均安装时效统计
     * @param bo
     * @return
     */
    @ApiOperation(value = "平均安装时效统计 ")
    @GetMapping(value = "/getAvgTimeStatistics")
    public  ResponseMessage2 getAvgTimeStatistics(ReportStatisticsBO bo) {
        return  ResponseMessage2.Success2(reportStatisticsService.getTimeStatistics(bo,"getAvgTimeStatistics"));
    }



    @ApiOperation(value = "安装完成率统计 ")
    @GetMapping(value = "/getInstallationCompletionRate")
    public  ResponseMessage2<PageData<InstallationCompletionRateBO>> getInstallationCompletionRate(ReportStatisticsBO bo, int pageIndex, int pageSize){
        List<InstallationCompletionRateBO> list= ProcessUtil.getPageInfo(pageIndex,pageSize,reportStatisticsService.getInstallationCompletionRate(bo));
        return  ResponseMessage2.Success2(PageData.create2((long)list.size() ,list));
    }



    @ApiOperation(value = "平均运行时长（非电力报装）统计 ")
    @GetMapping(value = "/getAverageRunningTime")
    public  ResponseMessage2 getAverageRunningTime(ReportStatisticsBO bo){
        return  ResponseMessage2.Success2(reportStatisticsService.getTimeStatistics(bo,"getAverageRunningTime"));
    }


    @ApiOperation(value = "平均运行时长（电力报装）统计 ")
    @GetMapping(value = "/getAverageRunningTime1")
    public  ResponseMessage2 getAverageRunningTime1(ReportStatisticsBO bo) {
        return  ResponseMessage2.Success2(reportStatisticsService.getTimeStatistics(bo,"getAverageRunningTime1"));
    }







}
