package com.boco.nbd.wios.flow.controller;

import com.boco.nbd.framework.common.PageData;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.workflow.entity.*;
import com.boco.nbd.framework.workflow.service.WorkflowService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @author:cao hai
 * @date:2022/6/27 8:44
 * @version:V1.0
 * @description:WorkflowController
 * @modify:
 */
@RestController
@Slf4j
@RequestMapping("/flow")
@Api(tags = "工作流管理")
public class WorkflowController {

    @Autowired
    private WorkflowService workflowService;


    @ApiOperation("获取流程的节点列表")
    @GetMapping(value = "/getFlowNodeList")
    public ResponseMessage2<List<FlowNodeResponse>> getFlowNodeList(String flowCode) {
        try {
            List<FlowNodeResponse> list = workflowService.getFlowNodeList(flowCode);
            return ResponseMessage2.Success2(list);
        } catch (Exception e) {
            log.error("WorkflowController.getFlowNodeList::" + e);
            return ResponseMessage2.Failed(e.getMessage());
        }
    }


    @ApiOperation("根据条件获取流程最后一个节点（包含已归档和流程中的）")
    @PostMapping(value = "/getProcessLastTask")
    @ApiIgnore
    public ResponseMessage2<WorkTaskResponse> getProcessLastTask(@RequestBody GetTaskRequest request) {
        try {
            return ResponseMessage2.Success2(workflowService.getProcessLastTask(request));
        } catch (Exception e) {
            log.error("WorkflowController.getProcessLastTask::" + e);
            return ResponseMessage2.Failed(e.getMessage());
        }
    }


    @ApiOperation("根据任务id获取任务详情")
    @GetMapping(value = "/getTaskInfo")
    public ResponseMessage2<TaskInfoResponse> getTaskByTaskId(String taskId) {
        try {
            TaskInfoResponse result = workflowService.getTaskByTaskId(taskId);
            return ResponseMessage2.Success2(result);
        } catch (Exception e) {
            log.error("WorkflowController.getTaskByTaskId::" + e);
            return ResponseMessage2.Failed(e.getMessage());
        }
    }


    @ApiOperation("根据工单id获取任务详情")
    @ApiIgnore
    @GetMapping(value = "/getTaskByBusinessKey")
    public ResponseMessage2<TaskInfoResponse> getTaskByBusinessKey(String businessKey) {
        try {
            TaskInfoResponse result = workflowService.getTaskByBusinessKey(businessKey);
            return ResponseMessage2.Success2(result);
        } catch (Exception e) {
            log.error("WorkflowController.getTaskByBusinessKey::" + e);
            return ResponseMessage2.Failed(e.getMessage());
        }
    }


    @ApiOperation("部署流程")
    @PostMapping(value = "/deployFlow")
    public ResponseMessage2<NewDeployFlowResponse> deployFlow(MultipartFile bpmnFile, String flowCode) {
        try {
            NewDeployFlowRequest request = new NewDeployFlowRequest();
            request.setBpmnFileStream(bpmnFile.getInputStream());
            request.setFlowCode(flowCode);
            request.setFileName(bpmnFile.getOriginalFilename());
            return ResponseMessage2.Success2(workflowService.deployeFlow(request));
        } catch (Exception e) {
            log.error("WorkflowController.deployFlow::" + e);
            return ResponseMessage2.Failed(e.getMessage());
        }
    }

    @ApiOperation("获取已发布流程")
    @GetMapping(value = "/getDeployedFlow")
    public ResponseMessage2<PageData<ProcessDef>> getDeployedFlow(ProcessDefParam param) {
        return ResponseMessage2.Success2(workflowService.getDeployedFlow(param));
    }

    @GetMapping("/getDiagram")
    @ApiOperation("下载动态流程图")
    @ApiIgnore
    public void getDiagram(String processInsId, HttpServletResponse response) throws IOException {
        workflowService.getDynamicOutputStream(processInsId, response.getOutputStream());
    }

    @GetMapping("/getProcessDiagram")
    @ApiOperation("下载流程图")
    @ApiIgnore
    public void getProcessDiagram(String processDefId, HttpServletResponse response) throws IOException {
        workflowService.getProcessDiagram(processDefId, response.getOutputStream());
    }

    @ApiOperation("获取流程变量信息")
    @GetMapping("/getFormProperties")
    @ApiIgnore
    public ResponseMessage2<List<FormProperty>> getFormProperties(String processInsId, String nodeName) {
        return ResponseMessage2.Success2(workflowService.getFormProperties(processInsId, nodeName));
    }


    @GetMapping("/deleteProcessInstanceByProcessInstanceId")
    @ApiOperation("根据流程实例号删除正在流转的流程记录")
    @ApiIgnore
    public ResponseMessage2 deleteProcessInstanceByProcessInstanceId(@ApiParam("流程实例") @RequestParam String processInstanceId,
                                                                     @ApiParam("删除原因") @RequestParam String deleteReason) {
        workflowService.deleteProcessInstanceByProcessInstanceId(processInstanceId, deleteReason);
        return ResponseMessage2.Success();
    }


    @GetMapping("/deleteFinishedProcessByProcessInstanceId")
    @ApiOperation("根据流程实例号删除全部的流程记录")
    @ApiIgnore
    public ResponseMessage2 deleteFinishedProcessByProcessInstanceId(@ApiParam("流程实例") @RequestParam String processInstanceId) {
        workflowService.deleteFinishedProcessByProcessInstanceId(processInstanceId);
        return ResponseMessage2.Success();
    }
}
