package com.boco.nbd.wios.flow.entity.bo;

import cn.hutool.core.collection.CollUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.flow.entity.FlowConstant;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.enums.FlowStateEnum;
import com.boco.nbd.wios.flow.enums.RgbEnum;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author:cao hai
 * @date:2022/8/2 17:14
 * @version:V1.0
 * @description:FlowChartBO
 * @modify:
 */
@Data
public class FlowChartBO {

    /**
     * 构造函数
     *
     * @param name
     * @param state
     * @param colour
     */
    public FlowChartBO(String name, Integer state, String colour) {
        this.name = name;
        this.state = state;
        this.color = colour;
    }


    /**
     * 构造函数
     *
     * @param nodeEnum
     * @param state
     * @param colour
     */
    public FlowChartBO(FlowNodeEnum nodeEnum, Integer state, String colour) {
        this.name = FlowNodeEnum.getFlowNodeStr(nodeEnum);
        this.current = nodeEnum.getKey() + "";
        this.state = state;
        this.color = colour;
    }

    @ApiModelProperty(value = "名称")
    private String name;

    @ApiModelProperty(value = "状态: >0:已完成节点;0:处于节点;<0:未完成节点")
    private Integer state;

    @ApiModelProperty(value = "颜色")
    private String color;

    @ApiModelProperty(value = "当前节点")
    private String current;

    @ApiModelProperty(value = "下一节点")
    private String next;

    /**
     * 判断是否包含
     *
     * @param item
     * @param name
     * @return
     */
    public boolean containNode(String item, String name) {
        if (name.contains(CamsConstant.DASH_FLAG)) {
            name = name.split(CamsConstant.DASH_FLAG)[0];
        }
        return Arrays.asList(item.split(CamsConstant.COMMA)).contains(name);
    }


    /**
     * @param items
     * @return
     */
    public static List<FlowChartBO> toCellChart(List<FlowChartBO> items) {
        List<FlowChartBO> list = new ArrayList<>(5);
        FlowStateEnum[] enums = FlowStateEnum.values();
        for (FlowStateEnum item : enums) {
            List<FlowChartBO> findItems = Linq.of(items).where(s -> s.containNode(item.getValue(), s.getName())).toList();
            if (CollUtil.isEmpty(findItems)) {
                list.add(new FlowChartBO(item.getKey(), -1, RgbEnum.GREY.getKey()));
                continue;
            }
            FlowChartBO loopBo = Linq.of(findItems).where(s -> RgbEnum.BLACK.getKey().equals(s.getColor())).firstOrDefault();
            if (loopBo != null) {
                list.add(new FlowChartBO(item.getKey(), loopBo.getState(), loopBo.getColor()));
                continue;
            }
            FlowChartBO executingBo = Linq.of(findItems).where(s -> s.state == 0).firstOrDefault();
            if (executingBo != null) {
                list.add(new FlowChartBO(item.getKey(), executingBo.getState(), executingBo.getColor()));
                continue;
            }
            FlowChartBO chartBo = findItems.get(findItems.size() - 1);
            list.add(new FlowChartBO(item.getKey(), chartBo.getState(), chartBo.getColor()));
        }
        return list;
    }
}
