package com.boco.nbd.wios.flow.entity.bo;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.workflow.entity.SubmitFlowBO;
import com.boco.nbd.framework.workflow.entity.SubmitTypeEnum;
import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.wios.flow.entity.FlowConstant;
import com.boco.nbd.wios.flow.entity.po.OrderInstallPO;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.entity.po.OrderSurveyPO;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.cams.enums.OverTimeType;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * @author:cao hai
 * @date:2022/7/1 10:41
 * @version:V1.0
 * @description:LocalSubmitFlowBO
 * @modify:
 */
@Data
public class LocalSubmitFlowBO extends SubmitFlowBO {

    @ApiModelProperty(value = "业务数据-工单", hidden = true)
    private OrderPO business;

    @ApiModelProperty(value = "已存在业务数据-工单", hidden = true)
    private OrderPO oldBusiness;

    @ApiModelProperty(value = "业务数据-勘测", hidden = true)
    private OrderSurveyPO businessSurvey;

    @ApiModelProperty(value = "业务数据-安装", hidden = true)
    private OrderInstallPO businessInstall;

    @ApiModelProperty(value = "提交表单参数", required = true)
    private Map<String, Object> formData;

    @ApiModelProperty(value = "当前节点", hidden = true)
    private FlowNodeEnum currentNodeEnum;

    @ApiModelProperty(value = "下个节点", hidden = true)
    private FlowNodeEnum nextNodeEnum;

    @ApiModelProperty(value = "提交人显示名")
    private String submitUserName;

    /**
     * 个性用户处理
     */
    public void userNamesHandler() {
        if (ProcessUtil.areSurveyTransfer(this.getCurrentNode())) {
            if (this.business.getSurveyStaffId() == null) {
                throw new FlowException("未选择下一节点操作用户.");
            }
            //选择电工作为下一环节流转对象
            this.getUserNames().add(this.business.getSurveyStaffId() + "");
        } else if (ProcessUtil.areInstallTransfer(this.getCurrentNode())) {
            if (this.business.getInstallStaffId() == null) {
                throw new FlowException("未选择下一节点操作用户.");
            }
            //选择电工作为下一环节流转对象
            this.getUserNames().add(this.business.getInstallStaffId() + "");
        }
    }

    /**
     * 添加业务参数
     */
    public void addTaskParam() {
        if (this.getTaskParam() == null) {
            this.setTaskParam(new HashMap<>(16));
        }
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getId), business.getId());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getType), business.getType());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getOutOrderId), business.getOutOrderId());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getUserName), business.getUserName());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getUserPhone), business.getUserPhone());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getOemAgentInfo), business.getOemAgentInfo());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getOemId), business.getOemId());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getStatus), business.getStatus());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getRegionId), business.findAreaId());
        //墙盒设备编号 t_order_install中字段
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderInstallPO::getWallboxNo), null);
        // 服务商名称 t_supplier =>install_supplier_id
        this.getTaskParam().put(FlowConstant.SUPPLIER_ID, business.getInstallSupplierId());
        this.getTaskParam().put(FlowConstant.SUPPLIER_NAME, business.getInstallSupplierName());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getSurveyFinishTime), null);
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getInstallFinishTime), null);
        //超时类型 t_system_info.overtime_flag(third_party_id=id and order_status=status)
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderPO::getOvertimeType), OverTimeType.UN_TIME_OUT.getType());
        this.getTaskParam().put(LambdaUtils.getFieldName(OrderInstallPO::getClientRate), null);
    }


    /**
     * 验证
     */
    public void localVerify() {
        //新建不验证taskId
        if (!SubmitTypeEnum.StartFlow.equals(this.getSubmitType())) {
            if (StrUtil.isEmpty(this.getTaskId())) {
                throw new FlowException("taskId is empty.");
            }
        }
        if (StrUtil.isEmpty(this.getWorkId())) {
            throw new FlowException("workId is empty.");
        }
        if (this.getCurrentNode() == null) {
            throw new FlowException("currentNode is empty.");
        }
        this.setIsFinished(false);
        this.setFlowCode(FlowConstant.FLOW_CODE);
        //避免后续null
        if (CollUtil.isEmpty(this.getUserNames())) {
            this.setUserNames(new ArrayList<>(1));
        }
        if (CollUtil.isEmpty(this.getTaskParam())) {
            this.setTaskParam(new HashMap<>(16));
        }
        FlowNodeEnum currentEnum = FlowNodeEnum.getEnum(this.getCurrentNode());
        if (currentEnum == null) {
            throw new FlowException(this.getCurrentNode() + " undefined.");
        }
        currentNodeEnum = currentEnum;
        //按实际字段赋值，不夹杂硬编码业务
        if (business == null) {
            business = ProcessUtil.fillBeanWithMap(this.formData, OrderPO.class);
            if (business != null) {
                business.setId(this.getWorkId());
            }

        }
        if (businessSurvey == null) {
            businessSurvey = ProcessUtil.fillBeanWithMap(this.formData, OrderSurveyPO.class);
            if (businessSurvey != null) {
                businessSurvey.setOrderId(this.getWorkId());
            }
        }
        if (businessInstall == null) {
            businessInstall = ProcessUtil.fillBeanWithMap(this.formData, OrderInstallPO.class);
            if (businessInstall != null) {
                businessInstall.setOrderId(this.getWorkId());
                if (StrUtil.isNotEmpty(businessInstall.getWallboxNo())) {
                    this.getTaskParam().put(LambdaUtils.getFieldName(OrderInstallPO::getWallboxNo), businessInstall.getWallboxNo());
                }
            }
        }
    }
}
