package com.boco.nbd.wios.flow.entity.bo;

import com.boco.nbd.wios.log.entity.po.InterfaceMonitorPO;
import com.boco.nbd.wios.log.enums.InterfaceMonitorEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;

import java.util.Date;


/**
 * @ClassName SapStatisticsBO
 * @Description sap统计接口信息返回实体类
 * @Author yuanlong
 * @Email yuanlong@boco.com.cn
 * @Date 2022/10/25 15:31
 * @Version V1.0
 */
@Data
@AllArgsConstructor
@ApiModel("sap接口信息返回实体类")
public class SapStatisticsBO {
    @ApiModelProperty("接口状态，正常true，异常false")
    private Boolean status;
    @ApiModelProperty("请求sap接口地址")
    private String sapUrl;
    @ApiModelProperty("请求sap接口返回值")
    private String responseResult;
    @ApiModelProperty("接口响应时间，单位秒")
    private Double responseCostTime;
    @ApiModelProperty("发起调用SAP接口的时间")
    private Date sapCallTime;
    @ApiModelProperty("工单号或其它主键")
    private String orderId;


    /**
     * 对象转换
     *
     * @param monitorEnum
     * @return
     */
    public InterfaceMonitorPO create(InterfaceMonitorEnum monitorEnum) {
        InterfaceMonitorPO po = new InterfaceMonitorPO();
        po.setOrderNo(this.orderId);
        po.setInterfaceName(monitorEnum.getValue());
        po.setInterfaceAddress(this.sapUrl);
        po.setResponseData(this.responseResult);
        po.setResponseTime(this.responseCostTime == null ? 0 : this.responseCostTime.intValue());
        po.setInterfaceState(this.status ? "1" : "2");
        po.setCallTime(this.sapCallTime);
        return po;
    }
}
