package com.boco.nbd.wios.flow.entity.bo;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.wios.flow.sms.aliyun.exception.SmsException;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;

/**
 * @ClassName TimeConfirmBO
 * @Description 安装预约时间变更确认模板
 * @Author songyanfei
 * @Email songyanfei@boco.com.cn
 * @Date 2022/10/30 9:57
 * @Version V1.0
 */
@Data
public class TimeConfirmBO implements Serializable {

    @ApiModelProperty("变更前时间")
    private String beforeTime;

    @ApiModelProperty("变更后时间")
    private String afterTime;

    public TimeConfirmBO() {

    }

    /**
     * 构造函数
     *
     * @param before
     * @param after
     */
    public TimeConfirmBO(Date before, Date after) {
        this(DateUtil.format(before, DatePattern.NORM_DATETIME_PATTERN), DateUtil.format(after, DatePattern.NORM_DATETIME_PATTERN));
    }

    /**
     * 构造函数
     *
     * @param before
     * @param after
     */
    public TimeConfirmBO(String before, String after) {
        if (StrUtil.isEmpty(before)) {
            throw new SmsException(LambdaUtils.getFieldApiModelProperty(TimeConfirmBO::getBeforeTime) + "为空");
        }
        if (StrUtil.isEmpty(after)) {
            throw new SmsException(LambdaUtils.getFieldApiModelProperty(TimeConfirmBO::getAfterTime) + "为空");
        }
        this.beforeTime = before;
        this.afterTime = after;
    }
}
