package com.boco.nbd.wios.flow.entity.bo;

import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * @author:cao hai
 * @date:2022/10/18 20:08
 * @version:V1.0
 * @description:TransferFlowBO
 * @modify:
 */
@Data
public class TransferFlowBO {

    @ApiModelProperty(value = "当前节点例如：101", required = true, example = "101")
    private Integer currentNode;

    @ApiModelProperty(value = "任务编号（必须）", required = true)
    private String taskId;

    @ApiModelProperty(value = "工单号（必须）", required = true)
    private String workId;

    @ApiModelProperty(value = "服务商Id（必须）", required = true)
    private Integer supplierId;

    @ApiModelProperty(value = "服务商（必须）", required = true)
    private String supplierName;

    @ApiModelProperty(value = "电工Id（必须）", required = true)
    private Integer staffId;

    @ApiModelProperty(value = "电工名（必须）", required = true)
    private String staffName;

    @ApiModelProperty(value = "电工号码（必须）", required = true)
    private String staffPhone;

    /**
     * 转换
     *
     * @return
     */
    public LocalSubmitFlowBO toBo() {
        LocalSubmitFlowBO bo = new LocalSubmitFlowBO();
        bo.setCurrentNode(this.getCurrentNode());
        bo.setTaskId(this.getTaskId());
        bo.setWorkId(this.workId);
        Map<String, Object> formData = new HashMap<>(3);
        if (this.supplierId == null) {
            throw new FlowException("服务商信息缺失.");
        }
        if (this.staffId == null) {
            throw new FlowException("电工信息缺失.");
        }
        if (ProcessUtil.areSurveyTransfer(this.getCurrentNode())) {
            formData.put(LambdaUtils.getFieldName(OrderPO::getSurveySupplierId), this.supplierId);
            formData.put(LambdaUtils.getFieldName(OrderPO::getSurveySupplierName), this.supplierName);
            formData.put(LambdaUtils.getFieldName(OrderPO::getSurveyStaffId), this.staffId);
            formData.put(LambdaUtils.getFieldName(OrderPO::getSurveyStaffName), this.staffName);
            formData.put(LambdaUtils.getFieldName(OrderPO::getSurveyStaffPhone), this.staffPhone);
        } else if (ProcessUtil.areInstallTransfer(this.getCurrentNode())) {
            formData.put(LambdaUtils.getFieldName(OrderPO::getInstallSupplierId), this.supplierId);
            formData.put(LambdaUtils.getFieldName(OrderPO::getInstallSupplierName), this.supplierName);
            formData.put(LambdaUtils.getFieldName(OrderPO::getInstallStaffId), this.staffId);
            formData.put(LambdaUtils.getFieldName(OrderPO::getInstallStaffName), this.staffName);
            formData.put(LambdaUtils.getFieldName(OrderPO::getInstallStaffPhone), this.staffPhone);
        } else {
            throw new FlowException("当前节点不支持转派.");
        }
        bo.setFormData(formData);

        return bo;
    }


}
