package com.boco.nbd.wios.flow.entity.po;

import cn.hutool.core.util.EnumUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.framework.workflow.annotation.EnumConvert;
import com.boco.nbd.framework.workflow.entity.constant.FlowConstants;
import com.boco.nbd.wios.flow.enums.RegionLevelEnum;
import com.boco.nbd.wios.manage.util.NumberUtil;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.util.StringUtils;

import javax.persistence.Column;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author:cao hai
 * @date:2022/6/27 13:54
 * @version:V1.0
 * @description:tOrder
 * @modify:
 */
@Data
@Table(name = "t_order")
public class OrderPO implements Serializable {


    @Id
    @Column(name = "id")
    @ApiModelProperty(value = "安装单号")
    @NotNull(message = "订单编号不可以为空", groups = {VerifyStart.class})
    private String id;

    @Column(name = "process_ins_id")
    @ApiModelProperty(value = "流程实例Id")
    private String processInsId;

    @Column(name = "oem_id")
    @ApiModelProperty(value = "主机厂ID", required = true)
    @NotNull(message = "主机厂不可以为空", groups = {VerifyStart.class})
    private Integer oemId;

    @Column(name = "oem_agent_id")
    @ApiModelProperty(value = "经销商id")
    private Integer oemAgentId;

    @Column(name = "oem_agent_info")
    @ApiModelProperty(value = "经销商信息")
    private String oemAgentInfo;

    @Column(name = "oem_agent_contact_name")
    @ApiModelProperty(value = "经销商联系人")
    private String oemAgentContactName;

    @Column(name = "oem_agent_contact_phone")
    @ApiModelProperty(value = "经销商联系方式")
    private String oemAgentContactPhone;

    @Column(name = "out_order_id")
    @ApiModelProperty(value = "外部订单编号,车企安装单号", required = true)
    @NotNull(message = "车企安装单号不可以为空", groups = {VerifyStart.class})
    private String outOrderId;

    @Column(name = "contract_id")
    @ApiModelProperty(value = "合同编号")
    private Integer contractId;

    @Column(name = "supplier_contract_id")
    @ApiModelProperty(value = "服务商合同id")
    private Integer supplierContractId;

    @Column(name = "user_name")
    @ApiModelProperty(value = "车主姓名", required = true)
    @NotNull(message = "车主姓名不可以为空", groups = {VerifyStart.class})
    private String userName;

    @Column(name = "user_phone")
    @ApiModelProperty(value = "车主电话", required = true)
    @NotNull(message = "车主电话不可以为空", groups = {VerifyStart.class})
    private String userPhone;

    @Column(name = "user_sex")
    @ApiModelProperty(value = "车主性别 0-女 1-男 2-未知")
    @EnumConvert()
    private Integer userSex;

    @Column(name = "contact_name")
    @ApiModelProperty(value = "联系人姓名")
    private String contactName;

    @Column(name = "contact_phone")
    @ApiModelProperty(value = "联系电话")
    private String contactPhone;

    @Column(name = "region_id")
    @ApiModelProperty(value = "区域id", required = true)
    private String regionId;

    @Column(name = "warehouse_id")
    @ApiModelProperty(value = "仓库id")
    private String warehouseId;

    @Column(name = "address")
    @ApiModelProperty(value = "地址")
    private String address;

    @Column(name = "vehicle_model")
    @ApiModelProperty(value = "车型")
    @NotNull(message = "车型不可以为空", groups = {VerifyStart.class})
    private String vehicleModel;

    @Column(name = "vehicle_chassis_no")
    @ApiModelProperty(value = "车辆底盘号")
    private String vehicleChassisNo;

    @Column(name = "vehicle_vin")
    @ApiModelProperty(value = "车辆识别号")
    private String vehicleVin;

    @Column(name = "vehicle_parking_no")
    @ApiModelProperty(value = "车位号")
    private String vehicleParkingNo;

    @Column(name = "vehicle_parking_status")
    @ApiModelProperty(value = "车位状态")
    private String vehicleParkingStatus;

    @Column(name = "survey_before_sale_enable")
    @ApiModelProperty(value = "是否为售车前预勘测 0:否 1：是")
    @EnumConvert(enumName = "yesOrNo")
    private Integer surveyBeforeSaleEnable;

    @Column(name = "manu_dispatch_enable")
    @ApiModelProperty(value = "是否需要人工派单 0：否 1：是")
    @EnumConvert()
    private Integer manuDispatchEnable;

    @Column(name = "wallbox_model")
    @ApiModelProperty(value = "墙盒品牌")
    @EnumConvert()
    private String wallboxModel;

    @Column(name = "status")
    @ApiModelProperty(value = "订单状态")
    @EnumConvert()
    private Integer status;

    @Column(name = "type")
    @ApiModelProperty(value = "类型 1：普通订单 2：移桩订单")
    @EnumConvert()
    private Integer type;

    @Column(name = "expand_status")
    @ApiModelProperty(value = "报装状态 1：通过 2：不通过")
    @EnumConvert()
    private Integer expandStatus;

    @Column(name = "expand_fail_reason")
    @ApiModelProperty(value = "报装不通过原因")
    private String expandFailReason;

    @Column(name = "expand_fail_operate")
    @ApiModelProperty(value = "报装不通过后订单后续操作")
    private Integer expandFailOperate;

    @Column(name = "expand_remark")
    @ApiModelProperty(value = "报装备注")
    private String expandRemark;

    @Column(name = "survey_staff_id")
    @ApiModelProperty(value = "勘测员工编号")
    private Integer surveyStaffId;

    @Column(name = "survey_finish_time")
    @ApiModelProperty(value = "勘测完成时间")
    private Date surveyFinishTime;

    @Column(name = "survey_reserve_time")
    @ApiModelProperty(value = "预约勘测时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date surveyReserveTime;

    @Column(name = "survey_client_name")
    @ApiModelProperty(value = "勘测客户姓名")
    private String surveyClientName;

    @Column(name = "survey_client_phone")
    @ApiModelProperty(value = "勘测客户手机号")
    private String surveyClientPhone;

    @Column(name = "survey_cams_phone")
    @ApiModelProperty(value = "注册CAMS PLUS 手机号")
    private String surveyCamsPhone;

    @Column(name = "survey_reserve_remark")
    @ApiModelProperty(value = "预约勘测备注")
    private String surveyReserveRemark;

    @Column(name = "install_supplier_id")
    @ApiModelProperty(value = "安装服务商Id")
    private Integer installSupplierId;

    @Column(name = "install_staff_id")
    @ApiModelProperty(value = "安装员工编号")
    private Integer installStaffId;

    @Column(name = "install_finish_time")
    @ApiModelProperty(value = "安装完成时间")
    private Date installFinishTime;

    @Column(name = "install_reserve_time")
    @ApiModelProperty(value = "预约安装时间")
    private Date installReserveTime;

    @Column(name = "install_client_name")
    @ApiModelProperty(value = "安装客户姓名")
    private String installClientName;

    @Column(name = "install_client_phone")
    @ApiModelProperty(value = "安装客户手机")
    private String installClientPhone;

    @Column(name = "install_cams_phone")
    @ApiModelProperty(value = "注册CAMS PLUS 手机号(安装)")
    private String installCamsPhone;

    @Column(name = "install_reserve_remark")
    @ApiModelProperty(value = "预约安装备注")
    private String installReserveRemark;

    @Column(name = "survey_finish_fail_reason")
    @ApiModelProperty(value = "勘察完成审核不通过原因")
    private String surveyFinishFailReason;

    @Column(name = "survey_rectify_fail_reason")
    @ApiModelProperty(value = "勘察整改审核不通过原因")
    private String surveyRectifyFailReason;

    @Column(name = "install_change_fail_reason")
    @ApiModelProperty(value = "安装变更审核不通过原因")
    private String installChangeFailReason;

    @Column(name = "install_finish_fail_reason")
    @ApiModelProperty(value = "安装完成审核不通过原因")
    private String installFinishFailReason;

    @Column(name = "install_rectify_fail_reason")
    @ApiModelProperty(value = "安装整改不通过原因")
    private String installRectifyFailReason;

    @Column(name = "close_order_reason")
    @ApiModelProperty(value = "关闭订单原因")
    private String closeOrderReason;

    @Column(name = "install_change_attachment_files")
    @ApiModelProperty(value = "变更安装附件")
    private String installChangeAttachmentFiles;

    @Column(name = "survey_reserve_fail_info")
    @ApiModelProperty(value = "预约勘察异常信息")
    private String surveyReserveFailInfo;

    @Column(name = "install_reserve_fail_info")
    @ApiModelProperty(value = "预约安装异常信息")
    private String installReserveFailInfo;

    @Column(name = "files")
    @ApiModelProperty(value = "附件地址")
    private String files;

    @Column(name = "description")
    @ApiModelProperty(value = "备注")
    private String description;

    @Column(name = "score")
    @ApiModelProperty(value = "评分")
    private Integer score;

    @Column(name = "visit_content")
    @ApiModelProperty(value = "回访内容，客户意见或建议")
    private String visitContent;

    @Column(name = "invoice_time")
    @ApiModelProperty(value = "发票日期")
    private Date invoiceTime;

    @Column(name = "estate_type")
    @ApiModelProperty(value = "小区类型")
    private Integer estateType;

    @Column(name = "has_independent_parking")
    @ApiModelProperty(value = "是否独立车位 0：否 1：是")
    @EnumConvert(enumName = "yesOrNo")
    private Integer hasIndependentParking;

    @Column(name = "install_position")
    @ApiModelProperty(value = "安装位置 1：室内 2：室外")
    @EnumConvert()
    private Integer installPosition;

    @Column(name = "property_agree")
    @ApiModelProperty(value = "物业态度 0：不同意 1：同意")
    @EnumConvert()
    private Integer propertyAgree;

    @Column(name = "electric_expand")
    @ApiModelProperty(value = "是否需要电力报装 0：否 1：是")
    @EnumConvert(enumName = "yesOrNo")
    private Integer electricExpand;

    @Column(name = "electric_expand_time")
    @ApiModelProperty(value = "电力报装时间")
    private Date electricExpandTime;

    @Column(name = "need_erect_pile")
    @ApiModelProperty(value = "是否需要立柱 0：否 1：是")
    @EnumConvert(enumName = "yesOrNo")
    private Integer needErectPile;

    @Column(name = "dispatch_time")
    @ApiModelProperty(value = "派单时间")
    private Date dispatchTime;

    @Column(name = "survey_submit_examine_time")
    @ApiModelProperty(value = "提交勘测审核时间")
    private Date surveySubmitExamineTime;

    @Column(name = "survey_approve_time")
    @ApiModelProperty(value = "勘测审核通过时间")
    private Date surveyApproveTime;

    @Column(name = "install_submit_examine_time")
    @ApiModelProperty(value = "安装提交审核时间")
    private Date installSubmitExamineTime;

    @Column(name = "install_approve_time")
    @ApiModelProperty(value = "安装审核通过时间")
    private Date installApproveTime;

    @Column(name = "return_visit_time")
    @ApiModelProperty(value = "回访时间")
    private Date returnVisitTime;

    @Column(name = "close_time")
    @ApiModelProperty(value = "订单关闭时间")
    private Date closeTime;

    @Column(name = "create_account")
    @ApiModelProperty(value = "创建人")
    private String createAccount;

    @Column(name = "create_account_name")
    @ApiModelProperty(value = "创建人")
    private String createAccountName;

    @Column(name = "create_time")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    @Column(name = "modify_account")
    @ApiModelProperty(value = "修改人")
    private String modifyAccount;

    @Column(name = "modify_time")
    @ApiModelProperty(value = "修改时间")
    private Date modifyTime;

    @Column(name = "item_model")
    @ApiModelProperty(value = "墙盒设备型号")
    @EnumConvert()
    private String itemModel;

    @Column(name = "survey_pack_type")
    @ApiModelProperty(value = "套包类型（勘测）")
    @EnumConvert(enumName = "packType")
    private String surveyPackType;

    @Column(name = "survey_wallbox_model")
    @ApiModelProperty(value = "墙盒型号（勘测）")
    @EnumConvert(enumName = "wallboxModel")
    private String surveyWallboxModel;

    @Column(name = "survey_item_model")
    @ApiModelProperty(value = "墙盒设备型号（勘测）")
    @EnumConvert(enumName = "itemModel")
    private String surveyItemModel;

    @Column(name = "install_pack_type")
    @ApiModelProperty(value = "套包类型（安装）")
    @EnumConvert(enumName = "packType")
    private String installPackType;

    @Column(name = "install_wallbox_model")
    @ApiModelProperty(value = "墙盒型号（安装）")
    @EnumConvert(enumName = "wallboxModel")
    private String installWallboxModel;

    @Column(name = "install_item_model")
    @ApiModelProperty(value = "墙盒设备型号（安装）")
    @EnumConvert(enumName = "itemModel")
    private String installItemModel;

    @Column(name = "address_lat")
    @ApiModelProperty(value = "地址纬度")
    private String addressLat;

    @Column(name = "address_lng")
    @ApiModelProperty(value = "地址经度")
    private String addressLng;

    @Column(name = "survey_follow_time")
    @ApiModelProperty(value = "预约勘测下次跟进时间")
    private Date surveyFollowTime;

    @Column(name = "install_follow_time")
    @ApiModelProperty(value = "预约安装下次跟进时间")
    private Date installFollowTime;

    @Column(name = "connect_person")
    @ApiModelProperty(value = "首联人")
    private String connectPerson;

    @Column(name = "connect_person_name")
    @ApiModelProperty(value = "首联人")
    private String connectPersonName;

    @Column(name = "connect_time")
    @ApiModelProperty(value = "首联时间")
    private Date connectTime;

    @Column(name = "survey_connect_time")
    @ApiModelProperty(value = "预约首联时间")
    private Date surveyConnectTime;

    @Column(name = "want_home_time")
    @ApiModelProperty(value = "客户希望上门时间")
    private Date wantHomeTime;

    @Column(name = "install_emergency_level")
    @ApiModelProperty(value = "安装紧急程度 1:一般 2:紧急")
    @EnumConvert()
    private Integer installEmergencyLevel;

    @Column(name = "remark")
    @ApiModelProperty(value = "其他说明")
    private String remark;

    @Column(name = "is_connect")
    @ApiModelProperty(value = "是否联系上客户：0-否，1：是")
    @EnumConvert(enumName = "yesOrNo")
    private Integer isConnect;


    @Column(name = "connect_fail_reason")
    @ApiModelProperty(value = "联系失败原因")
    private String connectFailReason;

    @Column(name = "is_need_survey")
    @ApiModelProperty(value = "是否需要勘测：0-否，1：是")
    @EnumConvert(enumName = "yesOrNo")
    private Integer isNeedSurvey;

    @Column(name = "shipno")
    @ApiModelProperty(value = "物流单号")
    private String shipno;

    /**
     * 新增扩展字段
     */
    @Column(name = "oem_name")
    @ApiModelProperty(value = "主机厂名称", required = true)
    private String oemName;

    @Column(name = "oem_code")
    @ApiModelProperty(value = "主机厂代码 organization_code")
    private String oemCode;

    /**
     * oem_agent_id
     */
    @Column(name = "oem_agent_name")
    @ApiModelProperty(value = "经销商名称", required = true)
    private String oemAgentName;

    @Column(name = "oem_agent_code")
    @ApiModelProperty(value = "经销商代码 organization_code")
    private String oemAgentCode;

    @ApiModelProperty(value = "区域级别：1 全国 2 区域 3 省份 4 地市 5 区县")
    @Column(name = "region_level")
    private Integer regionLevel;

    @ApiModelProperty(value = "国")
    @Column(name = "country_id")
    private String countryId;

    @ApiModelProperty(value = "区域")
    @Column(name = "country_name")
    private String countryName;

    @ApiModelProperty(value = "区域")
    @Column(name = "area_id")
    private String areaId;

    @ApiModelProperty(value = "区域")
    @Column(name = "area_name")
    private String areaName;

    @ApiModelProperty(value = "省")
    @Column(name = "province_id")
    private String provinceId;

    @ApiModelProperty(value = "省")
    @Column(name = "province_name")
    private String provinceName;


    @ApiModelProperty(value = "地市")
    @Column(name = "region_name")
    private String regionName;

    @ApiModelProperty(value = "区县")
    @Column(name = "city_id")
    private String cityId;

    @ApiModelProperty(value = "区县")
    @Column(name = "city_name")
    private String cityName;


    @Column(name = "pack_type")
    @ApiModelProperty(value = "套包类型", required = true)
    @EnumConvert()
    private String packType;

    @ApiModelProperty(value = "安装服务商")
    @Column(name = "install_supplier_name")
    private String installSupplierName;

    @ApiModelProperty(value = "电工")
    @Column(name = "install_staff_name")
    private String installStaffName;

    @ApiModelProperty(value = "安装员工电话", hidden = true)
    @Column(name = "install_staff_phone")
    private String installStaffPhone;

    @ApiModelProperty(value = "勘察员工姓名", hidden = true)
    @Column(name = "survey_staff_name")
    private String surveyStaffName;

    @ApiModelProperty(value = "勘察员工电话", hidden = true)
    @Column(name = "survey_staff_phone")
    private String surveyStaffPhone;

    @ApiModelProperty(value = " 勘测审核不通过次数")
    @Column(name = "survey_check_fail_num")
    private Integer surveyCheckFailNum;

    @ApiModelProperty(value = " 安装审核不通过次数")
    @Column(name = "install_check_fail_num")
    private Integer installCheckFailNum;

    @Column(name = "install_start_time")
    @ApiModelProperty(value = "安装入场时间")
    private Date installStartTime;

    @Column(name = "install_end_time")
    @ApiModelProperty(value = "安装离场时间")
    private Date installEndTime;


    @ApiModelProperty(value = "流程节点标识", hidden = true)
    @Column(name = "node_flag")
    private Integer nodeFlag;

    @ApiModelProperty(value = "是否同意:0 否，1 是", hidden = true)
    @Transient
    private Integer whether;


    @Column(name = "is_pick_car")
    @ApiModelProperty(value = "是否提车：是、否、已支付定金")
    private String isPickCar;

    @Column(name = "refuse_order_reason")
    @ApiModelProperty(value = "拒绝接单原因")
    private String refuseOrderReason;

    @Column(name = "overtime_type")
    @ApiModelProperty(value = "超时类型", hidden = true)
    @EnumConvert()
    private Integer overtimeType;

    /**
     * 新增扩展字段 2022-10-18
     */
    @Column(name = "survey_supplier_id")
    @ApiModelProperty(value = "勘测服务商")
    private Integer surveySupplierId;

    @Column(name = "survey_supplier_name")
    @ApiModelProperty(value = "勘测服务商", hidden = true)
    private String surveySupplierName;

    @Column(name = "survey_start_time")
    @ApiModelProperty(value = "勘测入场时间")
    private Date surveyStartTime;

    @Column(name = "survey_end_time")
    @ApiModelProperty(value = "勘测离场时间")
    private Date surveyEndTime;

    @Column(name = "survey_sign_lng")
    @ApiModelProperty(value = "勘测签到 经度", hidden = true)
    private String surveySignLng;

    @Column(name = "survey_sign_lat")
    @ApiModelProperty(value = "勘测签到 纬度", hidden = true)
    private String surveySignLat;

    @Column(name = "survey_sign_over_range")
    @ApiModelProperty(value = "勘测签到 是否超出一公里 0:否 1 是")
    private Integer surveySignOverRange;

    @Column(name = "survey_sign_out_lng")
    @ApiModelProperty(value = "勘测签退 经度", hidden = true)
    private String surveySignOutLng;

    @Column(name = "survey_sign_out_lat")
    @ApiModelProperty(value = "勘测签退 纬度", hidden = true)
    private String surveySignOutLat;

    @Column(name = "survey_sign_out_over_range")
    @ApiModelProperty(value = "勘测签退 是否超出一公里 0:否 1 是")
    private Integer surveySignOutOverRange;


    @Column(name = "install_sign_lng")
    @ApiModelProperty(value = "安装签到 经度", hidden = true)
    private String installSignLng;

    @Column(name = "install_sign_lat")
    @ApiModelProperty(value = "安装签到 纬度", hidden = true)
    private String installSignLat;

    @Column(name = "install_sign_over_range")
    @ApiModelProperty(value = "安装签到 是否超出一公里 0:否 1 是")
    private Integer installSignOverRange;

    @Column(name = "install_sign_out_lng")
    @ApiModelProperty(value = "安装签退 经度", hidden = true)
    private String installSignOutLng;

    @Column(name = "install_sign_out_lat")
    @ApiModelProperty(value = "安装签退 纬度", hidden = true)
    private String installSignOutLat;

    @Column(name = "install_sign_out_over_range")
    @ApiModelProperty(value = "安装签退 是否超出一公里 0:否 1 是")
    private Integer installSignOutOverRange;

    @Column(name = "transfer_user")
    @ApiModelProperty(value = "转派用户", hidden = true)
    private String transferUser;

    @Column(name = "know_after_sales_channel")
    @ApiModelProperty(value = "是否了解墙盒售后问题反馈渠道：0-否，1：是")
    @EnumConvert(enumName = "yesOrNo")
    private Integer knowAfterSalesChannel;

    @Column(name = "install_length")
    @ApiModelProperty(value = "墙盒线缆的安装长度")
    @EnumConvert()
    private Integer installLength;


    /**
     * 字段暂时不添加到数据库，处理到该部分业务时再完善
     */
    @ApiModelProperty(value = "发票号", hidden = true)
    @Transient
    private String invoiceNumber;

    @ApiModelProperty(value = "纬度", hidden = true)
    @Transient
    private String latitude;

    @ApiModelProperty(value = "经度", hidden = true)
    @Transient
    private String longitude;

    @ApiModelProperty(value = "增项费用", hidden = true)
    @Transient
    private String itemFees;

    @ApiModelProperty(value = "高附加值材料", hidden = true)
    @Transient
    private String itemHighAdd;

    @ApiModelProperty(value = "运行时长", hidden = true)
    @Transient
    private String runTime;

    @ApiModelProperty(value = "流程标识信息字段", hidden = true)
    @Transient
    private String message;


    /**
     * 设置转派人
     *
     * @param superUser
     * @param transferUser
     */
    public void setTransferUserValue(List<String> superUser, List<String> transferUser) {
        List<String> items = new ArrayList<>(8);
        items.addAll(transferUser);
        items.addAll(superUser);
        items = Linq.of(items).distinct().toList();
        this.transferUser = StringUtils.collectionToCommaDelimitedString(items);
    }

    /**
     * 设置运行时长
     */
    public void setRunTimeValue() {
        if (this.createTime == null) {
            this.runTime = FlowConstants.NULL_REPLACE;
            return;
        }
        Long second = (System.currentTimeMillis() - this.createTime.getTime()) / 1000;
        double hour = (second / 3600.0);
        this.runTime = NumberUtil.roundDouble(hour / 24) + "";
    }

    /**
     * 获取区域id
     *
     * @return
     */
    public String findAreaId() {
        if (this.regionLevel == null) {
            return "";
        }
        RegionLevelEnum levelEnum = EnumUtil.likeValueOf(RegionLevelEnum.class, this.regionLevel);
        if (levelEnum == null) {
            return "";
        }
        switch (levelEnum) {
            case COUNTRY:
                return this.countryId;
            case AREA:
                return this.areaId;
            case PROVINCE:
                return this.provinceId;
            case REGION:
                return this.regionId;
            case CITY:
                return this.cityId;
            default:
                return "";
        }
    }

    /**
     * 获取区域名称
     *
     * @return
     */
    public String findAreaName() {
        if (this.regionLevel == null) {
            return null;
        }
        RegionLevelEnum levelEnum = EnumUtil.likeValueOf(RegionLevelEnum.class, this.regionLevel);
        if (levelEnum == null) {
            return null;
        }
        switch (levelEnum) {
            case COUNTRY:
                return this.countryName;
            case AREA:
                return this.areaName;
            case PROVINCE:
                return this.provinceName;
            case REGION:
                return this.regionName;
            case CITY:
                return this.cityName;
            default:
                return null;
        }

    }

    /**
     * 流程启动参数验证接口
     */
    public interface VerifyStart {

    }

}
