package com.boco.nbd.wios.flow.entity.qo;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.workflow.annotation.AnnotationUtils;
import com.boco.nbd.framework.workflow.annotation.DoubleOperatorHandle;
import com.boco.nbd.framework.workflow.annotation.NoHandle;
import com.boco.nbd.framework.workflow.annotation.OperatorHandle;
import com.boco.nbd.framework.workflow.entity.GetTaskRequest;
import com.boco.nbd.framework.workflow.entity.VariableCondition;
import com.boco.nbd.framework.workflow.entity.constant.FlowConstants;
import com.boco.nbd.framework.workflow.entity.enums.WhereOperatorEnum;
import com.boco.nbd.wios.flow.entity.FlowConstant;
import com.boco.nbd.wios.flow.entity.po.OrderInstallPO;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author:cao hai
 * @date:2022/7/4 10:45
 * @version:V1.0
 * @description:CamsOrderQo
 * @modify:
 */
@Data
public class CamsOrderQo extends BaseQo {

    @ApiModelProperty(value = "车企单号")
    private String outOrderId;

    @ApiModelProperty(value = "车主姓名")
    @OperatorHandle(operatorValue = WhereOperatorEnum.LIKE)
    private String userName;

    @ApiModelProperty(value = "车主电话")
    private String userPhone;

    @ApiModelProperty(value = "经销商信息")
    @OperatorHandle(operatorValue = WhereOperatorEnum.LIKE)
    private String oemAgentInfo;

    @ApiModelProperty(value = "经销商名称")
    @OperatorHandle(operatorValue = WhereOperatorEnum.LIKE)
    private String oemAgentName;

    @ApiModelProperty(value = "安装单号")
    private String id;

    @ApiModelProperty(value = "车企品牌")
    private Long oemId;

    @ApiModelProperty(value = "订单状态")
    @OperatorHandle(operatorValue = WhereOperatorEnum.IN, value = Integer.class)
    private String status;

    /**
     * modify 2022-10-10 db中把这个字段定义程String
     */
    @ApiModelProperty(value = "区域")
    @OperatorHandle(operatorValue = WhereOperatorEnum.IN)
    private String regionId;

    @ApiModelProperty(value = "墙盒设备编号")
    private String wallboxNo;

    @ApiModelProperty(value = "服务商名称")
    @OperatorHandle(operatorValue = WhereOperatorEnum.LIKE)
    private String supplierName;

    @ApiModelProperty(value = "服务商名称")
    @OperatorHandle(operatorValue = WhereOperatorEnum.IN, value = Integer.class)
    private String supplierId;

    @ApiModelProperty(value = "节点")
    @OperatorHandle(operatorValue = WhereOperatorEnum.IN, value = Integer.class)
    private String next;

    @ApiModelProperty(value = "订单类型:1 安装工单 2 移桩工单", example = "1")
    private Integer type;

    /**
     * 起止时间
     */
    @ApiModelProperty(value = "勘察完成时间")
    @DoubleOperatorHandle(operatorValue = WhereOperatorEnum.GREATER_THAN_OR_EQUAL, addOperatorValue = WhereOperatorEnum.LESS_THAN_OR_EQUAL, value = Date.class)
    private String surveyFinishTime;

    @ApiModelProperty(value = "安装完成时间")
    @DoubleOperatorHandle(operatorValue = WhereOperatorEnum.GREATER_THAN_OR_EQUAL, addOperatorValue = WhereOperatorEnum.LESS_THAN_OR_EQUAL, value = Date.class)
    private String installFinishTime;


    @ApiModelProperty(value = "超时类型")
    private Integer overtimeType;

    @ApiModelProperty(value = "客户评价分小于等于")
    @NoHandle
    private Float scoreLt;

    /**
     * 安装环节 client_rate
     */
    @ApiModelProperty(value = "客户评价分大于")
    @NoHandle
    private Float scoreGt;

    /**
     * modify 2022-11-18兼容一期归档查询
     */
    /**
     * 安装服务商编号
     */
    @ApiModelProperty(hidden = true)
    private Integer installSupplierId;

    @ApiModelProperty(hidden = true)
    private String countryId;

    @ApiModelProperty(hidden = true)
    private String areaId;

    @ApiModelProperty(hidden = true)
    private String provinceId;

    @ApiModelProperty(hidden = true)
    private String cityId;

    @ApiModelProperty(hidden = true)
    private String surveyFinishTimeStart;

    @ApiModelProperty(hidden = true)
    private String surveyFinishTimeEnd;

    @ApiModelProperty(hidden = true)
    private String installFinishTimeStart;

    @ApiModelProperty(hidden = true)
    private String installFinishTimeEnd;


    /**
     * 组装查询条件
     *
     * @param qo
     * @return
     */
    public static GetTaskRequest toDo(CamsOrderQo qo) {
        qo.setFlowCode(FlowConstant.FLOW_CODE);
        GetTaskRequest request = new GetTaskRequest();
        //status字段与GetTaskRequest字段重复 copy会异常
        String status = qo.getStatus();
        if (StrUtil.isNotEmpty(status)) {
            qo.setStatus(null);
        }
        BeanUtil.copyProperties(qo, request);
        if (request.getOtherParam() == null) {
            request.setOtherParam(new HashMap<>(16));
        }
        //type个性处理,默认安装工单
        if (qo.getType() == null) {
            qo.setType(1);
        }
        if (StrUtil.isNotEmpty(status)) {
            qo.setStatus(status);
        }
        Map<String, Object> otherParam = AnnotationUtils.createOtherParam(qo);
        //个性处理客户评价
        boolean addOperatorFlag = false;
        if (qo.getScoreGt() != null) {
            VariableCondition cond = new VariableCondition();
            cond.setOperator(WhereOperatorEnum.GREATER_THAN);
            cond.setObjValue(qo.getScoreGt());
            cond.setObjType(Float.class);
            otherParam.put(LambdaUtils.getFieldName(OrderInstallPO::getClientRate), cond);
            addOperatorFlag = true;
        }
        if (qo.getScoreLt() != null) {
            VariableCondition cond = new VariableCondition();
            cond.setOperator(WhereOperatorEnum.GREATER_THAN_OR_EQUAL);
            cond.setObjValue(qo.getScoreLt());
            cond.setObjType(Float.class);
            String paramName = LambdaUtils.getFieldName(OrderInstallPO::getClientRate);
            if (addOperatorFlag) {
                paramName = FlowConstants.PREFIX + paramName;
            }
            otherParam.put(paramName, cond);
        }
        request.getOtherParam().putAll(otherParam);
        return request;
    }

}
