package com.boco.nbd.wios.flow.imports.impl;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.imports.Import;
import com.boco.nbd.wios.flow.mapper.def.FlowOrderMapper;
import com.boco.nbd.wios.manage.entity.bo.Dictionary;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import com.boco.nbd.wios.manage.entity.bo.OemContractVo;
import com.boco.nbd.wios.manage.entity.bo.Region;
import com.boco.nbd.wios.manage.entity.cams.enums.ContractStatus;
import com.boco.nbd.wios.manage.entity.cams.enums.HouseType;
import com.boco.nbd.wios.manage.mapper.def.DictionaryDao;
import com.boco.nbd.wios.manage.mapper.def.OemContractDao;
import com.boco.nbd.wios.manage.mapper.def.OemDao;
import com.boco.nbd.wios.manage.mapper.def.RegionDao;
import com.boco.nbd.wios.manage.util.NumberUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 订单导入
 *
 * @author xgl
 * @version [版本号, 2020年10月22日]
 */
@Service
public class OrderImport implements Import {

    @Resource
    private FlowOrderMapper flowOrderMapper;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private RegionDao regionDao;

    @Autowired
    private DictionaryDao dicDao;

    @Autowired
    private OemContractDao oemContractDao;

    @SuppressWarnings("unchecked")
    @Override
    public List<?> handle(List<Object> list) {
        List<String> headList = (List<String>) list.get(0);

        List<Object> newList = new ArrayList<>();
        List<Object> childNewList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            childNewList = new ArrayList<>();
            List<String> childList = (List<String>) list.get(i);
            for (int j = 0; j < childList.size() + 1; j++) {
                if (j == 0) {
                    childNewList.add("CA" + NumberUtil.getRandomOrderId(7));
                } else {
                    childNewList.add(childList.get(j - 1));
                }
            }
            newList.add(childNewList);
        }
        List<Dictionary> dicList = dicDao.selectValidByType("wallBox", null);
        Map<String, String> dicMap = dicList.stream().collect(Collectors.toMap(Dictionary::getName, Dictionary::getValue));
        // 是否移桩订单
        boolean isYiZhuang = true;
        if (headList.size() - 25 > 0) {
            isYiZhuang = false;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        for (int i = 1; i < newList.size(); i++) {
            List<String> childList = (List<String>) newList.get(i);
            // 订单号
            String orderId = childList.get(0);
            // 主机厂名称
            String oemName = childList.get(1);
            // 经销商机构代码
            String oemCode = childList.get(2);
            // 区域名称
            String regionName = childList.get(3);
            // 安装地址
            String installAddress = childList.get(4);
            // 姓名
            String userName = childList.get(5);
            // 性别
            String userSex = childList.get(6);
            // 手机
            String userPhone = childList.get(7);
            // 是否为售车勘测
            String surveyBeforeSale = childList.get(8);
            // 墙盒品牌
            String wallbox = childList.get(11);
            // 车企安装单号
            String outOrderId = childList.get(14);
            // 发票日期
            String invoiceDate = childList.get(15);
            // 经销商名称
            String oemAgentName = childList.get(16);
            // 小区类型
            String estateType = childList.get(20);
            // 是否独立车位
            String hasIndependentParking = childList.get(21);
            // 室内/室外
            String installPosition = childList.get(22);
            // 物业是否同意安装
            String propertyAgree = childList.get(23);
            // 是否需要电力报装
            String electricExpand = childList.get(24);
            // 是否需要立柱
            String needErectPile = childList.get(25);
            String manuDiapatch = null;
            if (!isYiZhuang) {
                // 移桩订单不需要这个字段 // 是否需要人工派单
                manuDiapatch = childList.get(26);
            }

            if (StringUtils.isEmpty(orderId)) {
                childList.add("error:缺少订单号");
                continue;
            }
            if (StringUtils.isEmpty(oemName)) {
                childList.add("error:缺少主机厂名称");
                continue;
            }
            if (StringUtils.isEmpty(installAddress)) {
                childList.add("error:缺少安裝地址");
                continue;
            }

            if (StringUtils.isEmpty(oemCode)) {
                childList.add("error:缺少主机厂代码");
                continue;
            }

            if (StringUtils.isEmpty(regionName)) {
                childList.add("error:缺少区域");
                continue;
            }

            if (StringUtils.isEmpty(wallbox)) {
                childList.add("error:缺少墙盒品牌");
                continue;
            }
            if (StringUtils.isEmpty(outOrderId)) {
                childList.add("error:缺少车企单号");
                continue;
            }
            if (StringUtils.isEmpty(userName)) {
                childList.add("error:缺少用户姓名");
                continue;
            }

            String wallboxValue = dicMap.get(wallbox);
            if (StringUtils.isEmpty(wallboxValue)) {
                childList.add("error:墙盒品牌不存在");
                continue;
            } else {
                childList.set(11, wallbox + CamsConstant.UNDER_LINE + wallboxValue);
            }
            OrderPO order = flowOrderMapper.selectByPrimaryKey(orderId);
            if (order != null) {
                childList.add("error:订单号已存在");
                continue;
            }

            List<Oem> oemList = oemDao.selectByNameAndCode(oemName, oemCode);
            if (oemList.isEmpty()) {
                childList.add("error:无此主机厂或已被禁用");
                continue;
            } else {
                childList.set(1, oemName + CamsConstant.UNDER_LINE + oemList.get(0).getId());
            }
            Integer oemId = oemList.get(0).getId();
            final int tempOemId = oemId.intValue();

            if (!StringUtils.isEmpty(oemAgentName)) {
                List<Oem> oemAgentList = oemDao.selectByName(oemAgentName);
                oemAgentList = oemAgentList.stream().filter(
                        item -> item.getParentId() != null && item.getParentId().intValue() == tempOemId && item.getStatus().intValue() == 1)
                        .collect(Collectors.toList());
                if (!oemAgentList.isEmpty()) {
                    oemId = oemAgentList.get(0).getId();
                    childList.set(16, oemAgentList.get(0).getName() + CamsConstant.UNDER_LINE + oemAgentList.get(0).getId());
                } else {
                    childList.add("error:无此经销商或已被禁用");
                    continue;
                }
            }

            List<OemContractVo> oemContractList = oemContractDao.selectByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
            if (oemContractList.isEmpty()) {
                childList.add("error:该主机厂/经销商没有生效中的合约");
                continue;
            }

            Region region = regionDao.selectSingleByName(regionName);
            if (region == null) {
                childList.add("error:区域不存在");
                continue;
            } else {
                childList.set(3, regionName + CamsConstant.UNDER_LINE + region.getId());
            }

            if (!StringUtils.isEmpty(userSex)) {
                if ("男".equals(userSex)) {
                    childList.set(6, userSex + "_1");
                } else if ("女".equals(userSex)) {
                    childList.set(6, userSex + "_0");
                } else {
                    childList.add("error:性别不对");
                    continue;
                }
            }
            if (!StringUtils.isEmpty(userPhone)) {
                boolean phoneValidate = NumberUtil.validateMobilePhone(userPhone);
                if (!phoneValidate) {
                    childList.add("error:手机号格式不对");
                    continue;
                }
            } else {
                childList.add("error:缺少用户手机号");
                continue;
            }

            if (!StringUtils.isEmpty(invoiceDate)) {
                try {
                    sdf.parse(invoiceDate);
                } catch (ParseException e) {
                    childList.add("error:发票日期输入有误");
                    continue;
                }
            }

            if (!StringUtils.isEmpty(surveyBeforeSale)) {
                if ("是".equals(surveyBeforeSale)) {
                    childList.set(8, surveyBeforeSale + "_1");
                } else if ("否".equals(surveyBeforeSale)) {
                    childList.set(8, surveyBeforeSale + "_0");
                } else {
                    childList.add("error:是否为售车勘测字段输入有误");
                    continue;
                }
            } else {
                childList.add("error:是否为售车勘测字段缺少");
                continue;
            }

            if (!StringUtils.isEmpty(estateType)) {
                Integer houseType = HouseType.getTypeByValue(estateType);
                if (houseType == null) {
                    childList.add("error:小区类型输入有误");
                    continue;
                }
                childList.set(20, estateType + CamsConstant.UNDER_LINE + houseType);
            }

            if (!StringUtils.isEmpty(hasIndependentParking)) {
                if ("是".equals(hasIndependentParking)) {
                    childList.set(21, hasIndependentParking + "_1");
                } else if ("否".equals(hasIndependentParking)) {
                    childList.set(21, hasIndependentParking + "_0");
                } else {
                    childList.add("error:是否独立车位字段输入有误");
                    continue;
                }
            }

            if (!StringUtils.isEmpty(installPosition)) {
                if ("室内".equals(installPosition)) {
                    childList.set(22, installPosition + "_1");
                } else if ("室外".equals(installPosition)) {
                    childList.set(22, installPosition + "_2");
                } else {
                    childList.add("error:室内室外字段输入有误");
                    continue;
                }
            }

            if (!StringUtils.isEmpty(propertyAgree)) {
                if ("是".equals(propertyAgree)) {
                    childList.set(23, propertyAgree + "_1");
                } else if ("否".equals(propertyAgree)) {
                    childList.set(23, propertyAgree + "_0");
                } else {
                    childList.add("error:物业是否同意字段输入有误");
                    continue;
                }
            }

            if (!StringUtils.isEmpty(electricExpand)) {
                if ("是".equals(electricExpand)) {
                    childList.set(24, electricExpand + "_1");
                } else if ("否".equals(electricExpand)) {
                    childList.set(24, electricExpand + "_0");
                } else {
                    childList.add("error:是否需要电力报装字段输入有误");
                    continue;
                }
            }

            if (!StringUtils.isEmpty(needErectPile)) {
                if ("是".equals(needErectPile)) {
                    childList.set(25, needErectPile + "_1");
                } else if ("否".equals(needErectPile)) {
                    childList.set(25, needErectPile + "_0");
                } else {
                    childList.add("error:是否需要立柱字段输入有误");
                    continue;
                }
            }

            if (!isYiZhuang) {
                if (!StringUtils.isEmpty(manuDiapatch)) {
                    if ("是".equals(manuDiapatch)) {
                        childList.set(26, manuDiapatch + "_1");
                    } else if ("否".equals(manuDiapatch)) {
                        childList.set(26, manuDiapatch + "_0");
                    } else {
                        childList.add("error:是否人工派单字段输入有误");
                    }
                } else {
                    childList.add("error:是否人工派单字段缺少");
                }
            }

        }
        return newList;
    }

}
