package com.boco.nbd.wios.flow.imports.impl;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.flow.imports.Import;
import com.boco.nbd.wios.flow.service.IOrderService;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import com.boco.nbd.wios.manage.entity.bo.OemContractItemVo;
import com.boco.nbd.wios.manage.entity.bo.OemContractVo;
import com.boco.nbd.wios.manage.entity.bo.Region;
import com.boco.nbd.wios.manage.entity.cams.enums.ContractStatus;
import com.boco.nbd.wios.manage.entity.dto.MaterialsDTO;
import com.boco.nbd.wios.manage.mapper.def.OemContractDao;
import com.boco.nbd.wios.manage.mapper.def.OemContractItemDao;
import com.boco.nbd.wios.manage.mapper.def.OemDao;
import com.boco.nbd.wios.manage.mapper.def.RegionDao;
import com.boco.nbd.wios.manage.service.impl.OemContractService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import com.boco.nbd.wios.manage.service.impl.WarehouseService;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author yong
 */
@Service
public class SupplierContractPriceImport implements Import {

    @Autowired
    private RegionDao regionDao;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private OemContractService oemContractService;

    @Autowired
    private IOrderService flowOrderService;

    @SuppressWarnings("unchecked")
    @Override
    public List<?> handle(List<Object> list) {
        for (int i = 1; i < list.size(); i++) {
            List<String> childList = (List<String>) list.get(i);
            // 区域名称
            String regionName = childList.get(0);
            String packType = childList.get(2);

            // 主机厂名称
            String oemName = childList.get(1);
            // 设备名称
            String deviceName = childList.get(3);
            // String surveyPrice = childList.get(3);// 勘测价格（不含税）
            // String installPrice = childList.get(4);// 勘测价格（含税）
            // String remark = childList.get(5);// 备注
            if (StringUtils.isEmpty(regionName) || StringUtils.isEmpty(oemName)) {
                childList.add("error:参数不能未空");
                continue;
            }
            if (StringUtils.isEmpty(packType)) {
                childList.add("error:套包类型参数不能未空");
                continue;
            }
            Region region = regionDao.selectSingleByName(regionName);
            if (region == null) {
                childList.add("error:区域不存在");
                continue;
            } else {
                childList.set(0, regionName + CamsConstant.UNDER_LINE + region.getIdTree());
            }
            Oem oem = oemDao.selectSingleByName(oemName);
            if (oem == null) {
                childList.add("error:主机厂/经销商不存在");
                continue;
            } else {
                childList.set(1, oemName + CamsConstant.UNDER_LINE + oem.getId() + CamsConstant.UNDER_LINE + oem.getCode());
            }
            String packTypeId = flowOrderService.getPackTypeId(packType);
            if (packTypeId == null) {
                childList.add("error:墙盒类型不存在");
                continue;
            } else {
                childList.set(2, packType + CamsConstant.UNDER_LINE + packTypeId);
            }

            if (!StringUtils.isEmpty(deviceName)) {
                List<OemContractItemVo> contractItems = oemContractService.getSupplierInstallDevice(oem.getId().intValue(), packTypeId);
                List<OemContractItemVo> ociList =
                        contractItems.stream().filter(item -> deviceName.equals(item.getItemName())).collect(Collectors.toList());
                if (ociList.isEmpty()) {
                    childList.add("error:设备不存在");
                    continue;
                } else {
                    childList.set(3, deviceName + CamsConstant.UNDER_LINE + ociList.get(0).getItemId());
                }
            }
        }
        return list;
    }

}
