package com.boco.nbd.wios.flow.imports.impl;

import cn.hutool.core.util.ObjectUtil;
import com.boco.nbd.cams.core.constant.MessageConstant;
import com.boco.nbd.wios.flow.imports.Import;
import com.boco.nbd.wios.manage.entity.bo.Order;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.service.impl.OrderService;
import com.boco.nbd.wios.manage.service.impl.WallboxApplyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * description
 *
 * @author ning.chai@foxmail.com
 */
@Service
@Slf4j
public class WallboxApplyImport implements Import {

    @Autowired
    private WallboxApplyService wallboxApplyService;
    @Autowired
    private OrderService orderService;

    @Override
    public List<?> handle(List<Object> list) {
        for (int i = 1; i < list.size(); i++) {
            List<String> childList = (List<String>) list.get(i);
            // 安装订单号
            String orderId = childList.get(0);
            // 是否紧急
            String ifEmergency = childList.get(1);

            if (StringUtils.isEmpty(orderId) || StringUtils.isEmpty(ifEmergency)) {
                childList.add("error:" + MessageConstant.MISSING_PARAM);
                continue;
            }

            if (StringUtils.isNotBlank(orderId)) {
                Order order = orderService.get(orderId);
                if (ObjectUtil.isNull(order)) {
                    childList.add("error:不存在安装订单号");
                    continue;
                }
                Integer status = order.getStatus();
                // 检查订单状态是否在1-43之间
                boolean allowed = status >= OrderStatus.DISPATCHING.getType() && status <= OrderStatus.INSTALLING.getType();
                if (!allowed) {
                    childList.add("error:订单派单之后才能进行墙盒申请");
                    continue;
                }

                if (ObjectUtil.isNotNull(wallboxApplyService.selectWallboxDetail(orderId))) {
                    childList.add("error:重复订单申请");
                    continue;
                }

            }

        }


        return list;
    }
}
