package com.boco.nbd.wios.flow.mapper.def;

import com.boco.nbd.framework.persistence.BaseMapper3;
import com.boco.nbd.wios.flow.entity.bo.ConnectConfigBO;
import com.boco.nbd.wios.flow.entity.bo.NodeGroupBO;
import com.boco.nbd.wios.flow.entity.po.*;
import com.boco.nbd.wios.flow.entity.qo.FlowNodeUserQo;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author:cao hai
 * @date:2022/6/22 16:17
 * @version:V1.0
 * @description:NodeMapper
 * @modify:
 */
public interface NodeMapper extends BaseMapper3<NodePO> {

    /**
     * 获取流程节点配置的用户组
     *
     * @param nodeId
     * @return
     */
    List<NodeGroupBO> getNodeGroupsInfo(@Param(value = "nodeId") Long nodeId);

    /**
     * 删除节点和用户组关系
     *
     * @param nodeId
     */
    void deleteGroupNode(@Param(value = "nodeId") Long nodeId);

    /**
     * 绑定节点和用户组
     *
     * @param nodeId
     * @param groupId
     * @return
     */
    Integer bindGroupNode(@Param(value = "nodeId") Long nodeId, @Param(value = "groupId") Long groupId);


    /**
     * 根据流程Id 和节点Code获取流程节点记录
     *
     * @param flowCode
     * @param nodeCode
     * @return
     */
    List<FlowNodePO> selectByFlowIdAndNodeCode(@Param(value = "flowCode") String flowCode, @Param(value = "nodeCode") String nodeCode);

    /**
     * 删除用户组关系
     *
     * @param groupId
     * @return
     */
    Integer deleteGroupUser(@Param(value = "groupId") Long groupId);

    /**
     * 绑定用户组关系
     *
     * @param groupId
     * @param userId
     * @return
     */
    Integer bindGroupUser(@Param(value = "groupId") Long groupId, @Param(value = "userId") Long userId);

    /**
     * 加载用户
     * @param vendorId
     * @return
     */
    List<UserPO> loadUsers(@Param(value = "vendorId") Integer vendorId);

    /**
     * 加载用户组用户
     *
     * @param groupId
     * @return
     */
    List<UserPO> getUserByGroup(@Param(value = "groupId") Long groupId);

    /**
     * 获取流程超级用户
     *
     * @param groupId
     * @return
     */
    List<BaseUserPO> getSuperUserByGroup(@Param(value = "groupId") Long groupId);

    /**
     * 获取厂家信息
     *
     * @return
     */
    List<VendorPO> getSelectVendorInfo();

    /**
     * 加载节点用户信息
     *
     * @param qo
     * @return
     */
    List<FlowNodeUserPO> getFlowNodeUserInfo(FlowNodeUserQo qo);

    /**
     * 获取首联配置信息
     *
     * @param flowCode
     * @param nodeCode
     * @param vendorId
     * @return
     */
    List<ConnectConfigBO> getConnectConfig(@Param(value = "flowCode") String flowCode, @Param(value = "nodeCode") String nodeCode, @Param(value = "vendorId") Integer vendorId);


    /**
     * 删除首联配置信息
     * @param item
     * @return
     */
    Integer deleteConnectConfig(ConnectConfigBO item);

    /**
     * 绑定首联配置信息
     *
     * @param bo
     * @return
     */
    Integer bindConnectConfig(ConnectConfigBO bo);


}
