package com.boco.nbd.wios.flow.mapper.def;

import com.boco.nbd.wios.flow.entity.bo.TaskInstBO;
import com.boco.nbd.wios.flow.entity.vo.StatisticsProvinceVo;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @Description: TODO
 * @Author: ZQY
 * @Date: 2022/8/16
 **/
public interface StatisticsMapper {

    /**
     * 查询归档总数
     * @param createTime  时间
     * @param nodeFlag    节点
     * @param timeType    时间类型
     * @param oemId       主机厂ID
     * @param nodeName    节点中文名
     * @param startTime
     * @param endTime
     * @return
     */
    Integer selectFiledOrderCount(@Param("createTime") String createTime, @Param("nodeFlag") List<String> nodeFlag,@Param("timeType") String timeType,@Param("oemId")  String oemId,@Param("nodeName")  String nodeName,@Param("startTime")String startTime,@Param("endTime")String endTime);


    /**
     * 查询流转总数
     * @param createTime  时间
     * @param nodeFlag    节点
     * @param timeType    时间类型
     * @param oemId       主机厂ID
     * @param nodeName    节点中文名
     * @param startTime
     * @param endTime
     * @return
     */
    Integer selectFlowOrderCount(@Param("createTime") String createTime,@Param("nodeFlag") List<String> nodeFlag,@Param("timeType") String timeType,@Param("oemId")  String oemId,@Param("nodeName")  String nodeName,@Param("startTime")String startTime,@Param("endTime")String endTime);


    /**
     * 查询省份订单总数
     * @param createTime  时间
     * @param nodeFlag    节点
     * @param timeType    时间类型
     * @param oemId       主机厂ID
     * @param nodeName    节点中文名
     * @param startTime
     * @param endTime
     * @return
     */
    List<StatisticsProvinceVo> selectProvinceOrderCount(@Param("createTime") String createTime, @Param("nodeFlag") List<String> nodeFlag, @Param("timeType") String timeType, @Param("oemId")  String oemId, @Param("nodeName")  String nodeName, @Param("startTime")String startTime, @Param("endTime")String endTime);


    /**
     * 查询地市订单总数
     * @param provinceId 省份id
     * @param createTime  时间
     * @param nodeFlag    节点
     * @param timeType    时间类型
     * @param oemId       主机厂ID
     * @param nodeName    节点中文名
     * @param startTime
     * @param endTime
     * @return
     */
    List<Map<String,String>> selectAreaOrderCount(@Param("provinceId") String provinceId,@Param("createTime") String createTime, @Param("nodeFlag") List<String> nodeFlag, @Param("timeType") String timeType,@Param("oemId")  String oemId,@Param("nodeName")  String nodeName,@Param("startTime")String startTime,@Param("endTime")String endTime);


    /**
     * 查询月度订单总数
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @param nodeFlag    节点
     * @param oemId       主机厂ID
     * @return
     */
    List<Map<String,String>> selectMonthOrderQuantity(@Param("startTime") String startTime,@Param("endTime") String endTime,@Param("nodeFlag") List<String> nodeFlag,@Param("oemId")  String oemId);


    /**
     * 查询节点信息
     * @param procInstId
     * @param nodeName
     * @return
     */
    List<TaskInstBO> getTaskInstByInstIdAndName(@Param("procInstId") String procInstId,@Param("nodeName") String nodeName);


    /**
     * 根据服务商查询评分
     * @param time
     * @return
     */
    List<Map<String, Object>>  getScoreSupplierBySupplier(@Param("time") String time);


    /**
     * 根据区域查询评分
     * @param time
     * @return
     */
    List<Map<String, Object>>  getScoreSupplierByRegion(@Param("time") String time);

    /**
     * 派发订单总数
     * @param oemId
     * @param startTime
     * @param endTime
     * @return
     */
    Integer getOrderCount(@Param("oemId")  String oemId,@Param("startTime")String startTime,@Param("endTime")String endTime);


    /**
     * 根据节点查询订单总数
     * @param oemId
     * @param startTime
     * @param endTime
     * @param nodeFlag
     * @return
     */
    Integer getFlowOrderCount(@Param("oemId")  String oemId,@Param("startTime")String startTime,@Param("endTime")String endTime,@Param("nodeFlag") List<String> nodeFlag);


    /**
     * 区域派发订单总数
     * @param oemId
     * @param startTime
     * @param endTime
     * @return
     */
    @MapKey("key")
    List<Map<String, Object>> getOrderCountGroupArea(@Param("oemId")  String oemId,@Param("startTime")String startTime,@Param("endTime")String endTime);

    /**
     * 区域根据节点查询订单总数
     * @param oemId
     * @param startTime
     * @param endTime
     * @param nodeFlag
     * @return
     */
    @MapKey("key")
    List<Map<String, Object>> getFlowOrderCountGroupArea(@Param("oemId")  String oemId,@Param("startTime")String startTime,@Param("endTime")String endTime,@Param("nodeFlag") List<String> nodeFlag);


    /**
     * 获取不满意订单总数
     * @param time
     * @return
     */
    Integer getDissatisfiedOrderCount(@Param("time")String time);

    /**
     * 安装审核通过数量
     * @param time
     * @return
     */
    Integer getInstallationAuditPassedOrderCount(@Param("time")String time);

}
