package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.workflow.entity.SubmitTypeEnum;
import com.boco.nbd.framework.workflow.entity.TaskInfoResponse;
import com.boco.nbd.wios.flow.entity.bo.LocalSubmitFlowBO;
import com.boco.nbd.wios.flow.entity.po.OrderInstallPO;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.entity.vo.*;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.service.IAppOrderService;
import com.boco.nbd.wios.flow.service.IOrderService;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.mapper.def.MaterialConfigMapper;
import com.boco.nbd.wios.manage.mapper.def.OrderInstallItemMapper;
import com.boco.nbd.wios.manage.mapper.def.OrderMapper;
import com.boco.nbd.wios.manage.service.impl.UserPayBillService;
import com.boco.nbd.wios.wx.entity.bo.MinaOrderTrack;
import com.ihidea.core.support.exception.ServiceException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author:cao hai
 * @date:2022/10/27 15:28
 * @version:V1.0
 * @description:TODO AppOrderServiceImpl
 * @modify:
 */
@Service
@Slf4j
public class AppOrderServiceImpl implements IAppOrderService {

    @Resource
    private OrderMapper orderMapper;

    @Resource
    private OrderInstallItemMapper orderInstallItemMapper;

    @Autowired
    private IOrderService orderService;

    @Autowired
    private UserPayBillService userPayBillService;

    @Resource
    private MaterialConfigMapper materialConfigMapper;

    /**
     * 通过用户号码获取工单
     *
     * @param userPhone
     * @return
     */
    @Override
    public List<AppOrderVO> queryOrderList(String userPhone) {
        List<AppOrderVO> returnList = new ArrayList<>(10);
        List<OrderVo> orderList = orderMapper.selectByCamsPhoneForApp(userPhone);
        // 排序
        orderList = orderList.stream().sorted(Comparator.comparing(OrderVo::getAppSort)).collect(Collectors.toList());
        String datePattern = DatePattern.NORM_DATETIME_MINUTE_PATTERN;
        for (OrderVo ov : orderList) {
            //1:接单 2：预约 3：勘察 4：安装 5：评价
            Integer transformStatus = transformStatus(ov.getStatus(), ov.getBillStatus());
            AppOrderVO order = new AppOrderVO();
            order.setUserName(ov.getUserName());
            order.setUserSex(ov.getUserSex());
            order.setOrderId(ov.getId());
            order.setAddress(ov.getAddress());
            order.setContactName(ov.getInstallStaffName());
            order.setContactPhone(ov.getInstallStaffPhone());
            //modify 2022-11-23 勘测使用勘测人员信息
            if (transformStatus == 3 || transformStatus == 2) {
                order.setContactName(ov.getSurveyStaffName());
                order.setContactPhone(ov.getSurveyStaffPhone());
            }
            // 查询最近一条操作日志
            List<TaskInfoResponse> tracks = orderService.getTrackByWorkId(ov.getId());
            if (CollUtil.isNotEmpty(tracks)) {
                String executeTime = tracks.get(tracks.size() - 1).getLocalVariables().get(LambdaUtils.getFieldName(MinaOrderTrack::getExecuteTime)) + "";
                if (StrUtil.isNotEmpty(executeTime)) {
                    order.setTime(DateUtil.format(DateUtil.parse(executeTime, DatePattern.NORM_DATETIME_PATTERN), datePattern));
                }
                TaskInfoResponse surveyStart = Linq.of(tracks).where(s -> s.getNodeName().equals(FlowNodeEnum.NODE_105.getValue())).firstOrDefault();
                if (surveyStart != null) {
                    order.setSurveyStartTime(DateUtil.format(surveyStart.getCreateTime(), datePattern));
                }
                TaskInfoResponse installStart = Linq.of(tracks).where(s -> s.getNodeName().equals(FlowNodeEnum.NODE_110.getValue())).firstOrDefault();
                if (installStart != null) {
                    order.setInstallStartTime(DateUtil.format(installStart.getCreateTime(), datePattern));
                }
            }
            if (ov.getDispatchTime() != null) {
                order.setDispatchTime(DateUtil.format(ov.getDispatchTime(), datePattern));
            }
            if (ov.getSurveyReserveTime() != null) {
                order.setSurveyReserveTime(DateUtil.format(ov.getSurveyReserveTime(), datePattern));
            }
            if (ov.getInstallReserveTime() != null) {
                order.setInstallReserveTime(DateUtil.format(ov.getInstallReserveTime(), datePattern));
            }
            if (ov.getReturnVisitTime() != null) {
                order.setReturnVisitTime(DateUtil.format(ov.getReturnVisitTime(), datePattern));
                if (OrderStatus.INSTALL_EXAMINE_SUCCESS.getType() == ov.getStatus()) {
                    ov.setStatus(OrderStatus.RETURN_VISIT.getType());
                }
            }

            order.setRelativeStatus(transformStatus);
            order.setStatus(ov.getStatus());
            order.setStatusText(OrderStatus.getText(ov.getStatus()));
            order.setFee(ov.getBillFee());
            if (ov.getInstallFinishTime() != null) {
                order.setInstallReserveTime(DateUtil.format(ov.getInstallFinishTime(), datePattern));
            }
            order.setBillId(ov.getBillId());
            order.setBillStatus(ov.getBillStatus());
            order.setExpandStatus(ov.getExpandStatus());
            order.setExpandFailOperate(ov.getExpandFailOperate());
            returnList.add(order);
        }
        return returnList;
    }

    /**
     * 获取订单详情
     *
     * @param orderId
     * @return
     */
    @Override
    public AppOrderDetailVO getOrderDetail(String orderId) {
        AppOrderDetailVO returnValue = new AppOrderDetailVO();
        OrderPO order = orderService.getOrder(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        List<TaskInfoResponse> tracks = orderService.getTrackByWorkId(orderId);
        Map<Integer, String> needShowStatusMap = new HashMap<>(7);
        needShowStatusMap.put(OrderStatus.SURVEY_RESERVED.getType(), "已预约");
        needShowStatusMap.put(OrderStatus.SURVEY_DISPATCHED.getType(), "已指派电工，等待上门勘测");
        needShowStatusMap.put(OrderStatus.SURVEY_FINISH.getType(), "已完成上门勘测");
        needShowStatusMap.put(OrderStatus.INSTALL_RESERVED.getType(), "已完成预约安装");
        needShowStatusMap.put(OrderStatus.INSTALL_FINISH.getType(), "已完成上门安装，待评价");
        needShowStatusMap.put(OrderStatus.RETURN_VISIT.getType(), "已完成，感谢您的评价");
        needShowStatusMap.put(OrderStatus.CLOSED.getType(), "已为您关闭工单");
        List<AppOperateLogVO> list = new ArrayList<>(8);
        if (CollUtil.isNotEmpty(tracks)) {
            for (int i = 0; i < tracks.size(); i++) {
                TaskInfoResponse track = tracks.get(i);
                String nextNode = (i == 0 ? order.getNodeFlag() + "" : tracks.get(i - 1).getNodeFlag());
                OrderStatus orderStatus = FlowNodeEnum.getOrderStatus(track.getNodeFlag(), nextNode);
                if (orderStatus != null && needShowStatusMap.containsKey(orderStatus.getType())) {
                    AppOperateLogVO dto = new AppOperateLogVO();
                    dto.setName(orderStatus.getType() + "");
                    dto.setValue(needShowStatusMap.get(orderStatus.getType()));
                    String comment = track.getLocalVariables().get(LambdaUtils.getFieldName(MinaOrderTrack::getComment)) + "";
                    if (StrUtil.isNotEmpty(comment) && comment.contains("工单转派") && track.getNodeFlag().equals(FlowNodeEnum.NODE_104.getKey() + "")) {
                        dto.setValue("已更换电工，等待上门勘测");
                    }
                    if (OrderStatus.SURVEY_RESERVED.equals(orderStatus)) {
                        if (StrUtil.isNotEmpty(comment) && comment.contains("更新业务数据")) {
                            dto.setValue("已修改预约信息，等待上门勘察");
                        }
                    }
                    if (OrderStatus.INSTALL_RESERVED.equals(orderStatus)) {
                        if (StrUtil.isNotEmpty(comment) && comment.contains("更新业务数据")) {
                            dto.setValue("已修改预约信息，等待上门安装");
                        }
                    }
                    dto.setDate(DateUtil.format(track.getCreateTime(), DatePattern.NORM_DATE_PATTERN));
                    if (OrderStatus.RETURN_VISIT.equals(orderStatus)) {
                        dto.setShowRating(true);
                    }
                    list.add(dto);
                }
            }
        }
        returnValue.setAddress(order.getAddress());
        returnValue.setOrderId(orderId);
        returnValue.setStatusText(OrderStatus.getText(order.getStatus()));
        returnValue.setOperateLogList(list);
        returnValue.setUserName(order.getUserName());

        UserPayBill userPayBill = userPayBillService.getByRefId(orderId);
        if (userPayBill != null) {
            returnValue.setBillId(userPayBill.getId());
            returnValue.setBillStatus(userPayBill.getStatus());
        }
        return returnValue;
    }

    /**
     * 安装订单详情
     *
     * @param orderId
     * @return
     */
    @Override
    public AppOrderInstallVO getInstallDetail(String orderId) {
        AppOrderInstallVO returnValue = new AppOrderInstallVO();
        OrderPO order = orderService.getOrder(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        OrderInstallPO orderInstall = orderService.getOrderInstall(orderId);
        if (orderInstall == null) {
            throw new ServiceException("安装订单不存在");
        }
        // 查询账单
        UserPayBill userPayBill = userPayBillService.getByRefId(orderId);
        if (userPayBill != null) {
            returnValue.setBillId(userPayBill.getId());
            returnValue.setBillStatus(userPayBill.getStatus());
            returnValue.setBillPayMethod(userPayBill.getPayType());
            returnValue.setBillUserPhone(userPayBill.getUserPhone());
            if (userPayBill.getPayTime() != null) {
                returnValue.setBillPayTime(DateUtil.format(userPayBill.getPayTime(), DatePattern.NORM_DATETIME_PATTERN));
            }
        }

        BigDecimal additionTotalFee = orderInstall.getPredictTotalFee() == null ? BigDecimal.ZERO : BigDecimal.valueOf(orderInstall.getPredictTotalFee());
        BigDecimal expandFee = orderInstall.getExpandFee() == null ? BigDecimal.ZERO : BigDecimal.valueOf(orderInstall.getExpandFee());
        returnValue.setOrderId(orderId);
        returnValue.setAddress(order.getAddress());
        returnValue.setExpandFee(expandFee);
        returnValue.setAdditionTotalFee(additionTotalFee);
        returnValue.setTotalFee(additionTotalFee.add(expandFee));

        if (order.getInstallFinishTime() != null) {
            returnValue.setInstallFinishTime(DateUtil.format(order.getInstallFinishTime(), DatePattern.NORM_DATETIME_PATTERN));
        }
        returnValue.setOrderStatus(order.getStatus());
        returnValue.setOrderStatusText(OrderStatus.getText(order.getStatus()));
        List<AppOrderInstallVO.AdditionFeeItem> expandFeeList = new ArrayList<>(8);

        List<OrderInstallItemVo> oiiList = orderInstallItemMapper.selectByOrderIdAndType(orderId, 2);
        for (OrderInstallItemVo oii : oiiList) {
            AppOrderInstallVO.AdditionFeeItem expandFeeItem = returnValue.new AdditionFeeItem();
            expandFeeItem.setFee(oii.getFee());
            expandFeeItem.setName(oii.getItemName());
            expandFeeItem.setQuantity(oii.getQunatity().intValue());
            expandFeeList.add(expandFeeItem);
        }
        returnValue.setAdditionFeeList(expandFeeList);
        //回访数据不存在，所以也不村存在评价详情内容 returnValue.setOrderScore
        return returnValue;
    }

    /**
     * 查看报装说明
     *
     * @param orderId
     * @return
     */
    @Override
    public MaterialConfig getExpandIntro(String orderId) {
        OrderPO order = orderService.getOrder(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        if (StringUtils.isEmpty(order.findAreaId())) {
            return null;
        }
        MaterialConfigBo condition = new MaterialConfigBo();
        condition.setType(1);
        condition.setStatus(1);

        condition.setRegionId(Integer.valueOf(order.findAreaId()));
        List<MaterialConfigVo> list = materialConfigMapper.selectByExample(condition);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询账单
     *
     * @param phone
     * @param orderId
     * @param status
     * @return
     */
    @Override
    public List<AppPayBillVO> qryBill(String phone, String orderId, Integer status) {
        UserPayBill condition = new UserPayBill();
        condition.setUserPhone(phone);
        condition.setRefId(orderId);
        condition.setStatus(status);
        List<UserPayBillVo> list = userPayBillService.selectListForApp(condition);
        // 过滤订单状态为已关闭的账单
        list = list.stream().filter(item -> item.getOrderStatus() != null && item.getOrderStatus().intValue() != OrderStatus.CLOSED.getType())
                .collect(Collectors.toList());
        List<AppPayBillVO> returnList = new ArrayList<>(8);
        for (UserPayBillVo vo : list) {
            AppPayBillVO billDto = new AppPayBillVO();
            billDto.setAmount(vo.getAmount());
            billDto.setBillId(vo.getId());
            billDto.setPayStatus(vo.getStatus());
            billDto.setOrderStatus(vo.getOrderStatus());
            billDto.setOrderStatusText(OrderStatus.getText(vo.getOrderStatus()));
            if (vo.getPayTime() != null) {
                billDto.setPayTime(DateUtil.format(vo.getPayTime(), DatePattern.NORM_DATETIME_PATTERN));
            }
            billDto.setOrderId(vo.getRefId());
            if (vo.getInstallFinishTime() != null) {
                billDto.setInstallFinishTime(DateUtil.format(vo.getInstallFinishTime(), DatePattern.NORM_DATETIME_PATTERN));
            }
            returnList.add(billDto);
        }
        return returnList;
    }

    /**
     * 报装
     *
     * @param orderId
     * @param expandStatus
     * @param expandFailReason
     * @param expandFailOperate
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void expand(String orderId, Integer expandStatus, String expandFailReason, Integer expandFailOperate) {
        OrderPO orderPO = orderService.getOrder(orderId);
        if (orderPO == null) {
            throw new ServiceException("该笔订单已报装过");
        }
        //原先一期逻辑
//        if (orderPO.getExpandStatus() != null) {
//            throw new ServiceException("该笔订单已报装过");
//        }
        TaskInfoResponse task = orderService.getTaskByWorkId(orderId);
        if (task == null) {
            throw new ServiceException("订单流程数据不存在");
        }
        if (!Convert.toInt(task.getNodeFlag()).equals(FlowNodeEnum.NODE_115.getKey())) {
            throw new ServiceException("订单不处于" + FlowNodeEnum.NODE_115.getValue());
        }
        LocalSubmitFlowBO bo = new LocalSubmitFlowBO();
        bo.setTaskId(task.getTaskId());
        bo.setProcessInsId(task.getProcessInsId());
        //使用超级管理流转
        bo.setSubmitUserId("admin");
        bo.setWorkId(orderId);
        bo.setCurrentNode(FlowNodeEnum.NODE_115.getKey());
        bo.setSubmitType(SubmitTypeEnum.TaskFlow);
        Map<String, Object> formData = new HashMap<>(3);
        formData.put(LambdaUtils.getFieldName(OrderPO::getExpandStatus), expandStatus);
        if (expandStatus != 1) {
            formData.put(LambdaUtils.getFieldName(OrderPO::getExpandFailReason), expandFailReason);
            formData.put(LambdaUtils.getFieldName(OrderPO::getExpandFailOperate), expandFailOperate);
        }
        bo.setFormData(formData);
        try {
            orderService.submitFlow(bo);
        } catch (Exception ex) {
            log.error("安装异常", ex);
            throw new ServiceException("安装异常:" + ex.getMessage());
        }
    }

    /**
     * 原先一期代码逻辑==》也没相应的需求，直接拷贝原先的代码逻辑
     *
     * @param sourceStatus
     * @param billStatus
     * @return
     */
    public Integer transformStatus(int sourceStatus, Integer billStatus) {
        List<Integer> receiveOrderStatus = new ArrayList<>();
        receiveOrderStatus.add(OrderStatus.DISPATCHING.getType());
        receiveOrderStatus.add(OrderStatus.SURVEY_RESERVING.getType());
        if (receiveOrderStatus.contains(sourceStatus)) {
            return 1;
        }
        List<Integer> surveyReserveStatus = new ArrayList<>();
        surveyReserveStatus.add(OrderStatus.SURVEY_DISPATCHED.getType());
        surveyReserveStatus.add(OrderStatus.SURVEY_RESERVED.getType());
        surveyReserveStatus.add(OrderStatus.SURVEY_RESERVE_EXCEPTION.getType());
        if (surveyReserveStatus.contains(sourceStatus)) {
            return 2;
        }
        List<Integer> surveyStatus = new ArrayList<>();
        surveyStatus.add(OrderStatus.SURVEYING.getType());
        surveyStatus.add(OrderStatus.SURVEY_FINISH.getType());
        surveyStatus.add(OrderStatus.SURVEY_EXAMINING.getType());
        surveyStatus.add(OrderStatus.SURVEY_EXAMINE_SUCCESS.getType());
        surveyStatus.add(OrderStatus.SURVEY_RECTIFYING.getType());
        surveyStatus.add(OrderStatus.SURVEY_RECTIFIED_EXAMINING.getType());
        if (surveyStatus.contains(sourceStatus)) {
            return 3;
        }
        List<Integer> installStatus = new ArrayList<>();
        installStatus.add(OrderStatus.INSTALL_RESERVE_EXCEPTION.getType());
        installStatus.add(OrderStatus.INSTALL_RESERVED.getType());
        installStatus.add(OrderStatus.INSTALL_CHANGE_EXAMINING.getType());
        installStatus.add(OrderStatus.INSTALL_CHANGE_EXAMINE_FAIL.getType());
        installStatus.add(OrderStatus.INSTALL_CHANGE_EXAMINE_SUCCESS.getType());
        installStatus.add(OrderStatus.INSTALLING.getType());
        installStatus.add(OrderStatus.INSTALL_FINISH.getType());
        installStatus.add(OrderStatus.INSTALL_EXAMINING.getType());
        installStatus.add(OrderStatus.INSTALL_RECTIFYING.getType());
        installStatus.add(OrderStatus.INSTALL_RECTIFY_EXAMINING.getType());
        if (installStatus.contains(sourceStatus)) {
            return 4;
        }
        // 未支付订单还是4
        if (billStatus != null && billStatus.intValue() == 0) {
            return 4;
        }
        return 5;
    }
}
