package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.date.DateUtil;
import com.boco.nbd.wios.flow.entity.qo.EvaluationStatisticsQo;
import com.boco.nbd.wios.flow.entity.bo.TaskInstBO;
import com.boco.nbd.wios.flow.entity.po.OrderBlockedPO;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.enums.EvaluationStatisticsEnum;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.mapper.def.EvaluationStatisticsMapper;
import com.boco.nbd.wios.flow.mapper.def.OrderBlockedMapper;
import com.boco.nbd.wios.flow.mapper.def.StatisticsMapper;
import com.boco.nbd.wios.flow.service.IEvaluationStatisticsService;
import com.boco.nbd.wios.flow.service.IHolidayService;
import com.boco.nbd.wios.flow.util.CalculateHours;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;




/**
 * @Description: TODO
 * @Author: ZQY
 * @Date: 2022/10/11
 **/
@Service
public class EvaluationStatisticsServiceImpl implements IEvaluationStatisticsService {

    @Autowired
    private EvaluationStatisticsMapper evaluationStatisticsMapper;

    @Autowired
    private StatisticsMapper statisticsMapper;

    @Autowired
    private IHolidayService holidayService;

    @Autowired
    private OrderBlockedMapper orderBlockedMapper;

    /**
     * 接单时效
     * @param qo
     * @return
     */
    @Override
    public float getOrderTakingTime(EvaluationStatisticsQo qo) {

        List<OrderPO> orderPOList=evaluationStatisticsMapper.getNormalOrderFiled(qo);
        return  getTime(orderPOList,EvaluationStatisticsEnum.Evaluation_OrderTakingTime.getValue());
    }

    /**
     * 返工单数
     * @param qo
     * @return
     */
    @Override
    public Integer getReworkOrderCount(EvaluationStatisticsQo qo) {


        List<OrderPO> orderPOList=evaluationStatisticsMapper.getNormalOrderFiled(qo);

        Integer count=0;
            for(OrderPO orderPO : orderPOList){

            List<TaskInstBO> taskInstBOList = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_111));

            if(taskInstBOList.size()>1){
                count++;
            }

        }

        return count;
    }

    /**
     * 勘测时效
     * @param qo
     * @return
     */
    @Override
    public float getOrderSurveyTime(EvaluationStatisticsQo qo) {
        qo.setTimeType("2");
        List<OrderPO> orderPOList=evaluationStatisticsMapper.getOrderFiled(qo);

        return  getTime(orderPOList,EvaluationStatisticsEnum.Evaluation_OrderSurveyTime.getValue());
    }

    /**
     * 勘测资料上传时效
     * @param qo
     * @return
     */
    @Override
    public float getOrderSurveyDataUploadTime(EvaluationStatisticsQo qo) {
        List<OrderPO> orderPOList=evaluationStatisticsMapper.getNormalOrderFiled(qo);

        return getTime(orderPOList,EvaluationStatisticsEnum.Evaluation_OrderSurveyDataUploadTime.getValue());
    }

    /**
     * 安装完成时效
     * @param qo
     * @return
     */
    @Override
    public float getOrderInstallationTime(EvaluationStatisticsQo qo) {
        qo.setTimeType("2");
        List<OrderPO> orderPOList=evaluationStatisticsMapper.getOrderFiled(qo);

        return  getTime(orderPOList,EvaluationStatisticsEnum.Evaluation_OrderInstallationTime.getValue());
    }

    /**
     * 安装资料上传时效
     * @param qo
     * @return
     */
    @Override
    public float getOrderInstallationDataUploadTime(EvaluationStatisticsQo qo) {
        List<OrderPO> orderPOList=evaluationStatisticsMapper.getNormalOrderFiled(qo);

        return getTime(orderPOList,EvaluationStatisticsEnum.Evaluation_OrderInstallationDataUploadTime.getValue());
    }

    /**
     *总体安装时效(无电力报装)
     * @param qo
     * @return
     */
    @Override
    public float getAllOrderInstallationNoElectricExpandTime(EvaluationStatisticsQo qo) {
        qo.setTimeType("2");
        qo.setElectricExpand("0");
        List<OrderPO> orderPOList=evaluationStatisticsMapper.getOrderFiled(qo);
        return getTime(orderPOList,EvaluationStatisticsEnum.Evaluation_AllOrderInstallationNoElectricExpandTime.getValue());
    }

    /**
     *总体安装时效(电力报装)
     * @param qo
     * @return
     */
    @Override
    public float getAllOrderInstallationElectricExpandTime(EvaluationStatisticsQo qo) {
        qo.setTimeType("2");
        qo.setElectricExpand("1");
        List<OrderPO> orderPOList=evaluationStatisticsMapper.getOrderFiled(qo);
        return getTime(orderPOList,EvaluationStatisticsEnum.Evaluation_AllOrderInstallationElectricExpandTime.getValue());
    }

    /**
     * 服务商累计安装完成率
     * @param qo
     * @return
     */
    @Override
    public float getInstallSupplierCompletionRate(EvaluationStatisticsQo qo) {

        List<OrderPO> orderPOList1=evaluationStatisticsMapper.getNormalOrderFiled(qo);
        qo.setNodeFlag( Arrays.asList(new String[]{"113","114","199"}));
        List<OrderPO> orderPOList2=evaluationStatisticsMapper.getNormalOrderFiled(qo);


        return orderPOList1.size()>0 ? (float)Math.round(((float)orderPOList2.size()/orderPOList1.size())*100)/100 : 0;
    }


    /**
     * 当月安装完成率
     * @param qo
     * @return
     */
    @Override
    public float getInstallCompletionRate(EvaluationStatisticsQo qo) {
        List<OrderPO> orderPOList1=evaluationStatisticsMapper.getNormalOrderFiled(qo);
        qo.setNodeFlag( Arrays.asList(new String[]{"113","114","199"}));
        List<OrderPO> orderPOList2=evaluationStatisticsMapper.getNormalOrderFiled(qo);

        return orderPOList1.size()>0 ? (float)Math.round(((float)orderPOList2.size()/orderPOList1.size())*100)/100 : 0;
    }

    /**
     * 统计未勘测、安装按约定时间到场数量
     * @param qo
     * @return
     */
    @Override
    public Integer getAppointmentTimeStatistics(EvaluationStatisticsQo qo) {
        List<OrderPO> orderPOList = evaluationStatisticsMapper.getNormalOrderFiled(qo);
        Integer count = 0;

        for (OrderPO po : orderPOList) {

            if(FlowNodeEnum.NODE_111.getKey() <= po.getNodeFlag()){
                //计算勘测预约   勘测入场时间为添加先注释
//                if(po.getSurveyStartTime() != null ){
//                    float time = po.getSurveyReserveTime().getTime() - po.getSurveyStartTime().getTime;
//                    time=(float) Math.round((time / (60 * 1000)/orderPOList.size()) * 100) / 100;
//
//                    if(time < 30){
//                        count++;
//                    }
//
//                }else{
//                    count++;
//                }

                //计算安装预约
                if(po.getInstallStartTime() != null ){
                    float time = po.getInstallReserveTime().getTime() - po.getInstallStartTime().getTime();
                    time=(float) Math.round((time / (60 * 1000)/orderPOList.size()) * 100) / 100;

                    if(time < 30){
                        count++;
                    }

                }else{
                    count++;
                }

            }


        }

        return count;
    }


    /**
     * 根据不同方法获取不一样的时间算法
     * @param orderPOList
     * @param methodType
     * @return
     * @throws ParseException
     */
    private float getTime(List<OrderPO> orderPOList,String methodType)  {

        List<String> holidayPOList= holidayService.getListByType("节假日");
        List<String> workList= holidayService.getListByType("调休工作日");
        String[] workdayArr = holidayPOList.toArray(new String[0]);
        String[] naturalArr = workList.toArray(new String[0]);

        float time=0;
        for(OrderPO orderPO:orderPOList){
            List<TaskInstBO> taskInstBOList = null;
            List<TaskInstBO> taskInstBOList1 = null;
            List<TaskInstBO> taskInstBOList2 = null;
            if(EvaluationStatisticsEnum.Evaluation_OrderTakingTime.getValue().equals(methodType)){
                taskInstBOList = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_103));
            }else if(EvaluationStatisticsEnum.Evaluation_OrderSurveyTime.getValue().equals(methodType)){
                taskInstBOList = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_106));
            }else if(EvaluationStatisticsEnum.Evaluation_OrderSurveyDataUploadTime.getValue().equals(methodType)){
                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_106));
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_107));
            }else if(EvaluationStatisticsEnum.Evaluation_OrderInstallationTime.getValue().equals(methodType)){
                taskInstBOList = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_111));
            }else if(EvaluationStatisticsEnum.Evaluation_OrderInstallationDataUploadTime.getValue().equals(methodType)){
                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_111));
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_112));
            }else if(EvaluationStatisticsEnum.Evaluation_AllOrderInstallationNoElectricExpandTime.getValue().equals(methodType)||EvaluationStatisticsEnum.Evaluation_AllOrderInstallationElectricExpandTime.getValue().equals(methodType)){
                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_101));
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.getFlowNodeStr(FlowNodeEnum.NODE_112));
            }

            if(taskInstBOList1 != null && taskInstBOList2 != null){
                if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){
                    if(EvaluationStatisticsEnum.Evaluation_AllOrderInstallationNoElectricExpandTime.getValue().equals(methodType) || EvaluationStatisticsEnum.Evaluation_AllOrderInstallationElectricExpandTime.getValue().equals(methodType)){
                        //需要排除掉受阻时间
                        float workeTime = taskInstBOList2.get(0).getStartTime().getTime() - taskInstBOList1.get(0).getStartTime().getTime();
                        float blockedTime = getBlockedTime(orderPO);
                        time += (workeTime-blockedTime);

                    }else {
                        //自然日算法  单位毫秒
                        time += taskInstBOList2.get(0).getStartTime().getTime() - taskInstBOList1.get(0).getStartTime().getTime();
                    }
                }
            }

            if(taskInstBOList != null) {
                if (taskInstBOList.size() > 0) {
                    if (EvaluationStatisticsEnum.Evaluation_OrderSurveyTime.getValue().equals(methodType)) {
                        //105节点提交至106节点的时间-104节点填写的预约勘测时间
                        time += taskInstBOList.get(0).getStartTime().getTime() - orderPO.getSurveyReserveTime().getTime();
                    } else if (EvaluationStatisticsEnum.Evaluation_OrderInstallationTime.getValue().equals(methodType)) {
                        //110节点提交至111节点的时间-108节点填写的预约安装时间
                        time += taskInstBOList.get(0).getStartTime().getTime() - orderPO.getInstallReserveTime().getTime();
                    } else if (EvaluationStatisticsEnum.Evaluation_OrderTakingTime.getValue().equals(methodType)) {
                        //预约首联时间减去-103最后一个开始时间（计算工作日）
                        time += CalculateHours.calculateHour(DateUtil.formatDateTime(taskInstBOList.get(0).getStartTime()), DateUtil.formatDateTime(orderPO.getSurveyConnectTime()), workdayArr, naturalArr);
                    }
                }
            }
        }

        if(EvaluationStatisticsEnum.Evaluation_OrderTakingTime.getValue().equals(methodType)){
            time=(float)(Math.round((time/orderPOList.size())*100))/100;
        }else{
            //换算小时
            time=(float) Math.round((time / (60 * 60 * 1000)/orderPOList.size()) * 100) / 100;
        }

        return  time;
    }


    /**
     * 计算受阻时间
     * @param orderPO
     * @return
     */
    private float getBlockedTime(OrderPO orderPO ){

        float time=0;
        List<Integer> integerList=new ArrayList<>();
        List<OrderBlockedPO> orderBlockedPOList=orderBlockedMapper.getAllByIdList(orderPO.getId(),"");

        //筛选出没有受阻的位置
        if(orderBlockedPOList.size()>0){
            for(int i=0;i<orderBlockedPOList.size();i++){
                if("否".equals(orderBlockedPOList.get(i).getWhether())){
                    integerList.add(i);
                }
            }

            //计算时间
            for(int b=0;b<integerList.size();b++){
                if(b==0){
                    int index=integerList.get(0);
                    time += orderBlockedPOList.get(index).getUpdateTime().getTime()-orderBlockedPOList.get(0).getUpdateTime().getTime();
                }else{
                    //找出上一个为“否”的数据坐标+1（找出受阻订单为“是”的开头）
                    int indexStart=integerList.get(b-1)+1;
                    int indexEnd=integerList.get(b);
                    time +=orderBlockedPOList.get(indexEnd).getUpdateTime().getTime()-orderBlockedPOList.get(indexStart).getUpdateTime().getTime();
                }
            }
        }

        return time;
    }

}
