package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.util.StrUtil;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.wios.flow.entity.bo.FlowVO;
import com.boco.nbd.wios.flow.entity.po.FlowPO;
import com.boco.nbd.wios.flow.mapper.def.FlowMapper;
import com.boco.nbd.wios.flow.service.IFlowService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author:cao hai
 * @date:2022/6/22 14:58
 * @version:V1.0
 * @description: FlowServiceImpl
 * @modify:
 */
@Service
public class FlowServiceImpl extends BaseDaoService3<FlowPO, FlowMapper> implements IFlowService {

    /**
     * 查询所有功能
     *
     * @return
     */
    @Override
    public List<String> selectAllCategory() {
        List<FlowPO> list = selectAll();
        return list.stream().map(FlowPO::getCategoryName).distinct().collect(Collectors.toList());
    }

    /**
     * 核查流程标识
     *
     * @param po
     * @return
     */
    @Override
    public boolean checkFlow(FlowPO po) {
        if (po == null) {
            return false;
        }
        List<FlowPO> flowList = selectAll();
        for (FlowPO flow : flowList) {
            if (flow.getFlowCode().equals(po.getFlowCode()) || flow.getFlowName().equals(po.getFlowName())) {
                return false;
            }
        }
        return true;
    }

    /**
     * 查询所有流程
     *
     * @return
     */
    @Override
    public List<FlowVO> selectAllFlow() {
        List<FlowPO> list = selectAll();
        List<FlowVO> result = new ArrayList<>(2);
        list.stream().forEach(s -> {
            result.add(s.toVo());
        });
        return result;
    }

    /**
     * 根据名称查询流程
     *
     * @param categoryName
     * @param flowName
     * @return
     */
    @Override
    public List<FlowPO> getFlowByName(String categoryName, String flowName) {
        List<FlowPO> list = selectAll();
        List<FlowPO> tempList = new ArrayList<>();
        if (StrUtil.isNotEmpty(categoryName)) {
            tempList.addAll(list.stream().filter(po -> !po.getCategoryName().contains(categoryName))
                    .collect(Collectors.toList()));
            list.removeAll(tempList);
        }
        if (StrUtil.isNotEmpty(flowName)) {
            tempList = (list.stream().filter(po -> !po.getFlowName().contains(flowName))
                    .collect(Collectors.toList()));
            list.removeAll(tempList);
        }

        return list;
    }
}
