package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.common.excel.ExcelImporter;
import com.boco.nbd.framework.common.excel.ExcelParams;
import com.boco.nbd.framework.common.util.IdUtil;
import com.boco.nbd.framework.parent.microservice.exception.ErrorInfoException;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.wios.flow.entity.po.HolidayPO;
import com.boco.nbd.wios.flow.entity.po.PlanTaskPO;
import com.boco.nbd.wios.flow.mapper.def.HolidayMapper;
import com.boco.nbd.wios.flow.service.IHolidayService;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;

/**
*
*/
@Service
@Slf4j
public class HolidayServiceImpl extends BaseDaoService3<HolidayPO, HolidayMapper> implements IHolidayService {

    @Autowired
    private  HolidayMapper holidayMapper;

    @Override
    public List<HolidayPO> getList() {
        int year=DateUtil.year(new Date());
        return holidayMapper.getList(year+"");
    }

    @Override
    public ResponseMessage2<Boolean> importExcel(MultipartFile file) {
        try {
            LinkedHashMap<String, String> fieldMap = new LinkedHashMap<>(8);
            ExcelParams<HolidayPO> et = new ExcelParams(file.getOriginalFilename(), file.getInputStream(), fieldMap);
            et.setDateformat("yyyy-MM-dd");
            et.setEntityClass(HolidayPO.class);
            fieldMap.put("时间", "holidayDate");
            fieldMap.put("时间说明", "holidayDesc");
            fieldMap.put("时间类型", "holidayType");
            et.setFieldMap(fieldMap);
            List<HolidayPO> list = ExcelImporter.excelToList(et);

            for(int i=0;i<list.size();i++){
                if (list.get(i).getHolidayDate()==null){
                    throw new ErrorInfoException("第"+(i+1)+"行时间为空！");
                }
                if(StringUtils.isBlank(list.get(i).getHolidayDesc())){
                    throw new ErrorInfoException("第"+(i+1)+"行主机厂为空！");
                }
                if(StringUtils.isBlank(list.get(i).getHolidayType())){
                    throw new ErrorInfoException("第"+(i+1)+"行时间类型为空！");
                }

                list.get(i).setId(IdUtil.getLongUUID()+"");

                Example example = new Example(HolidayPO.class);
                Example.Criteria criteria = example.createCriteria();
                criteria.andEqualTo(LambdaUtils.getFieldName(HolidayPO::getHolidayDate), list.get(i).getHolidayDate().substring(0,10));
                List<HolidayPO> holidayPOList= selectByExample(example);
                if(CollUtil.isNotEmpty(holidayPOList)){
                    throw new ErrorInfoException("第"+(i+1)+"行存在（"+list.get(i).getHolidayDate()+")重复数据！");
                }

            }

            for(HolidayPO holidayPO:list){
                holidayPO.setHolidayDate(holidayPO.getHolidayDate().substring(0,10));
                holidayMapper.insert(holidayPO);
            }

        }catch (Exception e){
            e.printStackTrace();
            log.error("节日配置导入失败：{}",e.getMessage());
            return ResponseMessage2.Failed("节日配置导入失败："+e.getMessage());
        }
        return ResponseMessage2.Success();
    }

    @Override
    public List<String> getListByType(String holidayType) {
        int year=DateUtil.year(new Date());
        return holidayMapper.getListByType(holidayType,year+"");
    }
}
