package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.common.excel.ExcelImporter;
import com.boco.nbd.framework.common.excel.ExcelParams;
import com.boco.nbd.framework.parent.microservice.exception.ErrorInfoException;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.wios.flow.entity.bo.OrderTempBO;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.entity.po.OrderTempPO;
import com.boco.nbd.wios.flow.mapper.def.ImportOrderMapper;
import com.boco.nbd.wios.flow.service.IImportOrderService;
import com.boco.nbd.wios.flow.service.IOrderService;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.flow.util.ValidUtil;
import com.boco.nbd.wios.manage.entity.bo.Dictionary;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import com.boco.nbd.wios.manage.entity.bo.OemContractVo;
import com.boco.nbd.wios.manage.entity.bo.Region;
import com.boco.nbd.wios.manage.entity.cams.enums.ContractStatus;
import com.boco.nbd.wios.manage.mapper.def.DictionaryDao;
import com.boco.nbd.wios.manage.mapper.def.OemContractDao;
import com.boco.nbd.wios.manage.mapper.def.OemDao;
import com.boco.nbd.wios.manage.mapper.def.RegionDao;
import com.boco.nbd.wios.manage.util.NumberUtil;
import com.ihidea.core.support.session.SessionInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import javax.annotation.Resource;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: TODO
 * @Author: ZQY
 * @Date: 2022/7/12
 **/
@Service
@Slf4j
public class ImportOrderServiceImpl extends BaseDaoService3<OrderTempPO, ImportOrderMapper> implements IImportOrderService {

    @Autowired
    private DictionaryDao dicDao;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private OemContractDao oemContractDao;

    @Autowired
    private RegionDao regionDao;

    @Resource
    private ImportOrderMapper importOrderMapper;

    @Autowired
    private IOrderService orderService;

    /**
     * 订单导入
     * @param file
     * @param type
     * @return
     */
    @Override
    public ResponseMessage2 importExcel(MultipartFile file, Integer type) {
        try {
            Map<String, Map<Object, String>> maps = orderService.createEnumMap();
            LinkedHashMap<String, String> fieldMap = new LinkedHashMap<>();
            ExcelParams<OrderTempPO> et = new ExcelParams(file.getOriginalFilename(), file.getInputStream(), fieldMap);
            et.setDateformat("yyyy-MM-dd");
            et.setEntityClass(OrderTempPO.class);
            fieldMap.put("主机厂/业务名称", "oemName");
            fieldMap.put("主机厂/业务代码", "oemCode");
            fieldMap.put("区域", "regionName");
            fieldMap.put("安装地址", "address");
            fieldMap.put("车主姓名", "userName");
            fieldMap.put("车主性别", "userSex");
            fieldMap.put("车主电话", "userPhone");
            fieldMap.put("是否为售车前预勘测", "surveyBeforeSaleEnable");
            fieldMap.put("车型", "vehicleModel");
            fieldMap.put("车辆底盘号", "vehicleChassisNo");
            fieldMap.put("墙盒品牌", "wallboxModel");
            fieldMap.put("车位状态", "vehicleParkingStatus");
            fieldMap.put("车位号", "vehicleParkingNo");
            fieldMap.put("车企安装单号", "outOrderId");
            fieldMap.put("发票日期", "invoiceTime");
            fieldMap.put("经销商名称", "oemAgentName");
            fieldMap.put("经销商联系人", "oemAgentContactName");
            fieldMap.put("经销商联系方式", "oemAgentContactPhone");
            fieldMap.put("经销商信息", "oemAgentInfo");
            fieldMap.put("小区类型", "estateType");
            fieldMap.put("是否有独立车位", "hasIndependentParking");
            fieldMap.put("室内/室外", "installPosition");
            fieldMap.put("物业是否同意安装", "propertyAgree");
            fieldMap.put("是否需要电力报装", "electricExpand");
            fieldMap.put("是否需要立柱", "needErectPile");
            fieldMap.put("是否需要人工派单", "manuDispatchEnable");
            fieldMap.put("是否需要勘测", "isNeedSurvey");
            fieldMap.put("安装紧急程度", "installEmergencyLevel");
            et.setFieldMap(fieldMap);
            List<OrderTempPO> list = ExcelImporter.excelToList(et);

            List<Dictionary> dicList = dicDao.selectValidByType("wallBox", null);
            Map<String, String> dicMap = dicList.stream().collect(Collectors.toMap(Dictionary::getName, Dictionary::getValue));


            for (int i = 0; i < list.size(); i++) {
                if (StringUtils.isBlank(list.get(i).getUserName())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行车主姓名为空！");
                } else if (!ValidUtil.isMobileSimple(list.get(i).getUserPhone())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行车主电话有误！");
                } else if (StringUtils.isBlank(list.get(i).getVehicleModel())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行车型为空！");
                } else if (StringUtils.isBlank(list.get(i).getOemName())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行主机厂/业务名称为空！");
                } else if (list.get(i).getManuDispatchEnable() == null) {
                    throw new ErrorInfoException("第" + (i + 1) + "行是否需要人工派单为空！");
                } else if (StringUtils.isBlank(list.get(i).getRegionName())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行区域为空！");
                } else if (StringUtils.isBlank(list.get(i).getAddress())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行安装地址为空！");
                } else if (list.get(i).getSurveyBeforeSaleEnable() == null) {
                    throw new ErrorInfoException("第" + (i + 1) + "行是否为售车前预勘测为空！");
                } else if (list.get(i).getIsNeedSurvey() == null) {
                    throw new ErrorInfoException("第" + (i + 1) + "行是否需要勘测为空！");
                } else if (list.get(i).getInstallEmergencyLevel() == null) {
                    throw new ErrorInfoException("第" + (i + 1) + "行安装紧急程度为空！");
                } else if (StringUtils.isBlank(list.get(i).getWallboxModel())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行墙合品牌不能为空！");
                } else if (StringUtils.isBlank(list.get(i).getOemCode())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行主机厂/业务代码不能为空！");
                }


                String wallboxValue = dicMap.get(list.get(i).getWallboxModel());
                if (StringUtils.isBlank(wallboxValue)) {
                    throw new ErrorInfoException("第" + (i + 1) + "行墙合品牌不存在！");
                }

                List<Oem> oemList = oemDao.selectByNameAndCode(list.get(i).getOemName(), list.get(i).getOemCode());
                if (oemList.isEmpty()) {
                    throw new ErrorInfoException("第" + (i + 1) + "行无此主机厂或已被禁用！");
                }

                Integer oemId = oemList.get(0).getId();
                list.get(i).setOemId(oemId);
                final int tempOemId = oemId.intValue();

                if (!oemList.get(0).getCode().equals(list.get(i).getOemCode())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行主机厂名称与主机厂代码不匹配！");
                }

                if (!StringUtils.isBlank(list.get(i).getOemAgentName())) {
                    List<Oem> oemAgentList = oemDao.selectByName(list.get(i).getOemAgentName());
                    list.get(i).setOemAgentId(oemAgentList.get(0).getId());
                    oemAgentList = oemAgentList.stream().filter(
                            item -> item.getParentId() != null && item.getParentId().intValue() == tempOemId && item.getStatus().intValue() == 1)
                            .collect(Collectors.toList());
                    if (oemAgentList.isEmpty()) {
                        throw new ErrorInfoException("第" + (i + 1) + "行无此经销商或已被禁用！");
                    }
                }

                List<OemContractVo> oemContractList = oemContractDao.selectByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
                if (oemContractList.isEmpty()) {
                    throw new ErrorInfoException("第" + (i + 1) + "行该主机厂/经销商没有生效中的合约！");
                }


                Region region = regionDao.selectSingleByName(list.get(i).getRegionName());
                if (region == null) {
                    throw new ErrorInfoException("第" + (i + 1) + "行没有找到'" + list.get(i).getRegionName() + "'区域！");
                }

                list.get(i).setRegionId(region.getId() + "");
                list.get(i).setId("CA" + NumberUtil.getRandomOrderId(5));
            }


            for (OrderTempPO orderTempPO : list) {
                //是否枚举值转换
                Map<Object, String> yesOrNoMap = maps.get("yesOrNo");
                for (Object key : yesOrNoMap.keySet()) {
                    if (yesOrNoMap.get(key).equals(orderTempPO.getSurveyBeforeSaleEnable())) {
                        orderTempPO.setSurveyBeforeSaleEnable(key.toString());
                    }
                    if (yesOrNoMap.get(key).equals(orderTempPO.getHasIndependentParking())) {
                        orderTempPO.setHasIndependentParking(key.toString());
                    }
                    if (yesOrNoMap.get(key).equals(orderTempPO.getElectricExpand())) {
                        orderTempPO.setElectricExpand(key.toString());
                    }
                    if (yesOrNoMap.get(key).equals(orderTempPO.getNeedErectPile())) {
                        orderTempPO.setNeedErectPile(key.toString());
                    }
                    if (yesOrNoMap.get(key).equals(orderTempPO.getManuDispatchEnable())) {
                        orderTempPO.setManuDispatchEnable(key.toString());
                    }
                    if (yesOrNoMap.get(key).equals(orderTempPO.getIsNeedSurvey())) {
                        orderTempPO.setIsNeedSurvey(key.toString());
                    }
                }

                //车主性别
                Map<Object, String> userSexMap = maps.get("userSex");
                for (Object key : userSexMap.keySet()) {
                    if (userSexMap.get(key).equals(orderTempPO.getUserSex())) {
                        orderTempPO.setUserSex(key.toString());
                    }
                }


                //小区类别
                Map<Object, String> estateTypeMap = maps.get("houseType");
                for (Object key : estateTypeMap.keySet()) {
                    if (estateTypeMap.get(key).equals(orderTempPO.getEstateType())) {
                        orderTempPO.setEstateType(key.toString());
                    }
                }


                //室内、室外
                Map<Object, String> installPositionMap = maps.get("installPosition");
                for (Object key : installPositionMap.keySet()) {
                    if (installPositionMap.get(key).equals(orderTempPO.getInstallPosition())) {
                        orderTempPO.setInstallPosition(key.toString());
                    }
                }

                //物业是否同意安装
                Map<Object, String> propertyAgreeMap = maps.get("propertyAgree");
                for (Object key : propertyAgreeMap.keySet()) {
                    if (propertyAgreeMap.get(key).equals(orderTempPO.getPropertyAgree())) {
                        orderTempPO.setPropertyAgree(key.toString());
                    }
                }


                //安装紧急程度
                Map<Object, String> installEmergencyLevelMap = maps.get("installEmergencyLevel");
                for (Object key : installEmergencyLevelMap.keySet()) {
                    if (installEmergencyLevelMap.get(key).equals(orderTempPO.getInstallEmergencyLevel())) {
                        orderTempPO.setInstallEmergencyLevel(key.toString());
                    }
                }
                orderTempPO.setType(type);
                SessionInfo user = ProcessUtil.getUserInfo();
                if (user != null) {
                    orderTempPO.setCreateAccount(user.getUserId());
                    orderTempPO.setCreateAccountName(user.getUserName());
                }
                importOrderMapper.insertSelective(orderTempPO);
            }


        } catch (Exception e) {
            e.printStackTrace();
            log.error("订单导入失败：{}", e.getMessage());
            return ResponseMessage2.Failed("订单导入失败：" + e.getMessage());
        }

        return ResponseMessage2.Success();
    }

    /**
     * 查询列表
     * @param type
     * @return
     */
    @Override
    public List<OrderTempBO> getImportOrderAll(Integer type) {
        return importOrderMapper.getImportOrderAll(type);
    }

    /**
     * 删除导入订单
     * @param ids
     * @return
     */
    @Override
    public ResponseMessage2 deleteImportOrder(List<String> ids) {
        Example example = new Example(OrderTempPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andIn(LambdaUtils.getFieldName(OrderTempPO::getId), ids);
        importOrderMapper.deleteByExample(example);
        return ResponseMessage2.Success();
    }

    /**
     * 生成订单
     * @param ids
     * @param stateAll
     */
    @Override
    public void addOrder(List<String> ids, String stateAll) {

        Example example = new Example(OrderTempPO.class);
        Example.Criteria criteria = example.createCriteria();

        if ("1".equals(stateAll)) {
            SessionInfo user = ProcessUtil.getUserInfo();
            if (user != null) {
                criteria.andEqualTo(LambdaUtils.getFieldName(OrderTempPO::getCreateAccount), user.getUserId());
            }
        } else {
            criteria.andIn(LambdaUtils.getFieldName(OrderTempPO::getId), ids);
        }

        List<OrderTempPO> orderTempPOList = selectByExample(example);
        for (OrderTempPO orderTempPO : orderTempPOList) {
            OrderPO orderPO = new OrderPO();
            BeanUtil.copyProperties(orderTempPO, orderPO);
            orderService.startFlow(orderPO);
            importOrderMapper.delete(orderTempPO);
        }

    }
}
