package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.wios.flow.entity.FlowConstant;
import com.boco.nbd.wios.flow.entity.bo.*;
import com.boco.nbd.wios.flow.entity.po.*;
import com.boco.nbd.wios.flow.entity.qo.FlowNodeUserQo;
import com.boco.nbd.wios.flow.entity.qo.NodeQo;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.mapper.def.NodeMapper;
import com.boco.nbd.wios.flow.service.INodeService;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author:cao hai
 * @date:2022/6/22 16:19
 * @version:V1.0
 * @description:NodeServiceImpl
 * @modify:
 */
@Service
public class NodeServiceImpl extends BaseDaoService3<NodePO, NodeMapper> implements INodeService {

    /**
     * 主机厂首联人缓存
     */
    private static ConcurrentHashMap<Integer, ConcurrentHashMap<String, ConnectSelectBO>> omeConnectConfig;

    /**
     * 核查Node
     *
     * @param po
     * @return
     */
    @Override
    public boolean checkNode(NodePO po) {
        if (po == null) {
            return false;
        }
        Example example = new Example(NodePO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(NodePO::getFlowId), po.getFlowId());
        List<NodePO> nodeList = selectByExample(example);
        for (NodePO node : nodeList) {
            if (node.getNodeCode().equals(po.getNodeCode()) || node.getNodeName().equals(po.getNodeName())) {
                return false;
            }
        }

        return true;
    }

    /**
     * 查询节点数据
     *
     * @param qo
     * @return
     */
    @Override
    public List<NodeVO> searchNode(NodeQo qo) {
        List<FlowNodePO> list = this.mapper.selectByFlowIdAndNodeCode(null, null);
        List<FlowNodePO> tempList = new ArrayList<>();
        if (StrUtil.isNotEmpty(qo.getCategoryName())) {
            tempList.addAll(list.stream().filter(po -> !po.getCategoryName().contains(qo.getCategoryName()))
                    .collect(Collectors.toList()));
            list.removeAll(tempList);
        }
        if (StrUtil.isNotEmpty(qo.getNodeName())) {
            tempList = (list.stream().filter(po -> !po.getNodeName().contains(qo.getNodeName()))
                    .collect(Collectors.toList()));
            list.removeAll(tempList);
        }
        if (qo.getFlowId() != null) {
            tempList = (list.stream().filter(po -> !po.getFlowId().equals(qo.getFlowId()))
                    .collect(Collectors.toList()));
            list.removeAll(tempList);
        }
        List<NodeVO> result = new ArrayList<>(4);
        list.stream().forEach(s -> {
            NodeVO vo = new NodeVO();
            BeanUtil.copyProperties(s, vo);
            Long nodeId = s.getNodeId();
            List<NodeGroupBO> groups = getNodeGroupsInfo(nodeId);
            vo.setAreConfig(!groups.isEmpty());
            result.add(vo);
        });
        return result;
    }

    /**
     * 获取流程节点配置的用户组
     *
     * @param nodeId
     * @return
     */
    @Override
    public List<NodeGroupBO> getNodeGroupsInfo(Long nodeId) {
        return this.mapper.getNodeGroupsInfo(nodeId);
    }

    /**
     * 更新节点配置的用户组
     *
     * @param vo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateGroupNode(NodeGroupVO vo) {
        this.mapper.deleteGroupNode(vo.getNodeId());
        if (vo.getGroupIds().isEmpty()) {
            return;
        }
        String[] items = StrUtil.split(vo.getGroupIds(), ",");
        for (String item : items) {
            Long groupId = Convert.toLong(item, 0L);
            if (groupId != 0) {
                mapper.bindGroupNode(vo.getNodeId(), groupId);
            }
        }
    }

    /**
     * 根据流程ID与节点id查数据
     *
     * @param flowCode
     * @param nodeCode
     * @return
     */
    @Override
    public FlowNodePO selectDataByFlowAndNode(String flowCode, String nodeCode) {
        if (StringUtils.isBlank(flowCode) || StringUtils.isBlank(nodeCode)) {
            return null;
        }
        List<FlowNodePO> nodeList = this.mapper.selectByFlowIdAndNodeCode(flowCode, nodeCode);
        if (null == nodeList || nodeList.isEmpty()) {
            return null;
        }
        FlowNodePO po = nodeList.get(0);
        List<NodeGroupBO> groupBOS = this.mapper.getNodeGroupsInfo(po.getNodeId());
        po.setGroups(groupBOS);
        return po;
    }

    /**
     * 更新用户组绑定用户
     *
     * @param groupId
     * @param userIds
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateGroupUsers(Long groupId, String userIds) {
        if (groupId == null) {
            return false;
        }
        this.mapper.deleteGroupUser(groupId);
        if (StrUtil.isEmpty(userIds)) {
            return true;
        }
        String[] items = StrUtil.split(userIds, ",");
        for (String item : items) {
            Long userId = Convert.toLong(item, 0L);
            if (userId != 0) {
                mapper.bindGroupUser(groupId, userId);
            }
        }
        return true;
    }

    /**
     * 加载用户
     *
     * @param groupId
     * @param vendorId
     * @return
     */
    @Override
    public List<UserVO> loadUsers(Long groupId, Integer vendorId) {
        List<UserPO> list = (null == groupId) ? this.mapper.loadUsers(vendorId) : this.mapper.getUserByGroup(groupId);
        return loadUserData(list);
    }

    /**
     * 获取厂家信息
     *
     * @return
     */
    @Override
    public List<VendorPO> getSelectVendorInfo() {
        List<VendorPO> result = new ArrayList<>(10);
        result.add(0, VendorPO.getAllItem());
        result.add(1, VendorPO.getCamsItem());
        result.addAll(this.mapper.getSelectVendorInfo());
        return result;
    }

    /**
     * 加载节点用户信息
     *
     * @param nodeCode
     * @param vendorId
     * @param vendorId
     * @return
     */
    @Override
    public List<FlowNodeUserPO> getFlowNodeUserInfo(Integer nodeCode, Integer vendorId, String userName) {
        FlowNodeUserQo qo = FlowNodeUserQo.init(nodeCode, vendorId);
        qo.setUserName(userName);
        return this.mapper.getFlowNodeUserInfo(qo);
    }

    /**
     * 处理数据
     *
     * @param list
     * @return
     */
    private List<UserVO> loadUserData(List<UserPO> list) {
        //按厂家分组
        List<UserVO> items = new ArrayList<>(10);
        List<Integer> vendorType = list.stream().map(UserPO::getVendorType).distinct().collect(Collectors.toList());
        vendorType.stream().forEach(m -> {
            List<UserPO> data = list.stream().filter(s -> s.getVendorType().equals(m)).collect(Collectors.toList());
            Map<Long, String> vendors = data.stream().collect(Collectors.toMap(UserPO::getVendorId, UserPO::getVendorName, (s1, s2) -> s1));
            for (Map.Entry<Long, String> entry : vendors.entrySet()) {
                UserVO vo = new UserVO();
                vo.setVendorId(entry.getKey());
                vo.setVendorName(entry.getValue());
                List<UserPO> users = data.stream().filter(n -> n.getVendorId().equals(entry.getKey())).collect(Collectors.toList());
                List<UserBO> userList = users.stream().sorted(Comparator.comparing(UserPO::getRoleId)).map(UserPO::toBo).collect(Collectors.toList());
                vo.setUsers(userList);
                items.add(vo);
            }
        });

        return items;
    }

    /**
     * 获取首联配置信息
     *
     * @return
     */
    @Override
    public List<ConnectConfigBO> getConnectConfig() {
        return mapper.getConnectConfig(FlowConstant.FLOW_CODE, FlowNodeEnum.NODE_101.getKey() + "", null);
    }

    /**
     * 首联配置信息
     *
     * @param item
     */
    @Override
    public void saveConnectConfig(ConnectConfigBO item) {
        mapper.deleteConnectConfig(item);
        mapper.bindConnectConfig(item);
    }

    /**
     * 获取首联人
     *
     * @param id    :工单
     * @param oemId :主机厂Id
     * @return
     */
    @Override
    public synchronized ConnectUserConfigBO getConnectUser(String id, Integer oemId) {
        if (omeConnectConfig == null || CollUtil.isEmpty(omeConnectConfig)) {
            List<ConnectConfigBO> items = getConnectConfig();
            List<Integer> vendors = Linq.of(items).select(ConnectConfigBO::getVendorId).distinct().toList();
            omeConnectConfig = new ConcurrentHashMap<>(vendors.size());
            for (Integer vendor : vendors) {
                omeConnectConfig.put(vendor, ConnectSelectBO.create(Linq.of(items).where(s -> s.getVendorId().equals(vendor)).toList()));
            }
        }
        if (!omeConnectConfig.containsKey(oemId)) {
            throw new FlowException("未配置首联人.");
        }
        if (StrUtil.isEmpty(id)) {
            throw new FlowException("安装单号为空.");
        }
        String lastStr = id.substring(id.length() - 1);
        if (!omeConnectConfig.get(oemId).containsKey(lastStr) || CollUtil.isEmpty(omeConnectConfig.get(oemId).get(lastStr).getItems())) {
            throw new FlowException("未配置处理工单尾号为" + lastStr + "首联人.");
        }
        int index = omeConnectConfig.get(oemId).get(lastStr).getIndex();
        int maxIndex = omeConnectConfig.get(oemId).get(lastStr).getItems().size();
        int setIndex = (maxIndex == 0 || index == maxIndex - 1) ? 0 : index + 1;
        omeConnectConfig.get(oemId).get(lastStr).setIndex(setIndex);
        return omeConnectConfig.get(oemId).get(lastStr).getItems().get(setIndex);
    }
}
