package com.boco.nbd.wios.flow.service.impl;

import com.boco.nbd.wios.flow.entity.Constant;
import com.boco.nbd.wios.flow.entity.bo.AppointmentUserBO;
import com.boco.nbd.wios.flow.entity.bo.BusinessRefusedBO;
import com.boco.nbd.wios.flow.entity.bo.TimeConfirmBO;
import com.boco.nbd.wios.flow.entity.bo.TimeNotifyBO;
import com.boco.nbd.wios.flow.service.INoticeService;
import com.boco.nbd.wios.flow.sms.aliyun.entity.NoticeData;
import com.boco.nbd.wios.flow.util.NoticeUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;

/**
 * @ClassName NoticeServiceImpl
 * @Description 阿里云短信通知 业务实现
 * @Author songyanfei
 * @Email songyanfei@boco.com.cn
 * @Date 2022/10/31 10:06
 * @Version V1.0
 */
@Service
public class NoticeServiceImpl implements INoticeService {

    private static final String NAME = "name";
    private static final String USER_PHONE = "phonenumber";
    private static final String ARRIVAL_TIME = "time";
    private static final String BUSINESS_WORK_ID = "Workid";
    private static final String BEFORE_TIME = "oldtime";
    private static final String AFTER_TIME = "newtime";
    private static final String NOTICE_WORK_ID = "Workid";
    private static final String CHANGE_TIME = "time";

    @Resource
    private NoticeUtil noticeUtil;

    /**
     * 预约用户短信模板
     *
     * @param appointmentUser 预约用户信息
     * @param phone           接收人手机号码
     * @return 是否发送成功
     */
    @Override
    public boolean appointmentUser(AppointmentUserBO appointmentUser, String phone) {
        NoticeData noticeData = new NoticeData();
        // 预约用户短信模板
        noticeData.setType(Constant.APPOINTMENT_USER);
        // 设置短信模板的参数
        HashMap<String, String> params = new HashMap<>(3);
        params.put(ARRIVAL_TIME, appointmentUser.getArrivalTime());
        params.put(NAME, appointmentUser.getName());
        params.put(USER_PHONE, appointmentUser.getPhone());
        noticeData.setParams(params);
        return noticeUtil.send(noticeData,phone);
    }

    /**
     * 服务商拒单通知模板
     *
     * @param businessRefused 服务商拒单信息
     * @param phone           接收人手机号码
     * @return 是否发送成功
     */
    @Override
    public boolean businessRefused(BusinessRefusedBO businessRefused, String phone) {
        NoticeData noticeData = new NoticeData();
        // 服务商拒单通知模板
        noticeData.setType(Constant.BUSINESS_REFUSED);
        // 设置短信模板的参数
        HashMap<String, String> params = new HashMap<>(3);
        params.put(BUSINESS_WORK_ID, businessRefused.getWorkId());
        noticeData.setParams(params);
        return noticeUtil.send(noticeData,phone);
    }

    /**
     * 安装预约时间变更确认模板
     *
     * @param timeConfirm 安装预约时间变更确认信息
     * @param phone       接收人手机号码
     * @return 是否发送成功
     */
    @Override
    public boolean appointmentTimeConfirm(TimeConfirmBO timeConfirm, String phone) {
        NoticeData noticeData = new NoticeData();
        // 安装预约时间变更确认模板
        noticeData.setType(Constant.APPOINTMENT_TIME_CONFIRM);
        // 设置短信模板的参数
        HashMap<String, String> params = new HashMap<>(3);
        params.put(BEFORE_TIME, timeConfirm.getBeforeTime());
        params.put(AFTER_TIME, timeConfirm.getAfterTime());
        noticeData.setParams(params);
        return noticeUtil.send(noticeData,phone);
    }

    /**
     * 安装预约时间变更通知模板
     *
     * @param timeNotify 安装预约时间变更通知模板
     * @param phone      接收人手机号码
     * @return 是否发送成功
     */
    @Override
    public boolean appointmentTimeNotify(TimeNotifyBO timeNotify, String phone) {
        NoticeData noticeData = new NoticeData();
        // 安装预约时间变更通知模板
        noticeData.setType(Constant.APPOINTMENT_TIME_NOTIFY);
        // 设置短信模板的参数
        HashMap<String, String> params = new HashMap<>(3);
        params.put(NOTICE_WORK_ID, timeNotify.getWorkId());
        params.put(CHANGE_TIME, timeNotify.getTime());
        noticeData.setParams(params);
        return noticeUtil.send(noticeData,phone);
    }

}
