package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.cams.core.entity.config.BaseConfigDic;
import com.boco.nbd.cams.core.entity.config.ConfigDic;
import com.boco.nbd.cams.core.enums.YesOrNoEnum;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.common.util.SpringContextUtil;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.framework.workflow.annotation.AnnotationUtils;
import com.boco.nbd.framework.workflow.entity.*;
import com.boco.nbd.framework.workflow.entity.constant.FlowConstants;
import com.boco.nbd.framework.workflow.entity.enums.WhereOperatorEnum;
import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.framework.workflow.flow.AbstractFlow;
import com.boco.nbd.wios.core.config.ConfigDicProperties;
import com.boco.nbd.wios.core.entity.AlgorithmDic;
import com.boco.nbd.wios.core.entity.ReportEntry;
import com.boco.nbd.wios.flow.entity.Constant;
import com.boco.nbd.wios.flow.entity.FlowConstant;
import com.boco.nbd.wios.flow.entity.bo.*;
import com.boco.nbd.wios.flow.entity.pdf.BlockPairBo;
import com.boco.nbd.wios.flow.entity.pdf.SurveyDataBo;
import com.boco.nbd.wios.flow.entity.po.*;
import com.boco.nbd.wios.flow.entity.qo.CamsOrderQo;
import com.boco.nbd.wios.flow.entity.qo.FlowChartQo;
import com.boco.nbd.wios.flow.entity.qo.FlowNodeUserQo;
import com.boco.nbd.wios.flow.entity.react.SelectBO;
import com.boco.nbd.wios.flow.entity.vo.FlowChartVO;
import com.boco.nbd.wios.flow.entity.vo.OrderInstallItemVO;
import com.boco.nbd.wios.flow.entity.vo.RegionInfoVO;
import com.boco.nbd.wios.flow.entity.vo.VisitOrderVO;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.enums.OrderInfoEnum;
import com.boco.nbd.wios.flow.enums.RegionLevelEnum;
import com.boco.nbd.wios.flow.enums.RgbEnum;
import com.boco.nbd.wios.flow.mapper.def.*;
import com.boco.nbd.wios.flow.process.OrderCommonHandle;
import com.boco.nbd.wios.flow.process.Process0;
import com.boco.nbd.wios.flow.service.INoticeService;
import com.boco.nbd.wios.flow.service.IOrderService;
import com.boco.nbd.wios.flow.util.PdfUtil;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.log.entity.po.InterfaceMonitorPO;
import com.boco.nbd.wios.log.enums.InterfaceMonitorEnum;
import com.boco.nbd.wios.log.service.InterfaceMonitorService;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.Dictionary;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.entity.cams.enums.OverTimeType;
import com.boco.nbd.wios.manage.entity.cams.enums.UploadFileType;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.mapper.def.WallboxApplyMapper;
import com.boco.nbd.wios.wx.entity.bo.MinaOrderBO;
import com.boco.nbd.wios.wx.entity.bo.MinaOrderTrack;
import com.deepoove.poi.data.PictureRenderData;
import com.deepoove.poi.util.BytePictureUtils;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.ihidea.core.support.session.SessionInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author:cao hai
 * @date: 2022/6/28 8:37
 * @version:V1.0
 * @description: OrderServiceImpl
 * @modify:
 */
@Service
@Slf4j
public class OrderServiceImpl extends BaseDaoService3<OrderPO, FlowOrderMapper> implements IOrderService {

    @Resource
    private Process0 process;

    @Resource
    private NodeMapper nodeMapper;

    @Resource
    private HolidayMapper holidayMapper;

    @Resource
    private MenuFieldMapper menuFieldMapper;

    @Autowired
    private ConfigDicProperties configDicProperties;

    @Resource
    private OemCascadeMapper oemCascadeMapper;

    @Resource
    private FlowOrderSurveyMapper flowOrderSurveyMapper;

    @Resource
    private FlowOrderInstallMapper flowOrderInstallMapper;

    @Autowired
    private OriginalService originalService;

    @Autowired
    private InterfaceMonitorService interfaceMonitorService;

    @Autowired
    INoticeService noticeService;

    @Autowired
    private WallboxApplyMapper wallboxApplyMapper;

    /**
     * 枚举转换字段
     */
    private static final Map<String, Map<Object, String>> ENUM_MAP = new HashMap<>(10);

    private static final Map<String, List<String>> HOLIDAY_MAP = new HashMap<>(10);

    /**
     * 获取节假日配置数据
     *
     * @return
     */
    private Map<String, List<String>> getHolidayMap() {
        if (CollUtil.isEmpty(HOLIDAY_MAP)) {
            String date = DateUtil.year(new Date()) + "";
            HOLIDAY_MAP.put(ProcessUtil.HOLIDAY_TYPE, holidayMapper.getListByType(ProcessUtil.HOLIDAY_TYPE, date));
            HOLIDAY_MAP.put(ProcessUtil.WORK_TYPE, holidayMapper.getListByType(ProcessUtil.WORK_TYPE, date));
        }
        return HOLIDAY_MAP;
    }


    /**
     * 流程流转
     *
     * @param bo
     * @return
     */
    @Override
    public ResponseMessage2<Boolean> submitFlow(LocalSubmitFlowBO bo) {
        log.info("start {} ,nodeFlag {} request param verify.", bo.getWorkId(), bo.getCurrentNode());
        bo.localVerify();
        if (bo.getBusiness() == null) {
            return ResponseMessage2.Failed("表单数据为空！");
        }
        if (CollUtil.isEmpty(bo.getFormData())) {
            log.info("{}FormData内容:{}", bo.getWorkId(), bo.getFormData());
        }
        AbstractFlow process = ProcessUtil.getBaseProcess(bo.getCurrentNode());
        if (null == process) {
            return ResponseMessage2.Failed(bo.getCurrentNode() + " not exists implements class.");
        }
        process.setFlowCommonHandle(SpringContextUtil.getBean(OrderCommonHandle.class));
        if (!SubmitTypeEnum.StartFlow.equals(bo.getSubmitType())) {
            bo.setOldBusiness(getOrder(bo.getWorkId()));
            if (bo.getOldBusiness() == null) {
                return ResponseMessage2.Failed(bo.getWorkId() + "工单数据不存在.");
            }
        }

        log.info("start {} ,nodeFlag {} process run.", bo.getWorkId(), bo.getCurrentNode());
        try {
            process.run(bo);
        } catch (FlowException ex) {
            log.error(bo.getWorkId() + ",nodeFlag " + bo.getCurrentNode() + " submitFlow FlowException:", ex);
            return ResponseMessage2.Failed(ex.getMessage());
        } catch (Exception ex) {
            log.error(bo.getWorkId() + ",nodeFlag " + bo.getCurrentNode() + " submitFlow Exception:", ex);
            return ResponseMessage2.Failed("工单流转异常:" + ex.getMessage());
        }
        log.info("end {} ,nodeFlag {} process run.", bo.getWorkId(), bo.getCurrentNode());
        return ResponseMessage2.Success();
    }

    /**
     * 启动流程
     *
     * @param po
     * @return
     */
    @Override
    public ResponseMessage2<Boolean> startFlow(OrderPO po) {
        //验证工单号是否重复
        OrderPO existPo = selectByPrimaryKey(po.getId());
        if (existPo != null) {
            throw new FlowException(LambdaUtils.getFieldName(OrderPO::getId) + po.getId() + "已存在");
        }
        LocalSubmitFlowBO bo = new LocalSubmitFlowBO();
        bo.setSubmitType(SubmitTypeEnum.StartFlow);
        bo.setCurrentNode(FlowNodeEnum.NODE_START.getKey());
        bo.setNextNode(FlowNodeEnum.NODE_101.getKey());
        if (StrUtil.isEmpty(po.getCreateAccount())) {
            SessionInfo user = ProcessUtil.getUserInfo();
            if (user != null) {
                po.setCreateAccount(user.getUserId());
                po.setCreateAccountName(user.getUserName());
            }
        }

        //区域级别为空,补充区域信息
        if (po.getRegionLevel() == null) {
            RegionInfoVO infoVO = getRegionInfo().get(Convert.toInt(po.getRegionId()));
            BeanUtil.copyProperties(infoVO, po, true);
        }
        //验证
        if (RegionLevelEnum.REGION.getKey().equals(po.getRegionLevel()) || RegionLevelEnum.CITY.getKey().equals(po.getRegionLevel())) {
            bo.setSubmitUserId(po.getCreateAccount());
            bo.setBusiness(po);
            bo.setWorkId(po.getId());
            return submitFlow(bo);
        }

        throw new FlowException("选择的区域数据非地市、区县维度数据");
    }

    /**
     * 获取工单数据
     *
     * @param ids
     * @return
     */
    private Map<String, OrderPO> getOrderData(List<String> ids) {
        Map<String, OrderPO> map = new HashMap<>(10);
        if (ids == null || ids.isEmpty()) {
            return map;
        }
        int length = ids.size();
        int space = 500;
        int size = length % space == 0 ? length / space : length / space + 1;
        List<OrderPO> list = new ArrayList<>(100);
        for (int i = 0; i < size; i++) {
            int end = Math.min((i + 1) * space, length);
            List<String> params = ids.subList(i * space, end);
            list.addAll(getOrderInfoData(params));
        }
        return list.stream().collect(Collectors.toMap(OrderPO::getId, Function.identity()));
    }

    /**
     * 获取工单数据
     *
     * @param ids
     * @return
     */
    private List<OrderPO> getOrderInfoData(List<String> ids) {
        Example example = new Example(OrderPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andIn(LambdaUtils.getFieldName(OrderPO::getId), ids);
        return selectByExample(example);
    }

    /**
     * 小程序接口
     *
     * @param taskResponse
     * @return
     */
    private WorkTaskVO toMinaVo(WorkTaskResponse taskResponse) {
        WorkTaskVO vo = new WorkTaskVO();
        if (taskResponse == null || CollUtil.isEmpty(taskResponse.getWorkTaskList())) {
            return vo;
        }
        vo.setCount(taskResponse.getPageCount());
        vo.setTaskList(new ArrayList<>(100));
        List<String> ids = Linq.of(taskResponse.getWorkTaskList()).select(WorkTaskInfo::getWorkId).toList();
        Map<String, OrderPO> orders = getOrderData(ids);
        for (WorkTaskInfo item : taskResponse.getWorkTaskList()) {
            MinaOrderBO bo = new MinaOrderBO();
            BeanUtil.copyProperties(item, bo);
            OrderPO order = orders.get(item.getWorkId());
            if (order == null) {
                order = new OrderPO();
            }
            BeanUtil.copyProperties(order, bo);
            bo.setStatusKey(order.getStatus());
            //状态
            bo.setStatusStr(order.getStatus() == null ? "" : OrderStatus.getText(order.getStatus()));
            bo.setElectricExpand(order.getElectricExpandTime() != null);
            bo.setInstallSupplierId(order.getInstallSupplierId());
            if (ProcessUtil.areSurveyTransfer(Convert.toInt(item.getNodeFlag()))) {
                bo.setInstallSupplierId(order.getSurveySupplierId());
            }
            vo.getTaskList().add(BeanUtil.beanToMap(bo));

        }
        return vo;
    }

    /**
     * 对象组装
     *
     * @param response
     * @param qo
     * @return
     */
    private WorkTaskVO toVo(WorkTaskResponse response, CamsOrderQo qo) {
        WorkTaskVO vo = new WorkTaskVO();
        if (response == null || CollUtil.isEmpty(response.getWorkTaskList())) {
            return vo;
        }
        FilterFieldsBO filterFieldsBO = new FilterFieldsBO(qo.getFields());
        Map<String, MenuFieldBO> fieldFieldsMap = qo.getFields().stream().collect(Collectors.toMap(MenuFieldBO::getDataIndex, Function.identity()));
        vo.setCount(response.getPageCount());
        vo.setTaskList(new ArrayList<>(100));
        List<String> ids = Linq.of(response.getWorkTaskList()).select(WorkTaskInfo::getWorkId).toList();
        Map<String, OrderPO> orders = getOrderData(ids);
        boolean areExport = (qo.getAreExport() != null && qo.getAreExport());
        boolean noFlag = fieldFieldsMap.containsKey(FlowConstant.NO);
        int num = 1;
        for (WorkTaskInfo item : response.getWorkTaskList()) {
            Map<String, Object> map = ProcessUtil.getFilterFields(item, filterFieldsBO, createEnumMap());
            OrderPO order = orders.get(item.getWorkId());

            if (order == null) {
                order = new OrderPO();
            }
            //导出序号赋值=>一期存在的业务
            if (areExport) {
                if (noFlag) {
                    map.put(FlowConstant.NO, num);
                    num++;
                }
                //个性处理运行时长
                order.setRunTimeValue();
            }
            //order中保留关闭节点的nodeFlag
            order.setNodeFlag(Convert.toInt(item.getNodeFlag()));
            map.putAll(ProcessUtil.getFilterFields(order, filterFieldsBO, createEnumMap()));
            //区域字段个性处理
            map.put(WiosConstant.AREA_FIELD, order.findAreaName());
            map.put(WiosConstant.REGION_ID, order.findAreaId());
            map.put("statusKey", order.getStatus());
            WallboxApply cond = new WallboxApply();
            cond.setOrderId(order.getId());
            //判断该订单是否为审核通过
            cond.setCheckStatus(2L);
            long count = wallboxApplyMapper.selectWallboxApplyList(cond).stream().count();
            map.put("isApply", count > 0 ? 1 : 0);
            //是否电力报装
            map.put(LambdaUtils.getFieldName(OrderPO::getElectricExpand), order.getElectricExpandTime() != null);
            //转派处理
            if (qo.getAreTransfer() != null && qo.getAreTransfer()) {
                //新增勘测服务商字段合并到原先
                String installSupplierId = LambdaUtils.getFieldName(OrderPO::getInstallSupplierId);
                if (map.containsKey(installSupplierId) && ProcessUtil.areSurveyTransfer(order.getNodeFlag())) {
                    map.put(installSupplierId, order.getSurveySupplierId());
                }
                map.put(LambdaUtils.getFieldName(OrderPO::getTransferUser), ProcessUtil.getTransferRightBO(order, qo.getUserId()));
            }

            vo.getTaskList().add(map);
        }
        return vo;
    }

    /**
     * 获取流程数据
     * <p>
     * 列表部分影响效率迁移出来
     * if (!areExport) {
     * Integer closeNode = order.getNodeFlag();
     * List<String> tracks = Linq.of(getTrackByWorkId(item.getWorkId())).select(s -> s.getNodeFlag()).toList();
     * map.put(FlowConstant.FLOW_CHART, RgbEnum.simpleFlowChart(item, closeNode, tracks));
     * }
     *
     * @param qo
     * @return ResponseMessage2
     */
    @Override
    public ResponseMessage2<FlowChartVO> getFlowChart(FlowChartQo qo) {
        OrderPO order = selectByPrimaryKey(qo.getWorkId());
        if (order == null) {
            return ResponseMessage2.Failed("工单数据不存在.");
        }
        List<String> tracks = Linq.of(getWorkTrackByProcessInsId(order.getProcessInsId())).select(TaskInfoResponse::getNodeFlag).toList();
        if (CollUtil.isEmpty(tracks)) {
            return ResponseMessage2.Failed("轨迹数据不存在.");
        }
        Collections.reverse(tracks);
        Integer closeNode = (order.getStatus() != null && OrderStatus.CLOSED.getType() == order.getStatus()) ? order.getNodeFlag() : null;
        return ResponseMessage2.Success2(RgbEnum.simpleFlowChart(qo, closeNode, tracks));
    }

    /**
     * 首字母小写
     *
     * @param str
     * @return
     */
    private String lowerFirstCase(String str) {
        char[] chars = str.toCharArray();
        //首字母小写方法，大写会变成小写，如果小写首字母会消失
        chars[0] += 32;
        return String.valueOf(chars);
    }

    /**
     * 生成枚举字段
     *
     * @return
     */
    @Override
    public Map<String, Map<Object, String>> createEnumMap() {
        if (ENUM_MAP.isEmpty()) {
            List<ConfigDic> configs = configDicProperties.getEnumNumberConfig();
            configs.addAll(configDicProperties.getEnumStringConfig());
            configs.stream().forEach(s -> {
                String[] items = StrUtil.split(s.getValue(), CamsConstant.SEMICOLON_FLAG);
                Map<Object, String> maps = new HashMap<>(4);
                for (String item : items) {
                    BaseConfigDic dic = BaseConfigDic.create(item);
                    maps.put(dic.getKey(), dic.getValue());
                }
                ENUM_MAP.put(s.getKey(), maps);
            });

            //手动添加
            ENUM_MAP.put(LambdaUtils.getFieldName(OrderPO::getStatus), OrderStatus.createMap());
            ENUM_MAP.put(LambdaUtils.getFieldName(OrderPO::getOvertimeType), OverTimeType.createMap());

            //墙盒
            initPackType();

        }
        return ENUM_MAP;
    }

    /**
     * 评分自动计算算法
     *
     * @param parentKey
     * @return
     */
    @Override
    public List<AlgorithmDic> getScoreAlgorithm(Integer parentKey) {
        List<AlgorithmDic> list = configDicProperties.getScoreAlgorithm();
        if (parentKey != null) {
            list = Linq.of(list).where(s -> s.getParentKey().equals(parentKey)).toList();
        }
        return list;
    }

    /**
     * 获取报告配置实体
     *
     * @param parentKey
     * @return
     */
    @Override
    public List<ReportEntry> getReportEntry(Integer parentKey) {
        List<ReportEntry> list = configDicProperties.getReportEntry();
        if (parentKey != null) {
            list = Linq.of(list).where(s -> s.getParentKey().equals(parentKey)).toList();
        }
        return list;
    }

    /**
     * 初始化墙盒套包类型、墙盒品牌、墙盒型号=>初始化数据特点 id长度为7,9,11
     */
    private void initPackType() {
        List<OemCascadePO> list = oemCascadeMapper.selectAll();
        ENUM_MAP.put(LambdaUtils.getFieldName(OrderPO::getPackType), OemCascadePO.createMap(list, WiosConstant.PACK_TYPE_FLAG));
        ENUM_MAP.put(LambdaUtils.getFieldName(OrderPO::getWallboxModel), OemCascadePO.createMap(list, WiosConstant.WALL_BOX_MODEL_FLAG));
        ENUM_MAP.put(LambdaUtils.getFieldName(OrderPO::getItemModel), OemCascadePO.createMap(list, WiosConstant.ITEM_MODEL_FLAG));
    }

    /**
     * 获取in 查询条件
     *
     * @param regionId
     * @return
     */
    @Override
    public String getRegionQo(String regionId) {
        if (StrUtil.isEmpty(regionId)) {
            return null;
        }
        List<Region> regions = originalService.getAllRegions(null);
        List<String> result = new ArrayList<>(8);
        List<String> items = Arrays.asList(regionId.split(CamsConstant.COMMA));
        for (String item : items) {
            Integer key = Convert.toInt(item);
            //等于1 根目录 可以不处理
            if (key == 1) {
                continue;
            }
            result.add(item);
            result.addAll(findChild(Linq.of(regions).where(s -> s.getParentId() != null).toList(), key));
        }
        if (CollUtil.isEmpty(result)) {
            return null;
        }
        result = Linq.of(result).distinct().toList();
        return String.join(FlowConstants.COMMA, result);
    }

    /**
     * 获取服务商有效合同
     *
     * @param supplierId
     * @return
     */
    @Override
    public SupplierContractVo getSupplierContract(int supplierId) {
        return originalService.getSupplierContract(supplierId);
    }

    /**
     * 获取pm用户
     *
     * @return
     */
    @Override
    public List<PmUserPO> getPmUser() {
        return this.mapper.getPmUser();
    }

    /**
     * 获取可用安装服务商
     *
     * @return
     */
    @Override
    public List<UsableSupplierPO> getUsableSupplier() {
        return this.mapper.getUsableSupplier();
    }

    /**
     * 获取服务商接单信息
     *
     * @param supplierId
     * @param regionId
     * @return
     */
    @Override
    public List<SupplierReceivePO> getSupplierReceiveInfo(Integer supplierId, String regionId) {
        return this.mapper.getSupplierReceiveInfo(supplierId, regionId);
    }

    /**
     * 写入接口监控日志
     *
     * @param po
     * @return
     */
    @Override
    public int addMonitorLog(InterfaceMonitorPO po) {
        return interfaceMonitorService.add(po);
    }

    /**
     * 获取区域详细信息
     *
     * @return
     */
    @Override
    public Map<Integer, RegionInfoVO> getRegionInfo() {
        List<Region> regions = originalService.getAllRegions(null);
        Map<Integer, String> dicMap = regions.stream().collect(Collectors.toMap(Region::getId, Region::getName));
        Map<Integer, RegionInfoVO> map = new HashMap<>(30);
        for (Region region : regions) {
            if (StrUtil.isEmpty(region.getIdTree())) {
                continue;
            }
            RegionInfoVO vo = new RegionInfoVO();
            String[] items = region.getIdTree().split(CamsConstant.UNDER_LINE);
            vo.setRegionLevel(items.length);
            for (int i = 0; i < items.length; i++) {
                Integer aimRegionId = Convert.toInt(items[i]);
                String aimRegionName = dicMap.get(aimRegionId);
                Integer regionLevel = i + 1;
                vo.addRegionData(regionLevel, aimRegionId, aimRegionName);
            }
            map.put(region.getId(), vo);
        }
        return map;
    }

    /**
     * 获取工单信息
     *
     * @param id
     * @param infoEnum
     * @return
     */
    @Override
    public Map<String, Object> getOrderInfo(String id, OrderInfoEnum infoEnum) {
        OrderPO po = selectByPrimaryKey(id);
        if (po == null) {
            log.error("工单" + id + "不存在.");
            return new HashMap<>(1);
        }
        List<String> fields = null;
        Map<String, Object> maps;
        switch (infoEnum) {
            case BASE_INFO:
                fields = Arrays.asList(Constant.BASE_ORDER_FIELD.split(FlowConstants.COMMA));
                maps = AnnotationUtils.convert(po, fields, createEnumMap());
                //基本信息
                maps.put(LambdaUtils.getFieldName(LocalSubmitFlowBO::getBusiness), po);
                break;
            case SURVEY_INFO:
                fields = Arrays.asList(Constant.SURVEY_ORDER_FIELD.split(FlowConstants.COMMA));
                OrderSurveyPO surveyPO = getOrderSurvey(id);
                maps = AnnotationUtils.convert(po, fields, createEnumMap());
                maps.putAll(AnnotationUtils.convert(surveyPO, fields, createEnumMap()));
                break;
            case INSTALL_INFO:
                fields = Arrays.asList(Constant.INSTALL_ORDER_FIELD.split(FlowConstants.COMMA));
                OrderInstallPO installPO = getOrderInstall(id);
                maps = AnnotationUtils.convert(po, fields, createEnumMap());
                maps.putAll(AnnotationUtils.convert(installPO, fields, createEnumMap()));
                break;
            default:
                log.error("type=" + infoEnum.getKey() + ",参数异常.");
                maps = new HashMap<>(1);
        }
        List<UploadFile> files = getOrderFile(OrderInfoEnum.getUploadFileType(infoEnum).getType(), id);
        maps.put(LambdaUtils.getFieldName(OrderPO::getFiles), files);
        Map<String, Object> filesData = ProcessUtil.handleFiles(files);
        maps.put(FlowConstant.PIC_FILES, filesData.get(FlowConstant.PIC_FILES));
        maps.put(FlowConstant.ATTACH_FILES, filesData.get(FlowConstant.ATTACH_FILES));
        //添加增项费用和附件
        if (OrderInfoEnum.SURVEY_INFO.equals(infoEnum) || OrderInfoEnum.INSTALL_INFO.equals(infoEnum)) {
            List<OrderInstallItemVO> itemFeeList = originalService.getOrderDetailItems(id, infoEnum.getKey(), po.getOemId());
            maps.put(FlowConstant.ITEM_FEES_FILED, Linq.of(itemFeeList).where(s -> WiosConstant.FEE_TYPE.equals(s.getItemType())).toList());
            maps.put(FlowConstant.ITEM_HIGH_FILED, Linq.of(itemFeeList).where(s -> WiosConstant.HIGH_ADD_TYPE.equals(s.getItemType())).toList());
        }
        //所属区域 需要个性处理
        if (fields != null && fields.contains(LambdaUtils.getFieldName(OrderPO::getRegionName))) {
            maps.put(LambdaUtils.getFieldName(OrderPO::getRegionName), po.findAreaName());
            maps.put(LambdaUtils.getFieldName(OrderPO::getRegionId), po.findAreaId());
        }

        return maps;
    }

    /**
     * 是否用户导出userPhone权限
     *
     * @param account
     * @return
     */
    @Override
    public boolean hasExportUserPhone(String account) {
        List<String> items = originalService.getInfoConfig(Constant.USER_PHONE_DICTIONARY_ID);
        if (CollUtil.isEmpty(items)) {
            return false;
        }

        return items.contains(account);
    }

    /**
     * 通过workId获取轨迹
     *
     * @param workId
     * @return
     */
    @Override
    public List<TaskInfoResponse> getTrackByWorkId(String workId) {
        OrderPO po = selectByPrimaryKey(workId);
        if (po == null || StrUtil.isEmpty(po.getProcessInsId())) {
            log.error(workId + "工单数据异常.");
            return new ArrayList<>(1);
        }
        return getWorkTrackByProcessInsId(po.getProcessInsId());
    }

    /**
     * 订单安装项目表更新=>全量数据处理
     *
     * @param request
     */
    @Override
    public void saveInstallItem(LocalSubmitFlowBO request) {
        if (request.getBusiness() == null) {
            return;
        }
        boolean emptyFees = StrUtil.isEmpty(request.getBusiness().getItemFees());
        boolean emptyHighAdd = StrUtil.isEmpty(request.getBusiness().getItemHighAdd());
        if (emptyFees && emptyHighAdd) {
            return;
        }
        Integer type = FlowNodeEnum.getInstallItemType(request.getCurrentNodeEnum());
        if (type == null) {
            log.error(request.getCurrentNodeEnum().getValue() + "获取订单安装明细类型[type]异常,请核查！");
        }
        List<OrderInstallItem> items = new ArrayList<>(10);
        if (!emptyFees) {
            log.info("开始添加增项费用明细数据");
            items.addAll(getOrderInstallItem(request.getBusiness().getItemFees(), request.getWorkId(), type, WiosConstant.FEE_TYPE));
        }
        if (!emptyHighAdd) {
            log.info("开始添加高附加值材料明细数据");
            items.addAll(getOrderInstallItem(request.getBusiness().getItemHighAdd(), request.getWorkId(), type, WiosConstant.HIGH_ADD_TYPE));
        }
        if (CollUtil.isEmpty(items)) {
            log.error("无有效订单安装明细");
            return;
        }

        //删除旧数据
        log.info("开始删除已有数据.");
        originalService.saveOrderInstallItem(request.getWorkId(), type, items);
    }

    /**
     * 保存附件
     *
     * @param files
     * @param type
     * @param thirdPartyId
     * @return
     */
    @Override
    public int saveFileItem(List<UploadFile> files, UploadFileType type, String thirdPartyId) {
        return originalService.saveFile(files, type, thirdPartyId);
    }

    /**
     * 保存报告==>需要做类型过滤
     *
     * @param files
     * @param type
     * @param thirdPartyId
     * @return
     */
    @Override
    public int saveReportItem(List<UploadFile> files, UploadFileType type, String thirdPartyId) {
        if (CollUtil.isEmpty(files)) {
            return 0;
        }
        List<UploadFile> filter = Linq.of(files).where(s -> type.getType() == s.getType()).toList();
        if (CollUtil.isEmpty(filter)) {
            return 0;
        }
        log.info("开始{}", type.getValue());
        return saveFileItem(filter, type, thirdPartyId);
    }

    /**
     * 获取短信推送数据
     *
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public List<SendMessagePO> getSendMessageData(Date startTime, Date endTime) {
        return this.mapper.getSendMessageData(startTime, endTime);
    }

    /**
     * 根据workId获取流程信息
     *
     * @param workId
     * @return
     */
    @Override
    public TaskInfoResponse getTaskByWorkId(String workId) {
        return process.getFlowService().getTaskByBusinessKey(FlowConstant.FLOW_CODE + "." + workId);
    }

    /**
     * 同步Sap
     *
     * @param bo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void pushSapOrder(ManualSapBo bo) {
        OrderPO po = ManualSapBo.toPo(bo);
        updateByPrimaryKeySelective(po);
        saveSapItem(po);
        ResponseMessage2<SapStatisticsBO> responseItems = originalService.pushAddNewItems(bo.getId());
        SapStatisticsBO itemsData = responseItems.getData();
        if (itemsData != null) {
            InterfaceMonitorPO itemsPo = itemsData.create(InterfaceMonitorEnum.Evaluation_ReCallSap);
            addMonitorLog(itemsPo);
        }
        ResponseMessage2<SapStatisticsBO> responsePush = originalService.pushOrder(bo.getId());
        SapStatisticsBO pushData = responsePush.getData();
        if (pushData != null) {
            InterfaceMonitorPO pushPo = pushData.create(InterfaceMonitorEnum.Evaluation_ReCallSap);
            addMonitorLog(pushPo);
        }
    }

    /**
     * 保存sap明细数据
     * 2022-11-15 取消增项、高附加材料非空验证
     *
     * @param po
     */
    private void saveSapItem(OrderPO po) {
        List<OrderInstallItem> items = new ArrayList<>(10);
        log.info("开始添加增项费用明细数据");
        List<OrderInstallItem> itemFees = getOrderInstallItem(po.getItemFees(), po.getId(), WiosConstant.INSTALL_TYPE, WiosConstant.FEE_TYPE);
        if (CollUtil.isNotEmpty(itemFees)) {
            items.addAll(itemFees);
            log.info("开始添加增项费用明细数据,共{}条", itemFees.size());
        }
        List<OrderInstallItem> itemHighAdd = getOrderInstallItem(po.getItemHighAdd(), po.getId(), WiosConstant.INSTALL_TYPE, WiosConstant.HIGH_ADD_TYPE);
        if (CollUtil.isNotEmpty(itemHighAdd)) {
            items.addAll(itemHighAdd);
            log.info("开始添加高附加值材料明细数据,共{}条", itemHighAdd.size());
        }
        if (CollUtil.isNotEmpty(items)) {
            log.info("开始删除已有数据.");
            originalService.saveOrderInstallItem(po.getId(), WiosConstant.INSTALL_TYPE, items);
            return;
        }
        log.info("增项、高附加明细数据为空");
    }

    /**
     * 数据转换
     *
     * @param json:json格式数据
     * @param orderId:
     * @param type
     * @param itemType
     * @return
     */
    private List<OrderInstallItem> getOrderInstallItem(String json, String orderId, Integer type, Integer itemType) {
        List<OrderInstallItem> items = OrderInstallItem.jsonHandle(json);
        if (CollUtil.isEmpty(items)) {
            log.error("明细数据类型转换异常.");
            return new ArrayList<>(1);
        }
        //数据赋值=>不为空不处理
        items.forEach(s -> {
            if (s.getType() == null) {
                s.setType(type);
            }
            if (StrUtil.isEmpty(s.getOrderId())) {
                s.setOrderId(orderId);
            }
            if (s.getItemType() == null) {
                s.setItemType(itemType);
            }
        });

        return items;
    }

    /**
     * 通过工单号和类型获取附件
     *
     * @param type
     * @param thirdPartyId
     * @return
     */
    private List<UploadFile> getOrderFile(int type, String thirdPartyId) {
        return originalService.getUploadFile(type, thirdPartyId);
    }

    /**
     * 获取子类regionId
     *
     * @param regions
     * @param regionId
     * @return
     */
    private List<String> findChild(List<Region> regions, Integer regionId) {
        List<String> list = new ArrayList<>(2);
        List<Region> children = Linq.of(regions).where(s -> s.getParentId().equals(regionId)).toList();
        if (CollUtil.isNotEmpty(children)) {
            List<String> items = Linq.of(children).select(s -> s.getId().toString()).toList();
            list.addAll(items);
            items.forEach(s -> list.addAll(findChild(regions, Convert.toInt(s))));
        }
        return list;
    }

    /**
     * 处理查询条件
     *
     * @param qo
     * @return
     */
    private GetTaskRequest createTaskRequest(CamsOrderQo qo) {
        qo.setRegionId(getRegionQo(qo.getRegionId()));
        GetTaskRequest request = CamsOrderQo.toDo(qo);
        //modify 2022-11-25 TIME_OUT(3, "超时订单") 需要个性出来 大于三的都是超时的
        if (qo.getOvertimeType() != null && qo.getOvertimeType() == OverTimeType.TIME_OUT.getType()) {
            VariableCondition cond = new VariableCondition();
            cond.setOperator(WhereOperatorEnum.GREATER_THAN);
            cond.setObjValue(qo.getOvertimeType());
            cond.setObjType(Integer.class);
            request.getOtherParam().put(LambdaUtils.getFieldName(OrderPO::getOvertimeType), cond);
        }
        return request;
    }

    /**
     * 获取待办列表
     *
     * @param qo
     * @return
     */
    @Override
    public WorkTaskVO getWaitDealWorkTask(CamsOrderQo qo) {
        GetTaskRequest request = createTaskRequest(qo);
        WorkTaskResponse result = process.getFlowService().getOrderedWaitDealTask(request);
        if (qo.getAreMina() != null && qo.getAreMina()) {
            return toMinaVo(result);
        }
        qo.setAreTransfer(true);
        return toVo(result, qo);
    }

    /**
     * 获取已办列表
     *
     * @param qo
     * @return
     */
    @Override
    public WorkTaskVO getMyFinishedWorkTask(CamsOrderQo qo) {
        GetTaskRequest request = createTaskRequest(qo);
        WorkTaskResponse result = process.getFlowService().getMyInvolvedProcess(request);
        return toVo(result, qo);
    }


    /**
     * 获取归档列表
     *
     * @param qo
     * @return
     */
    @Override
    public WorkTaskVO getArchivedWorkTask(CamsOrderQo qo) {
        if (qo.getType() != null && qo.getType() == 1) {
            //2022-11-18 兼容一期迁移工单
            return localArchivedWorkTask(qo);
        }
        GetTaskRequest request = createTaskRequest(qo);
        WorkTaskResponse result = process.getFlowService().getArchivedWorkTask(request);
        return toVo(result, qo);
    }

    /**
     * 生成区域数据
     *
     * @param regionIds
     * @return
     */
    private Map<RegionLevelEnum, List<Integer>> createRegionLevelData(String regionIds) {
        Map<RegionLevelEnum, List<Integer>> map = new HashMap<>(5);
        List<Integer> items = Arrays.stream(regionIds.split(CamsConstant.COMMA)).map(Integer::parseInt).collect(Collectors.toList());
        List<Region> regions = originalService.getAllRegions(items);
        for (Region aim : regions) {
            if (aim.getIdTree().contains(CamsConstant.UNDER_LINE)) {
                String[] details = aim.getIdTree().split(CamsConstant.UNDER_LINE);
                switch (details.length) {
                    case 1:
                        if (!map.containsKey(RegionLevelEnum.COUNTRY)) {
                            map.put(RegionLevelEnum.COUNTRY, new ArrayList<>(1));
                        }
                        map.get(RegionLevelEnum.COUNTRY).add(aim.getId());
                        break;
                    case 2:
                        if (!map.containsKey(RegionLevelEnum.AREA)) {
                            map.put(RegionLevelEnum.AREA, new ArrayList<>(8));
                        }
                        map.get(RegionLevelEnum.AREA).add(aim.getId());
                        break;
                    case 3:
                        if (!map.containsKey(RegionLevelEnum.PROVINCE)) {
                            map.put(RegionLevelEnum.PROVINCE, new ArrayList<>(8));
                        }
                        map.get(RegionLevelEnum.PROVINCE).add(aim.getId());
                        break;
                    case 4:
                        if (!map.containsKey(RegionLevelEnum.REGION)) {
                            map.put(RegionLevelEnum.REGION, new ArrayList<>(8));
                        }
                        map.get(RegionLevelEnum.REGION).add(aim.getId());
                        break;
                    case 5:
                        if (!map.containsKey(RegionLevelEnum.CITY)) {
                            map.put(RegionLevelEnum.CITY, new ArrayList<>(8));
                        }
                        map.get(RegionLevelEnum.CITY).add(aim.getId());
                        break;
                    default:
                        break;
                }
            }
        }
        return map;
    }

    /**
     * 用户处理
     *
     * @param qo
     */
    private void userHandle(CamsOrderQo qo) {
        if (StrUtil.isEmpty(qo.getUserId())) {
            return;
        }
        Integer accountId = Integer.parseInt(qo.getUserId());
        CamsAccount camsAccount = originalService.getCamsAccount(accountId);
        if (camsAccount != null && StrUtil.isNotEmpty(camsAccount.getRegionId())) {
            if (StrUtil.isEmpty(qo.getRegionId())) {
                qo.setRegionId(camsAccount.getRegionId());
                return;
            }
        }
        SupplierStaff supplierStaff = originalService.getSupplierStaff(accountId);
        if (supplierStaff != null && supplierStaff.getDataAuth() != null && supplierStaff.getDataAuth().intValue() == 1) {
            qo.setInstallSupplierId(supplierStaff.getId());
        }
    }

    /**
     * 本地获取归档数据=>兼容一期归档工单
     *
     * @param qo
     * @return
     */
    private WorkTaskVO localArchivedWorkTask(CamsOrderQo qo) {
        qo.setStatus(OrderStatus.CLOSED.getType() + "");
        userHandle(qo);
        if (StrUtil.isNotEmpty(qo.getInstallFinishTime())) {
            String[] items = StrUtil.split(qo.getInstallFinishTime(), FlowConstants.SEMICOLON);
            qo.setInstallFinishTimeStart(items[0]);
            if (items.length > 1) {
                qo.setInstallFinishTimeEnd(items[1]);
            }
        }
        if (StrUtil.isNotEmpty(qo.getSurveyFinishTime())) {
            String[] items = StrUtil.split(qo.getSurveyFinishTime(), FlowConstants.SEMICOLON);
            qo.setSurveyFinishTimeStart(items[0]);
            if (items.length > 1) {
                qo.setSurveyFinishTimeEnd(items[1]);
            }
        }
        if (StrUtil.isNotEmpty(qo.getRegionId())) {
            Map<RegionLevelEnum, List<Integer>> map = createRegionLevelData(qo.getRegionId());
            qo.setRegionId(null);
            map.entrySet().forEach(s -> {
                switch (s.getKey()) {
                    case COUNTRY:
                        qo.setCountryId(StringUtils.join(s.getValue(), CamsConstant.COMMA));
                        break;
                    case AREA:
                        qo.setAreaId(StringUtils.join(s.getValue(), CamsConstant.COMMA));
                        break;
                    case PROVINCE:
                        qo.setProvinceId(StringUtils.join(s.getValue(), CamsConstant.COMMA));
                        break;
                    case REGION:
                        qo.setRegionId(StringUtils.join(s.getValue(), CamsConstant.COMMA));
                        break;
                    case CITY:
                        qo.setCityId(StringUtils.join(s.getValue(), CamsConstant.COMMA));
                        break;
                    default:
                        break;
                }
            });
        }
        Page<OrderPO> result = PageHelper.startPage(qo.getPageIndex(), qo.getPageSize());
        this.mapper.selectOrderPO(qo);
        WorkTaskVO vo = new WorkTaskVO();
        vo.setCount((int) result.getTotal());
        vo.setTaskList(new ArrayList<>(result.getResult().size()));

        FilterFieldsBO filterFieldsBO = new FilterFieldsBO(qo.getFields());
        Map<String, MenuFieldBO> fieldFieldsMap = qo.getFields().stream().collect(Collectors.toMap(MenuFieldBO::getDataIndex, Function.identity()));
        vo.setTaskList(new ArrayList<>(result.getResult().size()));
        boolean areExport = (qo.getAreExport() != null && qo.getAreExport());
        boolean noFlag = fieldFieldsMap.containsKey(FlowConstant.NO);
        int num = 1;
        for (OrderPO order : result.getResult()) {
            Map<String, Object> map = ProcessUtil.getFilterFields(order, filterFieldsBO, createEnumMap());
            //导出序号赋值=>一期存在的业务
            if (areExport) {
                if (noFlag) {
                    map.put(FlowConstant.NO, num);
                    num++;
                }
                //个性处理运行时长
                order.setRunTimeValue();
            }
            //区域字段个性处理
            map.put(WiosConstant.AREA_FIELD, order.findAreaName());
            map.put(WiosConstant.REGION_ID, order.findAreaId());
            map.put(LambdaUtils.getFieldName(BaseSubmitFlowBO::getCurrentNode), FlowNodeEnum.NODE_END.getValue());
            //是否电力报装
            map.put(LambdaUtils.getFieldName(OrderPO::getElectricExpand), order.getElectricExpandTime() != null);

            vo.getTaskList().add(map);
        }
        return vo;
    }


    /**
     * 获取menuId字段数据
     *
     * @param menuId
     * @return
     */
    @Override
    public List<MenuFieldPO> getMenuFields(String menuId) {
        List<MenuFieldPO> list = new ArrayList<>(10);
        if (StrUtil.isEmpty(menuId)) {
            return list;
        }
        Example example = new Example(MenuFieldPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(MenuFieldPO::getMenuId), menuId);
        list = menuFieldMapper.selectByExample(example);
        return list;
    }

    /**
     * 轨迹统一做倒序处理
     *
     * @param processInsId
     * @return
     */
    private List<TaskInfoResponse> getWorkTrackByProcessInsId(String processInsId) {
        List<TaskInfoResponse> tracks = process.getFlowService().getWorkTrack(processInsId);
        if (CollUtil.isEmpty(tracks)) {
            return tracks;
        }
        Collections.reverse(tracks);
        return tracks;
    }

    /**
     * 获取流转轨迹
     *
     * @param processInsId
     * @return
     */
    @Override
    public List<TaskInfoResponse> getWorkTrack(String processInsId) {
        List<TaskInfoResponse> tracks = getWorkTrackByProcessInsId(processInsId);
        if (CollUtil.isEmpty(tracks)) {
            return tracks;
        }
        List<String> userIds = Linq.of(tracks).select(TaskInfoResponse::getDealUser).distinct().toList();
        List<SessionInfo> userInfos = this.mapper.getUserMoreInfo(userIds);
        for (TaskInfoResponse track : tracks) {
            SessionInfo sessionInfo = Linq.of(userInfos).where(t -> t.getUserId().equals(track.getDealUser())).firstOrDefault();
            if (sessionInfo == null) {
                sessionInfo = new SessionInfo();
            }
            if (null == track.getLocalVariables()) {
                track.setLocalVariables(new HashMap<>(16));
            }
            track.getLocalVariables().put(LambdaUtils.getFieldName(SessionInfo::getUserName), sessionInfo.getUserName());
            track.getLocalVariables().put(LambdaUtils.getFieldName(SessionInfo::getLoginname), sessionInfo.getLoginname());
        }
        return tracks;
    }

    /**
     * 获取流程轨迹
     *
     * @param processInsId
     * @return
     */
    @Override
    public List<MinaOrderTrack> getMinaWorkTrack(String processInsId) {
        List<MinaOrderTrack> result = new ArrayList<>(4);
        List<TaskInfoResponse> tracks = getWorkTrack(processInsId);
        if (CollUtil.isEmpty(tracks)) {
            return result;
        }
        for (TaskInfoResponse response : tracks) {
            MinaOrderTrack track = new MinaOrderTrack();
            track.setNodeName(response.getNodeName());
            if (CollUtil.isNotEmpty(response.getLocalVariables())) {
                track.setExecuteTime(response.getLocalVariables().get(LambdaUtils.getFieldName(MinaOrderTrack::getExecuteTime)) + "");
                track.setComment(response.getLocalVariables().get(LambdaUtils.getFieldName(MinaOrderTrack::getComment)) + "");
                track.setUserName(response.getLocalVariables().get(LambdaUtils.getFieldName(UserPO::getUserName)) + "");
            }
            track.setTaskId(response.getTaskId());
            track.setNodeFlag(response.getNodeFlag());
            result.add(track);
        }
        return result;
    }

    /**
     * 获取节点时限配置
     */
    private Map<String, String> getDueDataConfig() {
        List<Dictionary> items = originalService.getDictionary(FlowConstant.TIMING_CONFIG, "");
        return items.stream().collect(Collectors.toMap(Dictionary::getName, Dictionary::getValue));
    }

    /**
     * 获取处理时限
     *
     * @param nodeId
     * @param time   nacos配置方式
     *               List<ConfigIntDic> list = configDicProperties.getDueDate();
     *               Map<Integer, String> map = list.stream().collect(Collectors.toMap(ConfigIntDic::getKey, ConfigIntDic::getValue));
     * @return
     */
    @Override
    public Date getDueDate(int nodeId, Date time) {
        Map<String, String> timeConfig = getDueDataConfig();
        if (CollUtil.isEmpty(timeConfig)) {
            log.error("未配置处理时限.");
            return null;
        }
        if (timeConfig.containsKey(nodeId + "")) {
            String configValue = timeConfig.get(nodeId + "");
            int limit = Convert.toInt(configValue, 0);
            if (limit == 0) {
                log.error("error:nodeId={} time limit config={}", nodeId, configValue);
                return null;
            }
            if (FlowConstant.DAY_CONFIG.contains(nodeId)) {
                return ProcessUtil.offsetDay(time, limit, getHolidayMap());
            }
            return ProcessUtil.offsetHour(time, limit, getHolidayMap());
        }
        log.info("nodeId={} does not config time limit.", nodeId);
        return null;
    }

    /**
     * 根据oemId获取主机厂级联关系
     *
     * @param oemId
     * @return
     */
    @Override
    public List<OemCascadePO> getOemCascade(String oemId) {
        List<OemCascadePO> cascadePO = oemCascadeMapper.getCascadePO(oemId);
        //查询的是设备型号的话 主键调整成设备编码
        if (StrUtil.isNotEmpty(oemId) && oemId.length() == WiosConstant.WALL_BOX_MODEL_FLAG) {
            for (OemCascadePO po : cascadePO) {
                po.setId(po.getMaterialsCode());
            }
        }
        return cascadePO;
    }

    /**
     * 获取packType标识
     *
     * @param packType
     * @return
     */
    @Override
    public String getPackTypeId(String packType) {
        if (StrUtil.isEmpty(packType)) {
            return null;
        }
        Map<Object, String> maps = createEnumMap().get(LambdaUtils.getFieldName(OrderPO::getPackType));
        if (CollUtil.isEmpty(maps)) {
            return null;
        }
        for (Map.Entry<Object, String> entry : maps.entrySet()) {
            if (packType.equals(entry.getValue())) {
                return entry.getKey() + "";
            }
        }
        return null;
    }

    /**
     * 保存勘测数据
     *
     * @param po
     * @return
     */
    @Override
    public boolean saveOrderSurvey(OrderSurveyPO po) {
        OrderSurveyPO existPo = getOrderSurveyByOrderId(po.getOrderId());
        if (existPo == null) {
            return flowOrderSurveyMapper.insertSelective(po) > 0;
        }
        po.setId(existPo.getId());
        return flowOrderSurveyMapper.updateByPrimaryKeySelective(po) > 0;
    }

    /**
     * 保存安装数据
     *
     * @param po
     * @param submitUserName
     * @return
     */
    @Override
    public boolean saveOrderInstall(OrderInstallPO po, String submitUserName) {
        OrderInstallPO existPo = getOrderInstall(po.getOrderId());
        if (existPo == null) {
            po.setCreateTime(new Date());
            po.setCreateAccount(submitUserName);
            return flowOrderInstallMapper.insertSelective(po) > 0;
        }
        //modify 2022-09-21 验证墙盒设备编号
        if (StrUtil.isNotEmpty(po.getWallboxNo()) && checkWallBoxNo(po.getWallboxNo(), po.getOrderId())) {
            throw new FlowException(LambdaUtils.getFieldApiModelProperty(OrderInstallPO::getWallboxNo) + "重复.");
        }
        po.setId(existPo.getId());
        po.setModifyTime(new Date());
        po.setModifyAccount(submitUserName);
        return flowOrderInstallMapper.updateByPrimaryKeySelective(po) > 0;
    }

    /**
     * true 存在
     *
     * @param wallBoxNo
     * @param orderId
     * @return
     */
    private boolean checkWallBoxNo(String wallBoxNo, String orderId) {
        Example example = new Example(OrderInstallPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(OrderInstallPO::getWallboxNo), wallBoxNo);
        criteria.andNotEqualTo(LambdaUtils.getFieldName(OrderInstallPO::getOrderId), orderId);
        List<OrderInstallPO> list = flowOrderInstallMapper.selectByExample(example);
        return CollUtil.isNotEmpty(list);
    }

    /**
     * 通过订单编号获取安装数据
     *
     * @param orderId
     * @return
     */
    @Override
    public OrderInstallPO getOrderInstall(String orderId) {
        Example example = new Example(OrderInstallPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(OrderInstallPO::getOrderId), orderId);
        return flowOrderInstallMapper.selectOneByExample(example);
    }


    /**
     * 通过订单编号获取勘测数据
     *
     * @param orderId
     * @return
     */
    private OrderSurveyPO getOrderSurveyByOrderId(String orderId) {
        Example example = new Example(OrderSurveyPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(OrderSurveyPO::getOrderId), orderId);
        return flowOrderSurveyMapper.selectOneByExample(example);
    }

    /**
     * 获取流程用户组
     *
     * @return
     */
    @Override
    public List<BaseUserPO> getSuperUserByGroup() {
        //存在超级用户组 统一添加
        if (FlowConstant.FLOW_SUPER_GROUP_ID != null) {
            return this.nodeMapper.getSuperUserByGroup(FlowConstant.FLOW_SUPER_GROUP_ID);
        }
        return new ArrayList<>(1);
    }

    /**
     * 加载流程节点用户
     *
     * @param qo
     * @return
     */
    @Override
    public List<FlowNodeUserPO> getFlowNodeUserInfo(FlowNodeUserQo qo) {
        return nodeMapper.getFlowNodeUserInfo(qo);
    }

    /**
     * 获取工单信息
     *
     * @param id
     * @return
     */
    @Override
    public OrderPO getOrder(String id) {
        return selectByPrimaryKey(id);
    }

    /**
     * 通过车企安装单号获取工单
     *
     * @param outOrderId
     * @return
     */
    @Override
    public OrderPO getOrderByOutOrderId(String outOrderId) {
        Example example = new Example(OrderPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(OrderPO::getOutOrderId), outOrderId);
        return this.mapper.selectOneByExample(example);
    }

    /**
     * 获取勘测数据
     *
     * @param orderId
     * @return
     */
    @Override
    public OrderSurveyPO getOrderSurvey(String orderId) {
        Example example = new Example(OrderInstallPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(OrderInstallPO::getOrderId), orderId);
        return flowOrderSurveyMapper.selectOneByExample(example);
    }

    /**
     * 更新业务数据
     *
     * @param business
     */
    @Override
    public void updateBusinessData(BusinessDataBO business) {
        if (business.getBusiness() != null) {
            business.getBusiness().setId(business.getWorkId());
            if (StrUtil.isNotEmpty(business.getBusiness().getRegionId())) {
                //补充区域信息
                RegionInfoVO infoVO = getRegionInfo().get(Convert.toInt(business.getBusiness().getRegionId()));
                BeanUtil.copyProperties(infoVO, business.getBusiness(), true);
            }

            //修改预约时间发短信
            if (business.getBusiness().getInstallReserveTime() != null) {
                try {
                    OrderPO existPo = getOrder(business.getWorkId());
                    if (existPo == null) {
                        log.error("工单不存在");
                        return;
                    }
                    if (existPo.getInstallReserveTime() == null) {
                        log.error(LambdaUtils.getFieldApiModelProperty(OrderPO::getInstallReserveTime) + "为空");
                        return;
                    }
                    TimeConfirmBO timeConfirmBO = new TimeConfirmBO(existPo.getInstallReserveTime(), business.getBusiness().getInstallReserveTime());
                    String phone = StrUtil.isEmpty(business.getBusiness().getInstallClientPhone()) ? existPo.getInstallClientPhone() : business.getBusiness().getInstallClientPhone();
                    noticeService.appointmentTimeConfirm(timeConfirmBO, phone);
                    log.info(business.getWorkId() + "安装预约时间变更确认发送短信成功");
                } catch (Exception ex) {
                    log.error(business.getWorkId() + "安装预约时间变更确认发送短信异常:", ex);
                }
            }
            this.mapper.updateByPrimaryKeySelective(business.getBusiness());
        }

        if (business.getBusinessSurvey() != null) {
            business.getBusinessSurvey().setOrderId(business.getWorkId());
            OrderSurveyPO surveyPO = getOrderSurveyByOrderId(business.getWorkId());
            if (surveyPO != null) {
                business.getBusinessSurvey().setId(surveyPO.getId());
                this.flowOrderSurveyMapper.updateByPrimaryKeySelective(business.getBusinessSurvey());
            }
        }

        if (business.getBusinessInstall() != null) {
            OrderInstallPO installPO = getOrderInstall(business.getWorkId());
            if (installPO != null) {
                business.getBusinessSurvey().setId(installPO.getId());
                this.flowOrderInstallMapper.updateByPrimaryKeySelective(business.getBusinessInstall());
            }
        }
    }

    /**
     * 获取表单操作界面数据
     *
     * @param workId
     * @return
     */
    @Override
    public BusinessDataBO getBusinessData(String workId) {
        BusinessDataBO business = new BusinessDataBO();
        OrderPO po = this.mapper.selectByPrimaryKey(workId);
        if (null == po || StrUtil.isEmpty(po.getId())) {
            log.error("未发现订单编号为" + workId + ".");
            return business;
        }

        business.setWorkId(workId);
        business.setBusiness(po);
        business.setBusinessSurvey(getOrderSurveyByOrderId(workId));
        business.setBusinessInstall(getOrderInstall(workId));
        return business;
    }

    /**
     * 订单附件生成pdf
     */
    @Override
    public void getAutogeneratePdf(BlockPairBo blockPairBo, HttpServletResponse response) {
        try {
            List<ReportEntry> reports = configDicProperties.getReportEntry();
            ReportEntry entry = Linq.of(reports).where(s -> s.getKey().equals(blockPairBo.getType())).firstOrDefault();
            if (entry == null) {
                throw new IOException("未定义附件类型");
            }
            List<UploadFile> files = originalService.getUploadFile(blockPairBo.getType(), blockPairBo.getId());
            if (CollUtil.isEmpty(files)) {
                throw new IOException("附件不存在");
            }
            PdfUtil pdfUtil = new PdfUtil();
            blockPairBo.setExperiences(blockPairBo.getExperiences());
            String path = "static/templates/" + WiosConstant.SURVEY_TEMPLATE_DOCX;
            //  "key": 10,"name": "勘测报告材料";"key": 11,"name": "勘测照片";"key": 20,"name": "安装验收单材料";"key": 21,"name": "安装照片";
            String fileName = "";
            boolean photo = false;
            switch (blockPairBo.getType()) {
                case 10:
                    fileName = WiosConstant.SURVEY_REPORT_DOCX;
                    break;
                case 11:
                    fileName = WiosConstant.SURVEY_PHOTOS_DOCX;
                    photo = true;
                    break;
                case 20:
                    fileName = WiosConstant.INSTALL_REPORT_DOCX;
                    break;
                case 21:
                    fileName = WiosConstant.INSTALL_PHOTOS_DOCX;
                    photo = true;
                    break;
                default:
                    log.info("<====== getAutogeneratePdf  default ======>");
                    throw new IOException("未定义附件类型");
            }
            //报告:title 配置描述，describe 默认值
            //照片：title 配置名称，describe 配置描述
            List<SurveyDataBo> data = new ArrayList<>(files.size());
            List<Integer> useTypes = Linq.of(files).orderBy(UploadFile::getUseType).select(UploadFile::getUseType).distinct().toList();
            int indexOne = 1;
            for (Integer useType : useTypes) {
                if (null == useType) {
                    log.error("bad useType:null");
                    continue;
                }
                ReportEntry useReport = Linq.of(reports).where(s -> s.getKey().equals(useType)).firstOrDefault();
                if (useReport == null) {
                    log.error("undefined useType:" + useType);
                    continue;
                }
                List<UploadFile> aims = Linq.of(files).where(s -> s.getUseType().equals(useType)).toList();
                boolean moreAim = aims.size() > 1;
                int indexTwo = 1;
                for (UploadFile file : aims) {
                    SurveyDataBo dataBo = new SurveyDataBo();
                    dataBo.setUrlPath(file.getUrl());
                    String title = photo ? indexOne + "、" + useReport.getName() : useReport.getDescribe();
                    if (moreAim) {
                        title = title + (photo ? "(" + indexTwo + ")" : " -" + indexTwo);
                        indexTwo++;
                    }
                    dataBo.setTitle(title);
                    String describe = photo ? useReport.getDescribe() : WiosConstant.REPORT_DESCRIBE;
                    dataBo.setDescribe(describe);
                    dataBo.setPicture(new PictureRenderData(560, 750, ".png", BytePictureUtils.getUrlBufferedImage(dataBo.getUrlPath())));
                    data.add(dataBo);
                }
                indexOne++;
            }
            blockPairBo.setExperiences(data);
            pdfUtil.getWord(path, blockPairBo, fileName, response);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 获取回访订单数据
     *
     * @param orderBo
     * @return
     */
    @Override
    public List<VisitOrderVO> getVisitOrderDate(OrderBo orderBo) {
        List<VisitOrderPO> list = this.mapper.getVisitOrderScore(orderBo);
        List<VisitOrderVO> items = new ArrayList<>(list.size());
        if (CollUtil.isEmpty(list)) {
            return items;
        }
        Map<Object, String> maps1 = createEnumMap().get("yesOrNo");
        Map<Object, String> maps2 = createEnumMap().get(LambdaUtils.getFieldName(OrderPO::getInstallLength));
        for (VisitOrderPO po : list) {
            VisitOrderVO vo = VisitOrderVO.toVo(po);
            if (po.getKnowAfterSalesChannel() != null) {
                vo.setKnowAfterSalesChannel(maps1.get(po.getKnowAfterSalesChannel()));
            }
            if (po.getInstallLength() != null) {
                vo.setKnowAfterSalesChannel(maps2.get(po.getInstallLength()));
            }
            items.add(vo);
        }
        return items;
    }


}
