package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.wios.flow.entity.bo.SapStatisticsBO;
import com.boco.nbd.wios.flow.entity.vo.OrderInstallItemVO;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.ContractStatus;
import com.boco.nbd.wios.manage.entity.cams.enums.UploadFileType;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.boco.nbd.wios.manage.service.impl.OemContractService;
import com.boco.nbd.wios.manage.service.impl.OrderService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author:cao hai
 * @date:2022/10/18 16:27
 * @version:V1.0
 * @description:原先一期相关接口
 * @modify:
 */
@Service
@Slf4j
public class OriginalService {

    @Resource
    private RegionDao regionDao;

    @Resource
    private DictionaryDao dictionaryDao;

    @Autowired
    private SupplierStaffDao supplierStaffDao;

    @Autowired
    private CamsAccountDao camsAccountDao;

    @Resource
    private InfoConfigDao infoConfigDao;

    @Autowired
    private UploadFileDao uploadFileDao;

    @Autowired
    private OemContractService oemContractService;

    @Autowired
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    private OrderService oldOrderService;

    @Autowired
    private SupplierContractDao supplierContractDao;

    /**
     * 获取服务商有效合同
     *
     * @param supplierId
     * @return
     */
    public SupplierContractVo getSupplierContract(int supplierId) {
        List<SupplierContractVo> supplierContractList =
                supplierContractDao.selectByStatusAndSupplierId(supplierId, ContractStatus.EFFECTIVE.getType());
        if (CollUtil.isEmpty(supplierContractList)) {
            throw new FlowException("服务商没有有效合同");
        }
        return supplierContractList.get(0);
    }

    /**
     * sap同步增项
     *
     * @param orderId
     */
    public ResponseMessage2<SapStatisticsBO> pushAddNewItems(String orderId) {
        return  oldOrderService.pushAddNewItems(orderId);
    }

    /**
     * 同步订单到sap
     *
     * @param orderId
     */
    public ResponseMessage2<SapStatisticsBO> pushOrder(String orderId) {
        return oldOrderService.pushOrder(orderId);
    }


    /**
     * 保存工单安装明细
     *
     * @param orderId
     * @param type
     * @param list
     * @return
     */
    public int saveOrderInstallItem(String orderId, Integer type, List<OrderInstallItem> list) {
        orderInstallItemDao.delete(orderId, type);
        return orderInstallItemDao.batchInsert(list);
    }

    /**
     * 获取工单明细
     *
     * @param orderId
     * @param type
     * @param oemId
     * @return
     */
    public List<OrderInstallItemVO> getOrderDetailItems(String orderId, Integer type, Integer oemId) {
        return oemContractService.getOrderDetailItems(orderId, type, oemId);
    }

    /**
     * 保存附件
     *
     * @param files
     * @param type
     * @param thirdPartyId
     * @return
     */
    public int saveFile(List<UploadFile> files, UploadFileType type, String thirdPartyId) {
        uploadFileDao.deleteByTypeAndThirdPartyId(type.getType(), thirdPartyId);
        return uploadFileDao.batchSave(files, type, thirdPartyId);
    }

    /**
     * 根据类型和三方id获取附件
     *
     * @param type
     * @param thirdPartyId
     * @return
     */
    public List<UploadFile> getUploadFile(int type, String thirdPartyId) {
        return uploadFileDao.selectValidByTypeAndThirdPartyId(type, thirdPartyId);
    }

    /**
     * 获取所有区域数据
     *
     * @return
     */
    public List<Region> getAllRegions(List<Integer> ids) {
        return regionDao.selectAll(1,ids);
    }

    /**
     * 获取字典数据
     *
     * @param type
     * @param name
     * @return
     */
    public List<Dictionary> getDictionary(String type, String name) {
        return dictionaryDao.selectValidByType(type, name);
    }

    /**
     * 获取信息配置
     *
     * @param type
     * @return
     */
    public List<String> getInfoConfig(Integer type) {
        return infoConfigDao.selectValidConfigName(type);
    }

    /**
     * 获取服务商用户
     * @param accountId
     * @return
     */
    public SupplierStaff getSupplierStaff(Integer accountId){
        List<SupplierStaff> supplierStaffList = supplierStaffDao.selectByAccountId(accountId);
        if(CollUtil.isNotEmpty(supplierStaffList)){
            return supplierStaffList.get(0);
        }
        return null;
    }

    /**
     * 获取cams用户
     * @param accountId
     * @return
     */
    public CamsAccount getCamsAccount(Integer accountId){
        List<CamsAccount> list = camsAccountDao.selectByAccountIdAndDeptId(accountId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            return list.get(0);
        }
        return null;
    }




}
