package com.boco.nbd.wios.flow.service.impl;


import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.common.excel.ExcelImporter;
import com.boco.nbd.framework.common.excel.ExcelParams;
import com.boco.nbd.framework.common.util.IdUtil;
import com.boco.nbd.framework.parent.microservice.exception.ErrorInfoException;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.wios.flow.entity.po.PlanTaskPO;
import com.boco.nbd.wios.flow.mapper.def.PlanTaskMapper;
import com.boco.nbd.wios.flow.service.IPlanTaskService;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import com.boco.nbd.wios.manage.mapper.def.OemDao;
import com.ihidea.core.support.session.SessionInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import javax.annotation.Resource;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * @Description: TODO
 * @Author: ZQY
 * @Date: 2022/8/10
 **/
@Service
@Slf4j
public class PlanTaskServiceImpl extends BaseDaoService3<PlanTaskPO, PlanTaskMapper> implements IPlanTaskService {


    @Autowired
    private OemDao oemDao;


    @Resource
    private PlanTaskMapper planTaskMapper;

    /**
     * 计划任务配置导入
     * @param file
     * @return
     */
    @Override
    public ResponseMessage2 importExcel(MultipartFile file) {

        try {
            LinkedHashMap<String, String> fieldMap = new LinkedHashMap<>();
            ExcelParams<PlanTaskPO> et = new ExcelParams(file.getOriginalFilename(), file.getInputStream(), fieldMap);
            et.setDateformat("yyyy");
            et.setEntityClass(PlanTaskPO.class);
            fieldMap.put("年份", "year");
            fieldMap.put("主机厂", "oemName");
            fieldMap.put("1月", "janCount");
            fieldMap.put("2月", "febCount");
            fieldMap.put("3月", "marCount");
            fieldMap.put("4月", "aprCount");
            fieldMap.put("5月", "mayCount");
            fieldMap.put("6月", "junCount");
            fieldMap.put("7月", "julCount");
            fieldMap.put("8月", "augCount");
            fieldMap.put("9月", "sepCount");
            fieldMap.put("10月", "octCount");
            fieldMap.put("11月", "novCount");
            fieldMap.put("12月", "decCount");
            et.setFieldMap(fieldMap);
            List<PlanTaskPO> list = ExcelImporter.excelToList(et);
            SessionInfo sessionInfo = ProcessUtil.getUserInfo();
            for (int i = 0; i < list.size(); i++) {
                if (StringUtils.isBlank(list.get(i).getYear())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行年份为空！");
                } else if (StringUtils.isBlank(list.get(i).getOemName())) {
                    throw new ErrorInfoException("第" + (i + 1) + "行主机厂为空！");
                }

                List<Oem> oemList = oemDao.selectByName(list.get(i).getOemName());
                if (oemList.isEmpty()) {
                    throw new ErrorInfoException("第" + (i + 1) + "行无此主机厂或已被禁用！");
                }

                list.get(i).setId(IdUtil.getLongUUID() + "");
                if (sessionInfo != null) {
                    list.get(i).setCreateUser(sessionInfo.getUserName());
                }

                Example example = new Example(PlanTaskPO.class);
                Example.Criteria criteria = example.createCriteria();
                criteria.andEqualTo(LambdaUtils.getFieldName(PlanTaskPO::getYear), list.get(i).getYear());
                criteria.andEqualTo(LambdaUtils.getFieldName(PlanTaskPO::getOemName), list.get(i).getOemName());
                List<PlanTaskPO> planTaskPOList = selectByExample(example);
                if (planTaskPOList.size() > 0) {
                    throw new ErrorInfoException("第" + (i + 1) + "行存在（" + list.get(i).getYear() + "+" + list.get(i).getOemName() + ")重复数据！");
                }

            }

            for (PlanTaskPO planTaskPO : list) {
                planTaskMapper.insert(planTaskPO);
            }

        } catch (Exception e) {
            e.printStackTrace();
            log.error("计划任务配置导入失败：{}", e.getMessage());
            return ResponseMessage2.Failed("计划任务配置导入失败：" + e.getMessage());
        }
        return ResponseMessage2.Success();
    }

    @Override
    public PlanTaskPO getPlanTaskByYearAndOemId(String year, Integer oemId) {
        Oem oem = oemDao.selectById(oemId);

        return planTaskMapper.selectByYearAndOemName(oem == null ? "" : oem.getName(), year);
    }
}
