package com.boco.nbd.wios.flow.service.impl;

import cn.hutool.core.date.DateUtil;
import com.boco.nbd.framework.common.util.StringUtil;
import com.boco.nbd.wios.flow.entity.bo.*;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.enums.RegionLevelEnum;
import com.boco.nbd.wios.flow.mapper.def.ReportStatisticsMapper;
import com.boco.nbd.wios.flow.mapper.def.StatisticsMapper;
import com.boco.nbd.wios.flow.service.IHolidayService;
import com.boco.nbd.wios.flow.service.IReportStatisticsService;
import com.boco.nbd.wios.flow.util.CalculateHours;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import com.boco.nbd.wios.manage.entity.bo.Region;
import com.boco.nbd.wios.manage.entity.bo.Supplier;
import com.boco.nbd.wios.manage.mapper.def.OemMapper;
import com.boco.nbd.wios.manage.mapper.def.RegionMapper;
import com.boco.nbd.wios.manage.mapper.def.SupplierMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: TODO
 * @Author: ZQY
 * @Date: 2022/8/29
 **/
@Service
@Slf4j
public class ReportStatisticsServiceImpl implements IReportStatisticsService {


    @Autowired
    private ReportStatisticsMapper reportStatisticsMapper;

    @Autowired
    private StatisticsMapper statisticsMapper;

    @Autowired
    private RegionMapper regionMapper;

    @Autowired
    private IHolidayService holidayService;

    @Autowired
    private SupplierMapper supplierMapper;

    @Autowired
    private OemMapper oemMapper;


    @Override
    public List<InstallationDataStatisticsBO> getInstallationDataStatistics(ReportStatisticsBO reportStatisticsBO) {

        if(StringUtil.isEmpty(reportStatisticsBO.getStartTime())){
            reportStatisticsBO.setStartTime(DateUtil.format(DateUtil.lastWeek(), "yyyy-MM-dd"));
        }
        if(StringUtil.isEmpty(reportStatisticsBO.getEndTime())){

            reportStatisticsBO.setEndTime(DateUtil.today());
        }

        List<InstallationDataStatisticsBO> list=new ArrayList<>();
        //区域级别:1 全国 2 区域 3 省份 4 地市
        if("1".equals(reportStatisticsBO.getRegionLevel())){

            list.addAll(getCountryInstallationDataStatistics(reportStatisticsBO));
        }
        if("2".equals(reportStatisticsBO.getRegionLevel())){

            list.addAll(getAreaInstallationDataStatistics(reportStatisticsBO));
        }
        if("3".equals(reportStatisticsBO.getRegionLevel())){

            list.addAll(getProvinceInstallationDataStatistics(reportStatisticsBO));
        }
        if("4".equals(reportStatisticsBO.getRegionLevel())){

            list.addAll(getRegionInstallationDataStatistics(reportStatisticsBO));
        }

        for (InstallationDataStatisticsBO bo:list) {
            if(bo.getOrdersTotal()==null){
                bo.setOrdersTotal(0L);
            }
            if(bo.getRefuseTotal()==null){
                bo.setRefuseTotal(0L);
            }
            if(bo.getInstallationFailedTotal()==null){
                bo.setInstallationFailedTotal(0L);
            }
            if(bo.getInstallationCompleteTotal()==null){
                bo.setInstallationCompleteTotal(0L);
            }

        }
        return list;
    }

    @Override
    public List<InstallationDurationStatisticsBO> getInstallationDurationStatistics(ReportStatisticsBO reportStatisticsBO) {
        if(StringUtil.isEmpty(reportStatisticsBO.getStartTime())){
            reportStatisticsBO.setStartTime(DateUtil.format(DateUtil.lastWeek(), "yyyy-MM-dd"));
        }
        if(StringUtil.isEmpty(reportStatisticsBO.getEndTime())){

            reportStatisticsBO.setEndTime(DateUtil.today());
        }

        List<InstallationDurationStatisticsBO> list=new ArrayList<>();
        //区域级别:1 全国 2 区域 3 省份 4 地市
        if("1".equals(reportStatisticsBO.getRegionLevel())){

            List<InstallationDurationStatisticsBO> regionInstallationDurationStatisticsList = reportStatisticsMapper.getCountryOrderCountNodeFlag199(reportStatisticsBO);
            list.addAll(getInstallationDurationStatistics(regionInstallationDurationStatisticsList,"1",reportStatisticsBO));
        }
        if("2".equals(reportStatisticsBO.getRegionLevel())){

            List<InstallationDurationStatisticsBO> regionInstallationDurationStatisticsList = reportStatisticsMapper.getAreaOrderCountNodeFlag199(reportStatisticsBO);
            list.addAll(getInstallationDurationStatistics(regionInstallationDurationStatisticsList,"2",reportStatisticsBO));
        }
        if("3".equals(reportStatisticsBO.getRegionLevel())){

            List<InstallationDurationStatisticsBO> regionInstallationDurationStatisticsList = reportStatisticsMapper.getProvinceOrderCountNodeFlag199(reportStatisticsBO);
            list.addAll(getInstallationDurationStatistics(regionInstallationDurationStatisticsList,"3",reportStatisticsBO));
        }
        if("4".equals(reportStatisticsBO.getRegionLevel())){

            List<InstallationDurationStatisticsBO> regionInstallationDurationStatisticsList = reportStatisticsMapper.getRegionOrderCountNodeFlag199(reportStatisticsBO);
            list.addAll(getInstallationDurationStatistics(regionInstallationDurationStatisticsList,"4",reportStatisticsBO));

        }
        return list;
    }

    @Override
    public List<Map<String, String>> getTimeStatistics(ReportStatisticsBO reportStatisticsBO,String methodType)  {

        List<Map<String, String>> mapList=new ArrayList<>();

        if(StringUtil.isEmpty(reportStatisticsBO.getStartTime())){
            reportStatisticsBO.setStartTime(DateUtil.format(DateUtil.lastWeek(), "yyyy-MM-dd"));
        }
        if(StringUtil.isEmpty(reportStatisticsBO.getEndTime())){

            reportStatisticsBO.setEndTime(DateUtil.today());
        }

        if("getAverageRunningTime".equals(methodType)){
            reportStatisticsBO.setElectricExpand("0");
        }
        if("getAverageRunningTime1".equals(methodType)){
            reportStatisticsBO.setElectricExpand("1");
        }

        List<String> holidayPOList= holidayService.getListByType("节假日");

        List<String> workList= holidayService.getListByType("调休工作日");

        String[] workdayArr = holidayPOList.toArray(new String[0]);
        String[] naturalArr = workList.toArray(new String[0]);

        Map<String,String> map1= new HashMap<>(8);
        Map<String,String> map2= new HashMap<>(8);

        //区域级别统计
        if("1".equals(reportStatisticsBO.getStatisticsType())){

            //时长统计
            if("1".equals(reportStatisticsBO.getRegionLevel())){
                //全国
                List<Region> countryList=new ArrayList<>();
                countryList.add(regionMapper.selectById(Integer.parseInt(reportStatisticsBO.getRegionId().get(0))));
                Map<String,String> countryMap=getTimeStatisticsMap(reportStatisticsBO,countryList,workdayArr,naturalArr,methodType);

                //全国未超时
                Map<String,String> countryNoTimeoutMap = getNoTimeout(reportStatisticsBO,countryList,methodType);


                //区域
                List<Region> areaList=regionMapper.selectByParentId(Integer.parseInt(reportStatisticsBO.getRegionId().get(0)));
                reportStatisticsBO.setRegionLevel("2");
                Map<String,String>  areaMap=getTimeStatisticsMap(reportStatisticsBO,areaList,workdayArr,naturalArr,methodType);

                //区域未超时
                Map<String,String>areaNoTimeoutMap = getNoTimeout(reportStatisticsBO,areaList,methodType);


                map1.putAll(areaMap);
                map1.putAll(countryMap);
                //map1.put("全国",countryMap.get("全国"));
                map2.putAll(areaNoTimeoutMap);
                //map2.put("全国",countryNoTimeoutMap.get("全国"));
                map2.putAll(countryNoTimeoutMap);
            }else if("2".equals(reportStatisticsBO.getRegionLevel())){
                //省份
                List<Region> provinceList=regionMapper.selectByParentId(Integer.parseInt(reportStatisticsBO.getRegionId().get(0)));
                reportStatisticsBO.setRegionLevel("3");
                Map<String,String> provinceMap=getTimeStatisticsMap(reportStatisticsBO,provinceList,workdayArr,naturalArr,methodType);

                //省份未超时
                Map<String,String> provinceNoTimeoutMap = getNoTimeout(reportStatisticsBO,provinceList,methodType);

                map1.putAll(provinceMap);
                map2.putAll(provinceNoTimeoutMap);
            }else if("3".equals(reportStatisticsBO.getRegionLevel())){
                //地市
                List<Region> regionList=regionMapper.selectByParentId(Integer.parseInt(reportStatisticsBO.getRegionId().get(0)));
                reportStatisticsBO.setRegionLevel("4");
                Map<String,String> regionMap=getTimeStatisticsMap(reportStatisticsBO,regionList,workdayArr,naturalArr,methodType);


                //地市未超时
                Map<String,String> regionNoTimeoutMap = getNoTimeout(reportStatisticsBO,regionList,methodType);

                map1.putAll(regionMap);
                map2.putAll(regionNoTimeoutMap);

            }else if("4".equals(reportStatisticsBO.getRegionLevel())){
                //地市
                List<Region> regionList=new ArrayList<>();
                for(String regionId:reportStatisticsBO.getRegionId()){
                    Region region=regionMapper.selectById(Integer.parseInt(regionId));
                    regionList.add(region);
                }
                reportStatisticsBO.setRegionLevel("4");
                Map<String,String> regionMap=getTimeStatisticsMap(reportStatisticsBO,regionList,workdayArr,naturalArr,methodType);


                //地市未超时
                Map<String,String> regionNoTimeoutMap = getNoTimeout(reportStatisticsBO,regionList,methodType);

                map1.putAll(regionMap);
                map2.putAll(regionNoTimeoutMap);
            }

        }else if("2".equals(reportStatisticsBO.getStatisticsType())){
            //服务商统计
            List<Supplier> supplierList = new ArrayList<>();
            if(reportStatisticsBO.getInstallSupplierId()==null){
                supplierList=supplierMapper.selectAll(1);
            }else {
                for(String id:reportStatisticsBO.getInstallSupplierId()){
                    Supplier supplier=supplierMapper.selectById(Integer.parseInt(id));
                    supplierList.add(supplier);
                }
            }


            map1=getServiceTimeStatisticsMap(reportStatisticsBO,supplierList,workdayArr,naturalArr,methodType);

            map2=getServiceNoTimeout(reportStatisticsBO,supplierList,methodType);

        }else if("3".equals(reportStatisticsBO.getStatisticsType())){
            //厂家统计
            List<Oem> oemList=oemMapper.selectAllOem();
            map1=getOmeTimeStatisticsMap(reportStatisticsBO,oemList,workdayArr,naturalArr,methodType);
            map2=getOemNoTimeout(reportStatisticsBO,oemList,methodType);
        }

        mapList.add(map1);
        mapList.add(map2);
        return  mapList;
    }

    @Override
    public List<InstallationCompletionRateBO> getInstallationCompletionRate(ReportStatisticsBO reportStatisticsBO) {

        List<InstallationCompletionRateBO> list=new ArrayList<>();

        if(StringUtil.isEmpty(reportStatisticsBO.getStartTime())){
            reportStatisticsBO.setStartTime(DateUtil.format(DateUtil.lastWeek(), "yyyy-MM-dd"));
        }
        if(StringUtil.isEmpty(reportStatisticsBO.getEndTime())){

            reportStatisticsBO.setEndTime(DateUtil.today());
        }

        if("2".equals(reportStatisticsBO.getStatisticsType())){
            //服务商统计
            List<Supplier> supplierList = new ArrayList<>();
            if(reportStatisticsBO.getInstallSupplierId()==null){
                supplierList=supplierMapper.selectAll(1);
            }else {
                for(String id:reportStatisticsBO.getInstallSupplierId()){
                    Supplier supplier=supplierMapper.selectById(Integer.parseInt(id));
                    supplierList.add(supplier);
                }
            }

            if(reportStatisticsBO.getRegionLevel()==null){
                //当没有选择区域时默认全国+区域
                reportStatisticsBO.setRegionLevel("1");
                List<Region> countryList=new ArrayList<>();
                countryList.add(regionMapper.selectById(1));
                List<InstallationCompletionRateBO> list1=getSupplierInstallationCompletionRate(countryList,supplierList,reportStatisticsBO);

                reportStatisticsBO.setRegionLevel("2");
                List<Region> areaList=regionMapper.selectByParentId(1);
                List<InstallationCompletionRateBO> list2=getSupplierInstallationCompletionRate(areaList,supplierList,reportStatisticsBO);

                list.addAll(list1);
                list.addAll(list2);
            }else{
                List<Region> regionList=new ArrayList<>();
                for(String regionId:reportStatisticsBO.getRegionId()){
                    Region region=regionMapper.selectById(Integer.parseInt(regionId));
                    regionList.add(region);
                }

                list.addAll(getSupplierInstallationCompletionRate(regionList,supplierList,reportStatisticsBO));
            }


        }else if("3".equals(reportStatisticsBO.getStatisticsType())){
            //厂家统计
            List<Oem> oemList=oemMapper.selectAllOem();

            if(reportStatisticsBO.getRegionLevel()==null){
                //当没有选择区域时默认全国+区域
                reportStatisticsBO.setRegionLevel("1");
                List<Region> countryList=new ArrayList<>();
                countryList.add(regionMapper.selectById(1));
                List<InstallationCompletionRateBO> list1=getOemInstallationCompletionRate(countryList,oemList,reportStatisticsBO);

                reportStatisticsBO.setRegionLevel("2");
                List<Region> areaList=regionMapper.selectByParentId(1);
                List<InstallationCompletionRateBO> list2=getOemInstallationCompletionRate(areaList,oemList,reportStatisticsBO);

                list.addAll(list1);
                list.addAll(list2);
            }else{
                List<Region> regionList=new ArrayList<>();
                for(String regionId:reportStatisticsBO.getRegionId()){
                    Region region=regionMapper.selectById(Integer.parseInt(regionId));
                    regionList.add(region);
                }

                list.addAll(getOemInstallationCompletionRate(regionList,oemList,reportStatisticsBO));
            }

        }
        return  list;
    }


    private List<InstallationCompletionRateBO> getSupplierInstallationCompletionRate(List<Region> regionList,List<Supplier> supplierList,ReportStatisticsBO reportStatisticsBO){
        List<InstallationCompletionRateBO> list=new ArrayList<>();
        for(Region region:regionList){

            List<String> stringRegionList=new ArrayList<>();
            stringRegionList.add(region.getId()+"");
            reportStatisticsBO.setRegionId(stringRegionList);

            for(Supplier supplier:supplierList){
                List<String> stringSupplierList=new ArrayList<>();
                stringSupplierList.add(supplier.getId());
                reportStatisticsBO.setInstallSupplierId(stringSupplierList);

                List<OrderPO> orderPOList=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);
                InstallationCompletionRateBO installationCompletionRateBO=new InstallationCompletionRateBO();
                installationCompletionRateBO.setRegion(region.getName());
                installationCompletionRateBO.setInstallSupplierName(supplier.getName());
                installationCompletionRateBO.setOrdersTotal(orderPOList.size());

                Integer count10=0;
                Integer count20=0;
                Integer count30=0;
                Integer count40=0;
                Integer count50=0;
                Integer count60=0;


                for(OrderPO orderPO:orderPOList){
                    float time=0;
                    List<TaskInstBO> taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                    List<TaskInstBO> taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
                    if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){
                         //自然日算法  单位天
                            time = taskInstBOList2.get(0).getStartTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                            time=time / (24 * 60 * 60 * 1000);

                            if(time<=10){
                                count10++;
                            }
                            if(time<=20){
                                count20++;
                            }
                            if(time<=30){
                                count30++;
                            }
                            if(time<=40){
                                count40++;
                            }
                            if(time<=50){
                                count50++;
                            }
                            if(time<=60){
                                count60++;
                            }
                    }
                }


                installationCompletionRateBO.setOrdersTotal10(count10);
                installationCompletionRateBO.setOrdersTotal10Rate(count10==0?0:(float)((Math.round((count10/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal20(count20);
                installationCompletionRateBO.setOrdersTotal20Rate(count20==0?0:(float)((Math.round((count20/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal30(count30);
                installationCompletionRateBO.setOrdersTotal30Rate(count30==0?0:(float)((Math.round((count30/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal40(count40);
                installationCompletionRateBO.setOrdersTotal40Rate(count40==0?0:(float)((Math.round((count40/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal50(count50);
                installationCompletionRateBO.setOrdersTotal50Rate(count50==0?0:(float)((Math.round((count50/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal60(count60);
                installationCompletionRateBO.setOrdersTotal60Rate(count60==0?0:(float)((Math.round((count60/orderPOList.size())*100))/100)*100);
                list.add(installationCompletionRateBO);

            }
        }
          return list;

    }


    private List<InstallationCompletionRateBO> getOemInstallationCompletionRate(List<Region> regionList,List<Oem> oemList,ReportStatisticsBO reportStatisticsBO){
        List<InstallationCompletionRateBO> list=new ArrayList<>();
        for(Region region:regionList){

            List<String> stringRegionList=new ArrayList<>();
            stringRegionList.add(region.getId()+"");
            reportStatisticsBO.setRegionId(stringRegionList);

            for(Oem oem:oemList){

                reportStatisticsBO.setOemId(oem.getId()+"");
                List<OrderPO> orderPOList=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);
                InstallationCompletionRateBO installationCompletionRateBO=new InstallationCompletionRateBO();
                installationCompletionRateBO.setRegion(region.getName());
                installationCompletionRateBO.setInstallSupplierName(oem.getShortName());
                installationCompletionRateBO.setOrdersTotal(orderPOList.size());

                Integer count10=0;
                Integer count20=0;
                Integer count30=0;
                Integer count40=0;
                Integer count50=0;
                Integer count60=0;


                for(OrderPO orderPO:orderPOList){
                    float time=0;
                    List<TaskInstBO> taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                    List<TaskInstBO> taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
                    if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){
                        //自然日算法  单位天
                        time = taskInstBOList2.get(0).getStartTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                        time=time / (24 * 60 * 60 * 1000);

                        if(time<=10){
                            count10++;
                        }
                        if(time<=20){
                            count20++;
                        }
                        if(time<=30){
                            count30++;
                        }
                        if(time<=40){
                            count40++;
                        }
                        if(time<=50){
                            count50++;
                        }
                        if(time<=60){
                            count60++;
                        }
                    }
                }


                installationCompletionRateBO.setOrdersTotal10(count10);
                installationCompletionRateBO.setOrdersTotal10Rate(count10==0?0:(float)((Math.round((count10/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal20(count20);
                installationCompletionRateBO.setOrdersTotal20Rate(count20==0?0:(float)((Math.round((count20/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal30(count30);
                installationCompletionRateBO.setOrdersTotal30Rate(count30==0?0:(float)((Math.round((count30/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal40(count40);
                installationCompletionRateBO.setOrdersTotal40Rate(count40==0?0:(float)((Math.round((count40/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal50(count50);
                installationCompletionRateBO.setOrdersTotal50Rate(count50==0?0:(float)((Math.round((count50/orderPOList.size())*100))/100)*100);
                installationCompletionRateBO.setOrdersTotal60(count60);
                installationCompletionRateBO.setOrdersTotal60Rate(count60==0?0:(float)((Math.round((count60/orderPOList.size())*100))/100)*100);
                list.add(installationCompletionRateBO);

            }
        }
        return list;

    }


    /**
     * 区域统计计算时效
     * @param reportStatisticsBO
     * @param regionList
     * @param workdayArr
     * @param naturalArr
     * @return
     * @throws ParseException
     */
    private Map<String,String> getTimeStatisticsMap(ReportStatisticsBO reportStatisticsBO,List<Region> regionList,String[] workdayArr,String[] naturalArr,String methodType){
        Map<String,String> map= new HashMap<>(8);


        for(Region region:regionList){


            List<String> stringList=new ArrayList<>();

            stringList.add(region.getId().toString());

            reportStatisticsBO.setRegionId(stringList);



            List<OrderPO> orderPOList=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);

            float time=getTime(orderPOList,workdayArr,naturalArr,methodType);


            map.put(region.getName(),time+"");

        }
        return  map;
    }



    /**
     * 服务商统计计算时效
     * @param reportStatisticsBO
     * @param supplierList
     * @param workdayArr
     * @param naturalArr
     * @return
     * @throws ParseException
     */
    private Map<String,String> getServiceTimeStatisticsMap(ReportStatisticsBO reportStatisticsBO,List<Supplier> supplierList,String[] workdayArr,String[] naturalArr,String methodType) {
        Map<String,String> map= new HashMap<>(8);

        reportStatisticsBO.setInstallSupplierId(null);
        List<OrderPO> orderPOList1=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);
        float time1=getTime(orderPOList1,workdayArr,naturalArr,methodType);
        map.put("全服务商汇总",time1+"");


        for(Supplier supplier:supplierList){

            List<String> stringList=new ArrayList<>();

            stringList.add(supplier.getId());

            reportStatisticsBO.setInstallSupplierId(stringList);

            List<OrderPO> orderPOList=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);

            float time=getTime(orderPOList,workdayArr,naturalArr,methodType);

            map.put(supplier.getName(),time+"");

        }
        return  map;
    }

    /**
     * 厂商统计计算时效
     * @param reportStatisticsBO
     * @param oemList
     * @param workdayArr
     * @param naturalArr
     * @param methodType
     * @return
     * @throws ParseException
     */
    private Map<String,String> getOmeTimeStatisticsMap(ReportStatisticsBO reportStatisticsBO,List<Oem> oemList,String[] workdayArr,String[] naturalArr,String methodType) {
        Map<String,String> map= new HashMap<>(8);

        reportStatisticsBO.setOemId("");
        List<OrderPO> orderPOList1=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);
        float time1=getTime(orderPOList1,workdayArr,naturalArr,methodType);
        map.put("全厂家汇总",time1+"");



        for(Oem oem:oemList){

            reportStatisticsBO.setOemId(oem.getId()+"");

            List<OrderPO> orderPOList2=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);

            float time2=getTime(orderPOList2,workdayArr,naturalArr,methodType);

            map.put(oem.getShortName(),time2+"");

        }
        return  map;
    }

    /**
     * 根据不同方法获取不一样的时间算法
     * @param orderPOList
     * @param workdayArr
     * @param naturalArr
     * @param methodType
     * @return
     * @throws ParseException
     */
    private float getTime(List<OrderPO> orderPOList,String[] workdayArr,String[] naturalArr,String methodType)  {
        float time=0;
        for(OrderPO orderPO:orderPOList){
            List<TaskInstBO> taskInstBOList1 = null;
            List<TaskInstBO> taskInstBOList2 = null;
            if("getFirstTimeStatistics".equals(methodType)){
                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_102.getKey()+"-"+FlowNodeEnum.NODE_102.getValue());
            }else if("getSurveyTimeStatistics".equals(methodType)){

                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_104.getKey()+"-"+FlowNodeEnum.NODE_104.getValue());
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_107.getKey()+"-"+FlowNodeEnum.NODE_107.getValue());
            }else if("getSurveyExamineTimeStatistics".equals(methodType)){

                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_107.getKey()+"-"+FlowNodeEnum.NODE_107.getValue());
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_108.getKey()+"-"+FlowNodeEnum.NODE_108.getValue());
            }else if("getAvgTimeStatistics".equals(methodType)){
                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_108.getKey()+"-"+FlowNodeEnum.NODE_108.getValue());
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
            }else if("getAverageRunningTime".equals(methodType)||"getAverageRunningTime1".equals(methodType)){
                taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
            }
            if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){
                if("getFirstTimeStatistics".equals(methodType)) {
                    //工作日算法  单位小时
                    time += CalculateHours.calculateHour(DateUtil.formatDateTime(taskInstBOList1.get(0).getStartTime()), DateUtil.formatDateTime(taskInstBOList2.get(0).getStartTime()), workdayArr, naturalArr);
                }else {
                    //自然日算法  单位小时
                    time+= taskInstBOList2.get(0).getStartTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                }
            }
        }

        if("getFirstTimeStatistics".equals(methodType)){
            time=(float)(Math.round((time/orderPOList.size())*100))/100;
        }else{
            time=(float) Math.round((time / (60 * 60 * 1000)/orderPOList.size()) * 100) / 100;
        }

        return  time;
    }


    private float getTime1(List<OrderPO> orderPOList,String methodType) {

        Integer count=0;
        for(OrderPO orderPO:orderPOList){
            float time=0;
            List<TaskInstBO>    taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
            List<TaskInstBO>    taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());

            if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){
                    //自然日算法  单位天
                    time= taskInstBOList2.get(0).getStartTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                    time=time / (24 * 60 * 60 * 1000);

                    if("getAverageRunningTime".equals(methodType)){

                        if(time<=12){
                            count++;
                        }

                    }else if("getAverageRunningTime1".equals(methodType)){
                        if(time<=20){
                            count++;
                        }
                    }
            }
        }



        return  count==0 ? 0:(float)((Math.round((count/orderPOList.size())*100))/100)*100;
    }




    /**
     * 区域统计计算未超时比率
     * @param reportStatisticsBO
     * @param regionList
     * @return
     */
    private Map<String,String> getNoTimeout(ReportStatisticsBO reportStatisticsBO,List<Region> regionList,String methodType){
        Map<String,String> map= new HashMap<>(8);


        for(Region region:regionList){

            List<String> stringList=new ArrayList<>();

            stringList.add(region.getId().toString());

            reportStatisticsBO.setRegionId(stringList);

            List<OrderPO> orderPOList=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);



            if("getAverageRunningTime".equals(methodType)||"getAverageRunningTime1".equals(methodType)){

                float count=getTime1(orderPOList,methodType);
                map.put(region.getName(),count+"");

            }else{

                Integer count=getNoTimeoutCount(orderPOList,methodType);

                if(count!=0){
                    map.put(region.getName(),(float)((Math.round((count/orderPOList.size())*100))/100)*100+"");
                }else{
                    map.put(region.getName(),"0");
                }
            }



        }
        return  map;
    }


    /**
     * 服务商统计计算未超时比率
     * @param reportStatisticsBO
     * @param supplierList
     * @return
     */
    private Map<String,String> getServiceNoTimeout(ReportStatisticsBO reportStatisticsBO,List<Supplier> supplierList,String methodType){
        Map<String,String> map= new HashMap<>(8);



        reportStatisticsBO.setInstallSupplierId(null);
        List<OrderPO> orderPOList1=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);

        if("getAverageRunningTime".equals(methodType)||"getAverageRunningTime1".equals(methodType)){

            map.put("全服务商汇总",getTime1(orderPOList1,methodType)+"");

        }else {
            Integer count1=getNoTimeoutCount(orderPOList1,methodType);
            if(count1!=0){
                map.put("全服务商汇总",(float)((Math.round((count1/orderPOList1.size())*100))/100)*100+"");
            }else{
                map.put("全服务商汇总","0");
            }
        }


        for(Supplier supplier:supplierList){

            List<String> stringList=new ArrayList<>();

            stringList.add(supplier.getId());

            reportStatisticsBO.setInstallSupplierId(stringList);

            List<OrderPO> orderPOList=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);

            if("getAverageRunningTime".equals(methodType)||"getAverageRunningTime1".equals(methodType)){

                float count=getTime1(orderPOList,methodType);
                map.put(supplier.getName(),count+"");

            }else{

                Integer count=getNoTimeoutCount(orderPOList,methodType);

                if(count!=0){
                    map.put(supplier.getName(),(float)((Math.round((count/orderPOList.size())*100))/100)*100+"");
                }else{
                    map.put(supplier.getName(),"0");
                }
            }


        }
        return  map;
    }


    /**
     * 厂商统计计算未超时比率
     * @param reportStatisticsBO
     * @param oemList
     * @return
     */
    private Map<String,String> getOemNoTimeout(ReportStatisticsBO reportStatisticsBO,List<Oem> oemList,String methodType){
        Map<String,String> map= new HashMap<>(8);
        reportStatisticsBO.setOemId("");
        List<OrderPO> orderPOList1=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);

        if("getAverageRunningTime".equals(methodType)||"getAverageRunningTime1".equals(methodType)){

            map.put("全厂家汇总",getTime1(orderPOList1,methodType)+"");

        }else {
            Integer count1=getNoTimeoutCount(orderPOList1,methodType);
            if(count1!=0){
                map.put("全厂家汇总",(float)((Math.round((count1/orderPOList1.size())*100))/100)*100+"");
            }else{
                map.put("全厂家汇总","0");
            }
        }




        for(Oem oem:oemList){

            reportStatisticsBO.setOemId(oem.getId()+"");

            List<OrderPO> orderPOList=reportStatisticsMapper.getOrderFiled(reportStatisticsBO);

            if("getAverageRunningTime".equals(methodType)||"getAverageRunningTime1".equals(methodType)){

                float count=getTime1(orderPOList,methodType);
                map.put(oem.getShortName(),count+"");

            }else{

                Integer count=getNoTimeoutCount(orderPOList,methodType);

                if(count!=0){
                    map.put(oem.getShortName(),(float)((Math.round((count/orderPOList.size())*100))/100)*100+"");
                }else{
                    map.put(oem.getShortName(),"0");
                }
            }


        }
        return  map;
    }


    /**
     * 获取未超时数量
     * @param orderPOList
     * @param methodType
     * @return
     */
    private Integer getNoTimeoutCount(List<OrderPO> orderPOList,String methodType){
        Integer count=0;

        for(OrderPO orderPO:orderPOList){
            if("getFirstTimeStatistics".equals(methodType)){
                List<TaskInstBO> taskInstBOList = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_102.getKey()+"-"+FlowNodeEnum.NODE_102.getValue());
                if(taskInstBOList.size()>0) {
                    if (taskInstBOList.get(0).getDueDate() == null) {
                        count++;
                    } else if (taskInstBOList.get(0).getEndTime().getTime() <= taskInstBOList.get(0).getDueDate().getTime()) {
                        count++;
                    }
                }
            }else if("getSurveyTimeStatistics".equals(methodType)){
                List<TaskInstBO> taskInstBOList104 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_104.getKey()+"-"+FlowNodeEnum.NODE_104.getValue());
                List<TaskInstBO> taskInstBOList105 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_105.getKey()+"-"+FlowNodeEnum.NODE_105.getValue());
                List<TaskInstBO> taskInstBOList106 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_106.getKey()+"-"+FlowNodeEnum.NODE_106.getValue());
                boolean flag=true;

                if (taskInstBOList104.get(0).getDueDate() != null&&taskInstBOList104.get(0).getEndTime().getTime() > taskInstBOList104.get(0).getDueDate().getTime()) {
                    flag=false;
                }
                if (taskInstBOList105.get(0).getDueDate() != null&&taskInstBOList105.get(0).getEndTime().getTime() > taskInstBOList105.get(0).getDueDate().getTime()) {
                    flag=false;
                }
                if (taskInstBOList106.get(0).getDueDate() != null&&taskInstBOList106.get(0).getEndTime().getTime() > taskInstBOList106.get(0).getDueDate().getTime()) {
                    flag=false;
                }

                if(flag){
                    count++;
                }
            }else if("getSurveyExamineTimeStatistics".equals(methodType)){
                List<TaskInstBO> taskInstBOList107 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_107.getKey()+"-"+FlowNodeEnum.NODE_107.getValue());
                if(taskInstBOList107.size()>0) {
                    if (taskInstBOList107.get(0).getDueDate() == null) {
                        count++;
                    } else if (taskInstBOList107.get(0).getEndTime().getTime() <= taskInstBOList107.get(0).getDueDate().getTime()) {
                        count++;
                    }
                }
            }else if("getAvgTimeStatistics".equals(methodType)){
                List<TaskInstBO> taskInstBOList108 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_108.getKey()+"-"+FlowNodeEnum.NODE_108.getValue());
                List<TaskInstBO> taskInstBOList109 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_109.getKey()+"-"+FlowNodeEnum.NODE_109.getValue());
                List<TaskInstBO> taskInstBOList110 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_110.getKey()+"-"+FlowNodeEnum.NODE_110.getValue());
                List<TaskInstBO> taskInstBOList111 = statisticsMapper.getTaskInstByInstIdAndName(orderPO.getProcessInsId(), FlowNodeEnum.NODE_111.getKey()+"-"+FlowNodeEnum.NODE_111.getValue());
                boolean flag=true;

                if (taskInstBOList108.get(0).getDueDate() != null&&taskInstBOList108.get(0).getEndTime().getTime() > taskInstBOList108.get(0).getDueDate().getTime()) {
                    flag=false;
                }
                if(taskInstBOList109.size()>0){
                    if (taskInstBOList109.get(0).getDueDate() != null&&taskInstBOList109.get(0).getEndTime().getTime() > taskInstBOList109.get(0).getDueDate().getTime()) {
                        flag=false;
                    }
                }
                if (taskInstBOList110.get(0).getDueDate() != null&&taskInstBOList110.get(0).getEndTime().getTime() > taskInstBOList110.get(0).getDueDate().getTime()) {
                    flag=false;
                }
                if (taskInstBOList111.get(0).getDueDate() != null&&taskInstBOList111.get(0).getEndTime().getTime() > taskInstBOList111.get(0).getDueDate().getTime()) {
                    flag=false;
                }

                if(flag){
                    count++;
                }
            }
        }
        return count;
    }

    /**
     * 全国充电桩安装数据统计列表
     * @return
     */
    private List<InstallationDataStatisticsBO> getCountryInstallationDataStatistics(ReportStatisticsBO reportStatisticsBO){
        //获取全国订单总数
        List<InstallationDataStatisticsBO> countryOrderCountList=reportStatisticsMapper.getCountryOrderCount(reportStatisticsBO);

        //获取全国拒接数量
        List<InstallationDataStatisticsBO> countryRefuseTotalList=reportStatisticsMapper.getCountryRefuseTotal(reportStatisticsBO);
        countryOrderCountList.addAll(countryRefuseTotalList);

        List<InstallationDataStatisticsBO>  mergeRefuseList=new ArrayList<>();
        countryOrderCountList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),b.getRefuseTotal(),a.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeRefuseList::add);
        });

        //获取全国安装失败数量
        List<InstallationDataStatisticsBO> countryInstallationFailedTotal=reportStatisticsMapper.getCountryInstallationFailedTotal(reportStatisticsBO);
        mergeRefuseList.addAll(countryInstallationFailedTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationFailedTotalList=new ArrayList<>();
        mergeRefuseList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),b.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationFailedTotalList::add);
        });


        //获取全国安装完成数量
        List<InstallationDataStatisticsBO> countryInstallationCompleteTotal=reportStatisticsMapper.getCountryInstallationCompleteTotal(reportStatisticsBO);
        mergeInstallationFailedTotalList.addAll(countryInstallationCompleteTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationCompleteTotal=new ArrayList<>();
        mergeInstallationFailedTotalList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),a.getInstallationFailedTotal(),b.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationCompleteTotal::add);
        });


        for (InstallationDataStatisticsBO bo:mergeInstallationCompleteTotal) {
            //获取全国平均安装时长
            List<String> processInsIdList=reportStatisticsMapper.getOrderByNodeFlag("1",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime(),"1");
            long time=0;
            for(String processInsId:processInsIdList){
                List<TaskInstBO> taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                List<TaskInstBO> taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
                if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){

                    time+=taskInstBOList2.get(0).getEndTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                }

            }
            time=time==0?0:time/processInsIdList.size();
            bo.setAvgInstallationTime(ProcessUtil.timeFormat(time).toString());


            //全国订单完成率
            Integer total=reportStatisticsMapper.getInstallationCompleteTotal("1",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime());
            bo.setOrderRate(bo.getOrdersTotal() != null ? (total/bo.getOrdersTotal())*100+"%" : "0%");

        }
        return  mergeInstallationCompleteTotal;
    }


    /**
     * 区域充电桩安装数据统计列表
     * @return
     */
    private List<InstallationDataStatisticsBO> getAreaInstallationDataStatistics(ReportStatisticsBO reportStatisticsBO){
        //获取区域订单总数
        List<InstallationDataStatisticsBO> areaOrderCountList=reportStatisticsMapper.getAreaOrderCount(reportStatisticsBO);

        //获取区域拒接数量
        List<InstallationDataStatisticsBO> areaRefuseTotalList=reportStatisticsMapper.getAreaRefuseTotal(reportStatisticsBO);
        areaOrderCountList.addAll(areaRefuseTotalList);

        List<InstallationDataStatisticsBO>  mergeRefuseList=new ArrayList<>();
        areaOrderCountList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),b.getRefuseTotal(),a.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeRefuseList::add);
        });


        //获取区域安装失败数量
        List<InstallationDataStatisticsBO> areaInstallationFailedTotal=reportStatisticsMapper.getAreaInstallationFailedTotal(reportStatisticsBO);
        mergeRefuseList.addAll(areaInstallationFailedTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationFailedTotalList=new ArrayList<>();
        mergeRefuseList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),b.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationFailedTotalList::add);
        });


        //获取区域安装完成数量
        List<InstallationDataStatisticsBO> areaInstallationCompleteTotal=reportStatisticsMapper.getAreaInstallationCompleteTotal(reportStatisticsBO);
        mergeInstallationFailedTotalList.addAll(areaInstallationCompleteTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationCompleteTotal=new ArrayList<>();
        mergeInstallationFailedTotalList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),a.getInstallationFailedTotal(),b.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationCompleteTotal::add);
        });


        for (InstallationDataStatisticsBO bo:mergeInstallationCompleteTotal) {
            //获取区域平均安装时长
            List<String> processInsIdList=reportStatisticsMapper.getOrderByNodeFlag("2",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime(),"1");
            long time=0;
            for(String processInsId:processInsIdList){
                List<TaskInstBO> taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                List<TaskInstBO> taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
                if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){

                    time+=taskInstBOList2.get(0).getEndTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                }

            }
            time=processInsIdList.size()>0 ? time/processInsIdList.size() : 0;
            bo.setAvgInstallationTime(ProcessUtil.timeFormat(time).toString());


            //区域订单完成率
            Integer total=reportStatisticsMapper.getInstallationCompleteTotal("2",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime());
            bo.setOrderRate(bo.getOrdersTotal() != null ? (total/bo.getOrdersTotal())*100+"%" : "0%");

        }
        return mergeInstallationCompleteTotal;
    }


    /**
     * 省份充电桩安装数据统计列表
     * @return
     */
    private List<InstallationDataStatisticsBO> getProvinceInstallationDataStatistics(ReportStatisticsBO reportStatisticsBO){
        //获取省份订单总数
        List<InstallationDataStatisticsBO> provinceOrderCountList=reportStatisticsMapper.getProvinceOrderCount(reportStatisticsBO);

        //获取省份拒接数量
        List<InstallationDataStatisticsBO> provinceRefuseTotalList=reportStatisticsMapper.getProvinceRefuseTotal(reportStatisticsBO);
        provinceOrderCountList.addAll(provinceRefuseTotalList);

        List<InstallationDataStatisticsBO>  mergeRefuseList=new ArrayList<>();
        provinceOrderCountList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),b.getRefuseTotal(),a.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeRefuseList::add);
        });



        //获取省份安装失败数量
        List<InstallationDataStatisticsBO> provinceInstallationFailedTotal=reportStatisticsMapper.getProvinceInstallationFailedTotal(reportStatisticsBO);
        mergeRefuseList.addAll(provinceInstallationFailedTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationFailedTotalList=new ArrayList<>();
        mergeRefuseList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),b.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationFailedTotalList::add);
        });


        //获取省份安装完成数量
        List<InstallationDataStatisticsBO> provinceInstallationCompleteTotal=reportStatisticsMapper.getProvinceInstallationCompleteTotal(reportStatisticsBO);
        mergeInstallationFailedTotalList.addAll(provinceInstallationCompleteTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationCompleteTotal=new ArrayList<>();
        mergeInstallationFailedTotalList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),a.getInstallationFailedTotal(),b.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationCompleteTotal::add);
        });

        for (InstallationDataStatisticsBO bo:mergeInstallationCompleteTotal) {
            //获取省份平均安装时长
            List<String> processInsIdList=reportStatisticsMapper.getOrderByNodeFlag("3",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime(),"1");
            long time=0;
            for(String processInsId:processInsIdList){
                List<TaskInstBO> taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                List<TaskInstBO> taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
                if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){

                    time+=taskInstBOList2.get(0).getEndTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                }

            }
            time=processInsIdList.size()>0 ? time/processInsIdList.size() : 0;
            bo.setAvgInstallationTime(ProcessUtil.timeFormat(time).toString());


            //省份订单完成率
            Integer total=reportStatisticsMapper.getInstallationCompleteTotal("3",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime());
            bo.setOrderRate(bo.getOrdersTotal() != null ? (total/bo.getOrdersTotal())*100+"%" : "0%");

        }

        return mergeInstallationCompleteTotal;

    }


    /**
     * 地市充电桩安装数据统计列表
     * @return
     */
    private List<InstallationDataStatisticsBO> getRegionInstallationDataStatistics(ReportStatisticsBO reportStatisticsBO){
        //获取地市订单总数
        List<InstallationDataStatisticsBO> regionOrderCountList=reportStatisticsMapper.getRegionOrderCount(reportStatisticsBO);

        //获取地市拒接数量
        List<InstallationDataStatisticsBO> regionRefuseTotalList=reportStatisticsMapper.getRegionRefuseTotal(reportStatisticsBO);
        regionOrderCountList.addAll(regionRefuseTotalList);

        List<InstallationDataStatisticsBO>  mergeRefuseList=new ArrayList<>();
        regionOrderCountList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),b.getRefuseTotal(),a.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeRefuseList::add);
        });

        //获取地市安装失败数量
        List<InstallationDataStatisticsBO> regionInstallationFailedTotal=reportStatisticsMapper.getRegionInstallationFailedTotal(reportStatisticsBO);
        mergeRefuseList.addAll(regionInstallationFailedTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationFailedTotalList=new ArrayList<>();
        mergeRefuseList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),b.getInstallationFailedTotal(),a.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationFailedTotalList::add);
        });

        //获取地市安装完成数量
        List<InstallationDataStatisticsBO> regionInstallationCompleteTotal=reportStatisticsMapper.getRegionInstallationCompleteTotal(reportStatisticsBO);
        mergeInstallationFailedTotalList.addAll(regionInstallationCompleteTotal);

        List<InstallationDataStatisticsBO>  mergeInstallationCompleteTotal=new ArrayList<>();
        mergeInstallationFailedTotalList.parallelStream().collect(Collectors.groupingBy(o -> (o.getRegion()+o.getInstallSupplierName()), Collectors.toList())).forEach((id, transfer)->{
            transfer.stream().reduce((a,b)->new InstallationDataStatisticsBO(a.getRegion(),a.getInstallSupplierName(),a.getOrdersTotal(),a.getRefuseTotal(),a.getInstallationFailedTotal(),b.getInstallationCompleteTotal(),a.getAvgInstallationTime(),a.getOrderRate())).ifPresent(mergeInstallationCompleteTotal::add);
        });

        for (InstallationDataStatisticsBO bo:mergeInstallationCompleteTotal) {
            //获取地市平均安装时长
            List<String> processInsIdList=reportStatisticsMapper.getOrderByNodeFlag("4",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime(),"1");
            long time=0;
            for(String processInsId:processInsIdList){
                List<TaskInstBO> taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_101.getKey()+"-"+FlowNodeEnum.NODE_101.getValue());
                List<TaskInstBO> taskInstBOList2 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_112.getKey()+"-"+FlowNodeEnum.NODE_112.getValue());
                if(taskInstBOList1.size()>0&&taskInstBOList2.size()>0){

                    time+=taskInstBOList2.get(0).getEndTime().getTime()-taskInstBOList1.get(0).getStartTime().getTime();
                }

            }
            time=time/processInsIdList.size();
            bo.setAvgInstallationTime(ProcessUtil.timeFormat(time).toString());


            //地市订单完成率
            Integer total=reportStatisticsMapper.getInstallationCompleteTotal("4",bo.getRegion(),bo.getInstallSupplierName(),reportStatisticsBO.getStartTime(),reportStatisticsBO.getEndTime());
            bo.setOrderRate((total/bo.getOrdersTotal())*100+"%");

        }
        return mergeInstallationCompleteTotal;
    }

    /**
     * 充电桩安装订单时长统计
     * @param list
     * @param regionLevel
     * @param reportStatisticsBO
     * @return
     */
    private List<InstallationDurationStatisticsBO>  getInstallationDurationStatistics(List<InstallationDurationStatisticsBO> list,String regionLevel,ReportStatisticsBO reportStatisticsBO){
        for (InstallationDurationStatisticsBO bo : list) {
            List<String> processInsIdList = reportStatisticsMapper.getOrderByNodeFlag(regionLevel, bo.getRegion(), bo.getInstallSupplierName(), reportStatisticsBO.getStartTime(), reportStatisticsBO.getEndTime(), "2");
            long surveyAppointment = 0;
            long survey = 0;
            long installationAppointment = 0;
            long installation = 0;
            long settlement = 0;
            for (String processInsId : processInsIdList) {
                List<TaskInstBO> taskInstBOList1 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_101.getKey() + "-" + FlowNodeEnum.NODE_101.getValue());
                //勘察预约平均时长
                List<TaskInstBO> taskInstBOList105 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_105.getKey() + "-" + FlowNodeEnum.NODE_105.getValue());
                //勘察平均时长
                List<TaskInstBO> taskInstBOList107 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_107.getKey() + "-" + FlowNodeEnum.NODE_107.getValue());
                //安装预约平均时长
                List<TaskInstBO> taskInstBOList110 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_110.getKey() + "-" + FlowNodeEnum.NODE_110.getValue());
                //安装平均时长
                List<TaskInstBO> taskInstBOList112 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_112.getKey() + "-" + FlowNodeEnum.NODE_112.getValue());
                //结算时长
                List<TaskInstBO> taskInstBOList114 = statisticsMapper.getTaskInstByInstIdAndName(processInsId, FlowNodeEnum.NODE_114.getKey() + "-" + FlowNodeEnum.NODE_114.getValue());


                if (taskInstBOList1.size() > 0 && taskInstBOList105.size() > 0) {

                    surveyAppointment += taskInstBOList105.get(0).getStartTime().getTime() - taskInstBOList1.get(0).getStartTime().getTime();
                }

                if (taskInstBOList1.size() > 0 && taskInstBOList107.size() > 0) {

                    survey += taskInstBOList107.get(0).getStartTime().getTime() - taskInstBOList1.get(0).getStartTime().getTime();
                }

                if (taskInstBOList1.size() > 0 && taskInstBOList110.size() > 0) {

                    installationAppointment += taskInstBOList110.get(0).getStartTime().getTime() - taskInstBOList1.get(0).getStartTime().getTime();
                }

                if (taskInstBOList1.size() > 0 && taskInstBOList112.size() > 0) {

                    installation += taskInstBOList112.get(0).getStartTime().getTime() - taskInstBOList1.get(0).getStartTime().getTime();
                }

                if (taskInstBOList1.size() > 0 && taskInstBOList114.size() > 0) {

                    settlement += taskInstBOList114.get(0).getEndTime().getTime() - taskInstBOList1.get(0).getStartTime().getTime();
                }

            }

            bo.setSurveyAppointment((float) (Math.round(((float) surveyAppointment / (24 * 60 * 60 * 1000)/processInsIdList.size()) * 100)) / 100);
            bo.setSurvey((float) (Math.round(((float) survey / (24 * 60 * 60 * 1000)) * 100)) / 100);
            bo.setInstallationAppointment((float) (Math.round(((float) installationAppointment / (24 * 60 * 60 * 1000)/processInsIdList.size()) * 100)) / 100);
            bo.setInstallation((float) (Math.round(((float) installation / (24 * 60 * 60 * 1000)) * 100)/processInsIdList.size()) / 100);
            bo.setReturnVisit(0);
            bo.setSettlement((float) (Math.round(((float) settlement / (24 * 60 * 60 * 1000)) * 100)/processInsIdList.size()) / 100);

        }
        return  list;
    }


}
