package com.boco.nbd.wios.flow.service.impl;

import com.boco.nbd.framework.common.util.StringUtil;
import com.boco.nbd.wios.flow.entity.bo.VisualManagementBO;
import com.boco.nbd.wios.flow.entity.bo.VisualManagementTowNumBO;
import com.boco.nbd.wios.flow.entity.qo.VisualManagementQo;
import com.boco.nbd.wios.flow.entity.vo.VisualManagementNodeFlagCountVO;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import com.boco.nbd.wios.flow.mapper.def.VisualManagementMapper;
import com.boco.nbd.wios.flow.service.IVisualManagementService;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.mapper.def.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 目视化管理接口实现类
 * @Author: ZQY
 * @Date: 2022/10/26
 **/
@Service
public class VisualManagementServiceImpl implements IVisualManagementService {

    @Resource
    private VisualManagementMapper visualManagementMapper;

    @Resource
    private RegionMapper regionMapper;

    @Resource
    private SupplierMapper supplierMapper;

    @Resource
    private AccountMapper accountMapper;


    @Resource
    private AccountRoleMapper accountRoleMapper;


    @Resource
    private CamsAccountMapper camsAccountMapper;

    @Override
    public List<VisualManagementNodeFlagCountVO> getNodeFlagByArea(String startTime,String endTime,String userId) {


        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementBO> visualManagementBOList = visualManagementMapper.getNodeFlagByArea(qo);


        return getVisualManagementAreaNodeFlagCountVOList(visualManagementBOList,qo);
    }

    @Override
    public List<VisualManagementNodeFlagCountVO> getNodeFlagByInstallSupplierl(String startTime, String endTime, String areaId,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        if(!"-1".equals(areaId)){
            qo.setAreaId(areaId);
        }

        List<VisualManagementBO> visualManagementBOList = visualManagementMapper.getNodeFlagByInstallSupplierl(qo);


        return  getVisualManagementSupplierlNodeFlagCountVOList(visualManagementBOList,areaId,qo);

    }

    @Override
    public List<VisualManagementNodeFlagCountVO> getOvertimeNodeFlagByAreaCount(String startTime, String endTime,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementBO> visualManagementBOList = visualManagementMapper.getOvertimeNodeFlagByAreaCount(qo);

        return getVisualManagementAreaNodeFlagCountVOList(visualManagementBOList,qo);
    }

    @Override
    public List<VisualManagementNodeFlagCountVO> getOvertimeNodeFlagByInstallSupplierl(String startTime, String endTime,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementBO> visualManagementBOList = visualManagementMapper.getOvertimeNodeFlagByInstallSupplierl(qo);

        return  getVisualManagementSupplierlNodeFlagCountVOList(visualManagementBOList,"",qo);
    }

    @Override
    public List<VisualManagementNodeFlagCountVO> getAboutOvertimeNodeFlagByAreaCount(String startTime, String endTime, String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementBO> visualManagementBOList = visualManagementMapper.getAboutOvertimeNodeFlagByAreaCount(qo);

        return getVisualManagementAreaNodeFlagCountVOList(visualManagementBOList,qo);
    }

    @Override
    public List<VisualManagementNodeFlagCountVO> getAboutOvertimeNodeFlagByInstallSupplierl(String startTime, String endTime, String userId) {
        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementBO> visualManagementBOList = visualManagementMapper.getAboutOvertimeNodeFlagByInstallSupplierl(qo);

        return  getVisualManagementSupplierlNodeFlagCountVOList(visualManagementBOList,"",qo);
    }

    @Override
    public List<VisualManagementTowNumBO> getCustomerNode101ByArea(String startTime, String endTime,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementTowNumBO> visualManagementTowNumBOList =  visualManagementMapper.getCustomerNode101ByArea(qo);

        List<Region> regionList = regionMapper.selectByParentId(1);

        //补齐没有统计区域
        for(Region region : regionList){

            if(!StringUtil.isEmpty(qo.getAreaId())){
                if(!qo.getAreaId().contains(region.getId().toString())){
                    continue;
                }
            }


            Integer count= Math.toIntExact(visualManagementTowNumBOList.stream().filter(v -> region.getId().toString().equals(v.getId())).count());

            if(count == 0){
                visualManagementTowNumBOList.add(new VisualManagementTowNumBO(region.getId().toString(),region.getName(),0,0));
            }
        }


        return visualManagementTowNumBOList;
    }

    @Override
    public List<VisualManagementTowNumBO> getOrderReceivingBySupplierl(String startTime, String endTime,String id,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementTowNumBO> visualManagementTowNumBOList =  visualManagementMapper.getOrderReceivingBySupplierl(qo);


        return getVisualManagementTowNumBOList(visualManagementTowNumBOList,id,qo);
    }

    @Override
    public List<VisualManagementTowNumBO> getOrderReceivingByAreaAndSupplierlId(String startTime, String endTime, String installSupplierId,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        qo.setInstallSupplierId(installSupplierId);

        List<VisualManagementTowNumBO> visualManagementTowNumBOList =  visualManagementMapper.getOrderReceivingByAreaAndSupplierlId(qo);

        List<Region> regionList = regionMapper.selectByParentId(1);

        //补齐没有统计区域
        for(Region region : regionList){

            if(!StringUtil.isEmpty(qo.getAreaId())){
                if(!qo.getAreaId().contains(region.getId().toString())){
                    continue;
                }
            }

            Integer count= Math.toIntExact(visualManagementTowNumBOList.stream().filter(v -> region.getId().toString().equals(v.getId())).count());

            if(count == 0){
                visualManagementTowNumBOList.add(new VisualManagementTowNumBO(region.getId().toString(),region.getName(),0,0));
            }
        }


        return visualManagementTowNumBOList;

    }

    @Override
    public List<VisualManagementTowNumBO> getOrderSettlementBySupplierl(String startTime, String endTime,String id,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        List<VisualManagementTowNumBO> visualManagementTowNumBOList =  visualManagementMapper.getOrderSettlementBySupplierl(qo);

        return getVisualManagementTowNumBOList(visualManagementTowNumBOList,id,qo);
    }

    @Override
    public List<VisualManagementTowNumBO> getOrderSettlementByAreaAndSupplierlId(String startTime, String endTime, String installSupplierId,String userId) {

        VisualManagementQo qo = getVisualManagementQo( startTime, endTime, userId);

        qo.setInstallSupplierId(installSupplierId);

        List<VisualManagementTowNumBO> visualManagementTowNumBOList =  visualManagementMapper.getOrderSettlementByAreaAndSupplierlId(qo);

        List<Region> regionList = regionMapper.selectByParentId(1);

        //补齐没有统计区域
        for(Region region : regionList){

            if(!StringUtil.isEmpty(qo.getAreaId())){
                if(!qo.getAreaId().contains(region.getId().toString())){
                    continue;
                }
            }

            Integer count= Math.toIntExact(visualManagementTowNumBOList.stream().filter(v -> region.getId().toString().equals(v.getId())).count());

            if(count == 0){
                visualManagementTowNumBOList.add(new VisualManagementTowNumBO(region.getId().toString(),region.getName(),0,0));
            }
        }


        return visualManagementTowNumBOList;
    }


    /**
     * 分权分域
     * @param startTime
     * @param endTime
     * @param userId
     * @return
     */
    private VisualManagementQo getVisualManagementQo(String startTime,String endTime,String userId){
        VisualManagementQo qo = new VisualManagementQo();
        qo.setStartTime(startTime);
        qo.setEndTime(endTime);

        if(! StringUtil.isEmpty(userId)){
            Account account =  accountMapper.selectByPrimaryKey(Integer.parseInt(userId));
            Integer accountId = 0;

            if(account.getParentId()!= null){
                AccountRole accountRole;
                if(account.getParentId()==1){
                    accountId = account.getId();
                    accountRole = accountRoleMapper.selectByAccountId(account.getId());
                }else{
                    accountId = account.getParentId();
                    accountRole = accountRoleMapper.selectByAccountId(account.getParentId());

                }

                if(accountRole.getRoleId() == 4){
                    Supplier supplier = supplierMapper.selectByAccountId(accountId);
                    if(supplier != null){
                        qo.setInstallSupplierId(supplier.getId());
                    }
                }else if(accountRole.getRoleId() !=1 && accountRole.getRoleId() !=2 && accountRole.getRoleId() !=3 && accountRole.getRoleId() !=6){
                    List<CamsAccount> camsAccountList = camsAccountMapper.selectByAccountIdAndDeptId(account.getId(),null);

                    if(camsAccountList.size() > 0){
                        qo.setAreaId(camsAccountList.get(0).getRegionId());
                    }
                }

            }
        }


        return  qo;
    }

    /**
     * 区域节点统计
     * @param visualManagementBOList
     * @return
     */
    private List<VisualManagementNodeFlagCountVO> getVisualManagementAreaNodeFlagCountVOList(List<VisualManagementBO> visualManagementBOList,VisualManagementQo qo){

        List<VisualManagementNodeFlagCountVO> voList = new ArrayList<>();

        List<Region> regionList = regionMapper.selectByParentId(1);


        for(Region region : regionList){

                if(!StringUtil.isEmpty(qo.getAreaId())){
                    if(!qo.getAreaId().contains(region.getId().toString())){
                            continue;
                    }
                }

                //筛选出区域，补齐没有统计到的节点为0
                List<VisualManagementBO> visualManagementBOAreaList = visualManagementBOList.stream().filter(v -> region.getId().toString().equals(v.getId())).collect(Collectors.toList());

                VisualManagementNodeFlagCountVO vo = new VisualManagementNodeFlagCountVO(String.valueOf(region.getId()), region.getName(), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

                if (visualManagementBOAreaList.size() > 0) {

                    //将对象VisualManagementBO转换为VisualManagementNodeFlagCountVO
                    Map<String, VisualManagementBO> boMap = visualManagementBOAreaList.stream().collect(Collectors.toMap(VisualManagementBO::getNodeFlag, bo -> bo));

                    getVisualManagementNodeFlagCountVO(boMap, vo);

                }

                voList.add(vo);

        }

        //voList.add(getAllVisualManagementNodeFlagCountVO("-1","全国",voList));

        return voList;
    }





    /**
     * 服务商合并A类服务商节点统计
     * @param visualManagementBOList
     * @return
     */
    private List<VisualManagementNodeFlagCountVO> getVisualManagementSupplierlNodeFlagCountVOList(List<VisualManagementBO> visualManagementBOList,String areaId,VisualManagementQo qo){

        List<VisualManagementNodeFlagCountVO> voList = new ArrayList<>();
        List<VisualManagementNodeFlagCountVO> voAList = new ArrayList<>();
        List<Supplier> supplierList ;
        if(!StringUtil.isEmpty(areaId)&&!"-1".equals(areaId)){

            supplierList  = supplierMapper.selectByRegionId(areaId);

        }else{
            supplierList =  supplierMapper.selectAll(1);
        }

        for(Supplier supplier : supplierList){


            if(!StringUtil.isEmpty(qo.getInstallSupplierId())){
                if(!qo.getInstallSupplierId().equals(supplier.getId())){
                    continue;
                }
            }

            List<VisualManagementBO> visualManagementBOSupplierList = visualManagementBOList.stream().filter(v -> supplier.getId().equals(v.getId())).collect(Collectors.toList());
            VisualManagementNodeFlagCountVO vo= new VisualManagementNodeFlagCountVO(String.valueOf(supplier.getId()),supplier.getName(),0,0,0,0,0,0,0,0,0,0,0,0,0);


            if(StringUtil.isEmpty(areaId)){
                //处理A类服务商
                if(!"A".equals(supplier.getLevel())){
                    if(visualManagementBOSupplierList.size()>0){

                        Map <String,VisualManagementBO> boMap = visualManagementBOSupplierList.stream().collect(Collectors.toMap(VisualManagementBO::getNodeFlag,bo -> bo));

                        getVisualManagementNodeFlagCountVO(boMap,vo);

                    }
                    voList.add(vo);
                }else{
                    if(visualManagementBOSupplierList.size()>0){

                        Map <String,VisualManagementBO> boMap = visualManagementBOSupplierList.stream().collect(Collectors.toMap(VisualManagementBO::getNodeFlag,bo -> bo));

                        getVisualManagementNodeFlagCountVO(boMap,vo);

                    }
                    voAList.add(vo);
                }
            }else {
                if(visualManagementBOSupplierList.size()>0){

                    Map <String,VisualManagementBO> boMap = visualManagementBOSupplierList.stream().collect(Collectors.toMap(VisualManagementBO::getNodeFlag,bo -> bo));

                    getVisualManagementNodeFlagCountVO(boMap,vo);

                }
                voList.add(vo);
            }


        }

        if(voAList.size() > 0){
            voList.add(getAllVisualManagementNodeFlagCountVO("-2","自建安装团队",voAList));
        }
        //voList.add(getAllVisualManagementNodeFlagCountVO("-1","总服务商",voList));

        return voList;
    }

    /**
     * 订单结算、接单情况A类服务商处理
     * @param id
     * @return
     */
    private List<VisualManagementTowNumBO> getVisualManagementTowNumBOList(List<VisualManagementTowNumBO> visualManagementTowNumBOList,String id,VisualManagementQo qo){

        List<VisualManagementTowNumBO> visualManagementTowNumBOListVo =  new ArrayList<>();
        List<VisualManagementTowNumBO> visualManagementTowNumBOSupplierAList =  new ArrayList<>();

        List<Supplier> supplierList = new ArrayList<>();
        if(!StringUtil.isEmpty(id)){
            if("-2".equals(id)){
                supplierList = supplierMapper.selectByLevel("A");
            }
        }else{
            supplierList = supplierMapper.selectAll(1);
        }


        //补齐没有统计服务商
        for(Supplier supplier : supplierList){

            if(!StringUtil.isEmpty(qo.getInstallSupplierId())){
                if(!qo.getInstallSupplierId().equals(supplier.getId())){
                    continue;
                }
            }

            List<VisualManagementTowNumBO> list=visualManagementTowNumBOList.stream().filter(v -> supplier.getId().equals(v.getId())).collect(Collectors.toList());
            if(StringUtil.isEmpty(id)){
                //处理A类服务商
                if("A".equals(supplier.getLevel())){
                    if(list.size() > 0){
                        visualManagementTowNumBOSupplierAList.addAll(list);
                    }else{
                        visualManagementTowNumBOSupplierAList.add(new VisualManagementTowNumBO(supplier.getId(),supplier.getName(),0,0));
                    }
                }else{
                    if(list.size() > 0){
                        visualManagementTowNumBOListVo.addAll(list);
                    }else{
                        visualManagementTowNumBOListVo.add(new VisualManagementTowNumBO(supplier.getId(),supplier.getName(),0,0));
                    }
                }

            }else{
                if(list.size() > 0){
                    visualManagementTowNumBOListVo.addAll(list);
                }else{
                    visualManagementTowNumBOListVo.add(new VisualManagementTowNumBO(supplier.getId(),supplier.getName(),0,0));
                }
            }
        }

        //汇总A类服务商
        if(visualManagementTowNumBOSupplierAList.size() > 0){

            VisualManagementTowNumBO bo = new VisualManagementTowNumBO("-2","自建安装团队",visualManagementTowNumBOSupplierAList.stream().collect(Collectors.summingInt(VisualManagementTowNumBO::getNum1)),visualManagementTowNumBOSupplierAList.stream().collect(Collectors.summingInt(VisualManagementTowNumBO::getNum2)));
            visualManagementTowNumBOListVo.add(bo);

        }

        return  visualManagementTowNumBOListVo;
    }

    /**
     * 给节点赋值
     * @return
     */
    private void getVisualManagementNodeFlagCountVO(Map <String,VisualManagementBO> boMap,VisualManagementNodeFlagCountVO vo){
        if(boMap.containsKey(FlowNodeEnum.NODE_101.getKey().toString())){
            vo.setNodeFlag101Count(boMap.get(FlowNodeEnum.NODE_101.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_103.getKey().toString())){
            vo.setNodeFlag103Count(boMap.get(FlowNodeEnum.NODE_103.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_104.getKey().toString())){
            vo.setNodeFlag104Count(boMap.get(FlowNodeEnum.NODE_104.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_105.getKey().toString())){
            vo.setNodeFlag105Count(boMap.get(FlowNodeEnum.NODE_105.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_106.getKey().toString())){
            vo.setNodeFlag106Count(boMap.get(FlowNodeEnum.NODE_106.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_115.getKey().toString())){
            vo.setNodeFlag115Count(boMap.get(FlowNodeEnum.NODE_115.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_107.getKey().toString())){
            vo.setNodeFlag107Count(boMap.get(FlowNodeEnum.NODE_107.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_108.getKey().toString())){
            vo.setNodeFlag108Count(boMap.get(FlowNodeEnum.NODE_108.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_110.getKey().toString())){
            vo.setNodeFlag110Count(boMap.get(FlowNodeEnum.NODE_110.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_111.getKey().toString())){
            vo.setNodeFlag111Count(boMap.get(FlowNodeEnum.NODE_111.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_112.getKey().toString())){
            vo.setNodeFlag112Count(boMap.get(FlowNodeEnum.NODE_112.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_113.getKey().toString())){
            vo.setNodeFlag113Count(boMap.get(FlowNodeEnum.NODE_113.getKey().toString()).getNum());
        }

        if(boMap.containsKey(FlowNodeEnum.NODE_114.getKey().toString())){
            vo.setNodeFlag114Count(boMap.get(FlowNodeEnum.NODE_114.getKey().toString()).getNum());
        }
    }

    /**
     * 计算总的节点数量
     * @param id
     * @param name
     * @param voList
     * @return
     */
    private VisualManagementNodeFlagCountVO getAllVisualManagementNodeFlagCountVO(String id,String name,List<VisualManagementNodeFlagCountVO> voList){
        VisualManagementNodeFlagCountVO allVo = new VisualManagementNodeFlagCountVO(id,name
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag101Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag103Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag104Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag105Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag106Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag115Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag107Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag108Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag110Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag111Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag112Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag113Count))
                ,voList.stream().collect(Collectors.summingInt(VisualManagementNodeFlagCountVO::getNodeFlag114Count)));

                        return  allVo;
    }

}
