package com.boco.nbd.wios.flow.sms.aliyun;

import com.boco.nbd.wios.flow.sms.aliyun.entity.NoticeData;
import org.apache.commons.lang3.StringUtils;
import org.springframework.lang.Nullable;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

/**
 * @ClassName SendHandler
 * @Description 发送处理
 * @Author songyanfei
 * @Email songyanfei@boco.com.cn
 * @Date 2022/10/28 10:02
 * @Version V1.0
 */
public interface SendHandler {

    /**
     * 是否允许发送通知.
     *
     * @param type 通知类型
     * @return 是否允许
     */
    boolean acceptSend(String type);

    /**
     * 发送通知.
     *
     * @param noticeData 通知内容
     * @param phones     手机号列表
     * @return 是否发送成功
     */
    boolean send(NoticeData noticeData, Collection<String> phones);

    /**
     * 发送通知.
     *
     * @param noticeData 通知内容
     * @param phone      手机号列表
     * @return 是否发送成功
     */
    default boolean send(NoticeData noticeData, String phone) {
        if (StringUtils.isBlank(phone)) {
            return false;
        }

        return send(noticeData, Collections.singletonList(phone));
    }

    /**
     * 发送通知.
     *
     * @param noticeData 通知内容
     * @param phones     手机号列表
     * @return 是否发送成功
     */
    default boolean send(NoticeData noticeData, @Nullable String... phones) {
        if (phones == null) {
            return false;
        }

        return send(noticeData, Arrays.asList(phones));
    }
}
