package com.boco.nbd.wios.log.service.impl;

import com.boco.nbd.framework.common.util.StringUtil;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.wios.log.entity.po.InterfaceMonitorPO;
import com.boco.nbd.wios.log.entity.qo.InterfaceMonitorQO;
import com.boco.nbd.wios.log.mapper.def.InterfaceMonitorMapper;
import com.boco.nbd.wios.log.service.InterfaceMonitorService;
import com.boco.nbd.wios.log.util.MapUtil;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * InterfaceMonitorServiceImpl
 *
 * @Author: ZQY
 * @Date: 2022/10/24
 */
@Service
public class InterfaceMonitorServiceImpl extends BaseDaoService3<InterfaceMonitorPO, InterfaceMonitorMapper> implements InterfaceMonitorService {


    @Autowired
    private InterfaceMonitorMapper interfaceMonitorMapper;

    @Override
    public Integer add(InterfaceMonitorPO po) {
        return interfaceMonitorMapper.insert(po);
    }

    @Override
    public List<InterfaceMonitorPO> getInterfaceMonitorList(InterfaceMonitorQO qo) {
        return interfaceMonitorMapper.getInterfaceMonitorList(qo);
    }

    @Override
    public Map<String, Integer> getInterfaceMonitorCount(InterfaceMonitorQO qo) {
        //查询正常数量
        qo.setInterfaceState(1);
        Integer normal = interfaceMonitorMapper.getInterfaceMonitorCount(qo);

        //查询异常数量
        qo.setInterfaceState(2);
        Integer abnormal = interfaceMonitorMapper.getInterfaceMonitorCount(qo);

        Map<String, Integer> map = new HashedMap(2);
        map.put("正常", normal);
        map.put("异常", abnormal);

        return map;
    }

    @Override
    public Map<String, Integer> getInterfaceMonitorCountMap(InterfaceMonitorQO qo) {
        //查询正常数量
        qo.setInterfaceState(1);
        Integer normal = interfaceMonitorMapper.getInterfaceMonitorCount(qo);
        Map<String, Integer> map = new HashedMap(2);
        map.put("正常", normal);

        //查询异常数量
        qo.setInterfaceState(2);
        List<Map<String, Object>> mapList = interfaceMonitorMapper.getInterfaceMonitorCountMap(qo);

        map.putAll(MapUtil.getMap(mapList));


        return map;
    }

    @Override
    public List<Map<String, Integer>> getInterfaceMonitorMothCountMap(String time, String interfaceName) {

        List<Map<String, Integer>> mapList = new ArrayList<>();


        String[] timeArry = time.split("-");
        int days = getDays(Integer.parseInt(timeArry[0]), Integer.parseInt(timeArry[1]));
        InterfaceMonitorQO qo = new InterfaceMonitorQO();
        qo.setStartTime(time + "-01");
        qo.setEndTime(time + "-" + days);
        qo.setTimeType("day");
        if (!StringUtil.isEmpty(interfaceName)) {
            qo.setInterfaceName(interfaceName);
        }
        qo.setInterfaceState(1);

        //正常调用统计
        Map<String, Integer> normalMap = new HashedMap(2);
        normalMap.putAll(MapUtil.getMap(interfaceMonitorMapper.getInterfaceMonitorMothCountMap(qo)));
        normalMap = getMothMap(normalMap, days, time);
        mapList.add(normalMap);

        //异常调用统计
        qo.setInterfaceState(2);
        Map<String, Integer> abnormal = new HashedMap(2);
        abnormal.putAll(MapUtil.getMap(interfaceMonitorMapper.getInterfaceMonitorMothCountMap(qo)));
        abnormal = getMothMap(abnormal, days, time);
        mapList.add(abnormal);


        return mapList;
    }


    /**
     * 计算当前月有多少天
     *
     * @return
     */
    private int getDays(int year, int month) {
        int days = 0;
        switch (month) {
            case 2:
                days = leapYear(year) ? 29 : 28;
                break;
            case 1:
            case 3:
            case 5:
            case 7:
            case 8:
            case 10:
            case 12:
                days = 31;
                break;
            case 4:
            case 6:
            case 9:
            case 11:
                days = 30;
                break;
            default:
                break;
        }
        return days;
    }

    /**
     * 是否闰月
     *
     * @param year
     * @return
     */
    private boolean leapYear(int year) {
        int four = 4;
        int hundred = 100;
        int fourHundred = 400;

        return (year % four == 0 && year % hundred != 0 || year % fourHundred == 0);
    }

    /**
     * 补齐统计为0的日期
     *
     * @param map
     * @param days
     * @param time
     * @return
     */
    private Map<String, Integer> getMothMap(Map<String, Integer> map, Integer days, String time) {

        for (int i = 1; i <= days; i++) {
            String day = i < 10 ? "0" + i : String.valueOf(i);
            if (!map.containsKey(time + "-" + day)) {
                map.put(time + "-" + day, 0);
            }
        }

        //map排序
        map = MapUtil.sortByKeyAsc(map);

        return map;
    }


}
