package com.boco.nbd.wios.log.service.impl;


import com.boco.nbd.framework.common.util.StringUtil;
import com.boco.nbd.framework.persistence.BaseDaoService;
import com.boco.nbd.wios.log.entity.po.*;
import com.boco.nbd.wios.log.entity.qo.SysLogQO;
import com.boco.nbd.wios.log.mapper.def.SysLogMapper;
import com.boco.nbd.wios.log.service.ISysLogService;
import com.boco.nbd.wios.manage.entity.bo.CamsAccountBo;
import com.boco.nbd.wios.manage.entity.bo.CamsAccountVo;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import com.boco.nbd.wios.manage.entity.bo.Supplier;
import com.boco.nbd.wios.manage.mapper.def.OemMapper;
import com.boco.nbd.wios.manage.mapper.def.SupplierMapper;
import com.boco.nbd.wios.manage.service.impl.CamsAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * SysLogServiceImpl
 * @Author: ZQY
 * @Date: 2022/8/2022/10/17
 **/
@Service
public class SysLogServiceImpl extends BaseDaoService<SysLogPO, SysLogQO, String, SysLogMapper>
		implements ISysLogService {

	@Autowired
	private SysLogMapper customerUseLogMapper;

	@Autowired
	private OemMapper oemMapper;

	@Autowired
	private SupplierMapper supplierMapper;

	@Autowired
	private CamsAccountService camsAccountService;


	/** ###################登录日志分割线######################### */

	@Override
	public List<UserLogCountInfoPO> getUsersLoginStat(String start, String end)
			throws DataAccessException {
		return customerUseLogMapper.getUsersLoginStat(start, end);
	}

	@Override
	public List<SysLogInfoPO> getUsersLoginDetails(String start, String end, String userId) throws DataAccessException {

		List<SysLogInfoPO> sysLogInfoPOList = customerUseLogMapper.getUsersLoginDetails(start, end, userId);

		//查询用户的归属
		if(sysLogInfoPOList.size() > 0){

			String ascriptionName;
			if(StringUtil.isEmpty(sysLogInfoPOList.get(0).getParentId())){
				 ascriptionName="超级管理员";
			} else if("1".equals(sysLogInfoPOList.get(0).getParentId())){
				ascriptionName=getAscriptionName(sysLogInfoPOList.get(0).getRoleId(),sysLogInfoPOList.get(0).getUserId(),sysLogInfoPOList.get(0).getUserName());
			}else{
				ascriptionName=getAscriptionName(sysLogInfoPOList.get(0).getParentRoleId(),sysLogInfoPOList.get(0).getParentId(),sysLogInfoPOList.get(0).getParentUserName());
			}
			sysLogInfoPOList.forEach(sysLogInfoPO -> sysLogInfoPO.setAscriptionName(ascriptionName));
		}


		return sysLogInfoPOList;
	}

	@Override
	public List<RoleLogCountInfoPO> getRolesLoginStat(String start, String end)
			throws DataAccessException {
		return customerUseLogMapper.getRolesLoginStat(start, end);
	}


	@Override
	public List<SysLogInfoPO> getRolesLoginDetails(String start, String end, String roleName)
			throws DataAccessException {
		List<SysLogInfoPO> sysLogInfoPOList = customerUseLogMapper.getRolesLoginDetails(start, end, roleName);

		for(SysLogInfoPO po : sysLogInfoPOList){

			String ascriptionName;
			if(StringUtil.isEmpty(po.getParentId())){
				ascriptionName="超级管理员";
			} else if("1".equals(po.getParentId())){
				ascriptionName=getAscriptionName(po.getRoleId(),po.getUserId(),po.getUserName());
			}else{
				ascriptionName=getAscriptionName(po.getParentRoleId(),po.getParentId(),po.getParentUserName());
			}
			po.setAscriptionName(ascriptionName);
		}

		return sysLogInfoPOList;
	}


	/** ###################访问日志统计分割线######################### */

	@Override
	public List<UserLogCountInfoPO> countByUserDate(String start, String end) throws DataAccessException {

		return customerUseLogMapper.countByUserDate(start, end);
	}

	@Override
	public List<SysLogInfoPO> selectLogInfoByUserIdDate(String start, String end, String userId)
			throws DataAccessException {

		List<SysLogInfoPO> sysLogInfoPOList = customerUseLogMapper.selectLogInfoByUserIdDate(start, end, userId);

		//查询用户的归属
		if(sysLogInfoPOList.size() > 0){

			String ascriptionName;
			if(StringUtil.isEmpty(sysLogInfoPOList.get(0).getParentId())){
				ascriptionName="超级管理员";
			} else if("1".equals(sysLogInfoPOList.get(0).getParentId())){
				ascriptionName=getAscriptionName(sysLogInfoPOList.get(0).getRoleId(),sysLogInfoPOList.get(0).getUserId(),sysLogInfoPOList.get(0).getUserName());
			}else{
				ascriptionName=getAscriptionName(sysLogInfoPOList.get(0).getParentRoleId(),sysLogInfoPOList.get(0).getParentId(),sysLogInfoPOList.get(0).getParentUserName());
			}
			sysLogInfoPOList.forEach(sysLogInfoPO -> sysLogInfoPO.setAscriptionName(ascriptionName));
		}


		return customerUseLogMapper.selectLogInfoByUserIdDate(start, end, userId);
	}



	@Override
	public List<RoleLogCountInfoPO> countByRoleDate(String start, String end) throws DataAccessException {
		return customerUseLogMapper.countByRoleDate(start, end);
	}


	@Override
	public List<SysLogInfoPO> selectByRoleDate(String start, String end, String roleName)
			throws DataAccessException {

		List<SysLogInfoPO> sysLogInfoPOList = customerUseLogMapper.selectByRoleDate(start, end, roleName);

		for(SysLogInfoPO po : sysLogInfoPOList){

			String ascriptionName;
			if(StringUtil.isEmpty(po.getParentId())){
				ascriptionName="超级管理员";
			} else if("1".equals(po.getParentId())){
				ascriptionName=getAscriptionName(po.getRoleId(),po.getUserId(),po.getUserName());
			}else{
				ascriptionName=getAscriptionName(po.getParentRoleId(),po.getParentId(),po.getParentUserName());
			}
			po.setAscriptionName(ascriptionName);
		}
		return sysLogInfoPOList;
	}


	/** ###################模块访问日志统计分割线######################### */
	@Override
	public List<ModuleLogCountInfoPO> countByModuleDate(String start, String end)
			throws DataAccessException {
		return customerUseLogMapper.countByModuleDate(start, end);
	}

	@Override
	public List<SysLogInfoPO> selectLogInfoByModuleIdDate(String start, String end, String moduleId)
			throws DataAccessException {

		List<SysLogInfoPO> sysLogInfoPOList = customerUseLogMapper.selectLogInfoByModuleIdDate(start, end, moduleId);

		for(SysLogInfoPO po : sysLogInfoPOList){

			String ascriptionName;
			if(StringUtil.isEmpty(po.getParentId())){
				ascriptionName="超级管理员";
			} else if("1".equals(po.getParentId())){
				ascriptionName=getAscriptionName(po.getRoleId(),po.getUserId(),po.getUserName());
			}else{
				ascriptionName=getAscriptionName(po.getParentRoleId(),po.getParentId(),po.getParentUserName());
			}
			po.setAscriptionName(ascriptionName);
		}
		return sysLogInfoPOList;
	}


	/**
	 * 查询归属
	 * @param roleId
	 * @param id
	 * @return
	 */
	private String getAscriptionName(String roleId,String id,String account){
		String ascriptionName="";
		if("2".equals(roleId) || "3".equals(roleId)){
			List<Oem> oemList = oemMapper.selectByAccountId(Integer.parseInt(id));
			if(oemList.size() > 0){
				ascriptionName = oemList.get(0).getName();
			}

		}else if("4".equals(roleId)){
			  Supplier supplier = supplierMapper.selectByAccountId(Integer.parseInt(id));
			  if(supplier != null){
				  ascriptionName = supplier.getName();
			  }

		}else{
			CamsAccountBo condition=new CamsAccountBo();
			condition.setAccount(account);
			List<CamsAccountVo> camsAccountVoList = camsAccountService.getList(condition);
			if(camsAccountVoList.size() > 0){
				ascriptionName = camsAccountVoList.get(0).getRegionName();
			}
		}

		return ascriptionName;
	}

}