package com.boco.nbd.wios.manage.controller;

import com.boco.nbd.wios.manage.service.impl.OrderScoreService;
import com.boco.nbd.wios.manage.service.impl.OrderService;
import com.ihidea.component.api.v2.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

/**
 * 订单评分表 管理 API
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@RestController
@RequestMapping("api")
@Api(tags = "订单评分表的接口")
@ApiIgnore
public class OrderScoreController {

    @Autowired
    private OrderScoreService orderScoreService;

    @Autowired
    private OrderService orderService;

    /**
     * 查询订单评分
     *
     * @return
     */
    @GetMapping(value = "orderScore/getOrderScoreByOrderId")
    @ApiOperation(value = "查询订单评分")
    @ApiImplicitParam(paramType = "query", name = "orderId", value = "订单表ID", required = true, dataType = "String")
    public BaseResponse<Object> list(String orderId) {
        return new BaseResponse<>(orderScoreService.getOrderScoreByOrderId(orderId));
    }

    /**
     * 保存订单回访信息
     *
     * @param list
     */
    @PostMapping(value = "orderScore/save")
    @ApiOperation(value = "保存订单回访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "订单id", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "visitContent", value = "意见或建议", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "list", value = "格式:[{\"id\":9,\"score\":70},{\"id\":12,\"score\":60}]", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "knowAfterSalesChannel", value = "是否了解墙盒售后问题反馈渠道：0-否，1：是", dataType = "int", paramType = "query", required = true),
            @ApiImplicitParam(name = "installLength", value = "墙盒线缆的安装长度", dataType = "int", paramType = "query", required = true),
            })
    public BaseResponse<Object> save(String orderId, String visitContent, String list,Integer knowAfterSalesChannel,Integer installLength) {
        return orderService.saveOrderScore(orderId,visitContent,null,list,knowAfterSalesChannel,installLength);
    }

    /**
     * 评分小于60的订单，用于评分报表
     *
     * @return
     */
    @GetMapping(value = "orderScore/getListLessThan60")
    @ApiOperation(value = "评分小于60的订单，用于评分报表")
    @ApiImplicitParam(paramType = "query", name = "scoreSupplierId", value = "订单表ID", required = true, dataType = "int")
    public BaseResponse<Object> getListLessThan60(Long scoreSupplierId) {
        return new BaseResponse<>(orderScoreService.getListLessThan60(scoreSupplierId));
    }


}
