package com.boco.nbd.wios.manage.entity.bo;

import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.ihidea.core.support.exception.ServiceException;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @author haica
 */
public class OemContractBo extends OemContract {

    @ApiModelProperty(value = "充电桩及增项费用json字符串")
    private String itemsJson;
    
    @ApiModelProperty(hidden = true)
    private List<OemContractItem> items;
    
    @ApiModelProperty(value = "区域价格json字符串")
    private String pricesJson;
    
    @ApiModelProperty(hidden = true)
    private List<OemContractPrice> prices;
    
    @ApiModelProperty(value = "文件json字符串")
    private String filesJson;
    
    @ApiModelProperty(hidden = true)
    private List<UploadFile> files;
    
    @ApiModelProperty(value = "开始日期 yyyy-MM-dd")
    private String startDateStr;
    
    @ApiModelProperty(value = "结束日期 yyyy-MM-dd")
    private String endDateStr;
    
    @ApiModelProperty(value = "操作类型 0:变更 1：新增")
    private Integer operateType;
    
    public String getItemsJson() {
        return itemsJson;
    }
    
    public List<OemContractItem> getItems() {
        return items;
    }
    
    public String getPricesJson() {
        return pricesJson;
    }
    
    public List<OemContractPrice> getPrices() {
        return prices;
    }

    /**
     * json转换
     * @param itemsJson
     */
    public void setItemsJson(String itemsJson) {
        this.items = OemContractItem.jsonHandle(itemsJson);
    }
    
    public void setItems(List<OemContractItem> items) {
        this.items = items;
    }

    /**
     * json转换
     * @param pricesJson
     */
    public void setPricesJson(String pricesJson) {
        this.prices = OemContractPrice.jsonHandle(pricesJson);
    }
    
    public void setPrices(List<OemContractPrice> prices) {
        this.prices = prices;
    }
    
    public String getStartDateStr() {
        return startDateStr;
    }
    
    public String getEndDateStr() {
        return endDateStr;
    }
    
    public void setStartDateStr(String startDateStr) {
        if (!StringUtils.isEmpty(startDateStr)) {
            this.startDateStr = startDateStr;
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date start = sdf.parse(startDateStr);
                setStartTime(start);
            } catch (Exception e) {
                throw new ServiceException("开始时间转换失败");
            }
        }
    }
    
    public void setEndDateStr(String endDateStr) {
        if (!StringUtils.isEmpty(endDateStr)) {
            this.endDateStr = endDateStr;
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date end = sdf.parse(endDateStr);
                setEndTime(end);
            } catch (Exception e) {
                throw new ServiceException("结束时间转换失败");
            }
        }
    }
    
    public String getFilesJson() {
        return filesJson;
    }
    
    public List<UploadFile> getFiles() {
        return files;
    }

    /**
     * json
     * @param filesJson
     */
    public void setFilesJson(String filesJson) {
        this.files=UploadFile.jsonHandle(filesJson);
    }
    
    public void setFiles(List<UploadFile> files) {
        this.files = files;
    }
    
    public Integer getOperateType() {
        return operateType;
    }
    
    public void setOperateType(Integer operateType) {
        this.operateType = operateType;
    }
    
}
