package com.boco.nbd.wios.manage.entity.bo;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.json.JSONUtil;
import com.ihidea.core.support.exception.ServiceException;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author yong
 */
@Data
public class OemContractItem {
    /**
     * id
     */
    @ApiModelProperty(hidden = true)
    private Integer id;
    
    /**
     * 合同id
     */
    @ApiModelProperty(value = "合同id")
    private String contractId;
    
    /**
     * 物料/增项编号
     */
    @ApiModelProperty(value = "物料/增项编号")
    private String itemId;
    
    /**
     * 类型
     */
    @ApiModelProperty(value = "类型 1-物料 2-增项 3-充电桩 4-高附加")
    private String type;
    
    /**
     * 单价（不含税）
     */
    @ApiModelProperty(value = "单价（不含税）")
    private BigDecimal price;
    
    /**
     * 税率（%）
     */
    @ApiModelProperty(value = "税率（%）")
    private BigDecimal taxRate;
    
    /**
     * 单价（含税）
     */
    @ApiModelProperty(value = "单价（含税）")
    private BigDecimal ratePrice;
    
    /**
     * 备注说明
     */
    @ApiModelProperty(value = "备注说明")
    private String description;
    
    /**
     * 创建人
     */
    @ApiModelProperty(hidden = true)
    private String createAccount;
    
    /**
     * 创建时间
     */
    @ApiModelProperty(hidden = true)
    private Date createTime;
    
    /**
     * 修改人
     */
    @ApiModelProperty(hidden = true)
    private String modifyAccount;
    
    /**
     * 修改时间
     */
    @ApiModelProperty(hidden = true)
    private Date modifyTime;

    /**
     * json 处理
     *
     * @param json
     * @return
     */
    public static List<OemContractItem> jsonHandle(String json) {
        if (StringUtils.isEmpty(json)) {
            return new ArrayList<>(1);
        }
        try {
            return JSONUtil.toBean(JSONUtil.parse(json), new TypeReference<List<OemContractItem>>() {
            }, true);
        } catch (Exception ex) {
            throw new ServiceException("数据转换异常");
        }
    }
}