package com.boco.nbd.wios.manage.entity.bo;

import cn.hutool.json.JSONUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author yong
 */
@Data
public class OrderInstallItem {
    /**
     * id
     */
    @ApiModelProperty(hidden = true)
    private Integer id;

    /**
     * 订单编号
     */
    @ApiModelProperty(hidden = true)
    private String orderId;

    /**
     * 1：勘察 2：安装 3：安装变更
     */
    @ApiModelProperty(hidden = true)
    private Integer type;

    /**
     * 物料类型 1-物料 2-增项 3-充电桩 4-高附加值材料
     */
    @ApiModelProperty(hidden = true)
    private Integer itemType;

    /**
     * 物料编号
     */
    @ApiModelProperty(value = "物料编号")
    private String itemId;

    /**
     * 项目名称
     */
    @ApiModelProperty(value = "项目名称")
    private String itemName;

    /**
     * 型号
     */
    @ApiModelProperty(value = "型号")
    private String itemModel;

    /**
     * 仓库id
     */
    @ApiModelProperty(value = "仓库id")
    private String itemWarehouseId;

    /**
     * 对应的合同itemId
     */
    @ApiModelProperty(value = "对应的合同itemId")
    private Integer contractItemId;

    /**
     * 数量
     */
    @ApiModelProperty(value = "数量")
    private BigDecimal qunatity;

    /**
     * 总费用
     */
    @ApiModelProperty(value = "总费用")
    private BigDecimal fee;

    /**
     * 备注说明
     */
    @ApiModelProperty(value = "备注说明")
    private String description;

    /**
     * 创建人
     */
    @ApiModelProperty(hidden = true)
    private String createAccount;

    /**
     * 创建时间
     */
    @ApiModelProperty(hidden = true)
    private Date createTime;

    /**
     * 修改人
     */
    @ApiModelProperty(hidden = true)
    private String modifyAccount;

    /**
     * 修改时间
     */
    @ApiModelProperty(hidden = true)
    private Date modifyTime;

    /**
     * 数据校验
     *
     * @param items
     */
    private static void dataCheck(List<OrderInstallItem> items) {
        OrderInstallItem badItem = Linq.of(items).where(s -> StringUtils.isEmpty(s.getItemId())).firstOrDefault();
        if (badItem != null) {
            throw new ServiceException("itemId不能为空");
        }
        badItem = Linq.of(items).where(s -> StringUtils.isEmpty(s.getItemName())).firstOrDefault();
        if (badItem != null) {
            throw new ServiceException("itemName不能为空");
        }
        badItem = Linq.of(items).where(s -> StringUtils.isEmpty(s.getContractItemId())).firstOrDefault();
        if (badItem != null) {
            throw new ServiceException("contractItemId不能为空");
        }

    }

    /**
     * json处理
     * @param json
     * @return
     */
    public static List<OrderInstallItem> jsonHandle(String json) {
        if (StringUtils.isEmpty(json)) {
            return new ArrayList<>(1);
        }
        try {
            List<OrderInstallItem> itemList = JSONUtil.toBean(JSONUtil.parse(json), new cn.hutool.core.lang.TypeReference<List<OrderInstallItem>>() {
            }, true);
            itemList.stream().forEach(item -> {
                // detail接口 id属性就是合同id
                if (StringUtils.isEmpty(item.getContractItemId())) {
                    item.setContractItemId(item.getId());
                }
                item.setId(null);
            });
            dataCheck(itemList);
            return itemList;
        } catch (Exception ex) {
            throw new ServiceException("数据转换异常:"+ex.getMessage());
        }
    }

    /**
     * 一期json处理方法重构
     *
     * @param json
     * @return
     */
    public static List<OrderInstallItem> wiosJsonHandle(String json) {
        if (StringUtils.isEmpty(json)) {
            return new ArrayList<>(1);
        }
        json = StringUtilsEx.unescapeXss(json);
        try {
            //原先代码，屏蔽处理该字段(后续转义报时间类型错误)
            json = json.replaceAll(LambdaUtils.getFieldName(OrderInstallItem::getCreateTime), LambdaUtils.getFieldName(OrderInstallItem::getCreateTime) + "1");
            ObjectMapper mapper = new ObjectMapper();
            mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
            List<OrderInstallItem> itemList = mapper.readValue(json, new TypeReference<List<OrderInstallItem>>() {
            });
            itemList.stream().forEach(item -> {
                // detail接口 id属性就是合同id
                if (StringUtils.isEmpty(item.getContractItemId())) {
                    item.setContractItemId(item.getId());
                }
                item.setId(null);
            });
            dataCheck(itemList);
            return itemList;
        } catch (JsonProcessingException e) {
            throw new ServiceException("数据转换异常");
        }
    }
}