package com.boco.nbd.wios.manage.entity.cams.enums;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.boco.nbd.framework.workflow.entity.WorkTaskInfo;
import com.boco.nbd.wios.flow.enums.FlowNodeEnum;
import lombok.extern.slf4j.Slf4j;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author haica
 */
@Slf4j
public enum OverTimeType implements IOperator {

    /**
     * OverTimeType
     * modify 2022-11-25 兼容日视化：正常工单、即将超时、超时工单逻辑
     */
    UN_TIME_OUT(1, "未超时"),
    WILL_TIME_OUT(2, "即将超时"),
    TIME_OUT(3, "超时订单"),
    FIRST_CONTACT_OVER(101, "首联超时"),
    ACCEPT_ORDER_OVER(103, "接单超时"),
    SURVEY_RESERVE_OVER(104, "勘测预约超时"),
    UPLOAD_SURVEY_DATA_OVER(106, "上传勘测资料超时"),
    APPROVE_SURVEY_DATA_OVER(107, "勘测资料审核超时"),
    RESERVE_INSTALL_OVER(108, "预约安装超时"),
    UPLOAD_INSTALL_DATA_OVER(111, "上传安装资料超时"),
    APPROVE_INSTALL_DATA_OVER(112, "安装资料审核超时"),
    ELECTRIC_EXPAND_DATA_OVER(115, "电力报装超时"),
    SURVEY(201, "勘测时效超时"),
    SURVEY_APPROVE(202, "勘测审核时效超时"),
    INSTALL_RESERVE(203, "安装预约时效（不报装）超时"),
    INSTALL(204, "安装时效（不报装）超时"),
    INSTALL_APPROVE(205, "安装审核时效超时"),
    INSTALL_RESERVE_EXPAND(207, "安装预约时效（报装）超时"),
    INSTALL_EXPAND(208, "安装时效（报装）超时"),
    NEXT_SURVEY(213, "下次预约勘测跟进超时"),
    NEXT_INSTALL(214, "下次预约安装跟进超时");

    private int type;

    private String value;

    OverTimeType(int type, String value) {
        this.type = type;
        this.value = value;
    }

    public static String getText(int type) {
        OverTimeType[] values = OverTimeType.values();
        for (OverTimeType os : values) {
            if (os.getType() == type) {
                return os.getValue();
            }
        }
        return null;
    }

    /**
     * 生成map对象
     *
     * @return
     */
    public static Map<Object, String> createMap() {
        Map<Object, String> map = new HashMap<>(10);
        Arrays.stream(OverTimeType.values()).forEach(s -> {
            map.put(s.getType(), s.getValue());
        });
        return map;
    }

    /**
     * 获取超时类型
     *
     * @param info
     * @return
     */
    public static OverTimeType getOverTimeType(WorkTaskInfo info) {
        if (info == null || info.getDueDate() == null || info.getCurrentStartTime() == null) {
            return UN_TIME_OUT;
        }
        if (DateUtil.compare(info.getDueDate(), new Date()) > 0) {
            return UN_TIME_OUT;
        }
        //modify 2022-11-28 截止日期在任务创建时间之前
        long space = info.getDueDate().getTime() - info.getCurrentStartTime().getTime();
        long timeSpace = System.currentTimeMillis() - info.getCurrentStartTime().getTime();
        double ratio = 0.8;
        if (timeSpace > space * ratio && timeSpace < space) {
            return WILL_TIME_OUT;
        }

        FlowNodeEnum nodeEnum = FlowNodeEnum.getEnum(Convert.toInt(info.getNodeFlag()));
        if (nodeEnum == null) {
            log.error(info.getNodeFlag() + "get FlowNodeEnum error.");
            return UN_TIME_OUT;
        }
        switch (nodeEnum) {
            case NODE_101:
                return FIRST_CONTACT_OVER;
            case NODE_103:
                return ACCEPT_ORDER_OVER;
            case NODE_104:
                return SURVEY_RESERVE_OVER;
            case NODE_106:
                return UPLOAD_SURVEY_DATA_OVER;
            case NODE_107:
                return APPROVE_SURVEY_DATA_OVER;
            case NODE_108:
            case NODE_109:
                return RESERVE_INSTALL_OVER;
            case NODE_111:
                return UPLOAD_INSTALL_DATA_OVER;
            case NODE_112:
                return APPROVE_INSTALL_DATA_OVER;
            case NODE_115:
                return ELECTRIC_EXPAND_DATA_OVER;
            default:
                return UN_TIME_OUT;
        }
    }

    @Override
    public int getType() {
        return type;
    }

    @Override
    public String getValue() {
        return value;
    }
}
