package com.boco.nbd.wios.manage.entity.common.bo;

import cn.hutool.json.JSONUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.wios.manage.entity.bo.OrderInstallItem;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author yong
 * t_upload_file 附件
 */
@Data
public class UploadFile {

    @ApiModelProperty(hidden = true)
    private Integer id;

    @ApiModelProperty(hidden = true, value = "类型 1：勘察订单 2：安装订单 3：主机厂合同 4：服务商合同")
    private Integer type;

    @ApiModelProperty(value = "使用类型 随type字段变化 比如1：图片 2：附件")
    private Integer useType;

    @ApiModelProperty(hidden = true)
    private String thirdPartyId;

    @ApiModelProperty(value = "文件名字")
    private String name;

    @ApiModelProperty(value = "文件地址")
    private String url;

    @ApiModelProperty(hidden = true, value = "状态 1：正常 2：删除")
    private Integer status;

    @ApiModelProperty(value = "备注")
    private String remark;

    @ApiModelProperty(hidden = true, value = "更新人姓名")
    private String updateBy;

    @ApiModelProperty(hidden = true, value = "创建人姓名")
    private String createBy;

    @ApiModelProperty(hidden = true, value = "更新时间")
    private Date updateTime;

    @ApiModelProperty(hidden = true, value = "创建时间")
    private Date createTime;

    /**
     * json 处理
     *
     * @param json
     * @param checkUseType
     * @return
     */
    public static List<UploadFile> jsonHandle(String json, boolean checkUseType) {
        if (StringUtils.isEmpty(json)) {
            return new ArrayList<>(1);
        }
        try {
            List<UploadFile> itemList = JSONUtil.toBean(JSONUtil.parse(json), new cn.hutool.core.lang.TypeReference<List<UploadFile>>() {
            }, true);
            if (checkUseType) {
                UploadFile badItem = Linq.of(itemList).where(s -> s.getUseType() == null).firstOrDefault();
                if (badItem != null) {
                    throw new ServiceException("文件useType不能为空");
                }
            }
            return itemList;
        } catch (Exception ex) {
            throw new ServiceException("数据转换异常");
        }
    }

    /**
     * json 处理
     *
     * @param json
     * @return
     */
    public static List<UploadFile> jsonHandle(String json) {
       return jsonHandle(json, true);
    }

    /**
     * 一期处理==存在问题
     *
     * @param json
     * @return
     */
    public static List<UploadFile> wiosJsonHandle(String json) {
        if (StringUtils.isEmpty(json)) {
            return new ArrayList<>(1);
        }
        json = StringUtilsEx.unescapeXss(json);
        try {
            //原先代码，屏蔽处理该字段(后续转义报时间类型错误)
            json = json.replaceAll(LambdaUtils.getFieldName(OrderInstallItem::getCreateTime), LambdaUtils.getFieldName(OrderInstallItem::getCreateTime) + "1");
            ObjectMapper mapper = new ObjectMapper();
            mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
            List<UploadFile> itemList = mapper.readValue(json, new TypeReference<List<UploadFile>>() {
            });
            UploadFile badItem = Linq.of(itemList).where(s -> s.getUseType() == null).firstOrDefault();
            if (badItem != null) {
                throw new ServiceException("文件useType不能为空");
            }
            return itemList;
        } catch (JsonProcessingException e) {
            throw new ServiceException("数据转换异常");
        }
    }
}
