package com.boco.nbd.wios.manage.entity.settlement.po;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.json.JSONUtil;
import com.boco.nbd.wios.manage.enums.InvoiceSubmitType;
import com.ihidea.core.support.exception.ServiceException;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author yong
 * t_invoice 发票表
 */
@Data
@Slf4j
public class Invoice {

    @ApiModelProperty(value = "车企安装单号")
    private String orderId;

    @ApiModelProperty(value = "增项费用开票状态 0:未开票 1:已开票")
    private Integer additionalCostInvoiceStatus;

    @ApiModelProperty(value = "增项开票日期")
    private Date additionalCostInvoiceDate;

    @ApiModelProperty(value = "增项费用发票号")
    private String additionalCostInvoiceNumber;

    @ApiModelProperty(value = "套包内（设备）开票状态0:未开票 1:已开票")
    private Integer packageDeviceInvoiceStatus;

    @ApiModelProperty(value = "套包内（设备）开票日期")
    private Date packageDeviceInvoiceDate;

    @ApiModelProperty(value = "套包内（设备）收款金额")
    private BigDecimal packageDeviceFeeWithTax;

    @ApiModelProperty(value = "套包内（设备）税额")
    private BigDecimal packageDeviceTax;

    @ApiModelProperty(value = "套包内（设备）发票号")
    private String packageDeviceInvoiceNumber;

    @ApiModelProperty(value = "套包内（设备）收款状态0:未收款 1:已收款")
    private Integer packageDeviceReceiveStatus;

    @ApiModelProperty(value = "套包内（安装）开票状态0:未开票 1:已开票")
    private Integer packageInstallInvoiceStatus;

    @ApiModelProperty(value = "套包内（安装）开票日期")
    private Date packageInstallInvoiceDate;

    @ApiModelProperty(value = "套包内（安装）收款金额")
    private BigDecimal packageInstallFeeWithTax;

    @ApiModelProperty(value = "套包内（安装）税额")
    private BigDecimal packageInstallTax;

    @ApiModelProperty(value = "套包内（安装）发票号")
    private String packageInstallInvoiceNumber;

    @ApiModelProperty(value = "套包内（安装）收款状态0:未收款 1:已收款")
    private Integer packageInstallReceiveStatus;

    @ApiModelProperty(value = "服务商收票状态：0：未收票，1：已收票")
    private Integer supplierReceiveInvoiceStatus;

    @ApiModelProperty(value = "服务商收票日期")
    private Date supplierInvoiceDate;

    @ApiModelProperty(value = "服务商付款金额")
    private BigDecimal supplierFeeWithTax;

    @ApiModelProperty(value = "服务商税额")
    private BigDecimal supplierTax;

    @ApiModelProperty(value = "服务商发票号")
    private String supplierInvoiceNumber;

    @ApiModelProperty(value = "服务商付款状态：0：未付款，1：已付款")
    private Integer supplierPayStatus;

    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    @ApiModelProperty(value = "修改时间")
    private Date modifyTime;

    @ApiModelProperty(value = "0:正常 1:删除")
    private Integer isDel;

    @ApiModelProperty(value = "提交项（1-服务开票，2-设备开票，两个同时-3）")
    private Integer submitItem;


    /**
     * 2022-11-11新增字段 满足增项费用结算
     */
    @ApiModelProperty(value = "收票状态：0：未收票，1：已收票")
    private Integer receiveInvoiceStatus;

    @ApiModelProperty(value = "收票日期")
    private Date invoiceDate;

    @ApiModelProperty(value = "付款金额")
    private BigDecimal feeWithTax;

    @ApiModelProperty(value = "税额")
    private BigDecimal tax;

    @ApiModelProperty(value = "发票号")
    private String invoiceNumber;

    @ApiModelProperty(value = "付款状态：0：未付款，1：已付款")
    private Integer payStatus;

    @ApiModelProperty(value = "票据类型：1：增项，2 其它后续扩展")
    private Integer invoiceType;

    /**
     * 对象转换
     *
     * @param json
     * @return
     */
    public static List<Invoice> jsonHandle(String json) {
        if (StringUtils.isEmpty(json)) {
            return new ArrayList<>(1);
        }
        try {
            List<Invoice> itemList = JSONUtil.toBean(JSONUtil.parse(json), new TypeReference<List<Invoice>>() {
            }, true);
            return itemList;
        } catch (Exception ex) {
            log.error(json + "数据转换异常:", ex);
            throw new ServiceException("数据转换异常");
        }
    }

    /**
     * 是否服务开票
     *
     * @return
     */
    public boolean areInstallInvoice() {
        if (this.getSubmitItem() == null || this.getPackageDeviceInvoiceStatus() == null) {
            return false;
        }
        if (InvoiceSubmitType.DEVICE.getType().equals(this.getSubmitItem())) {
            return false;
        }
        return (this.getPackageInstallInvoiceStatus() == 1);
    }

    /**
     * 是否设备开票
     *
     * @return
     */
    public boolean areDeviceInvoice() {
        if (this.getSubmitItem() == null || this.getPackageDeviceInvoiceStatus() == null) {
            return false;
        }
        if (InvoiceSubmitType.SUPPLIER.getType().equals(this.getSubmitItem())) {
            return false;
        }
        return (this.getPackageDeviceInvoiceStatus() == 1);
    }
}