package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.manage.entity.bo.Department;
import com.boco.nbd.wios.manage.entity.bo.DepartmentVo;
import com.ihidea.core.support.exception.ServiceException;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 部门dao
 *
 * @author xgl
 * @version [版本号, 2020年9月24日]
 */
@Repository
public class DepartmentDao {

    @Resource
    private DepartmentMapper departmentMapper;

    public Department selectById(int id) {
        return departmentMapper.selectById(id);
    }

    public List<Department> selectByParentId(int parentId) {
        return departmentMapper.selectByParentId(parentId);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int insert(Department department) {
        List<Department> deptList = selectByParentId(department.getParentId());
        deptList = deptList.stream().filter(r -> r.getName().equals(department.getName())).collect(Collectors.toList());
        // 筛选同级别的部门
        if (!deptList.isEmpty()) {
            throw new ServiceException("同一级别下的部门名称不可相同");
        }

        department.setCreateTime(new Date());
        department.setStatus(1);
        departmentMapper.insert(department);

        Department pDepartment = departmentMapper.selectById(department.getParentId());
        String idTree = pDepartment.getIdTree();
        if (StringUtils.isEmpty(idTree)) {
            department.setIdTree(pDepartment.getId() + CamsConstant.UNDER_LINE + department.getId());
        } else {
            department.setIdTree(idTree + CamsConstant.UNDER_LINE + department.getId());
        }

        Department updateDepartment = new Department();
        updateDepartment.setId(department.getId());
        updateDepartment.setIdTree(department.getIdTree());
        return departmentMapper.updateDynamic(updateDepartment);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int update(int id, Department department) {
        department.setId(id);
        department.setUpdateTime(new Date());
        Department dept = departmentMapper.selectById(id);
        if (department.getParentId() != null) {
            Department parentDept = departmentMapper.selectById(department.getParentId());
            String parentIdTree = parentDept.getIdTree();
            if (StringUtils.isEmpty(parentIdTree)) {
                parentIdTree = parentDept.getId().toString();
            }
            String idTree = (parentIdTree + CamsConstant.UNDER_LINE + id);

            List<Department> deptList = selectByParentId(dept.getParentId());
            deptList = deptList.stream().filter(r -> r.getName().equals(department.getName())).collect(Collectors.toList());
            // 筛选同级别的部门
            if (!deptList.isEmpty() && deptList.get(0).getId().intValue() != id) {
                throw new ServiceException("同一级别下的部门名称不可相同");
            }

            department.setIdTree(idTree);
            departmentMapper.updateIdTree(idTree, dept.getIdTree());
        }
        departmentMapper.updateDynamic(department);
        return 1;
    }

    public List<DepartmentVo> getList(Integer status) {
        return departmentMapper.selectAll(status);
    }
}
