package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.wios.manage.entity.bo.Menu;
import com.ihidea.core.base.CoreDao;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * 菜单扩展mapper
 *
 * @author lilin
 * @version [版本号, 2020年5月18日]
 */
public interface MenuMapperEx extends CoreDao {
    /**
     * 根据用户权限查询菜单列表
     *
     * @param accountId 账户id
     * @return 菜单列表
     */
    @Select("select distinct m.id,m.parent_id as parentId,m.icon,m.e_name as eName,m.c_name as cName,m.path,m.type as type from wb_menu m "
            + "inner join wb_role_menu rm on m.id=rm.menu_id "
            + "inner join wb_role r on rm.role_id=r.id "
            + "inner join wb_account_role ar on ar.role_id=r.id "
            + "where ar.account_id=#{accountId} and m.`status`=1 and r.`status`=1 and m.type=0 and m.app_type=0 order by m.sort_order,m.id")
    List<Menu> findAccountMenuList(@Param("accountId") Integer accountId);

    /**
     * 根据角色权限查询菜单列表
     *
     * @param roleId 角色id
     * @return 菜单列表
     */
    @Select("select m.id,m.parent_id as parentId,m.icon,m.e_name as eName,m.c_name as cName,m.path,rm.role_id as status from wb_menu m "
            + "inner join wb_role_menu rm on m.id=rm.menu_id and rm.role_id=#{roleId} " + "where m.`status`=1 order by m.sort_order,m.id")
    List<Menu> findRoleMenuList(@Param("roleId") Integer roleId);

    /**
     * 查询按钮列表
     *
     * @param roleId
     * @param menuId
     * @return
     */
    @Select("SELECT m.* FROM wb_role_menu rm "
            + "LEFT JOIN wb_menu m ON rm.menu_id = m.id "
            + "LEFT JOIN wb_role r ON rm.role_id = r.id "
            + "WHERE role_id = #{roleId} AND m.parent_id = #{menuId} AND m.`status` = 1 and r.status=1")
    List<Menu> findBtnList(@Param("roleId") Integer roleId, @Param("menuId") String menuId);

    /**
     * findAll
     *
     * @return
     */
    @Select("select * from wb_menu")
    List<Menu> findAll();
}
