package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.OemContract;
import com.boco.nbd.wios.manage.entity.bo.OemContractBo2;
import com.boco.nbd.wios.manage.entity.bo.OemContractVo;
import com.boco.nbd.wios.manage.entity.cams.enums.ContractStatus;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 主机厂合约dao
 *
 * @author xgl
 * @version [版本号, 2020年9月16日]
 */
@Repository
public class OemContractDao {

    @Resource
    private OemContractMapper oemContractMapper;

    /**
     * 查询
     *
     * @param id
     * @return
     */
    public OemContract selectById(int id) {
        return oemContractMapper.selectById(id);
    }

    /**
     * 查询
     *
     * @param code
     * @return
     */
    public List<OemContract> selectByCode(String code) {
        return oemContractMapper.selectByCode(code);
    }

    /**
     * 增
     *
     * @param contract
     * @return
     */
    public int insert(OemContract contract) {
        contract.setCreateTime(new Date());
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null) {
            contract.setCreateAccount(sessionInfo.getUserName());
        }
        if (contract.getStartTime() != null && contract.getEndTime() != null) {
            dateCheck(contract);
        }
        return oemContractMapper.insert(contract);
    }

    /**
     * 时间验证
     *
     * @param contract
     */
    private void dateCheck(OemContract contract) {
        Date date1;
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            date1 = sdf.parse(sdf.format(new Date()));
        } catch (ParseException e) {
            throw new ServiceException("数据转换失败");
        }
        if (contract.getStartTime().compareTo(date1) > 0) {
            contract.setStatus(ContractStatus.NOT_START.getType());
        } else {
            if (contract.getEndTime().compareTo(date1) >= 0) {
                contract.setStatus(ContractStatus.EFFECTIVE.getType());
            } else {
                throw new ServiceException("结束日期必须大于今天");
            }
        }
    }


    /**
     * 根据主机厂Id和合并编号查询合同
     *
     * @param oemId
     * @param status
     * @return
     */
    public List<OemContractVo> selectByStatusAndOemId(int oemId, int status) {
        OemContractBo2 condition = new OemContractBo2();
        condition.setOemId(oemId + "");
        condition.setStatus(status);
        return oemContractMapper.selectByExample(condition);
    }

    /**
     * 根据主机厂Id和合并编号查询合同
     *
     * @param oemId
     * @param status
     * @return
     */
    public List<OemContract> selectSimpleByStatusAndOemId(int oemId, int status) {
        return oemContractMapper.selectByStatusAndOemId(status, oemId);
    }

    /**
     * 有效
     *
     * @param oemId
     * @return
     */
    public List<OemContractVo> selectValid(Integer oemId) {
        OemContractBo2 condition = new OemContractBo2();
        if (oemId != null) {
            condition.setOemId(oemId.intValue() + "");
        }
        List<Integer> statusInList = new ArrayList<>();
        statusInList.add(ContractStatus.NOT_START.getType());
        statusInList.add(ContractStatus.EFFECTIVE.getType());
        condition.setStatusInList(statusInList);
        condition.setOrderBy(2);
        return oemContractMapper.selectByExample(condition);
    }

    public List<OemContractVo> getList(OemContractBo2 condition) {
        return oemContractMapper.selectByExample(condition);
    }

    /**
     * 改
     *
     * @param id
     * @param contract
     * @return
     */
    public int update(int id, OemContract contract) {
        contract.setId(id);
        contract.setModifyTime(new Date());
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null) {
            contract.setModifyAccount(sessionInfo.getUserName());
        }

        if (contract.getStartTime() != null && contract.getEndTime() != null) {
            dateCheck(contract);
        }
        return oemContractMapper.updateDynamic(contract);
    }

    /**
     * 查询
     *
     * @param contractId
     * @return
     */
    public OemContractVo selectDetailById(int contractId) {
        return oemContractMapper.selectDetailById(contractId);
    }

}
