package com.boco.nbd.wios.manage.mapper.def;


import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.OperateLog;
import com.boco.nbd.wios.manage.entity.bo.OperateLogBo;
import com.boco.nbd.wios.manage.entity.cams.enums.OperateLogType;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.ihidea.core.support.session.SessionInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 操作日志dao
 * 
 * @author
 * @version
 */
@Repository
public class OperateLogDao {
    private static final Logger logger= LoggerFactory.getLogger(OperateLogDao.class);

    @Resource
    private OperateLogMapper operateLogMapper;

    /**
     * insert
     * @param operateLog
     * @return
     */
    public int insert(OperateLog operateLog) {
        operateLog.setCreateTime(new Date());
        SessionInfo sessionInfo= ProcessUtil.getUserInfo();
        if (sessionInfo != null) {
            try{
                operateLog.setCreateAccount(Integer.valueOf(sessionInfo.getUserId()));
            }catch (Exception e){
                logger.error("获取当前用户编号出现异常:"+e.getMessage());
            }
            operateLog.setCreateAccountName(sessionInfo.getUserName());
        }
        return operateLogMapper.insert(operateLog);
    }

    /**
     * insertOrderLog
     * @param orderId
     * @param name
     * @param value
     * @param desc
     * @return
     */
    public int insertOrderLog(String orderId, Integer name, String value, String desc) {
        OperateLog operateLog = new OperateLog();
        operateLog.setThirdPartyId(orderId);
        operateLog.setName(name == null ? "" : name.toString());
        operateLog.setValue(value);
        operateLog.setDesc(desc);
        operateLog.setType(OperateLogType.ORDER_STATUS.getType());
        return insert(operateLog);
    }

    /**
     * insertOrderLog
     * @param orderId
     * @param name
     * @param desc
     * @return
     */
    public int insertOrderLog(String orderId, Integer name, String desc) {
        return insertOrderLog(orderId, name, OrderStatus.getText(name), desc);
    }

    /**
     * selectByExample
     * @param condition
     * @return
     */
    public List<OperateLog> selectByExample(OperateLogBo condition) {
        if (!StringUtils.isEmpty(condition.getTypes())) {
            String[] arr = condition.getTypes().split(CamsConstant.COMMA);
            condition.setTypeList(Arrays.asList(arr));
        }
        return operateLogMapper.selectByExample(condition);
    }

    /**
     * selectById
     * @param id
     * @return
     */
    public OperateLog selectById(Integer id) {
        return operateLogMapper.selectById(id);
    }
    
    /**
     * 查询最近一天记录
     *
     * @param thId
     * @param type
     * @return
     */
    public OperateLog selectLastestByThirdPartyId(String thId, Integer type) {
        return selectLastestByThirdPartyId(thId, type, "");
    }

    /**
     * 查询最近一天记录
     * @param thId
     * @param type
     * @param name
     * @return
     */
    public OperateLog selectLastestByThirdPartyId(String thId, Integer type, String name) {
        OperateLogBo condition = new OperateLogBo();
        condition.setThirdPartyId(thId);
        condition.setType(type);
        condition.setName(name);
        List<OperateLog> list = operateLogMapper.selectByExample(condition);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * getNewestCheckFailByOrderId
     * @param orderId
     * @param name
     * @return
     */
    public OperateLog getNewestCheckFailByOrderId(String orderId,String name) {
        return operateLogMapper.getNewestCheckFailByOrderId(orderId,name);
    }

    /**
     * deleteByOrderId
     * @param orderId
     */
    public void deleteByOrderId(String orderId) {
        operateLogMapper.deleteByOrderId(orderId);
    }
}
