package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.wios.manage.entity.bo.OemCascadeBo;
import com.boco.nbd.wios.manage.entity.bo.OrderInstallItem;
import com.boco.nbd.wios.manage.entity.bo.OrderInstallItemVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 安装订单增项dao
 *
 * @author xgl
 * @version [版本号, 2020年11月2日]
 */
@Repository
@Slf4j
public class OrderInstallItemDao {

    @Resource
    private OrderInstallItemMapper orderInstallItemMapper;

    /**
     * 查询
     *
     * @param id
     * @return
     */
    public OrderInstallItem selectById(int id) {
        return orderInstallItemMapper.selectById(id);
    }

    /**
     * 插入
     *
     * @param item
     * @return
     */
    public int insert(OrderInstallItem item) {
        item.setCreateTime(new Date());
        return orderInstallItemMapper.insert(item);
    }

    /**
     * 批量插入
     *
     * @param list
     * @return
     */
    public int batchInsert(List<OrderInstallItem> list) {
        list.stream().forEach(item -> item.setCreateTime(new Date()));
        return orderInstallItemMapper.batchInsert(list);
    }

    /**
     * 删除
     *
     * @param orderId
     * @param type
     * @return
     */
    public int delete(String orderId, Integer type) {
        return orderInstallItemMapper.deleteByOrderId(orderId, type);
    }

    /**
     * 查询订单安装明细
     *
     * @param orderId
     * @param type
     * @return
     */
    public List<OrderInstallItemVo> selectByOrderIdAndType(String orderId, Integer type) {
        return orderInstallItemMapper.selectByOrderIdAndType(orderId, type);
    }

    /**
     * 删除
     *
     * @param orderId
     * @return
     */
    public List<OrderInstallItem> selectByOrderId(String orderId) {
        return orderInstallItemMapper.selectByOrderId(orderId);
    }

    /**
     * 删除
     *
     * @param id
     */
    public void deleteById(Integer id) {
        orderInstallItemMapper.delete(id);
    }



    /**
     * 查询套包
     * @param id
     * @return
     */
    public List<OemCascadeBo> selectPack(String id) {
        return orderInstallItemMapper.selectPackById(id);
    }
}
