package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.Order;
import com.boco.nbd.wios.manage.entity.bo.OrderScore;
import com.boco.nbd.wios.manage.util.NumberUtil;
import com.ihidea.core.support.session.SessionInfo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;


/**
 * @author yong
 */
@Repository
public class OrderScoreDao {
    @Resource
    private OrderScoreMapper orderScoreMapper;

    @Resource
    private OrderMapper orderMapper;
    
    public int insert(OrderScore orderScore) {
        orderScore.setCreateAccount(this.getUserId());
        orderScore.setCreateTime(new Date());
        orderScore.setModifyAccount(this.getUserId());
        orderScore.setModifyTime(new Date());
        orderScore.setStatus(1);
        return orderScoreMapper.insert(orderScore);
    }

    public int insertOrderScore(String orderId){
        return orderScoreMapper.insertOrderScore(orderId);
    }

    public int update( OrderScore orderScore) {
        orderScore.setModifyAccount(this.getUserId());
        orderScore.setModifyTime(new Date());
        int ret = orderScoreMapper.update(orderScore);

        if(orderScore.getScore() !=null){
            //更新该订单的客户评分合计
            OrderScore tmpOrderScore = orderScoreMapper.selectById(orderScore.getId());
            if (tmpOrderScore != null) {
                Double sumScore = orderScoreMapper.getScoreByOrderId(tmpOrderScore.getOrderId());
                Order order = new Order();
                order.setId(String.valueOf(tmpOrderScore.getOrderId()));
                order.setScore(NumberUtil.roundDouble(sumScore));
                orderMapper.updateDynamic(order);
            }
        }
        return ret;
    }

    /**
     * 通过编码和主表id更新明细表记录
     * @param orderScore
     * @return
     */
    public int updateByCode( OrderScore orderScore) {
        orderScore.setModifyAccount(this.getUserId());
        orderScore.setModifyTime(new Date());
        return orderScoreMapper.updateByCode(orderScore);
    }

    public OrderScore selectById(Integer id) {
        return orderScoreMapper.selectById(id);
    }

    public Double getScoreByOrderId(String orderId){
        return orderScoreMapper.getScoreByOrderId(orderId);
    }
    
    public List<OrderScore> selectByTypeAndOrderIds(int type, List<String> orderIds) {
        return orderScoreMapper.selectByTypeAndOrderIds(type, orderIds);
    }

    public List<OrderScore> getListByOrderId(String orderId){
        return orderScoreMapper.getListByOrderId(orderId);
    }

    public List<Order> getOrderListLessThan60(Long supplierId, Long regionId, String month){
        return orderScoreMapper.getOrderListLessThan60(supplierId, regionId, month);
    }

    public String getUserId(){
        SessionInfo user = ProcessUtil.getUserInfo();
        if(user != null){
            return user.getUserId();
        }else{
            return "1";
        }
    }

}
