package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.manage.entity.bo.Region;
import com.ihidea.core.support.exception.ServiceException;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 区域dao
 *
 * @author xgl
 * @version [版本号, 2020年9月21日]
 */
@Repository
public class RegionDao {

    @Resource
    private RegionMapper regionMapper;

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int insert(Region region) {
        List<Region> regionList = selectByParentId(region.getParentId());
        regionList = regionList.stream().filter(r -> r.getName().equals(region.getName())).collect(Collectors.toList());
        if (!regionList.isEmpty()) {
            throw new ServiceException("同一级别下的区域名称不可相同");
        }

        region.setCreateTime(new Date());
        region.setStatus(1);
        regionMapper.insert(region);

        Region pRegion = selectById(region.getParentId());
        String idTree = pRegion.getIdTree();
        if (StringUtils.isEmpty(idTree)) {
            region.setIdTree(pRegion.getId() + CamsConstant.UNDER_LINE + region.getId());
        } else {
            region.setIdTree(idTree + CamsConstant.UNDER_LINE + region.getId());
        }

        Region updateRegion = new Region();
        updateRegion.setId(region.getId());
        updateRegion.setIdTree(region.getIdTree());
        return regionMapper.updateDynamic(updateRegion);
    }

    public Region selectById(int id) {
        return regionMapper.selectById(id);
    }

    public List<Region> selectByParentId(int parentId) {
        return regionMapper.selectByParentId(parentId);
    }

    public List<Region> selectAll(Integer status) {
        return regionMapper.selectAll(status, null);
    }

    public List<Region> selectAll(Integer status, List<Integer> ids) {
        return regionMapper.selectAll(status, ids);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int update(int id, Region region) {
        region.setId(id);
        region.setUpdateTime(new Date());
        Region reg = regionMapper.selectById(id);
        if (region.getParentId() != null) {
            Region parentRegion = regionMapper.selectById(region.getParentId().intValue());
            String parentIdTree = parentRegion.getIdTree();
            if (StringUtils.isEmpty(parentIdTree)) {
                parentIdTree = parentRegion.getId().toString();
            }
            String idTree = (parentIdTree + CamsConstant.UNDER_LINE + id);

            List<Region> regionList = selectByParentId(reg.getParentId());
            regionList = regionList.stream()
                .filter(r -> r.getName().equals(region.getName())).collect(Collectors.toList());
            if (!regionList.isEmpty() && regionList.get(0).getId().intValue() != id) {
                throw new ServiceException("同一级别下的区域名称不可相同");
            }

            region.setIdTree(idTree);
            regionMapper.updateIdTree(idTree, reg.getIdTree());
        }
        regionMapper.updateDynamic(region);
        return regionMapper.updateDynamic(region);
    }

    public List<Region> selectByName(String regionName) {
        return regionMapper.selectByName(regionName);
    }

    public Region selectSingleByName(String regionName) {
        List<Region> regionList = selectByName(regionName);
        return regionList.isEmpty() ? null : regionList.get(0);
    }

    public List<Region> selectSelfAndChildren(Integer id) {
        Region region = regionMapper.selectById(id);
        return regionMapper.selectSelfAndChildren(region.getIdTree());
    }

    public List<Region> selectByCode(String code) {
        return regionMapper.selectByCode(code);
    }
}
