package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.wios.manage.entity.bo.ScoreSupplier;
import com.boco.nbd.wios.manage.entity.bo.ScoreSupplierDTO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author haica
 */
public interface ScoreSupplierMapper {

    /**
     * delete
     * @param id
     * @return
     */
    int delete(Long id);

    /**
     * insert
     * @param scoreSupplier
     * @return
     */
    int insert(ScoreSupplier scoreSupplier);

    /**
     * update
     * @param scoreSupplier
     * @return
     */
    int update(ScoreSupplier scoreSupplier);

    /**
     * selectById
     * @param id
     * @return
     */
    ScoreSupplierDTO selectById(Long id);

    /**
     * selectLastBySupplierId
     * @param supplierId
     * @return
     */
    ScoreSupplierDTO selectLastBySupplierId(Long supplierId);

    /**
     * selectByExample
     * @param scoreSupplierDTO
     * @return
     */
    List<ScoreSupplierDTO> selectByExample(ScoreSupplierDTO scoreSupplierDTO);

    /**
     * insertMonthDetail
     * @param scoreSupplierId
     * @return
     */
    int insertMonthDetail(Long scoreSupplierId);

    /**
     * 统计某个状态的订单数
     *
     * @param supplierId
     * @param reginId
     * @param orderStatus
     * @param month
     * @return
     */
    int countOrder(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("orderStatus") String orderStatus,
                   @Param("month") String month);

    /**
     * 统计更换服务商次数
     * @param supplierId
     * @param threeMonthBefore
     * @param month
     * @return
     */
    int countChangeSupplier(@Param("supplierId") Long supplierId, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * 计算工单完成的平均时间（小时）
     *
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    Integer calcOrderFinishedTime(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("month") String month);

    /**
     * 计算接单响应的平均时间（小时）
     *
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    Integer calcOrderResponseTime(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("month") String month);

    /**
     * 计算勘查完成的平均时间（小时），T3上门服务
     *
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    int calcOrderSurveyTime(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("month") String month);

    /**
     * selectLastestScoreInfo
     * @param supplierId
     * @param regionId
     * @return
     */
    ScoreSupplier selectLastestScoreInfo(@Param("supplierId") int supplierId, @Param("regionId") int regionId);

    /**
     * 计算客户评价的平均分
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    Double calcOrderScore(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderByCondition
     * @param supplierId
     * @param regionIdTree
     * @param status
     * @param threeMonthBefore
     * @param month
     * @return
     */
    int countOrderByCondition(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("status") Integer status, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderHasFinished
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    int countOrderHasFinished(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderFinishedTime
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    Integer countOrderFinishedTime(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderResponseTime
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    Integer countOrderResponseTime(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderSurveyTime
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    Integer countOrderSurveyTime(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderResponse
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    int countOrderResponse(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderFinished
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    int countOrderFinished(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * countOrderSurvey
     * @param supplierId
     * @param regionIdTree
     * @param threeMonthBefore
     * @param month
     * @return
     */
    int countOrderSurvey(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);
}