package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.SupplierContract;
import com.boco.nbd.wios.manage.entity.bo.SupplierContractBo;
import com.boco.nbd.wios.manage.entity.bo.SupplierContractVo;
import com.boco.nbd.wios.manage.entity.cams.enums.ContractStatus;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 服务商合同dao
 *
 * @author
 */
@Repository
public class SupplierContractDao {
    @Resource
    private SupplierContractMapper supplierContractMapper;

    public SupplierContract selectById(int id) {
        return supplierContractMapper.selectById(id);
    }

    public List<SupplierContract> selectByCode(String code) {
        return supplierContractMapper.selectByCode(code);
    }

    public int insert(SupplierContract contract) {
        contract.setCreateTime(new Date());
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null) {
            contract.setCreateAccount(sessionInfo.getUserName());
        }
        Date date1;
        try {
            date1 = WiosConstant.SIMPLE_DATE_FORMAT.parse(WiosConstant.SIMPLE_DATE_FORMAT.format(new Date()));
        } catch (ParseException e) {
            throw new ServiceException("数据转换失败");
        }
        if (contract.getStartTime().compareTo(date1) > 0) {
            contract.setStatus(ContractStatus.NOT_START.getType());
        } else {
            if (contract.getEndTime().compareTo(date1) >= 0) {
                contract.setStatus(ContractStatus.EFFECTIVE.getType());
            } else {
                throw new ServiceException("结束日期必须大于今天");
            }
        }
        return supplierContractMapper.insert(contract);
    }

    public int update(int id, SupplierContract contract) {
        contract.setId(id);
        contract.setModifyTime(new Date());
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null) {
            contract.setModifyAccount(sessionInfo.getUserName());
        }

        if (contract.getStartTime() != null && contract.getEndTime() != null) {
            Date date1;
            try {
                date1 = WiosConstant.SIMPLE_DATE_FORMAT.parse(WiosConstant.SIMPLE_DATE_FORMAT.format(new Date()));
            } catch (ParseException e) {
                throw new ServiceException("数据转换失败");
            }
            if (contract.getStartTime().compareTo(date1) > 0) {
                contract.setStatus(ContractStatus.NOT_START.getType());
            } else {
                if (contract.getEndTime().compareTo(date1) >= 0) {
                    contract.setStatus(ContractStatus.EFFECTIVE.getType());
                } else {
                    throw new ServiceException("结束日期必须大于今天");
                }
            }
        }
        return supplierContractMapper.updateDynamic(contract);
    }

    public List<SupplierContractVo> selectByStatusAndSupplierId(int supplierId, int status) {
        SupplierContractBo condition = new SupplierContractBo();
        condition.setSupplierId(supplierId + "");
        condition.setStatus(status);
        List<SupplierContractVo> list = supplierContractMapper.selectByExample(condition);
        return list;
    }

    public List<SupplierContractVo> selectValid(Integer supplierId) {
        SupplierContractBo condition = new SupplierContractBo();
        if (supplierId != null) {
            condition.setSupplierId(supplierId.intValue() + "");
        }
        List<Integer> statusInList = new ArrayList<>();
        statusInList.add(ContractStatus.NOT_START.getType());
        statusInList.add(ContractStatus.EFFECTIVE.getType());
        condition.setStatusInList(statusInList);
        condition.setOrderBy(2);
        List<SupplierContractVo> list = supplierContractMapper.selectByExample(condition);
        return list;
    }

    public List<SupplierContractVo> getList(SupplierContractBo condition) {
        return supplierContractMapper.selectByExample(condition);
    }

    public List<SupplierContract> selectByRegionId(List<String> regionIdList, Integer checkStatus) {
        return supplierContractMapper.selectByRegionId(regionIdList, ContractStatus.EFFECTIVE.getType(), checkStatus);
    }

    public List<SupplierContract> selectEffectiveContract(String supplierId, String regionId, Integer status) {
        return supplierContractMapper.selectEffectiveContract(supplierId, regionId, status);
    }
}
