package com.boco.nbd.wios.manage.mapper.def;

import com.boco.nbd.wios.manage.entity.bo.Supplier;
import com.boco.nbd.wios.manage.entity.bo.SupplierBo;
import com.boco.nbd.wios.manage.entity.bo.SupplierVo;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 服务商dao
 *
 * @author xgl
 * @version [版本号, 2020年9月16日]
 */
@Repository
public class SupplierDao {
    @Resource
    private SupplierMapper supplierMapper;

    public Supplier selectById(Integer id) {
        return supplierMapper.selectById(id);
    }

    public Supplier selectByAccountId(Integer accountId) {
        return supplierMapper.selectByAccountId(accountId);
    }

    public List<SupplierVo> getList(SupplierBo condition) {
        return supplierMapper.selectByExample(condition);
    }

    public List<Supplier> selectAllValid() {
        return supplierMapper.selectAll(1);
    }

    public List<Supplier> selectAll() {
        return supplierMapper.selectAll(null);
    }

    public List<SupplierVo> getByRegionId(String regionId) {
        SupplierBo condition = new SupplierBo();
        condition.setRegionId(regionId);
        return supplierMapper.selectByExample(condition);
    }

    public List<SupplierVo> selectInProcessOrder(List<String> supplierIdList) {
        List<Integer> notInList = new ArrayList<>();
        notInList.add(OrderStatus.RETURN_VISIT.getType());
        notInList.add(OrderStatus.CLOSED.getType());
        notInList.add(OrderStatus.INSTALL_EXAMINE_SUCCESS.getType());
        return supplierMapper.selectInProcessOrder(notInList, supplierIdList);
    }

    public SupplierVo selectInProcessOrder(int supplierId) {
        List<String> supplierIdList = new ArrayList<>();
        supplierIdList.add(supplierId + "");
        List<SupplierVo> list = selectInProcessOrder(supplierIdList);
        return list.isEmpty() ? null : list.get(0);
    }

    public int insert(Supplier supplier) {
        supplier.setCreateTime(new Date());
        supplier.setStatus(1);
        return supplierMapper.insert(supplier);
    }

    public int batchInsert(List<Supplier> list) {
        list.stream().forEach(supplier -> {
            supplier.setCreateTime(new Date());
            supplier.setStatus(1);
        });
        return supplierMapper.batchInsert(list);
    }

    public int update(int id, Supplier supplier) {
        supplier.setId(id + "");
        supplier.setModifyTime(new Date());
        return supplierMapper.updateDynamic(supplier);
    }

    public List<Supplier> selectOnly(String code, String name) {
        return supplierMapper.selectOnly(code, name);
    }

    public void updateDynamic(Supplier supplierCheck) {
        supplierMapper.updateDynamic(supplierCheck);
    }
}
