package com.boco.nbd.wios.manage.mapper.def;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.cams.enums.UploadFileType;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.util.CommonUtils;
import com.ihidea.core.support.session.SessionInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 上传文件dao
 *
 * @author xgl
 * @version [版本号, 2020年10月10日]
 */
@Repository
@Slf4j
public class UploadFileDao {
    @Resource
    private UploadFileMapper ufMapper;

    /**
     * 查询
     *
     * @param id
     * @return
     */
    public UploadFile selectById(Integer id) {
        return ufMapper.selectById(id);
    }

    /**
     * 插入文件
     *
     * @param file
     * @return
     */
    public int insert(UploadFile file) {
        file.setCreateTime(new Date());
        file.setStatus(1);
        file.setCreateBy(getUserName(file.getCreateBy()));
        return ufMapper.insert(file);
    }

    /**
     * 附件保存
     *
     * @param files
     * @param type
     * @param thirdPartyId
     * @return
     */
    public int batchSave(List<UploadFile> files, UploadFileType type, String thirdPartyId) {
        if (CollUtil.isEmpty(files)) {
            log.error("附件为空！");
            return 0;
        }
        if (type == null) {
            log.error("附件类型为空！");
            return 0;
        }
        if (StrUtil.isEmpty(thirdPartyId)) {
            log.error("附件关联三方Id为空！");
            return 0;
        }
        List<UploadFile> list = new ArrayList<>(files.size());
        for (UploadFile file : files) {
            file.setId(null);
            file.setType(type.getType());
            file.setThirdPartyId(thirdPartyId);
            if (StrUtil.isEmpty(file.getUrl())) {
                log.error("{}附件地址为空！", thirdPartyId);
                continue;
            }
            file.setUrl(CommonUtils.urlHandle(file.getUrl()));
            list.add(file);
        }
        if (CollUtil.isEmpty(list)) {
            return 0;
        }
        return batchInsert(files);
    }

    /**
     * 批量插入
     *
     * @param list
     * @return
     */
    public int batchInsert(List<UploadFile> list) {
        list.stream().forEach(file -> {
            file.setCreateTime(new Date());
            file.setStatus(1);
            file.setCreateBy(getUserName(file.getCreateBy()));
        });
        return ufMapper.batchInsert(list);
    }

    /**
     * 通过token重新获取一下操作信息
     *
     * @param userName
     * @return
     */
    private String getUserName(String userName) {
        SessionInfo user = ProcessUtil.getUserInfo();
        if (user == null) {
            return userName;
        }
        return user.getUserName();
    }

    /**
     * 修改
     *
     * @param id
     * @param file
     * @return
     */
    public int update(int id, UploadFile file) {
        file.setId(id);
        file.setUpdateTime(new Date());
        file.setUpdateBy(getUserName(file.getUpdateBy()));
        return ufMapper.updateDynamic(file);
    }

    /**
     * 查询
     *
     * @param type
     * @param tpId
     * @return
     */
    public List<UploadFile> selectValidByTypeAndThirdPartyId(int type, String tpId) {
        UploadFile condition = new UploadFile();
        condition.setStatus(1);
        condition.setType(type);
        condition.setThirdPartyId(tpId);
        return ufMapper.selectByExample(condition);
    }

    /**
     * 删除
     *
     * @param type
     * @param tpId
     * @return
     */
    public int deleteByTypeAndThirdPartyId(int type, String tpId) {
        return ufMapper.deleteByTypeAndThirdPartyId(type, tpId);
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    public int delete(int id) {
        UploadFile file = new UploadFile();
        file.setId(id);
        file.setStatus(2);
        file.setUpdateTime(new Date());
        file.setUpdateBy(getUserName(file.getUpdateBy()));
        return ufMapper.updateDynamic(file);
    }
}
