/**
 * Project Name: warehouse
 * File Name: MaterialsMapperEx
 * Package Name: com.starcharge.warehouse.daoEx
 * Date: 2020/10/19 14:20
 * Copyright (c) 2020,All Rights Reserved.
 */
package com.boco.nbd.wios.manage.mapper.extdb;


import com.boco.nbd.wios.manage.entity.bo.Materials;
import com.boco.nbd.wios.manage.entity.bo.MaterialsEx;
import com.boco.nbd.wios.manage.entity.bo.WarehouseMaterials;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * Description
 * @author lvpeng
 * @date 2020/10/19 14:20
 */
public interface MaterialsMapperEx {

    /**
     * 批量新增物料
     * @param materialsList
     * @return
     */
    int insertMaterials(@Param("materialsList") List<Materials> materialsList);

    /**
     * 查询物料数据
     * @param appkey
     * @param warehouseIds
     * @param materialsName
     * @param materialsCode
     * @param regularModel
     * @param type
     * @param id
     * @param typesList
     * @param isBindWareHouse
     * @return
     */
    List<MaterialsEx> qryMaterialsList(@Param("appkey") String appkey, @Param("warehouseIds") List<String> warehouseIds, @Param("materialsName") String materialsName, @Param("materialsCode") String materialsCode, @Param("regularModel") String regularModel, @Param("type") Integer type, @Param("id") String id , @Param("typesList") List<String> typesList, @Param("isBindWareHouse") Integer isBindWareHouse);


    /**
     * 库存查询
     * @param appkey
     * @param warehouseIds
     * @param materialsName
     * @param materialsCode
     * @param regularModel
     * @param type
     * @param id
     * @param typesList
     * @param isBindWareHouse
     * @return
     */
    List<MaterialsEx> qryMaterialsStockList(@Param("appkey") String appkey,@Param("warehouseIds") List<String> warehouseIds,@Param("materialsName") String materialsName,@Param("materialsCode") String materialsCode,@Param("regularModel") String regularModel,@Param("type") Integer type,@Param("id") String id ,@Param("typesList") List<String> typesList,@Param("isBindWareHouse") Integer isBindWareHouse);

    /**
     * 查询数据是否存在
     * @param appkey
     * @param materialsName
     * @param materialsCode
     * @param regularModel
     * @param type
     * @return
     */
    List<MaterialsEx> qryMaterialsCount(@Param("appkey") String appkey,@Param("materialsName") String materialsName,@Param("materialsCode") String materialsCode,@Param("regularModel") String regularModel,@Param("type") Integer type);

    /**
     * 删除物料信息
     * @param materialsId
     * @return
     */
    int delMaterialsById(@Param("materialsId") String materialsId);


    /**
     * 根据仓库Id及物料ID查询相关物料数据
     * @param warehouseId
     * @param materialsId
     * @return
     */
    List<WarehouseMaterials> getWarehouseMaterials(@Param("warehouseId") String warehouseId, @Param("materialsId") String materialsId);

    /**
     * 根据仓库及物料更新仓库物料数量
     * @param warhouseId
     * @param materialsId
     * @param quantity
     * @param type
     * @return
     */
    int updateWarehouseMaterials(@Param("warhouseId") String warhouseId, @Param("materialsId")String materialsId, @Param("quantity")BigDecimal quantity,@Param("type") Integer type);
    /**
     * 库存增加
     * @param warhouseId
     * @param materialsId
     * @param quantity
     * @return
     */
    int increaseInventoryQuantity(@Param("warhouseId") String warhouseId, @Param("materialsId")String materialsId, @Param("quantity")BigDecimal quantity);
    /**
     * 库存减少
     * @param warhouseId
     * @param materialsId
     * @param quantity
     * @return
     */
    int reduceInventoryQuantity(@Param("warhouseId") String warhouseId, @Param("materialsId")String materialsId, @Param("quantity")BigDecimal quantity);


    /**
     * 新增或更新仓库物料数量
     * @param warehouseMaterials
     * @return
     */
    int insertWarehouseMaterials(WarehouseMaterials warehouseMaterials);


    /**
     * 查询物料编码
      * @param id
     * @return
     */
    List<String> selectMaterialsCode(@Param("id")String id);


    /**
     * 查询墙盒标号--yuanlong
     * @param materialsCode
     * @return
     */
    List<String> selectIdForPrice(@Param("materialsCode")String materialsCode);
    /**
     * 根据物料编码查询物料信息
     * @param materialsCode
     * @return
     */
    MaterialsEx materialsByMaterialsCode(@Param("materialsCode") String materialsCode);
}
