package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.AlarmType;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.ihidea.core.support.exception.ServiceException;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * cams账户service
 *
 * @author xgl
 * @version [版本号, 2020年9月25日]
 */
@Service
public class CamsAccountService {
    @Autowired
    private CamsAccountDao camsAccountDao;

    @Autowired
    private AccountService accountService;

    @Autowired
    private AccountRoleMapper arMapper;

    @Autowired
    private DepartmentDao deptDao;

    @Autowired
    private RegionService regionService;

    @Autowired
    private RegionDao regionDao;

    @Autowired
    private RoleMapper roleMapper;

    /**
     * 添加cams账号
     *
     * @param camsAccount
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(CamsAccountBo camsAccount) {
        AccountBo account = new AccountBo();
        account.setAccount(camsAccount.getAccount());
        account.setName(camsAccount.getUserName());
        account.setPassword(WiosConstant.ACCOUNT_INIT_PASSWORD);
        account.setPhone(camsAccount.getAccount());
        account.setEmail(camsAccount.getEmail());
        Integer roleId = camsAccount.getRoleId();
        Role role = roleMapper.selectByPrimaryKey(roleId);
        if (role == null)
            throw new ServiceException("角色不存在");
        account.setRoleIds(camsAccount.getRoleId().toString());
        accountService.addAccount(account);

        camsAccount.setAccountId(account.getId());
        camsAccountDao.insert(camsAccount);
    }

    /**
     * 修改cams账号
     *
     * @param id
     * @param accountBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(int id, CamsAccountBo accountBo) {
        CamsAccount account = new CamsAccount();
        BeanUtils.copyProperties(accountBo, account);
        camsAccountDao.update(id, account);
        if (!StringUtils.isEmpty(accountBo.getUserName()) || !StringUtils.isEmpty(accountBo.getAccount())
            || !StringUtils.isEmpty(accountBo.getEmail())) {
            AccountBo updateAccount = new AccountBo();
            CamsAccount ca = camsAccountDao.selectById(id);
            updateAccount.setId(ca.getAccountId());
            if (!StringUtils.isEmpty(accountBo.getUserName())) {
                updateAccount.setName(accountBo.getUserName());
            }
            if (!StringUtils.isEmpty(accountBo.getAccount())) {
                updateAccount.setAccount(accountBo.getAccount());
            }
            if (!StringUtils.isEmpty(accountBo.getEmail())) {
                updateAccount.setEmail(accountBo.getEmail());
            }

            accountService.updateAccount(updateAccount);
        }
        if (accountBo.getRoleId() != null) {
            // 修改角色
            Role role = roleMapper.selectByPrimaryKey(accountBo.getRoleId());
            if (role == null) {
                throw new ServiceException("角色不存在");
            }
            CamsAccount ca = camsAccountDao.selectById(accountBo.getId());
            AccountRoleCriteria example = new AccountRoleCriteria();
            example.createCriteria().andAccountIdEqualTo(ca.getAccountId());
            arMapper.deleteByExample(example);
            AccountRole record = new AccountRole();
            record.setAccountId(ca.getAccountId());
            record.setRoleId(accountBo.getRoleId());
            arMapper.insert(record);
        }
    }

    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<CamsAccountVo> getList(CamsAccountBo condition) {
        List<CamsAccountVo> returnList = camsAccountDao.selectByExample(condition);
        List<Region> regionList = regionDao.selectAll(null);
        Map<Integer, Region> regionMap = regionList.stream().collect(Collectors.toMap(Region::getId, region -> region));
        returnList.stream().forEach(item -> {
            String regionId = item.getRegionId();
            if (!StringUtils.isEmpty(regionId)) {
                String[] regionArr = regionId.split(CamsConstant.COMMA);
                List<String> idList = Arrays.asList(regionArr);
                Set<String> namelist = new HashSet<>();
                for (String rId : regionArr) {
                    Region region = regionMap.get(Integer.valueOf(rId));
                    if (region != null) {
                        Integer id = getRegionId(region.getIdTree(), idList);
                        if (id != null) {
                            Region region2 = regionMap.get(id);
                            if (region2 != null && !StringUtils.isEmpty(region2.getName())) {
                                namelist.add(region2.getName());
                            }
                        }
                    }
                }
                item.setRegionName(StringUtils.collectionToCommaDelimitedString(namelist));
            }
        });
        return returnList;
    }

    public Integer getRegionId(String idTree, List<String> idList) {
        String[] idTreeArr = idTree.split(CamsConstant.UNDER_LINE);
        for (int i = 0; i < idTreeArr.length; i++) {
            if (idList.contains(idTreeArr[i])) {
                return Integer.valueOf(idTreeArr[i]);
            }
        }
        return null;
    }

    /**
     * 根据账号id查询
     *
     * @param accountId
     * @return
     */
    public String getRegionIdByAccountId(Integer accountId){
        List<CamsAccount> list = camsAccountDao.selectByAccountIdAndDeptId(accountId, null);
        if(CollectionUtils.isNotEmpty(list)){
            return list.get(0).getRegionId();
        }
        return null;
    }

    /**
     * 根据账号id查询
     *
     * @param accountId
     * @return
     */
    public CamsAccount getByAccountId(Integer accountId) {
        List<CamsAccount> list = camsAccountDao.selectByAccountIdAndDeptId(accountId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            return list.get(0);
        }
        return null;
    }

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    public CamsAccountVo getDetailById(int id) {
        CamsAccount camsAccount = camsAccountDao.selectById(id);
        CamsAccountVo camsAccountVo = new CamsAccountVo();
        BeanUtils.copyProperties(camsAccount, camsAccountVo);
        if (camsAccount != null) {
            Integer accountId = camsAccount.getAccountId();
            AccountRoleCriteria example = new AccountRoleCriteria();
            example.createCriteria().andAccountIdEqualTo(accountId);
            List<AccountRole> roleList = arMapper.selectByExample(example);
            camsAccountVo.setRoleId(roleList.get(0).getRoleId());
            Role role = roleMapper.selectByPrimaryKey(roleList.get(0).getRoleId());
            if (role != null) {
                camsAccountVo.setRoleName(role.getName());
            }
            Department dept = deptDao.selectById(camsAccount.getDepartmentId());
            camsAccountVo.setDept(dept);

            String regionId = camsAccount.getRegionId();
            String[] regionIdArr = regionId.split(CamsConstant.COMMA);
            List<Integer> regionIdIntArr = new ArrayList<>();
            for (String s : regionIdArr) {
                regionIdIntArr.add(Integer.valueOf(s));
            }
            List<RegionVo> regionList = regionService.selectAll(1, regionIdIntArr);
            camsAccountVo.setRegions(regionList);
        }
        return camsAccountVo;
    }

    /**
     * 获取cams区域工程经理
     *
     * @param regionId
     * @return
     */
    public List<CamsAccountVo> getProjectManager(int regionId, AlarmType alarmType) {
        CamsAccountBo condition = new CamsAccountBo();
        condition.setRegionId(regionId + "");
        if(alarmType == null){
            // 内置数据不可修改
            condition.setRoleId(5);
        }else{
            condition.setAlarmId(alarmType.getType()+"");
        }
        return camsAccountDao.selectByExample(condition);
    }

}
