package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.cams.enums.IOperator;
import com.boco.nbd.wios.manage.mapper.def.UploadFileDao;
import com.boco.nbd.wios.manage.util.ClassUtil;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 通用service
 *
 * @author xgl
 * @version [版本号, 2020年9月17日]
 */
@Service
public class CommonService {

    /**
     * 枚举类map
     */
    public static Map<String, IOperator[]> enumMap = new HashMap<>();

    /**
     * 枚举packages
     */
    private final static String ENUM_PACKAGES = "com.boco.nbd.wios.manage.entity.cams.enums";

    @Resource
    private UploadFileDao uploadFileDao;

    /**
     * 初始化方法
     */
    @PostConstruct
    public void init() {
        // 获取包下所有枚举类
        List<Class<?>> enumClzList = ClassUtil.getClassFromPackage(ENUM_PACKAGES);
        for (Class<?> clz : enumClzList) {
            if (clz.isEnum()) {
                String clzName = clz.getName();
                String[] arr = clzName.split("\\.");
                String name = arr[arr.length - 1];
                enumMap.put(name, (IOperator[]) clz.getEnumConstants());
            }
        }
    }

    /**
     * 获取枚举map
     *
     * @param enumNames
     * @return
     */
    public Map<String, Map<Integer, String>> getEnums(String enumNames) {
        Map<String, Map<Integer, String>> returnMap = new HashMap<>(8);
        String[] arr = enumNames.split(CamsConstant.COMMA);
        for (String enumName : arr) {
            Map<Integer, String> map = new HashMap<>(4);
            IOperator[] iOperators = enumMap.get(enumName);
            if (iOperators == null) {
                continue;
            }
            for (IOperator io : iOperators) {
                map.put(io.getType(), io.getValue());
            }
            returnMap.put(enumName, map);
        }
        return returnMap;
    }

    /**
     * 获取枚举内容
     *
     * @param enumNames
     * @return
     */
    public Map<String, List<Map<String, Object>>> getEnumsArr(String enumNames) {
        Map<String, List<Map<String, Object>>> returnMap = new HashMap<>(8);
        String[] arr = enumNames.split(CamsConstant.COMMA);
        for (String enumName : arr) {
            List<Map<String, Object>> enumList = new ArrayList<>();
            IOperator[] iOperators = enumMap.get(enumName);
            if (iOperators == null) {
                continue;
            }
            for (IOperator io : iOperators) {
                Map<String, Object> map = new HashMap<>(2);
                map.put("id", io.getType());
                map.put("name", io.getValue());
                enumList.add(map);
            }
            returnMap.put(enumName, enumList);
        }
        return returnMap;
    }

    /**
     * 删除
     *
     * @param id
     */
    public void deleteUploadFile(int id) {
        uploadFileDao.delete(id);
    }
}
