package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.cams.core.enums.StatusEnum;
import com.boco.nbd.wios.manage.entity.bo.Department;
import com.boco.nbd.wios.manage.entity.bo.DepartmentVo;
import com.boco.nbd.wios.manage.entity.bo.Region;
import com.boco.nbd.wios.manage.mapper.def.DepartmentDao;
import com.boco.nbd.wios.manage.mapper.def.RegionDao;
import com.ihidea.core.support.exception.ServiceException;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 部门service
 *
 * @author xgl
 * @version [版本号, 2020年9月24日]
 */
@Service
public class DepartmentService {

    @Resource
    private DepartmentDao departmentDao;

    @Resource
    private RegionDao regionDao;

    /**
     * 增
     *
     * @param department
     */
    public void add(Department department) {
        departmentDao.insert(department);
    }

    /**
     * 改
     *
     * @param id
     * @param department
     */
    public void update(int id, Department department) {
        if (department.getStatus() != null && StatusEnum.DELETE.getKey().equals(department.getStatus())) {
            List<Department> childDeptList = departmentDao.selectByParentId(id);
            if (!childDeptList.isEmpty()) {
                throw new ServiceException("有下级部门，不可删除");
            }
        }
        departmentDao.update(id, department);
    }

    /**
     * 查询（状态）
     *
     * @param status
     * @return
     */
    public List<DepartmentVo> getList(Integer status) {
        List<DepartmentVo> list = departmentDao.getList(status);

        for (DepartmentVo item : list) {
            String[] idTreeArr = item.getIdTree().split(CamsConstant.UNDER_LINE);
            item.setIdLevelCount(idTreeArr.length);
        }
        // 排序
        list = list.stream().sorted(Comparator.comparing(DepartmentVo::getIdLevelCount)).collect(Collectors.toList());

        List<DepartmentVo> returnList = new ArrayList<>();
        for (Department department : list) {
            Integer parentId = department.getParentId();
            if (parentId != null) {
                DepartmentVo departmentVo = getDepartmentVo(returnList, parentId);
                if (departmentVo == null) {
                    continue;
                }
                DepartmentVo departmentVoNew = new DepartmentVo();
                BeanUtils.copyProperties(department, departmentVoNew);
                List<DepartmentVo> children = departmentVo.getChildren();
                if (children == null) {
                    children = new ArrayList<>();
                }
                children.add(departmentVoNew);
                departmentVo.setChildren(children);
            } else {
                DepartmentVo departmentVo = new DepartmentVo();
                BeanUtils.copyProperties(department, departmentVo);
                returnList.add(departmentVo);
            }
        }
        return returnList;
    }

    /**
     * 补充信息
     *
     * @param returnList
     * @param id
     * @return
     */
    public DepartmentVo getDepartmentVo(List<DepartmentVo> returnList, int id) {
        for (DepartmentVo departmentVo : returnList) {
            if (departmentVo.getId().intValue() == id) {
                return departmentVo;
            }
            List<DepartmentVo> children = departmentVo.getChildren();
            if (children != null) {
                DepartmentVo departmentVo2 = getDepartmentVo(children, id);
                if (departmentVo2 != null) {
                    return departmentVo2;
                }
            }
        }
        return null;
    }

    /**
     * 查询(id)
     *
     * @param id
     * @return
     */
    public DepartmentVo getById(Integer id) {
        Department dept = departmentDao.selectById(id);
        DepartmentVo returnValue = new DepartmentVo();
        BeanUtils.copyProperties(dept, returnValue);
        if (dept != null & dept.getRegionId() != null) {
            Region region = regionDao.selectById(dept.getRegionId());
            returnValue.setRegion(region);
        }
        return returnValue;
    }
}
