package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.wios.manage.entity.bo.Dictionary;
import com.boco.nbd.wios.manage.mapper.def.DictionaryDao;
import com.ihidea.core.support.exception.ServiceException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 字典service
 *
 * @author xgl
 * @version [版本号, 2020年9月30日]
 */
@Service
public class DictionaryService {

    @Resource
    private DictionaryDao dictionaryDao;

    /**
     * 查询
     * @param type
     * @param name
     * @return
     */
    public List<Dictionary> selectValidByType(String type, String name) {
        return dictionaryDao.selectValidByType(type, name);
    }

    public void add(Dictionary dic) {
        dictionaryDao.insert(dic);
    }

    /**
     * 修改
     * @param dic
     */
    public void update(Dictionary dic) {
        if (dic.getName()!=null) {
            Dictionary  dictionary=dictionaryDao.selectById(dic.getId());
            List<Dictionary> dictionaryList=dictionaryDao.selectValidById(dictionary.getType(), dic.getName(),dic.getId());
            if (!dictionaryList.isEmpty()) {
                throw new ServiceException("字典配置表中name重复");

            }
        }
        dictionaryDao.update(dic);
    }

    /**
     * 修改
     * @param list
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void timingConfigUpdate(List<Dictionary> list) {
        if (list.isEmpty()) {
            return;
        }
        List<Dictionary> typeNullList = list.stream().filter(dic -> StringUtils.isEmpty(dic.getType())).collect(Collectors.toList());
        if (!typeNullList.isEmpty()) {
            throw new ServiceException("缺少type");
        }
        List<Dictionary> nameNullList = list.stream().filter(dic -> StringUtils.isEmpty(dic.getName())).collect(Collectors.toList());
        if (!nameNullList.isEmpty()) {
            throw new ServiceException("缺少name");
        }
        List<Dictionary> valueNullList = list.stream().filter(dic -> StringUtils.isEmpty(dic.getValue())).collect(Collectors.toList());
        if (!valueNullList.isEmpty()) {
            throw new ServiceException("缺少value");
        }
        dictionaryDao.deleteByType(list.get(0).getType());
        dictionaryDao.batchInsert(list);
    }

    /**
     * 查询
     * @param type
     * @param value
     * @return
     */
    public List<Dictionary> selectValidByValue(String type, String value) {
        return dictionaryDao.selectValidByValue(type, value);
    }
}
