package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.lang.Assert;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.InfoConfig;
import com.boco.nbd.wios.manage.mapper.def.InfoConfigDao;
import com.ihidea.core.support.exception.ServiceException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * @Description InfoConfigService
 * @author liwenxiang
 * @date 2021/7/24 13:41
 * @return
 */
@Service
public class InfoConfigService {
    @Autowired
    private InfoConfigDao infoConfigDao;

    /**
     * @Description 添加信息配置
     * @Param [type, name]
     * @author liwenxiang
     * @date 2021/7/24 13:44
     * @return void
     */
    public void add(Integer type, String name) {
        Assert.notNull(type, "缺少type");
        Assert.notEmpty(name, "缺少name");
        List<InfoConfig> infoConfigList=infoConfigDao.selectValidNameType(type, name);
        if (!infoConfigList.isEmpty()) {
            if(infoConfigList.get(0).getName().equals(name)){
                throw new ServiceException("该类型下的名称已存在");
            }
        }
        InfoConfig infoConfig = new InfoConfig();
        infoConfig.setId(UUID.randomUUID().toString());
        infoConfig.setName(name);
        infoConfig.setType(type);
        infoConfig.setCreateTime(new Date());
        infoConfigDao.insertSelective(infoConfig);
    }

    /**
     * @Description 更新信息配置
     * @Param [id, name]
     * @author liwenxiang
     * @date 2021/7/26 9:30
     * @return void
     */
    public void update(String id, String name) {
        Assert.notEmpty(id, "缺少id");
        Assert.notEmpty(name, "缺少name");
        InfoConfig infoConfig = new InfoConfig();
        infoConfig.setId(id);
        infoConfig.setName(name);
        infoConfig.setModifyTime(new Date());
        infoConfigDao.updateByPrimaryKeySelective(infoConfig);
    }


    /**
     * @Description 条件查询信息配置列表
     * @Param [type, name]
     * @author liwenxiang
     * @date 2021/7/26 9:48
     * @return java.util.List<com.starcharge.wios.dao.entity.InfoConfig>
     */
    public List<InfoConfig> getInfoConfigListByCondition(Integer type, String name) {
        Assert.notNull(type, "缺少type");
        return infoConfigDao.selectValidNameType(type, name);
    }

    /**
     * @Description 删除基本信息配置
     * @Param [id]
     * @author liwenxiang
     * @date 2021/7/26 10:04
     * @return void
     */
    public void delete(String id) {
        InfoConfig infoConfig = new InfoConfig();
        infoConfig.setId(id);
        infoConfig.setIsDel(WiosConstant.IS_DEL_YES);
        infoConfig.setModifyTime(new Date());
        infoConfigDao.updateByPrimaryKeySelective(infoConfig);
    }
}
