package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.bestvike.linq.Linq;
import com.boco.nbd.wios.flow.entity.po.OemCascadePO;
import com.boco.nbd.wios.flow.entity.vo.OrderInstallItemVO;
import com.boco.nbd.wios.flow.service.IOrderService;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.ContractStatus;
import com.boco.nbd.wios.manage.entity.cams.enums.UploadFileType;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.entity.dto.MaterialsDTO;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 主机厂合同service
 *
 * @author
 */
@Service
public class OemContractService {

    @Autowired
    private OemContractDao oemContractDao;

    @Autowired
    private OemContractItemDao oemContractItemDao;

    @Autowired
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    private OemContractPriceDao oemContractPriceDao;

    @Autowired
    private UploadFileDao uploadFileDao;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private IOrderService flowOrderService;

    /**
     * 添加数据
     *
     * @param contractBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(OemContractBo contractBo) {
        List<OemContract> contractListByCode = oemContractDao.selectByCode(contractBo.getCode());
        if (!contractListByCode.isEmpty()) {
            throw new ServiceException("合同编号已存在");
        }
        // 获取已添加的有效的合同列表
        List<OemContractVo> validContractList = oemContractDao.selectValid(Integer.valueOf(contractBo.getOemId()));
        for (OemContractVo contract : validContractList) {
            if (contractBo.getOperateType() == null || contractBo.getOperateType().intValue() == 0) {
                OemContract updateContract = new OemContract();
                if (contract.getEndTime().compareTo(contractBo.getEndTime()) <= 0) {
                    if (contract.getStartTime().compareTo(contractBo.getStartTime()) >= 0) {
                        // 时间包含则设为失效
                        updateContract.setStatus(ContractStatus.INVALID.getType());
                    } else {
                        if (contract.getEndTime().compareTo(contractBo.getStartTime()) >= 0) {
                            // 时间有交集则更新该合同的结束时间
                            Calendar cal = Calendar.getInstance();
                            cal.setTime(contractBo.getStartTime());
                            cal.add(Calendar.DATE, -1);
                            updateContract.setEndTime(cal.getTime());
                            // 结束时间小于当前时间
                            if (contractBo.getStartTime().compareTo(new Date()) <= 0) {
                                updateContract.setStatus(ContractStatus.INVALID.getType());
                            }
                        } else {
                            continue;
                        }
                    }
                } else {
                    throw new ServiceException("结束日期不能小于已添加合同的结束日期");
                }
                oemContractDao.update(contract.getId(), updateContract);
            } else {
                if ((contractBo.getStartTime().compareTo(contract.getStartTime()) >= 0
                        && contractBo.getStartTime().compareTo(contract.getEndTime()) <= 0)
                        || (contractBo.getEndTime().compareTo(contract.getStartTime()) >= 0
                        && contractBo.getEndTime().compareTo(contract.getEndTime()) <= 0)) {
                    throw new ServiceException("与已有合同时间冲突");
                }
            }
        }
        // 添加合同数据
        oemContractDao.insert(contractBo);
        List<OemContractItem> items = contractBo.getItems();
        if (items != null && !items.isEmpty()) {
            items.stream().forEach(item -> {
                item.setContractId(contractBo.getId().toString());
                item.setId(null);
            });
            // 添加item
            oemContractItemDao.batchInsert(items);
        }

        List<OemContractPrice> prices = contractBo.getPrices();
        if (prices != null && !prices.isEmpty()) {
            prices.stream().forEach(price -> {
                price.setContractId(contractBo.getId());
                price.setId(null);
            });
            // 添加价格
            oemContractPriceDao.batchInsert(prices);
        }
        // 添加文件
        uploadFileDao.batchSave(contractBo.getFiles(), UploadFileType.OEM_CONTRACT, contractBo.getId() + "");
    }

    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<OemContractVo> getList(OemContractBo2 condition) {
        SessionInfo user = ProcessUtil.getUserInfo();
        if (user != null && !StringUtils.isEmpty(user.getAttribute().get("oemId"))) {
            condition.setOemId(user.getAttribute().get("oemId"));
        }
        return oemContractDao.getList(condition);
    }

    /**
     * 更新合同
     *
     * @param contractBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(OemContractBo contractBo) {
        if (!StringUtils.isEmpty(contractBo.getCode())) {
            List<OemContract> contractListByCode = oemContractDao.selectByCode(contractBo.getCode());
            if (CollUtil.isNotEmpty(contractListByCode) && (contractListByCode.get(0).getId().intValue() != contractBo.getId().intValue())) {
                throw new ServiceException("合同编号已存在");
            }
        }
        oemContractDao.update(contractBo.getId(), contractBo);
        oemContractItemDao.deleteByContractId(contractBo.getId());
        oemContractPriceDao.deleteByContractId(contractBo.getId());

        List<OemContractItem> items = contractBo.getItems();
        if (items != null && !items.isEmpty()) {
            items.stream().forEach(item -> item.setContractId(contractBo.getId().toString()));
            //添加item
            oemContractItemDao.batchInsert(items);
        }

        List<OemContractPrice> prices = contractBo.getPrices();
        if (prices != null && !prices.isEmpty()) {
            prices.stream().forEach(price -> price.setContractId(contractBo.getId()));
            // 添加价格
            oemContractPriceDao.batchInsert(prices);
        }
        // 添加文件
        uploadFileDao.deleteByTypeAndThirdPartyId(UploadFileType.OEM_CONTRACT.getType(), contractBo.getId() + "");
        uploadFileDao.batchSave(contractBo.getFiles(), UploadFileType.OEM_CONTRACT, contractBo.getId() + "");
    }

    /**
     * 查询详情
     *
     * @param contract
     * @return
     */
    public OemContractVo getDetail(OemContractVo contract) {
        if (contract == null) {
            return null;
        }
        List<OemContractItemVo> contractItems = oemContractItemDao.selectByContractId(contract.getId());
        contractItems.forEach(s -> {
            MaterialsDTO dto = warehouseService.getMaterialById(s.getItemId());
            if (dto != null) {
                s.setItemCode(dto.getMaterialsCode());
                s.setItemModel(dto.getRegularModel());
                s.setItemName(dto.getMaterialsName());
            }
        });

        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null) {
            pageLimit.setLimited(true);
        }
        List<OemContractPriceVo> contractPrices = oemContractPriceDao.selectByContractId(contract.getId());
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.OEM_CONTRACT.getType(), contract.getId() + "");

        contract.setItems(contractItems);
        contract.setPrices(contractPrices);
        contract.setFiles(files);
        return contract;
    }

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    public OemContractVo getDetail(int id) {
        OemContractVo contract = oemContractDao.selectDetailById(id);
        return getDetail(contract);
    }

    /**
     * 查询工单安装明细=》合并
     *
     * @param orderId
     * @param type
     * @param oemId
     * @return
     */
    public List<OrderInstallItemVO> getOrderDetailItems(String orderId, Integer type, Integer oemId) {
        List<OrderInstallItemVo> itemFeeList = orderInstallItemDao.selectByOrderIdAndType(orderId, type);
        //获取合同明细数据
        OemContractVo oemContractVo = getDetailByOemId(oemId);
        Map<String, OrderInstallItemVO> map = new HashMap<>(4);
        if (oemContractVo != null) {
            oemContractVo.getItems().forEach(s -> {
                map.put(s.getItemId(), s.toVo());
            });
        }
        itemFeeList.forEach(s -> {
            OrderInstallItemVO item = map.containsKey(s.getItemId()) ? map.get(s.getItemId()) : s.toVo();
            if (map.containsKey(s.getItemId())) {
                item.setAreSelect(true);
                item.setQunatity(s.getQunatity());
                item.setFee(s.getFee());
            }
            map.put(s.getItemId(), item);
        });

        return map.values().stream().collect(Collectors.toList());
    }

    /**
     * 主机厂id查询合同详情
     *
     * @param oemId
     * @return
     */
    public OemContractVo getDetailByOemId(int oemId) {
        List<OemContractVo> list = oemContractDao.selectByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
        if (list.isEmpty()) {
            return null;
        }
        return getDetail(list.get(0));
    }

    /**
     * 根据主机厂和墙盒类型查询安装设备详情
     *
     * @param oemId
     * @param packType
     * @return
     */
    public List<OemContractItemVo> getSupplierInstallDevice(Integer oemId, String packType) {
        List<OemContractItemVo> items = new ArrayList<>(5);
        List<OemContractVo> list = oemContractDao.selectByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
        if (list.isEmpty()) {
            return items;
        }
        OemContractVo contract = list.get(0);
        List<OemContractItemVo> contractItems = oemContractItemDao.selectByContractId(contract.getId());
        //packType过滤
        List<OemCascadePO> wallBoxModel = flowOrderService.getOemCascade(packType);
        List<OemCascadePO> itemModel = new ArrayList<>(5);
        for (OemCascadePO cascade : wallBoxModel) {
            itemModel.addAll(flowOrderService.getOemCascade(cascade.getId()));
        }
        List<String> aims = Linq.of(itemModel).select(s -> s.getId()).toList();
        contractItems.forEach(s -> {
            if (WiosConstant.CHARGING_TYPE.toString().equals(s.getType())) {
                MaterialsDTO dto = warehouseService.getMaterialById(s.getItemId());
                if (dto != null && aims.contains(dto.getMaterialsCode())) {
                    s.setItemCode(dto.getMaterialsCode());
                    s.setItemModel(dto.getRegularModel());
                    s.setItemName(dto.getMaterialsName());
                    items.add(s);
                }
            }
        });
        return items;
    }

    /**
     * 通过主机厂获取合同
     *
     * @param oemId
     * @return
     */
    public List<OemContract> getSimpleContractByOemId(int oemId) {
        return oemContractDao.selectSimpleByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
    }

    /**
     * 获取主机厂有效的增项列表
     *
     * @param oemId
     * @param regionId
     * @return
     */
    public List<OemContractItemVo> getContractItems(Integer oemId, String regionId) {
        OemContractVo contract = getDetailByOemId(oemId);
        return contract != null ? contract.getItems() : null;
    }

    /**
     * 更新合同状态
     *
     * @throws ParseException
     */
    public void contractTiming() throws ParseException {
        List<OemContractVo> contractList = oemContractDao.selectValid(null);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (OemContractVo contract : contractList) {
            Date startDate = contract.getStartTime();
            Date endDate = contract.getEndTime();
            OemContract updateContract = new OemContract();
            endDate = sdf1.parse(sdf.format(endDate) + " 23:59:59");
            if (endDate.compareTo(new Date()) < 0) {
                updateContract.setStatus(ContractStatus.INVALID.getType());
            } else {
                if (contract.getStatus() != null && contract.getStatus().intValue() == ContractStatus.NOT_START.getType()
                        && startDate.compareTo(new Date()) <= 0) {
                    updateContract.setStatus(ContractStatus.EFFECTIVE.getType());
                }
            }
            if (updateContract.getStatus() != null) {
                oemContractDao.update(contract.getId(), updateContract);
            }

        }
    }

}
