package com.boco.nbd.wios.manage.service.impl;


import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.AccountBo;
import com.boco.nbd.wios.manage.entity.bo.Oem;
import com.boco.nbd.wios.manage.entity.bo.OemBo;
import com.boco.nbd.wios.manage.entity.bo.OemVo;
import com.boco.nbd.wios.manage.entity.cams.enums.UploadFileType;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.mapper.def.OemDao;
import com.boco.nbd.wios.manage.mapper.def.UploadFileDao;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * 主机厂/4s店service
 * 
 * @author
 * @version
 */
@Service
public class OemService {
    
    @Autowired
    private OemDao oemDao;
    
    @Autowired
    private AccountService accountService;

    @Autowired
    private UploadFileDao uploadFileDao;
    
    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<OemVo> getList(OemBo condition) {
        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null) {
            pageLimit.setLimited(true);
        }
        
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null && !StringUtils.isEmpty(sessionInfo.getAttribute().get("oemId"))) {
            int oemId = Integer.valueOf(sessionInfo.getAttribute().get("oemId"));
            Oem oem = oemDao.selectById(oemId);
            
            if (oem.getType() != null && oem.getType().intValue() == 1) {
                if (condition.getType() != null && condition.getType().intValue() == 1) {
                    // 主机厂账号查主机厂信息
                    condition.setId(oemId);
                } else {
                    // 主机厂账号查经销商信息
                    condition.setParentId(oemId);
                }
            } else {
                // 经销商账号
                if (condition.getType() != null && condition.getType().intValue() == 2) {
                    // 经销商信息列表
                    condition.setId(oemId);
                } else {
                    // 经销商查主机厂信息
                    condition.setId(oem.getParentId());
                }
            }
            
            if (condition.getShowOemChildren() != null && condition.getShowOemChildren().intValue() == 1) {
                condition.setId(null);
                condition.setParentId(null);
                condition.setIdAndPId(oemId);
            }
        }
        if (pageLimit != null) {
            return pageLimit.limitList(oemDao.getList(condition));
        } else {
            return oemDao.getList(condition);
        }
    }
    
    /**
     * 查询自己及旗下经销商
     *
     * @param id
     * @return
     */
    public List<Oem> getSelfAndChildren(Integer id) {
        if (id == null) {
            SessionInfo sessionInfo = ProcessUtil.getUserInfo();
            if (!StringUtils.isEmpty(sessionInfo.getAttribute().get("oemId")))
                id = Integer.valueOf(sessionInfo.getAttribute().get("oemId"));
        }
        return oemDao.selectSelfAndChildren(id);
    }
    
    /**
     * 单个新增
     *
     * @param oem
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(OemBo oem) {

        
        AccountBo account = new AccountBo();
        account.setAccount(oem.getAccount());
        account.setName(oem.getContactName());
        account.setPassword(WiosConstant.ACCOUNT_INIT_PASSWORD);
        account.setPhone(oem.getAccount());
        if (oem.getType() != null && oem.getType().intValue() == 1) {
            // 默认角色为主机厂管理员
            account.setRoleIds("2");
        } else {
            // 默认角色为经销商管理员
            account.setRoleIds("3");
        }
        accountService.addAccount(account);
        oem.setWbAccountId(account.getId());
        oemDao.insert(oem);
    }
    
    /**
     * 单个更新
     *
     * @param oem
     */
    public void update(OemBo oem) {

        oemDao.update(oem);
    }
    
    /**
     * 附件上传
     *
     * @param oemId
     * @param fileName
     * @param fileUrl
     */
    public void fileUpload(Integer oemId, String fileName, String fileUrl) {
        
        UploadFile file = new UploadFile();
        file.setName(fileName);
        file.setUrl(fileUrl);
        file.setType(UploadFileType.OEM.getType());
        file.setThirdPartyId(oemId.toString());
        uploadFileDao.insert(file);
    }
    
    /**
     * 查询附件
     *
     * @param oemId
     * @return
     */
    public List<UploadFile> getFiles(Integer oemId) {
        return uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.OEM.getType(), oemId.toString());
    }
    
    /**
     * 更换账户
     *
     * @param accountId
     * @param account
     * @return
     */
    public int updateAccount(int accountId, String account) {
        AccountBo accountEntity = new AccountBo();
        accountEntity.setId(accountId);
        accountEntity.setAccount(account);
        accountEntity.setPhone(account);
        return accountService.updateAccount(accountEntity);
    }
    
    /**
     * 校验唯一性
     *
     * @param code
     * @param name
     * @return
     */
    public boolean checkExist(String code, String name, Integer id) {
        List<Oem> list = oemDao.selectOnly(code, name);
        if (id != null) {
            if (!list.isEmpty() && id.intValue() == list.get(0).getId().intValue()) {
                return false;
            }
        }
        return !list.isEmpty();
    }
}
