package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.lang.Assert;
import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.config.RedisClient;
import com.boco.nbd.wios.flow.entity.vo.OrderInstallItemVO;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.UploadFileType;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.entity.dto.MaterialsDTO;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

/**
 * 安装订单service
 *
 * @author xgl
 * @version [版本号, 2020年9月17日]
 */
@Service
public class OrderInstallService {

    @Resource
    private OrderInstallDao orderInstallDao;

    @Resource
    private UploadFileDao uploadFileDao;

    @Resource
    private ExpandPriceDao expandPriceDao;

    @Resource
    private OrderDao orderDao;

    @Resource
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    private UserPayBillService userPayBillService;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private OemContractService oemContractService;

    @Resource
    private OperateLogDao operateLogDao;

    @Autowired
    private RedisClient redisClient;

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(OrderInstallBo orderInstall) {
        String lockId = "addOrderInstall:" + orderInstall.getOrderId();
        String requestId = StringUtilsEx.getUUID();
        if (redisClient.lock(lockId, requestId, 60)) {
            try {
                OrderInstall oi = orderInstallDao.selectByOrderId(orderInstall.getOrderId());
                if (oi != null) {
                    throw new ServiceException("安装订单已存在");
                }
                Order order = orderDao.selectById(orderInstall.getOrderId());
                if (StringUtils.isEmpty(orderInstall.getWallboxItemId())) {
                    throw new ServiceException("缺少充电桩");
                }

                List<OrderInstallItem> itemList = orderInstall.getItemFees();
                if (itemList != null && !itemList.isEmpty()) {
                    itemList.stream().forEach(item -> {
                        item.setItemType(2);// 增项
                        item.setType(2);// 安装订单
                        item.setOrderId(orderInstall.getOrderId());
                    });

                    orderInstallItemDao.batchInsert(itemList);
                    orderInstall.setPredictTotalFee(itemList.stream().map(OrderInstallItem::getFee).reduce(BigDecimal.ZERO, BigDecimal::add));
                }

                if (orderInstall.getElectricExpand() != null && orderInstall.getElectricExpand().intValue() == 1) {
                    if (order != null) {
                        List<ExpandPrice> priceList = expandPriceDao.selectByRegionId(Integer.valueOf(order.getRegionId()));
                        if (!priceList.isEmpty()) {
                            orderInstall.setExpandFee(priceList.get(0).getPrice());
                        }
                    }
                }
                if (orderInstall.getExpandFee() == null) {
                    orderInstall.setExpandFee(BigDecimal.ZERO);
                }

                // 存入注册cams phone，用于后面开具发票
                orderInstall.setThirdUserId(order.getSurveyCamsPhone());
                orderInstallDao.insert(orderInstall);
                List<UploadFile> files = orderInstall.getFiles();
                // 添加文件
                uploadFileDao.batchSave(orderInstall.getFiles(), UploadFileType.INSTALL_ORDER, orderInstall.getId() + "");
                // 修改安装订单修改订单的墙盒型号和墙盒设备型号
                Order orderUpdate = new Order();
                orderUpdate.setInstallWallboxModel(orderInstall.getInstallWallboxModel());
                orderUpdate.setInstallItemModel(orderInstall.getWallboxItemId());
                orderDao.updateById(orderInstall.getOrderId(), orderUpdate);
            } finally {
                redisClient.releaseLock(lockId, requestId);
            }
        } else {
            throw new ServiceException("系统忙，请稍后");
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(int id, OrderInstallBo orderInstall) {
        OrderInstall oi = orderInstallDao.selectById(id);
        if (oi == null) {
            throw new ServiceException("无此安装单");
        }

        // 如果已经生成账单，1)不允许变更报装 2)不允许变更增项
        if (hasGenBill(oi.getOrderId())) {
            orderInstall.setElectricExpand(null);
            orderInstall.setExpandMethod(null);
            orderInstall.setItemFeesJson(null);
            orderInstall.setItemFees(null);
        }

        List<OrderInstallItem> itemList = orderInstall.getItemFees();
        if (CollectionUtils.isNotEmpty(itemList)) {
            itemList.stream().forEach(item -> {
                item.setItemType(2);// 增项
                item.setType(2);// 安装订单
                item.setOrderId(oi.getOrderId());
            });
            orderInstallItemDao.delete(oi.getOrderId(), 2);
            orderInstallItemDao.batchInsert(itemList);
            orderInstall.setPredictTotalFee(itemList.stream().map(OrderInstallItem::getFee).reduce(BigDecimal.ZERO, BigDecimal::add));
        }
        orderInstallDao.update(id, orderInstall);

        uploadFileDao.deleteByTypeAndThirdPartyId(UploadFileType.INSTALL_ORDER.getType(), orderInstall.getId().toString());
        // 添加文件
        uploadFileDao.batchSave(orderInstall.getFiles(), UploadFileType.INSTALL_ORDER, orderInstall.getId() + "");
        // 修改安装订单修改订单的墙盒型号和墙盒设备型号
        Order order = new Order();
        order.setInstallWallboxModel(orderInstall.getInstallWallboxModel());
        order.setInstallItemModel(orderInstall.getWallboxItemId());
        orderDao.updateById(oi.getOrderId(), order);
    }

    /**
     * 更新订单增项、报装
     *
     * @param orderId
     * @param type     订单类型 1-勘察订单 2-安装订单 3-安装变更
     * @param itemList
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveInstallItem(String orderId, Integer type, List<OrderInstallItem> itemList, Integer electricExpand, Integer expandMethod, BigDecimal expandFee) {
        Assert.notBlank(orderId, WiosConstant.EMPTY_ORDER_ID);
        Assert.notNull(type, WiosConstant.EMPTY_ORDER_TYPE);

        Order order = orderDao.selectById(orderId);
        Assert.notNull(order, WiosConstant.EMPTY_ORDER);

        if (type == 2) {
            if (hasGenBill(orderId)) {
                throw new ServiceException("当前订单已生成账单，不支持修改增项和报装");
            }
        }

        orderInstallItemDao.delete(orderId, type);
        BigDecimal totalFee = BigDecimal.ZERO;
        if (CollectionUtils.isNotEmpty(itemList)) {
            for (OrderInstallItem item : itemList) {
                // 增项
                item.setItemType(2);
                item.setType(type);
                item.setOrderId(orderId);

                totalFee = totalFee.add(item.getFee());
            }
            orderInstallItemDao.batchInsert(itemList);
        }

        if (type == 2) {
            OrderInstall oi = orderInstallDao.selectByOrderId(orderId);
            // 如果安装报告还未创建，则先创建
            if (oi == null) {
                OrderInstall orderInstall = new OrderInstall();
                orderInstall.setOrderId(orderId);
                orderInstall.setThirdUserId(order.getSurveyCamsPhone());
                orderInstall.setPayStatus(0);
                orderInstall.setPredictTotalFee(totalFee);
                orderInstall.setElectricExpand(electricExpand);
                orderInstall.setExpandMethod(expandMethod);
                orderInstall.setExpandFee(expandFee);
                orderInstallDao.insert(orderInstall);
            } else {
                OrderInstall orderInstall = new OrderInstall();
                orderInstall.setId(oi.getId());
                orderInstall.setPredictTotalFee(totalFee);
                orderInstall.setElectricExpand(electricExpand);
                orderInstall.setExpandMethod(expandMethod);
                orderInstall.setExpandFee(expandFee);
                orderInstallDao.update(oi.getId(), orderInstall);
            }
        }
    }

    /**
     * 查询安装订单详情
     *
     * @param orderId
     * @return
     */
    public OrderInstallVo getByOrderId(String orderId) {
        OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
        if (orderInstall == null) {
            return null;
        }
        OrderInstallVo result = new OrderInstallVo();
        String wallboxItemId = orderInstall.getWallboxItemId();
        if (!StringUtils.isEmpty(wallboxItemId)) {
            MaterialsDTO dto = warehouseService.getMaterialById(wallboxItemId);
            if (dto != null) {
                result.setWallboxItemName(dto.getMaterialsName());
                result.setWallboxItemModel(dto.getRegularModel());
            }
        }
        BeanUtils.copyProperties(orderInstall, result);
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            return null;
        }
        result.copyOrder(order);
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.INSTALL_ORDER.getType(), orderId);
        result.setFiles(files);
        List<OrderInstallItemVO> itemFeeList = oemContractService.getOrderDetailItems(orderId, 2, order.getOemId());
        result.setItemFees(Linq.of(itemFeeList).where(s -> WiosConstant.FEE_TYPE.equals(s.getItemType())).toList());
        result.setItemHighAdd(Linq.of(itemFeeList).where(s -> WiosConstant.HIGH_ADD_TYPE.equals(s.getItemType())).toList());
        result.setHasGenBill(hasGenBill(orderId) ? 1 : 0);
        result.setInstallWallboxModel(order.getInstallWallboxModel());

        //加入上次审核不通过原因描述
        OperateLog operateLog = operateLogDao.getNewestCheckFailByOrderId(orderId, "55");
        if (Objects.nonNull(operateLog)) {
            result.setCheckFailDesc(operateLog.getDesc());
        } else {
            result.setCheckFailDesc("");
        }

        return result;
    }

    public void updateByOrderId(String orderId, OrderInstall orderInstall) {
        orderInstallDao.updateByOrderId(orderId, orderInstall);
    }

    public boolean hasGenBill(String orderId) {
        return userPayBillService.countByRefId(orderId) > 0;
    }

}
