package com.boco.nbd.wios.manage.service.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.aliyun.oss.ServiceException;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.wios.flow.entity.Constant;
import com.boco.nbd.wios.flow.entity.bo.SapStatisticsBO;
import com.boco.nbd.wios.flow.entity.po.OrderSurveyPO;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.Dictionary;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.*;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.entity.dto.WarehouseDTO;
import com.boco.nbd.wios.manage.entity.dto.WarehouseMaterialDTO;
import com.boco.nbd.wios.manage.entity.dto.WarehouseQuery;
import com.boco.nbd.wios.manage.enums.ContractIdInfo;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.boco.nbd.wios.manage.util.DistanceUtil;
import com.boco.nbd.wios.manage.util.NumberUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.component.cache.redis.RedisClient;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import com.ihidea.core.util.JSONUtilsEx;
import com.ihidea.core.util.StringUtilsEx;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * 订单service
 *
 * @author
 */
@Service
public class OrderService {

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private RegionDao regionDao;

    @Autowired
    private OperateLogDao operateLogDao;

    @Autowired
    private DictionaryService dicService;

    @Autowired
    private SystemInfoDao systemInfoDao;

    @Autowired
    private CamsAccountService camsAccountService;

    @Autowired
    private PushService pushService;

    @Autowired
    private SupplierContractDao supplierContractDao;

    @Autowired
    private ScoreSupplierDao scoreSupplierDao;

    @Autowired
    private UploadFileDao uploadFileDao;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private SupplierStaffDao supplierStaffDao;

    @Autowired
    private OrderSurveyDao orderSurveyDao;

    @Autowired
    private OrderInstallDao orderInstallDao;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private OemContractDao oemContractDao;

    @Autowired
    private OrderScoreService orderScoreService;

    @Autowired
    private SecretPhoneService spService;

    @Autowired
    private InfoConfigDao infoConfigDao;

    @Autowired
    private OrderDeleteDao orderDeleteDao;

    @Autowired
    private OrderInstallDeleteDao orderInstallDeleteDao;

    @Autowired
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    private OrderInstallItemDeleteDao orderInstallItemDeleteDao;

    @Autowired
    private OrderSurveyDeleteDao orderSurveyDeleteDao;

    @Autowired
    private UserPayBillDao userPayBillDao;

    @Autowired
    private UserPayBillDeleteDao userPayBillDeleteDao;

    @Autowired
    private SystemInfoDeleteDao systemInfoDeleteDao;

    @Autowired
    private OperateLogDeleteDao operateLogDeleteDao;

    @Autowired
    private SettlementDetailDao settlementDetailDao;

    @Autowired
    private SettlementDetailSupplierDao settlementDetailSupplierDao;

    @Autowired
    private DispatchConfigDao dispatchConfigDao;

    @Autowired
    private DictionaryDao dictionaryDao;

    @Autowired
    private SapRemoteService sapRemoteService;

    @Autowired
    private RedissonClient redissonClient;

    @Resource
    OrderMapper orderMapper;

    private Logger logger = LoggerFactory.getLogger(OrderService.class);

    /**
     * 添加订单
     *
     * @param order
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(Order order) {
        Order selectOrder = orderDao.selectById(order.getId());
        if (selectOrder != null) {
            throw new ServiceException("订单号已存在");
        }

        //车企订单号唯一性判断
        OrderBo orderBo = new OrderBo();
        orderBo.setOutOrderId(order.getOutOrderId());
        List<Order> list = orderDao.selectAll(orderBo);
        for (Order o : list) {
            if (o.getStatus() != OrderStatus.CLOSED.getType()) {
                throw new ServiceException("车企订单号已存在");
            }
        }

        Integer oemId = order.getOemId();
        if (order.getOemAgentId() != null) {
            oemId = order.getOemAgentId();
        }
        List<OemContractVo> contractList = oemContractDao.selectByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
        if (contractList.isEmpty()) {
            throw new ServiceException("没有有效合同");
        }
        order.setContractId(contractList.get(0).getId());
        orderDao.insert(order);

        operateAfterAdd(order);
    }

    /**
     * 更新订单
     *
     * @param order
     */
    public void update(String id, Order order) {
        OrderVo dbOrderVo = orderDao.selectDetailById(id);
        if (dbOrderVo == null) {
            throw new ServiceException("订单不存在");
        }

        if (!StringUtils.isEmpty(dbOrderVo.getAddress())) {
            // 地址修改发短信
            if (!StringUtils.isEmpty(dbOrderVo.getAddress()) && !order.getAddress().equals(dbOrderVo.getAddress())) {
                dbOrderVo.setAddress(order.getAddress());
                pushService.sendSurveyReserveModify(dbOrderVo);
            }
        }

        // 2021-01-04注释
        // --------start-----------
        // if (!StringUtils.isEmpty(dbOrderVo.getWarehouseId())) {
        // // 区域修改了 更改仓库
        // if (!StringUtils.isEmpty(dbOrderVo.getRegionId()) && !order.getRegionId().equals(dbOrderVo.getRegionId())
        // && dbOrderVo.getInstallSupplierId() != null) {
        // WarehouseMaterialDTO dto =
        // getStubQuantityBySupplier(Integer.valueOf(order.getRegionId()), dbOrderVo.getInstallSupplierId().toString());
        // if (!StringUtils.isEmpty(dto.getWarehouseId())) {
        // order.setWarehouseId(dto.getWarehouseId());
        // } else {
        // throw new ServiceException("该区域没有仓库");
        // }
        // }
        // }
        // --------end-----------
        // 订单状态不修改
        order.setStatus(null);
        orderDao.updateById(id, order);
    }

    public void operateAfterAdd(Order order) {
        // 绑定虚拟号码
        try {
            spService.bindAsync(order.getUserPhone());
        } catch (Exception e) {
            logger.error("绑定隐私号码失败,message={}", e.getMessage());
        }

        // 以下是自动派单逻辑
        {
            // 是否需要自动派单
            Integer manuDispatchEnable = order.getManuDispatchEnable();
            if (manuDispatchEnable != null && manuDispatchEnable.intValue() == 1) {
                return;
            }
            if (order.getType() != null && order.getType().intValue() == 2) {
                // 移桩订单
                return;
            }
            SupplierVo supplier = null;
            RLock lock = redissonClient.getLock("dispatch_order_" + order.getRegionId());
            try {
                if (lock.tryLock(5, 5, TimeUnit.SECONDS)) {
                    supplier = getAutoDispatchOrderSupplier(Integer.valueOf(order.getRegionId()), order.getId());
                }
            } catch (InterruptedException e) {
                logger.error("自动派单失败,message={}", e.getMessage());
            } finally {
                if (lock.isLocked()) {
                    lock.unlock();
                }
            }

            if (supplier == null) {
                return;
            }

            // 更新安装服务商
            {
                Order updateOrder = new Order();
                updateOrder.setInstallSupplierId(Integer.valueOf(supplier.getId()));
                updateOrder.setStatus(OrderStatus.SURVEY_RESERVING.getType());
                if (!StringUtils.isEmpty(supplier.getWarehouseId())) {
                    updateOrder.setWarehouseId(supplier.getWarehouseId());
                }
                List<SupplierContractVo> supplierContractList =
                        supplierContractDao.selectByStatusAndSupplierId(Integer.valueOf(supplier.getId()), ContractStatus.EFFECTIVE.getType());
                if (supplierContractList.isEmpty()) {
                    throw new ServiceException("无服务商合同");
                }
                updateOrder.setSupplierContractId(supplierContractList.get(0).getId());
                updateOrder.setDispatchTime(new Date());
                orderDao.updateById(order.getId(), updateOrder);
            }

            // 插入操作日志
            operateLogDao.insertOrderLog(order.getId(), OrderStatus.SURVEY_RESERVING.getType(), "自动派单", "系统自动派单成功");

            // 通知管理员
            if (supplier.getAccountId() != null)
                systemInfoDao.insertForOrder("订单派单成功", supplier.getAccountId(), order.getId(), 1);
        }
    }

    /**
     * 批量添加
     *
     * @param ordersJson
     * @throws Exception
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void batchAdd(String ordersJson) {
        ObjectMapper mapper = new ObjectMapper();
        mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);

        try {
            List<Order> list = mapper.readValue(ordersJson, new TypeReference<List<Order>>() {
            });
            List<Order> idNullList = list.stream().filter(order -> StringUtils.isEmpty(order.getId())).collect(Collectors.toList());
            if (!idNullList.isEmpty()) {
                throw new ServiceException(WiosConstant.EMPTY_ORDER_ID);
            }
            List<String> idList = list.stream().map(Order::getId).collect(Collectors.toList());
            Set<String> idSet = new HashSet<>();
            idList.forEach(id -> {
                idSet.add(id);
            });
            if (idList.size() != idSet.size()) {
                throw new ServiceException("订单号不可重复");
            }
            OrderBo condition = new OrderBo();
            condition.setIdIn(StringUtils.collectionToCommaDelimitedString(idList));
            List<OrderVo> existOrderList = orderDao.getList(condition);
            if (!existOrderList.isEmpty()) {
                throw new ServiceException("订单号已存在");
            }

            list.stream().forEach(order -> {
                Integer oemId = order.getOemId();
                if (order.getOemAgentId() != null) {
                    oemId = order.getOemAgentId();
                }
                List<OemContract> contractList =
                        oemContractDao.selectSimpleByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
                if (contractList.isEmpty()) {
                    throw new ServiceException("没有有效合同");
                }
                order.setContractId(contractList.get(0).getId());
            });

            orderDao.batchInsert(list);
            list.stream().forEach(order -> {
                operateAfterAdd(order);
            });
        } catch (JsonProcessingException e) {
            throw new ServiceException("转换失败");
        }
    }

    /**
     * 自动派单
     *
     * @param regionId
     * @return
     */
    public SupplierVo getAutoDispatchOrderSupplier(Integer regionId, String orderId) {
        Region region = regionDao.selectById(regionId);
        if (region == null) {
            logger.error("orderId={},regionId={}自动派单失败，无此区域", orderId, regionId);
            operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVING.getType(), "自动派单", "自动派单失败，无此区域");
            return null;
        }
        //逐级查找
        String idTree = region.getIdTree();
        String[] idTreeArr = idTree.split(CamsConstant.UNDER_LINE);
        final List<String> idTreeList = Arrays.asList(idTreeArr);
        // 完全匹配 ，不再逐级查找 20201230
        // -------start---------
        /*final List<String> idTreeList = new ArrayList<>();
        idTreeList.add(region.getId().toString());*/
        // -------end-----------
        // 可用服务商(需要审核通过)
        List<SupplierContract> supplierContractList = supplierContractDao.selectByRegionId(idTreeList, 1);
        if (supplierContractList.isEmpty()) {
            logger.error("orderId={},regionId={}自动派单失败，无可用服务商", orderId, regionId);
            operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVING.getType(), "自动派单", "自动派单失败，无可用服务商");
            return null;
        }

        Map<Integer, Integer> supplierIdRegionIdMap = new HashMap<>(8);
        // 设置服务商区域
        {
            supplierContractList.stream().forEach(item -> {
                String rIdStr = item.getRegionId();
                String[] rIdStrArr = rIdStr.split(CamsConstant.COMMA);
                for (String rid : rIdStrArr) {
                    if (idTreeList.contains(rid)) {
                        supplierIdRegionIdMap.put(Integer.valueOf(item.getSupplierId()), Integer.valueOf(rid));
                        break;
                    }
                }
            });
        }

        List<String> supplierIdList = supplierContractList.stream().map(SupplierContract::getSupplierId).collect(Collectors.toList());
        logger.info("orderId={},regionId={}区域内服务商ids={}", orderId, regionId, supplierIdList);
        // 查询服务商当日接单数
        List<SupplierVo> supplierVoList = supplierDao.selectInProcessOrder(supplierIdList);
        // 筛选订单不饱和的服务商
        supplierVoList = supplierVoList.stream().filter(item -> item.getMaxOrderQuantity().intValue() > item.getOrderQuantity())
                .collect(Collectors.toList());
        if (supplierVoList.isEmpty()) {
            logger.error("orderId={},regionId={}自动派单失败，服务商订单都已饱和", orderId, regionId);
            operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVING.getType(), "自动派单", "自动派单失败，服务商订单都已饱和");
            return null;
        }
        // 整理服务商评分
        for (SupplierVo supplier : supplierVoList) {
            // 查询服务商评分
            ScoreSupplier score = scoreSupplierDao.selectLastestScoreInfo(Integer.valueOf(supplier.getId()),
                    supplierIdRegionIdMap.get(Integer.valueOf(supplier.getId())));
            logger.info("orderId={},服务商评分supplierId={},regionId={},score={}", orderId, supplier.getId(),
                    supplierIdRegionIdMap.get(Integer.valueOf(supplier.getId())), score == null ? 0 : score.getFinalScore());
            supplier.setScore(score == null ? 0 : (score.getFinalScore() == null ? 0 : score.getFinalScore()));
        }

        // 查询仓库2021-01-04注释
        // --------start-----------
        // {
        // supplierVoList.stream().forEach(supplier -> {
        // WarehouseMaterialDTO dto =
        // getStubQuantityBySupplier(supplierIdRegionIdMap.get(Integer.valueOf(supplier.getId())), supplier.getId());
        // supplier.setStubMaterialQuantity(dto.getStubMaterialQuantity());
        // supplier.setWarehouseId(dto.getWarehouseId());
        // });
        // supplierVoList =
        // supplierVoList.stream().filter(supplier ->
        // !StringUtils.isEmpty(supplier.getWarehouseId())).collect(Collectors.toList());
        // if (supplierVoList.isEmpty()) {
        // logger.error("orderId={},regionId={}自动派单失败，该区域的服务商没有仓库", orderId, regionId);
        // operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVING.getType(), "自动派单", "自动派单失败，该区域的服务商没有仓库");
        // return null;
        // }
        // }
        // --------end-----------

        // 判断满不满足库存 (2020.12.24注释)
        // {
        // supplierVoList = supplierVoList.stream()
        // .filter(item -> item.getStubMaterialQuantity() != null && item.getStubMaterialQuantity().intValue() > 0)
        // .collect(Collectors.toList());
        // if (supplierVoList.isEmpty()) {
        // logger.error("orderId={}自动派单失败，服务商仓库没有充电桩", orderId);
        // operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVING.getType(), "自动派单", "自动派单失败，服务商仓库没有充电桩");
        // return null;
        // }
        // }

        //先分配自建团队
        List<SupplierVo> levelSupplierList =
                supplierVoList.stream().filter(item -> "A".equals(item.getLevel())).collect(Collectors.toList());
        if (levelSupplierList.isEmpty()) {
            logger.info("orderId={},A类没有服务商regionId={}", orderId, regionId);

            //再分配经销商团队
            levelSupplierList = supplierVoList.stream().filter(item -> "B".equals(item.getLevel())).collect(Collectors.toList());
        }
        //AB团队存在则平均分配
        if (!levelSupplierList.isEmpty()) {
            return averDistribute(levelSupplierList, orderId);
        } else {
            logger.info("orderId={},B类没有服务商regionId={}", orderId, regionId);
            //C类服务商根据评分排序
            List<SupplierVo> supplierVoListSorted = supplierVoList.stream().sorted(Comparator.comparing(SupplierVo::getScore).reversed()).collect(Collectors.toList());
            Map<String, SupplierVo> supplierVoListMap = supplierVoListSorted.stream().collect(Collectors.toMap(SupplierVo::getId, a -> a, (k1, k2) -> k2));
            //派单比例包含了上级区域的服务商,若不存在则设置默认值
            List<DispatchConfig> dispatchConfigList = new ArrayList<>();
            for (Map.Entry<Integer, Integer> entry : supplierIdRegionIdMap.entrySet()) {
                List<Integer> regionIdList = new ArrayList<>();
                idTreeList.stream().forEach(str -> {
                    regionIdList.add(Integer.valueOf(str));
                });
//                regionIdList.add(entry.getValue());
                //region和supplierId只能查到一条数据
                List<DispatchConfig> dispatchConfigListByCondition = dispatchConfigDao.getDispatchConfigListByCondition(regionIdList, entry.getKey().toString());
                //若不存在，则插入一条默认派单值
                if (CollectionUtils.isEmpty(dispatchConfigListByCondition)) {
                    DispatchConfig dispatchConfig = new DispatchConfig();
                    dispatchConfig.setDispatchNum(Constant.DISPATCH_NUM);
                    dispatchConfig.setRegionId(entry.getValue());
                    dispatchConfig.setSupplierId(entry.getKey().toString());
                    dispatchConfigList.add(dispatchConfig);
                } else {
                    dispatchConfigList.add(dispatchConfigListByCondition.get(0));
                }

            }

            //可用服务商和配置表服务商取交集，获取生成派单比例值
            Map<String, Integer> dispatchConfigMap = dispatchConfigList.stream().collect(Collectors.toMap(DispatchConfig::getSupplierId, DispatchConfig::getDispatchNum, (k1, k2) -> k2));
            HashMap<String, Integer> configParamMap = new LinkedHashMap<>();
            for (int i = 0; i < supplierVoListSorted.size(); i++) {
                if (!ObjectUtils.isEmpty(dispatchConfigMap.get(supplierVoListSorted.get(i).getId()))) {
                    configParamMap.put(supplierVoListSorted.get(i).getId(), dispatchConfigMap.get(supplierVoListSorted.get(i).getId()));
                }
            }


            //redis获取当前区域派单的数量
            String redisKey = "dispatch_num_" + regionId;
            Map<Object, Object> dispatchNumMap = RedisClient.getMap(redisKey, HashMap.class, 2000);
            if (dispatchNumMap.size() == 0) {
                HashMap<Object, Object> map = new HashMap<>(8);
                map.put(supplierVoListSorted.get(0).getId(), 1);
                RedisClient.put("dispatch_num_" + regionId, map);
                return supplierVoListSorted.get(0);
            }
            for (String key : configParamMap.keySet()) {

                if (ObjectUtils.isEmpty(dispatchNumMap.get(key))) {
                    dispatchNumMap.put(key, 1);
                    RedisClient.put("dispatch_num_" + regionId, dispatchNumMap);
                    return supplierVoListMap.get(key);
                }
                //如果该人的分配数小于配置参数，则分配给此人
                if (Integer.parseInt(dispatchNumMap.get(key).toString()) < configParamMap.get(key)) {
                    dispatchNumMap.put(key, Integer.parseInt(dispatchNumMap.get(key).toString()) + 1);
                    RedisClient.put("dispatch_num_" + regionId, dispatchNumMap);
                    return supplierVoListMap.get(key);
                }
            }
            //开始新一轮
            RedisClient.remove("dispatch_num_" + regionId);
            HashMap<Object, Object> map = new HashMap<>(8);
            map.put(supplierVoListSorted.get(0).getId(), 1);
            RedisClient.put("dispatch_num_" + regionId, map);
            return supplierVoListSorted.get(0);
        }
    }

    public WarehouseMaterialDTO getStubQuantityBySupplier(Integer regionId, String supplierId) {
        WarehouseMaterialDTO returnValue = new WarehouseMaterialDTO();
        WarehouseQuery warehouseQuery = new WarehouseQuery();
        warehouseQuery.setServiceProvider(supplierId);
        warehouseQuery.setArea(regionId.toString());
        warehouseQuery.setIsEnable(1);
        warehouseQuery.setPage(1);
        warehouseQuery.setPagecount(1);
        List<WarehouseDTO> warehouseList = warehouseService.qryWarehouseList(warehouseQuery);

        if (!warehouseList.isEmpty()) {
            returnValue.setWarehouseId(warehouseList.get(0).getId());
            // (2020.12.24注释)
            // MaterialsQuery materialsQuery = new MaterialsQuery();
            // materialsQuery.setType(3);
            // materialsQuery.setWarehouseId(warehouseList.get(0).getId());
            // List<MaterialsDTO> materials = warehouseService.qryStockMaterials(materialsQuery);
            // materials = materials.stream().filter(item -> item != null && item.getRemainingNumber() !=
            // null).collect(Collectors.toList());
            // if (materials != null && !materials.isEmpty()) {
            // int stubMaterailQuantity =
            // materials.stream().map(MaterialsDTO::getRemainingNumber).reduce(BigDecimal.ZERO,
            // BigDecimal::add).intValue();
            // returnValue.setStubMaterialQuantity(stubMaterailQuantity);
            // return returnValue;
            // }
        }
        return returnValue;
    }

    /**
     * 按接单量平均分配
     *
     * @param supplierVoList
     * @return
     */
    public SupplierVo averDistribute(List<SupplierVo> supplierVoList, String orderId) {
        if (supplierVoList.isEmpty()) {
            return null;
        }
        if (supplierVoList.size() == 1) {
            logger.info("orderId={},自动分配的服务商1id={}", orderId, supplierVoList.get(0).getId());
            return supplierVoList.get(0);
        }
        int minOrderQuantity = supplierVoList.stream().min(Comparator.comparing(SupplierVo::getOrderQuantity)).get().getOrderQuantity();
        List<SupplierVo> minOrderQuantitySupplierList = new ArrayList<SupplierVo>();
        supplierVoList.stream().forEach(item -> {
            if (item.getOrderQuantity() == minOrderQuantity) {
                minOrderQuantitySupplierList.add(item);
            }
        });
        if (minOrderQuantitySupplierList.size() == 1) {
            logger.info("orderId={},自动分配的服务商2id={}", orderId, minOrderQuantitySupplierList.get(0).getId());
            return minOrderQuantitySupplierList.get(0);
        }
        int randomNum = NumberUtil.getRandomNum(0, minOrderQuantitySupplierList.size());
        SupplierVo selectedSupplier = minOrderQuantitySupplierList.get(randomNum);
        logger.info("orderId={},自动分配的服务商3id={}", orderId, selectedSupplier.getId());
        return selectedSupplier;
    }

    /**
     * 派单
     *
     * @param orderId
     * @param supplierId
     */
    public void dispatchOrder(String orderId, int supplierId) {
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        String regionId = order.getRegionId();
        Region region = regionDao.selectById(Integer.valueOf(regionId));
        if (region == null) {
            throw new ServiceException("无此区域");
        }
        String[] regionIdArr = region.getIdTree().split(CamsConstant.UNDER_LINE);
        List<String> regionIdList = Arrays.asList(regionIdArr);
        // 列表取反
        Collections.reverse(regionIdList);
        String warehouseId = "";
        // 逐级向上查找区域内仓库(2020-01-04注释)
        // --------start---------
        // for (String rId : regionIdList) {
        // WarehouseMaterialDTO dto = getStubQuantityBySupplier(Integer.valueOf(rId), supplierId + "");
        // if (!StringUtils.isEmpty(dto.getWarehouseId())) {
        // warehouseId = dto.getWarehouseId();
        // break;
        // }
        // }
        // if (StringUtils.isEmpty(warehouseId)) {
        // throw new ServiceException("该服务商在区域没有仓库");
        // }
        // --------end---------
        Supplier supplier = supplierDao.selectById(supplierId);
        List<String> supplierIdList = new ArrayList<>();
        supplierIdList.add(supplierId + "");
        List<SupplierVo> supplierVoList = supplierDao.selectInProcessOrder(supplierIdList);
        if (!supplierVoList.isEmpty() && supplier.getMaxOrderQuantity() <= supplierVoList.get(0).getOrderQuantity()) {
            throw new ServiceException("该服务商已到最大接单量");
        }

        Order updateOrder = new Order();
        updateOrder.setInstallSupplierId(Integer.valueOf(supplierId));
        updateOrder.setStatus(OrderStatus.SURVEY_RESERVING.getType());
        updateOrder.setWarehouseId(warehouseId);
        List<SupplierContractVo> supplierContractList =
                supplierContractDao.selectByStatusAndSupplierId(supplierId, ContractStatus.EFFECTIVE.getType());
        if (supplierContractList.isEmpty())
            throw new ServiceException("无服务商合同");
        updateOrder.setSupplierContractId(supplierContractList.get(0).getId());
        updateOrder.setDispatchTime(new Date());
        orderDao.updateById(orderId, updateOrder);

        // 操作日志
        operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVING.getType(), "派单", "手动派单");

        // 通知管理员
        if (supplier != null && supplier.getAccountId() != null) {
            systemInfoDao.insertForOrder("订单派单成功", supplier.getAccountId(), order.getId(), 1);
        }
    }

    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<OrderVo> getList(OrderBo condition) {
        if (!StringUtils.isEmpty(condition.getSurveyFinishStartTime())) {
            condition.setSurveyFinishStartTime(condition.getSurveyFinishStartTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(condition.getSurveyFinishEndTime())) {
            condition.setSurveyFinishEndTime(condition.getSurveyFinishEndTime() + " 23:59:59");
        }
        if (!StringUtils.isEmpty(condition.getInstallFinishStartTime())) {
            condition.setInstallFinishStartTime(condition.getInstallFinishStartTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(condition.getInstallFinishEndTime())) {
            condition.setInstallFinishEndTime(condition.getInstallFinishEndTime() + " 23:59:59");
        }
        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null) {
            pageLimit.setLimited(true);
        }
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        String roleIdType = sessionInfo.getAttribute().get("roleId");
        String accountId = sessionInfo.getUserId();
        if (!StringUtils.isEmpty(sessionInfo.getAttribute().get("oemId"))) {
            int oemId = Integer.valueOf(sessionInfo.getAttribute().get("oemId"));
            Oem oem = oemDao.selectById(oemId);
            if (!ObjectUtils.isEmpty(oem)) {
                if (OemType.OEM.getType() == (oem.getType().intValue())) {
                    condition.setOemId(oemId);
                } else {
                    condition.setOemAgentId(oemId);
                }

            } else {
                condition.setOemId(oemId);
            }
        }
        if (!StringUtils.isEmpty(sessionInfo.getAttribute().get("installSupplierId"))) {
            condition.setInstallSupplierId(Integer.valueOf(sessionInfo.getAttribute().get("installSupplierId")));
        }

        List<SupplierStaff> supplierStaffList = supplierStaffDao.selectByAccountId(Integer.valueOf(sessionInfo.getUserId()));
        if (!supplierStaffList.isEmpty()) {
            // 账号对应的角色是电工
            SupplierStaff supplierStaff = supplierStaffList.get(0);
            if (supplierStaff.getDataAuth() != null && supplierStaff.getDataAuth().intValue() == 1) {
                condition.setInstallStaffId(supplierStaff.getId());
            }
        }

        // cams账号只能查询负责区域的订单
        if (!StringUtils.isEmpty(roleIdType) && Integer.valueOf(roleIdType).intValue() == 999) {// cams账号
            if (!StringUtils.isEmpty(accountId)) {
                CamsAccount camsAccount = camsAccountService.getByAccountId(Integer.valueOf(accountId));
                if (camsAccount != null) {
                    String regionId = camsAccount.getRegionId();
                    List<String> regionIdList = Arrays.asList(regionId.split(CamsConstant.COMMA));
                    List<Integer> regionIdIntList = new ArrayList<>();
                    for (String id : regionIdList) {
                        regionIdIntList.add(Integer.valueOf(id));
                    }
                    condition.setRegionIdList(regionIdIntList);
                }
            }
        }

        List<OrderVo> list = orderDao.getList(condition);
        List<Dictionary> dic = dicService.selectValidByType("timing", OldTimingConfig.RESERVE_EXCEPTION.getType() + "");
        if (!dic.isEmpty()) {
            list.stream().forEach(order -> {
                setShowReserveExpBtn(order, dic);
            });

        }
        return list;
    }

    /**
     * 查询订单安装完成之前列表
     * @return
     */
    public List<InstallComplete> installCompleteList(OrderBo condition) {
        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null) {
            pageLimit.setLimited(true);
        }
        return orderMapper.installCompleteList(condition);
    }

    private void setShowReserveExpBtn(OrderVo order, List<Dictionary> dic) {
        if (order.getStatus() != null
                && (order.getStatus().intValue() == OrderStatus.SURVEY_RESERVING.getType()
                || order.getStatus().intValue() == OrderStatus.SURVEY_EXAMINE_SUCCESS.getType())
                && order.getModifyTime() != null && !StringUtils.isEmpty(dic.get(0).getValue())) {
            Date modifyTime = order.getModifyTime();
            Calendar cal = Calendar.getInstance();
            cal.setTime(modifyTime);
            cal.add(Calendar.HOUR, Integer.valueOf(dic.get(0).getValue()));
            // cal.add(Calendar.MINUTE, 5);// 测试用的

            if (cal.getTime().compareTo(new Date()) < 0) {
                order.setShowReserveExpBtn(true);
            }
        }
    }

    /**
     * 选择电工
     *
     * @param orderId
     * @param staffId
     */
    public void selectStaff(String orderId, int staffId) {
        Order updateOrder = new Order();
        updateOrder.setInstallStaffId(staffId);
        updateOrder.setStatus(OrderStatus.SURVEY_DISPATCHED.getType());
        orderDao.updateById(orderId, updateOrder);

        operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_DISPATCHED.getType(), "选择电工", "");

        // 给客户发短信
        OrderVo order = orderDao.selectDetailById(orderId);
        if (order != null && !StringUtils.isEmpty(order.getSurveyClientName()) && !StringUtils.isEmpty(order.getSurveyClientPhone())) {
            pushService.sendSurveyReserveSaveSucess(order);
        }
    }

    /**
     * 更换电工
     *
     * @param orderId
     * @param staffId
     */
    public void updateStaff(String orderId, int staffId) {

        Order order = orderDao.selectById(orderId);
        SupplierStaff staff = supplierStaffDao.selectById(staffId);
        if (order != null && order.getInstallStaffId() != null) {
            SupplierStaff preStaff = supplierStaffDao.selectById(order.getInstallStaffId());

            if (preStaff != null && staff != null) {
                OperateLog operateLog = new OperateLog();
                operateLog.setType(OperateLogType.STAFF_CHANGE.getType());
                operateLog.setThirdPartyId(orderId);
                operateLog.setPreName(order.getInstallStaffId().toString());
                operateLog.setPreValue(preStaff.getName());
                operateLog.setName(staffId + "");
                operateLog.setValue("更换电工");
                operateLog.setDesc("电工更换为" + staff.getName());
                operateLogDao.insert(operateLog);
            }
        }

        Order updateOrder = new Order();
        updateOrder.setInstallStaffId(staffId);
        orderDao.updateById(orderId, updateOrder);

        // 给客户发短信
        OrderVo orderVo = orderDao.selectDetailById(orderId);
        if (orderVo != null) {
            if (!StringUtils.isEmpty(orderVo.getInstallClientName()) && !StringUtils.isEmpty(orderVo.getInstallClientPhone())) {
                // 安装阶段
                pushService.sendIntallReserveModify(orderVo);
            } else if (!StringUtils.isEmpty(orderVo.getSurveyClientName()) && !StringUtils.isEmpty(orderVo.getSurveyClientPhone())) {
                // 勘测阶段
                pushService.sendSurveyReserveModify(orderVo);
            }
        }
    }

    /**
     * 更换安装服务商
     *
     * @param orderId
     * @param supplierId
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void updateInstallSupplier(String orderId, int supplierId) {
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            Assert.notNull(order, "无此订单");
        }
        Supplier preSupplier = supplierDao.selectById(order.getInstallSupplierId());
        Supplier supplier = supplierDao.selectById(supplierId);
        if (supplier == null) {
            Assert.notNull(supplier, "无此服务商");
        }
        // 查询服务商当日接单数
        SupplierVo vo = supplierDao.selectInProcessOrder(supplierId);
        if (vo != null && supplier.getMaxOrderQuantity() != null && vo.getOrderQuantity() >= supplier.getMaxOrderQuantity()) {
            throw new ServiceException("该服务商当日接单数已达上限");
        }

        // 2020-01-04注释
        // WarehouseMaterialDTO dto = getStubQuantityBySupplier(Integer.valueOf(order.getRegionId()), supplierId + "");
        // if (StringUtils.isEmpty(dto.getWarehouseId())) {
        // throw new ServiceException("服务商在该区域没有仓库");
        // }

        List<SupplierContractVo> supplierContractList =
                supplierContractDao.selectByStatusAndSupplierId(supplierId, ContractStatus.EFFECTIVE.getType());
        if (supplierContractList.isEmpty()) {
            throw new ServiceException("此服务商没有有效合同");
        }

        OperateLog operateLog = new OperateLog();
        operateLog.setType(OperateLogType.SUPPLIER_CHANGE.getType());
        operateLog.setThirdPartyId(orderId);
        operateLog.setPreName(order.getInstallSupplierId().toString());
        operateLog.setPreValue(preSupplier.getName());
        operateLog.setName(supplierId + "");
        operateLog.setValue("安装服务商变更");
        operateLog.setDesc("安装服务商变更为" + supplier.getName());
        operateLogDao.insert(operateLog);

        Order updateOrder = new Order();
        updateOrder.setInstallSupplierId(supplierId);
        // updateOrder.setWarehouseId(dto.getWarehouseId());// 更换仓库 2020-01-04注释
        // 更换合约
        updateOrder.setSupplierContractId(supplierContractList.get(0).getId());
        if (order.getStatus().intValue() == OrderStatus.SURVEYING.getType()
                || order.getStatus().intValue() == OrderStatus.SURVEY_EXAMINING.getType()
                || order.getStatus().intValue() == OrderStatus.SURVEY_RECTIFYING.getType()
                || order.getStatus().intValue() == OrderStatus.SURVEY_RECTIFIED_EXAMINING.getType()) {
            updateOrder.setStatus(OrderStatus.SURVEY_RESERVING.getType());
        } else if (order.getStatus().intValue() == OrderStatus.INSTALL_CHANGE_EXAMINING.getType()
                || order.getStatus().intValue() == OrderStatus.INSTALL_CHANGE_EXAMINE_FAIL.getType()
                || order.getStatus().intValue() == OrderStatus.INSTALLING.getType()
                || order.getStatus().intValue() == OrderStatus.INSTALL_EXAMINING.getType()
                || order.getStatus().intValue() == OrderStatus.INSTALL_RECTIFYING.getType()
                || order.getStatus().intValue() == OrderStatus.INSTALL_RECTIFY_EXAMINING.getType()) {
            updateOrder.setStatus(OrderStatus.SURVEY_EXAMINE_SUCCESS.getType());
        }
        orderDao.updateById(orderId, updateOrder);
        // 清空电工
        orderDao.clearInstallStaff(orderId);
    }

    /**
     * 更新状态
     *
     * @param orderId
     * @param status
     */
    public void updateStatus(String orderId, int status) {
        String msg = "";
        Order updateOrder = new Order();
        Order order = orderDao.selectById(orderId);
        if (status == OrderStatus.SURVEYING.getType() || status == OrderStatus.INSTALLING.getType()) {
            if (order.getInstallStaffId() == null)
                throw new ServiceException("请先选择电工");

        }
        if (status == OrderStatus.SURVEY_FINISH.getType() || status == OrderStatus.SURVEY_EXAMINING.getType()) {
            if (order.getType() != null && order.getType().intValue() == 1) {
                OrderSurvey orderSurvey = orderSurveyDao.selectByOrderId(orderId, 1);
                if (orderSurvey == null) {
                    throw new ServiceException("请先填写勘测报告");
                }
            } else {
                List<UploadFile> files = uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.ORDER.getType(), orderId);
                if (files.isEmpty()) {
                    throw new ServiceException("请先上传附件");
                }
            }
            if (status == OrderStatus.SURVEY_FINISH.getType()) {
                updateOrder.setSurveyFinishTime(new Date());
            }
            if (status == OrderStatus.SURVEY_EXAMINING.getType()) {
                updateOrder.setSurveySubmitExamineTime(new Date());
                msg = "您有一笔勘测报告待审核";
            }
        }
        if (status == OrderStatus.INSTALL_FINISH.getType() || status == OrderStatus.INSTALL_EXAMINING.getType()) {
            if (order.getType() != null && order.getType().intValue() == 1) {
                OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
                if (orderInstall == null) {
                    throw new ServiceException("请先填写安装报告");
                }
            } else {
                List<UploadFile> files = uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.ORDER.getType(), orderId);
                if (files.isEmpty()) {
                    throw new ServiceException("请先上传附件");
                }
            }
            if (status == OrderStatus.INSTALL_FINISH.getType()) {
                updateOrder.setInstallFinishTime(new Date());
            }
            if (status == OrderStatus.INSTALL_EXAMINING.getType()) {
                updateOrder.setInstallSubmitExamineTime(new Date());
                msg = "您有一笔安装报告待审核";

                UserPayBill entity = new UserPayBill();
                entity.setRefId(orderId);
                List<UserPayBill> list = userPayBillDao.list(entity);
                UserPayBill payBill = org.apache.commons.collections.CollectionUtils.isNotEmpty(list) ? list.get(0) : null;
                if (payBill != null && payBill.getStatus() != null && payBill.getStatus().intValue() == 0) {
                    throw new ServiceException("账单未支付，请支付后再试");
                }
            }
        }
        if (status == OrderStatus.SURVEY_RECTIFIED_EXAMINING.getType()) {
            msg = "您有一笔勘测整改报告待审核";
        }
        if (status == OrderStatus.INSTALL_RECTIFY_EXAMINING.getType()) {
            msg = "您有一笔安装整改报告待审核";
        }
        if (!StringUtils.isEmpty(msg)) {
            notifyRegionPM(msg, orderId, Integer.valueOf(order.getRegionId()), 1, null);
        }

        updateOrder.setStatus(status);
        orderDao.updateById(orderId, updateOrder);

        String text = getOperateTextByStatus(status);
        if (StringUtils.isEmpty(text)) {
            text = OrderStatus.getText(status);
        }
        if (!StringUtils.isEmpty(text)) {
            operateLogDao.insertOrderLog(orderId, status, text, "");
        }
    }

    public String getOperateTextByStatus(int status) {
        if (status == OrderStatus.SURVEY_RESERVED.getType()) {
            return "勘测预约";
        }
        if (status == OrderStatus.SURVEYING.getType()) {
            return "勘测开始";
        }
        if (status == OrderStatus.SURVEY_EXAMINING.getType()) {
            return "勘测完成，提交审核";
        }
        if (status == OrderStatus.SURVEY_RECTIFIED_EXAMINING.getType()) {
            return "勘测整改完成，提交审核";
        }
        if (status == OrderStatus.INSTALL_RESERVED.getType()) {
            return "安装预约";
        }
        if (status == OrderStatus.INSTALL_CHANGE_EXAMINING.getType()) {
            return "安装变更完成，提交审核";
        }
        if (status == OrderStatus.INSTALL_EXAMINING.getType()) {
            return "安装完成，提交审核";
        }
        if (status == OrderStatus.INSTALL_RECTIFY_EXAMINING.getType()) {
            return "安装整改完成，提交审核";
        }
        if (status == OrderStatus.INSTALLING.getType()) {
            return "安装开始";
        }

        if (status == OrderStatus.CLOSED.getType()) {
            return "关闭工单";
        }
        return null;
    }

    public void updateStatus(String orderId, int status, String desc) {
        orderDao.updateStatusById(orderId, status);

        String text = OrderStatus.getText(status);
        if (!StringUtils.isEmpty(text)) {
            operateLogDao.insertOrderLog(orderId, status, desc);
        }

        //勘测、安装开始 签到超时、超出范围判断
        if (OrderStatus.SURVEYING.getType() == status || OrderStatus.INSTALLING.getType() == status) {
            Order order = orderDao.selectById(orderId);
            //超时
            Date time = OrderStatus.SURVEYING.getType() == status ? order.getSurveyReserveTime() : order.getInstallReserveTime();
            Date now = new Date();
            String type = OrderStatus.SURVEYING.getType() == status ? "勘测" : "安装";
            AlarmType alarmType = OrderStatus.SURVEYING.getType() == status ? AlarmType.SIGN_SURVEY : AlarmType.SIGN_INSTALL;
            if (now.compareTo(time) > 0) {
                long second = DateUtil.between(time, now, DateUnit.SECOND);
                double minutes = NumberUtil.roundDouble(Double.valueOf(second) / 60);
                List<CamsAccountVo> camsAccounts = camsAccountService.getProjectManager(Integer.valueOf(order.getRegionId()), alarmType);
                camsAccounts.stream().forEach(item -> {
                    systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单" + type + "签到超时" + minutes + "分钟", item.getAccountId(), order.getId(),
                            0, order.getStatus(), alarmType.getType());
                    if (!StringUtils.isEmpty(item.getUserEmail()))
                        pushService.sendEmailAsync(item.getUserEmail(), "订单消息通知",
                                "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单" + type + "签到超时" + minutes + "分钟，请及时查看</p><p>本邮件是系统自动发送，无需直接回复</p>");
                });

                //抄送指定邮箱（配置）
                List<InfoConfig> mailList = infoConfigDao.selectValidNameType(alarmType.getType(), null);
                mailList.stream().forEach(item -> {
                    if (!StringUtils.isEmpty(item.getName())) {
                        pushService.sendEmailAsync(item.getName(), "订单消息通知",
                                "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单" + type + "签到超时" + minutes + "分钟，请及时查看</p><p>本邮件是系统自动发送，无需直接回复</p>");
                    }
                });
            }

            //超出范围
            if (StrUtil.isNotBlank(desc) && desc.contains(",") && StrUtil.isNotBlank(order.getAddressLat())) {
                Dictionary dictionary = dictionaryDao.selectById(InfoConfigSingleTypes.XCX_SIGN_SCOPE.getType() + "");
                if (dictionary != null) {
                    String[] split = desc.split(CamsConstant.COMMA);
                    double distance = DistanceUtil.getDistance(Double.valueOf(split[0]), Double.valueOf(split[1]), Double.valueOf(order.getAddressLng()), Double.valueOf(order.getAddressLat()));
                    if (distance > Integer.valueOf(dictionary.getValue())) {
                        List<CamsAccountVo> camsAccounts = camsAccountService.getProjectManager(Integer.valueOf(order.getRegionId()), alarmType);
                        camsAccounts.stream().forEach(item -> {
                            systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单" + type + "签到超出指定范围(实际距离：" + distance + "米)", item.getAccountId(), order.getId(),
                                    0, order.getStatus(), alarmType.getType());
                            if (!StringUtils.isEmpty(item.getUserEmail()))
                                pushService.sendEmailAsync(item.getUserEmail(), "订单消息通知",
                                        "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单" + type + "签到超出指定范围(实际距离：" + distance + "米)，请及时查看</p><p>本邮件是系统自动发送，无需直接回复</p>");
                        });

                        //抄送指定邮箱（配置）
                        List<InfoConfig> mailList = infoConfigDao.selectValidNameType(alarmType.getType(), null);
                        mailList.stream().forEach(item -> {
                            if (!StringUtils.isEmpty(item.getName())) {
                                pushService.sendEmailAsync(item.getName(), "订单消息通知",
                                        "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单" + type + "签到超出指定范围(实际距离：" + distance + "米)，请及时查看</p><p>本邮件是系统自动发送，无需直接回复</p>");
                            }
                        });
                    }
                }
            }
        }
    }

    /**
     * 添加备注
     *
     * @param orderId
     * @param remark
     */
    public void addRemark(String orderId, String remark, String nextTime, String tag) {
        OperateLog operateLog = new OperateLog();
        operateLog.setThirdPartyId(orderId);
        operateLog.setValue(remark);
        operateLog.setType(OperateLogType.ORDER_REMARK.getType());
        JSONObject obj = new JSONObject();
        obj.set("nextTime", nextTime);
        obj.set("tag", tag);
        operateLog.setDesc(obj.toString());
        operateLogDao.insert(operateLog);
    }

    /**
     * 查询订单备注
     *
     * @param orderId
     * @return
     */
    public List<OperateLog> getRemark(String orderId) {
        OperateLogBo condition = new OperateLogBo();
        condition.setType(OperateLogType.ORDER_REMARK.getType());
        condition.setThirdPartyId(orderId);
        return operateLogDao.selectByExample(condition);
    }

    /**
     * 关闭订单
     *
     * @param orderId
     * @param closeReason
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void closeOrder(String orderId, String closeReason) {
        Order updateOrder = new Order();
        updateOrder.setStatus(OrderStatus.CLOSED.getType());
        updateOrder.setCloseOrderReason(closeReason);
        updateOrder.setCloseTime(new Date());
        orderDao.updateById(orderId, updateOrder);

        // 添加操作日志
        operateLogDao.insertOrderLog(orderId, OrderStatus.CLOSED.getType(), "关闭订单", closeReason);

        // 2020-01-04注释
        // Order order = orderDao.selectById(orderId);
        // if (order != null) {
        // OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
        // if (orderInstall != null && !StringUtils.isEmpty(orderInstall.getWallboxItemId())
        // && !StringUtils.isEmpty(order.getWarehouseId())
        // && order.getStatus().intValue() != OrderStatus.INSTALL_EXAMINE_SUCCESS.getType()
        // && order.getStatus().intValue() != OrderStatus.RETURN_VISIT.getType()) {
        // // 解锁仓库充电桩
        // warehouseService.materailNumberUpdate(orderInstall.getWallboxItemId(), 1, 2, order.getWarehouseId());
        // }
        // }
    }

    /**
     * 报装
     *
     * @param orderId
     * @param expandStatus
     * @param expandFailReason
     * @param expandFailOperate
     */
    public void expand(String orderId, Integer expandStatus, String expandFailReason, Integer expandFailOperate) {
        Order updateOrder = new Order();
        // if (expandStatus != null && expandStatus.intValue() == 2 && expandFailOperate != null &&
        // expandFailOperate.intValue() == 2) {
        // updateOrder.setStatus(OrderStatus.CLOSED.getType());
        // }
        updateOrder.setExpandStatus(expandStatus);
        updateOrder.setExpandFailReason(expandFailReason);
        updateOrder.setExpandFailOperate(expandFailOperate);
        orderDao.updateById(orderId, updateOrder);

        OperateLog operateLog = new OperateLog();
        operateLog.setType(OperateLogType.EXPAND.getType());
        operateLog.setThirdPartyId(orderId);
        operateLog.setName(expandStatus.toString());
        operateLog.setValue("报装");
        operateLog.setDesc(expandStatus.intValue() == 1 ? "报装通过" : "报装不通过");
        operateLogDao.insert(operateLog);

        Order order = orderDao.selectById(orderId);
        if (order != null) {
            Integer installSupplierId = order.getInstallSupplierId();
            SystemInfo systemInfo = new SystemInfo();
            systemInfo.setTitle("报装结果通知");
            systemInfo.setContent("您有一笔订单报装" + (expandStatus.intValue() == 1 ? "通过" : "不通过"));
            systemInfo.setThirdPartyId(orderId);
            systemInfo.setType(1);
            Supplier supplier = supplierDao.selectById(installSupplierId);
            systemInfo.setAccountId(supplier.getAccountId());
            systemInfoDao.insert(systemInfo);

            Integer installStaffId = order.getInstallStaffId();
            if (installStaffId != null) {
                SystemInfo info = new SystemInfo();
                info.setTitle("报装结果通知");
                info.setContent("您有一笔订单报装" + (expandStatus.intValue() == 1 ? "通过" : "不通过"));
                info.setThirdPartyId(orderId);
                info.setType(1);
                SupplierStaff staff = supplierStaffDao.selectById(installStaffId);
                info.setAccountId(staff.getAccountId());
                systemInfoDao.insert(info);
            }
        }
    }

    /**
     * 预约勘测
     *
     * @param orderId
     * @param surveyReserveTime
     * @param surveyClientName
     * @param surveyClientPhone
     * @param surveyCamsPhone
     */
    public void reserveSurvey(String orderId, Date surveyReserveTime, String surveyClientName, String surveyClientPhone,
                              String surveyCamsPhone, String surveyReserveRemark, int surveyReserveStatus, Date followDate) {
        OrderVo order = orderDao.selectDetailById(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        // 是否是修改
        boolean isModify = true;
        if (StringUtils.isEmpty(order.getSurveyClientPhone())) {
            isModify = false;
        }
        Order updateOrder = new Order();
        updateOrder.setSurveyClientName(surveyClientName);
        updateOrder.setSurveyClientPhone(surveyClientPhone);
        updateOrder.setSurveyCamsPhone(surveyCamsPhone);
        updateOrder.setSurveyReserveRemark(surveyReserveRemark);
        //预约成功
        if (surveyReserveStatus == 1) {
            updateOrder.setSurveyReserveTime(surveyReserveTime);
            if (order.getStatus().intValue() == OrderStatus.SURVEY_RESERVING.getType())
                updateOrder.setStatus(OrderStatus.SURVEY_RESERVED.getType());
            orderDao.updateById(orderId, updateOrder);
            //将预约勘测下次跟进时间设置为null
            orderDao.clearSurveyFollowTime(orderId);
            Map<String, Object> descMap = new HashMap<>(8);
            descMap.put("reserveTime", surveyReserveTime);
            descMap.put("clientName", surveyClientName);
            descMap.put("clientPhone", surveyClientPhone);
            descMap.put("camsPhone", surveyCamsPhone);
            operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVED.getType(), isModify ? "修改预约勘测信息" : "预约勘测",
                    JSONUtilsEx.serialize(descMap));

            // 发短信
            {
                if (!isModify) {
                    // pushService.sendSurveyReserveSaveSucess(surveyClientPhone, surveyClientName);

                    Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
                    if (supplier != null && supplier.getAccountId() != null)
                        systemInfoDao.insertForOrder("预约勘测信息录入成功", supplier.getAccountId(), orderId, 1);
                } else {
                    order.setSurveyReserveTime(surveyReserveTime);
                    pushService.sendSurveyReserveModify(order);
                }
            }
        } else {
            //预约失败
            if (order.getStatus().intValue() == OrderStatus.SURVEY_RESERVING.getType()) {
                updateOrder.setSurveyFollowTime(followDate);
                orderDao.updateById(orderId, updateOrder);
            } else {
                throw new ServiceException("当前状态不能预约勘察");
            }

        }

    }

    /**
     * 修改勘测备注
     *
     * @param orderId
     * @param remark
     */
    public void updateSurveyRemark(String orderId, String remark) {
        Order updateOrder = new Order();
        updateOrder.setSurveyReserveRemark(remark);
        orderDao.updateById(orderId, updateOrder);
    }

    /**
     * 预约安装
     *
     * @param orderId
     * @param installReserveTime
     * @param installClientName
     * @param installClientPhone
     * @param installCamsPhone
     */
    public void reserveInstall(String orderId, Date installReserveTime, String installClientName, String installClientPhone,
                               String installCamsPhone, String installReserveRemark, int installReserveStatus, Date followDate) {
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        // 是否是修改
        boolean isModify = true;
        if (StringUtils.isEmpty(order.getInstallClientPhone())) {
            isModify = false;
        }
        Order updateOrder = new Order();
        updateOrder.setInstallClientName(installClientName);
        updateOrder.setInstallClientPhone(installClientPhone);
        updateOrder.setInstallCamsPhone(installCamsPhone);
        updateOrder.setInstallReserveRemark(installReserveRemark);
        if (installReserveStatus == 1) {
            updateOrder.setInstallReserveTime(installReserveTime);
            updateOrder.setStatus(OrderStatus.INSTALL_RESERVED.getType());
            orderDao.updateById(orderId, updateOrder);
            //将预约安装下次跟进时间设置为null
            orderDao.clearInstallFollowTime(orderId);
            Map<String, Object> descMap = new HashMap<>(8);
            descMap.put("reserveTime", installReserveTime);
            descMap.put("clientName", installClientName);
            descMap.put("clientPhone", installClientPhone);
            descMap.put("camsPhone", installCamsPhone);
            operateLogDao.insertOrderLog(orderId, OrderStatus.INSTALL_RESERVED.getType(), isModify ? "修改预约安装信息" : "预约安装",
                    JSONUtilsEx.serialize(descMap));

            // 发短信
            {
                OrderVo orderVo = orderDao.selectDetailById(orderId);
                if (!isModify) {
                    pushService.sendIntallReserveSaveSuccess(orderVo);

                    Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
                    if (supplier != null && supplier.getAccountId() != null)
                        systemInfoDao.insertForOrder("预约安装信息录入成功", supplier.getAccountId(), orderId, 1);
                } else {
                    pushService.sendIntallReserveModify(orderVo);
                }
            }
        } else {
            updateOrder.setInstallFollowTime(followDate);
            orderDao.updateById(orderId, updateOrder);
        }

    }

    /**
     * 修改安装备注
     *
     * @param orderId
     * @param remark
     */
    public void updateInstallRemark(String orderId, String remark) {
        Order updateOrder = new Order();
        updateOrder.setInstallReserveRemark(remark);
        orderDao.updateById(orderId, updateOrder);
    }

    /**
     * 勘测完成审批
     *
     * @param orderId
     * @param status
     * @param surveyFinishFailReason
     */
    public void surveyFinishExamine(String orderId, Integer status, String surveyFinishFailReason) {
        Order updateOrder = new Order();
        if (status.intValue() == 0) {
            updateOrder.setStatus(OrderStatus.SURVEY_RECTIFYING.getType());
            updateOrder.setSurveyFinishFailReason(surveyFinishFailReason);
            operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RECTIFYING.getType(), "勘测完成审批不通过", surveyFinishFailReason);

            // 推送给安装服务商
            Order order = orderDao.selectById(orderId);
            if (order != null && order.getInstallSupplierId() != null) {
                Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
                if (supplier != null && supplier.getAccountId() != null) {
                    systemInfoDao.insertForOrder("勘测完成审批不通过", supplier.getAccountId(), orderId, 1);
                }
            }
        } else {
            updateOrder.setStatus(OrderStatus.SURVEY_EXAMINE_SUCCESS.getType());
            updateOrder.setSurveyApproveTime(new Date());
            operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_EXAMINE_SUCCESS.getType(), "勘测审批通过");
        }
        orderDao.updateById(orderId, updateOrder);
    }

    public void surveyRectifyExamine(String orderId, Integer status, String surveyRectifyFailReason) {
        Order updateOrder = new Order();
        if (status.intValue() == 0) {
            updateOrder.setStatus(OrderStatus.SURVEY_RECTIFYING.getType());
        } else {
            updateOrder.setStatus(OrderStatus.SURVEY_EXAMINE_SUCCESS.getType());
        }
        updateOrder.setSurveyRectifyFailReason(surveyRectifyFailReason);
        orderDao.updateById(orderId, updateOrder);
    }

    /**
     * 安装变更审批
     *
     * @param orderId
     * @param status
     * @param installChangeFailReason
     */
    public void installChangeExamine(String orderId, Integer status, String installChangeFailReason) {
        Order updateOrder = new Order();
        if (status.intValue() == 0) {
            updateOrder.setStatus(OrderStatus.INSTALL_CHANGE_EXAMINE_FAIL.getType());
            updateOrder.setInstallChangeFailReason(installChangeFailReason);
            operateLogDao.insertOrderLog(orderId, OrderStatus.INSTALL_CHANGE_EXAMINE_FAIL.getType(), "安装变更审批不通过", installChangeFailReason);
        } else {
            updateOrder.setStatus(OrderStatus.INSTALL_CHANGE_EXAMINE_SUCCESS.getType());
            operateLogDao.insertOrderLog(orderId, OrderStatus.INSTALL_CHANGE_EXAMINE_SUCCESS.getType(), "安装变更审批通过");
        }
        orderDao.updateById(orderId, updateOrder);
    }

    /**
     * 安装完成审批
     *
     * @param orderId
     * @param status
     * @param installFinishFailReason
     */
    public void installFinishExamine(String orderId, Integer status, String installFinishFailReason) {
        Order updateOrder = new Order();
        Order order = orderDao.selectById(orderId);
        if (status.intValue() == 0) {
            updateOrder.setStatus(OrderStatus.INSTALL_RECTIFYING.getType());
            updateOrder.setInstallFinishFailReason(installFinishFailReason);
            operateLogDao.insertOrderLog(orderId, OrderStatus.INSTALL_RECTIFYING.getType(), "安装完成审批不通过", installFinishFailReason);

            // 推送给安装服务商
            if (order != null && order.getInstallSupplierId() != null) {
                Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
                if (supplier != null && supplier.getAccountId() != null) {
                    systemInfoDao.insertForOrder("安装完成审批不通过", supplier.getAccountId(), orderId, 1);
                }
            }
        } else {
            updateOrder.setStatus(OrderStatus.INSTALL_EXAMINE_SUCCESS.getType());
            // 安装完成审批时间
            updateOrder.setInstallApproveTime(new Date());
            operateLogDao.insertOrderLog(orderId, OrderStatus.INSTALL_EXAMINE_SUCCESS.getType(), "安装完成审批通过");

            order.setInstallApproveTime(updateOrder.getInstallApproveTime());
            ResponseMessage2<SapStatisticsBO> sapStatisticsBOResponseMessage2 = sapRemoteService.pushOrder(order, null);
            if (sapStatisticsBOResponseMessage2.getStatus() == -1) {
                logger.error("sap订单" + orderId + "状态推送失败,失败原因：{}", sapStatisticsBOResponseMessage2.getMessage());
                //throw new ServiceException("sap订单状态推送失败，请稍后重试");
            }

            // 判断客户是否已评价当前订单, 如果已评价，则订单状态直接变更已回访
            if (orderScoreService.isOrderScore(orderId)) {
                updateOrder.setStatus(OrderStatus.RETURN_VISIT.getType());
            }

            // OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
            // 添加出库单 (2021-01-05注释)
            // if (orderInstall != null && !StringUtils.isEmpty(orderInstall.getWallboxItemId())
            // && !StringUtils.isEmpty(order.getWarehouseId()))
            //TODO 添加对墙盒品牌是否结算的状态判断，结算的才需要出库

            // warehouseService.addOutSingle(orderId, orderInstall.getWallboxItemId(), order.getWarehouseId(), 1);

            // 发短信
            if (order != null && !StringUtils.isEmpty(order.getInstallCamsPhone()) && !StringUtils.isEmpty(order.getInstallClientName()))
                pushService.sendIntallApproveFinish(order.getInstallClientPhone(), order.getInstallClientName());

            // 解绑隐私号码
            try {
                spService.unbind(order.getUserPhone());
            } catch (Exception e) {
                logger.error("解绑隐私号码失败，message:{}", e.getMessage());
            }
        }
        int result = orderDao.updateById(orderId, updateOrder);

    }


    /**
     * 填写预约异常内容
     *
     * @param orderId
     * @param content
     */
    public void reserveException(String orderId, String content) {
        Order order = orderDao.selectById(orderId);
        Assert.notNull(order, "订单不存在");
        if (order.getStatus().intValue() != OrderStatus.SURVEY_RESERVING.getType()
                && order.getStatus().intValue() != OrderStatus.SURVEY_EXAMINE_SUCCESS.getType()) {
            throw new ServiceException("订单状态错误");
        }
        Order updateOrder = new Order();
        if (order.getStatus().intValue() == OrderStatus.SURVEY_RESERVING.getType()) {
            updateOrder.setSurveyReserveFailInfo(content);
            operateLogDao.insertOrderLog(orderId, OrderStatus.SURVEY_RESERVE_EXCEPTION.getType(), "填写勘测预约异常", content);
        } else {
            updateOrder.setInstallReserveFailInfo(content);
            operateLogDao.insertOrderLog(orderId, OrderStatus.INSTALL_RESERVE_EXCEPTION.getType(), "填写安装预约异常", content);
        }
        orderDao.updateById(orderId, updateOrder);

    }

    public void backToManuDispatch(String orderId) {
        orderDao.updateStatusById(orderId, OrderStatus.DISPATCHING.getType());
    }

    /**
     * 查询维保订单列表
     *
     * @return
     */
    public List<WarrantyOrder> getWarrantyOrders(WarrantyOrderBo condition) {
        return orderDao.selectWarrantyOrders(condition);
    }

    /**
     * 评分变化时判断是否发送提醒
     *
     * @param orderId
     * @param score
     */
    @Async
    public void doWhenScoreUpdate(String orderId, double score) {
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            return;
        }
        List<Dictionary> dic = dicService.selectValidByType("timing", OldTimingConfig.CLIENT_RATING.getType() + "");
        if (dic.isEmpty()) {
            return;
        }
        Double configScore = Double.valueOf(dic.get(0).getValue());
        if (configScore != null && score < configScore) {
            List<CamsAccountVo> camsAccounts = camsAccountService.getProjectManager(Integer.valueOf(order.getRegionId()), AlarmType.CLIENT_RATING);
            camsAccounts.stream().forEach(item -> {
                systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单评分较低", item.getAccountId(), order.getId(), 0);
                if (!StringUtils.isEmpty(item.getUserEmail())) {
                    pushService.sendEmailAsync(item.getUserEmail(), "订单消息通知",
                            "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单评分较低</p><p>本邮件是系统自动发送，无需直接回复</p>");
                }
            });

            List<InfoConfig> mailList206 = infoConfigDao.selectValidNameType(AlarmType.CLIENT_RATING.getType(), null);
            //抄送指定邮箱（配置）
            mailList206.stream().forEach(item -> {
                if (!StringUtils.isEmpty(item.getName())) {
                    pushService.sendEmailAsync(item.getName(), "订单消息通知",
                            "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单评分较低</p><p>本邮件是系统自动发送，无需直接回复</p>");
                }
            });
        }
    }

    /**
     * 保存评分
     *
     * @param orderId
     * @param visitContent
     * @param files
     * @param list
     * @param knowAfterSalesChannel
     * @param installLength
     * @return
     */
    public BaseResponse<Object> saveOrderScore(String orderId, String visitContent, String files, String list, Integer knowAfterSalesChannel, Integer installLength) {
        if (StringUtils.isEmpty(orderId)) {
            throw new ServiceException("缺少orderId参数");
        }
        if (knowAfterSalesChannel == null) {
            throw new ServiceException("缺少knowAfterSalesChannel参数");
        }
        if (knowAfterSalesChannel != 0 && knowAfterSalesChannel != 1) {
            throw new ServiceException("knowAfterSalesChannel参数有效值为0, 1");
        }
        if (installLength == null) {
            throw new ServiceException("缺少installLength参数");
        }
        Integer[] items = {1, 2, 3, 4, 5};
        if (!Arrays.asList(items).contains(installLength)) {
            throw new ServiceException("installLength参数有效值为:1, 2, 3, 4, 5");

        }
        //更新评分明细
        JSONArray jsonArray = JSONUtil.parseArray(list);
        if (jsonArray.size() > 0) {
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                OrderScore orderScore = new OrderScore();
                orderScore.setId(Integer.parseInt(jsonObject.get(LambdaUtils.getFieldName(OrderScore::getId)) + ""));
                orderScore.setScore(Double.parseDouble(jsonObject.get(LambdaUtils.getFieldName(OrderScore::getScore)) + ""));
                orderScore.setDescription(jsonObject.get(LambdaUtils.getFieldName(OrderScore::getDescription)) + "");
                orderScoreService.update(orderScore);
            }
        }
        if (!StringUtils.isEmpty(files)) {
            files = StringUtilsEx.unescapeXss(files);
        }
        //增加已回访的状态
        visitOrder(orderId, visitContent, files, jsonArray, knowAfterSalesChannel, installLength);
        return new BaseResponse<>();
    }

    /**
     * 订单回访
     * modify 2022-11-06 回访已经不是订单流程中状态了
     * if (OrderStatus.INSTALL_EXAMINE_SUCCESS.getType() == oldOrder.getStatus()) {
     * updateOrder.setStatus(OrderStatus.RETURN_VISIT.getType());
     * }
     *
     * @param orderId
     * @param visitContent
     */
    public void visitOrder(String orderId, String visitContent, String files, JSONArray jsonArray, Integer knowAfterSalesChannel, Integer installLength) {
        Order oldOrder = orderDao.selectById(orderId);
        Assert.notNull(oldOrder, "订单不存在");
        Order updateOrder = new Order();
        updateOrder.setId(orderId);
        if (!StringUtils.isEmpty(visitContent)) {
            updateOrder.setVisitContent(visitContent);
        }
        if (!StringUtils.isEmpty(files)) {
            updateOrder.setFiles(files);
        }
        updateOrder.setKnowAfterSalesChannel(knowAfterSalesChannel);
        updateOrder.setInstallLength(installLength);
        updateOrder.setReturnVisitTime(new Date());
        orderDao.updateById(orderId, updateOrder);

        operateLogDao.insertOrderLog(orderId, OrderStatus.RETURN_VISIT.getType(), "订单回访", visitContent);
        double totalScore = 0;
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            Double score = Double.valueOf(String.valueOf(jsonObject.get("score")));
            if (score != null) {
                totalScore += score;
            }
        }
        double averScore = totalScore / jsonArray.size();
        doWhenScoreUpdate(orderId, averScore);
    }

    /**
     * 更新车架号
     *
     * @param orderId
     * @param vehicleVin
     */
    public void updateVehicleVin(String orderId, String vehicleVin) {
        Order updateOrder = new Order();
        updateOrder.setVehicleVin(vehicleVin);
        orderDao.updateById(orderId, updateOrder);
    }

    /**
     * 变更安装提交审核
     *
     * @param orderId
     * @param fileNames
     * @param fileUrls
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void installChange(String orderId, List<String> fileNames, List<String> fileUrls) {
        Order updateOrder = new Order();
        updateOrder.setStatus(OrderStatus.INSTALL_CHANGE_EXAMINING.getType());
        orderDao.updateById(orderId, updateOrder);

        List<UploadFile> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(fileUrls)) {
            for (int i = 0; i < fileUrls.size(); i++) {
                UploadFile file = new UploadFile();
                file.setUrl(fileUrls.get(i));
                file.setName(fileNames.get(i));
                file.setThirdPartyId(orderId);
                file.setType(UploadFileType.INSTALL_CHANGE.getType());
                list.add(file);
            }
            uploadFileDao.batchInsert(list);
        }

        operateLogDao.insertOrderLog(orderId, OrderStatus.INSTALL_CHANGE_EXAMINING.getType(), "安装变更，提交审核", "");

        notifyRegionPM("您有一笔安装变更订单待审批", orderId, 1, null);
    }

    /**
     * 查看订单详情
     *
     * @param orderId
     * @return
     */
    public Map<String, Object> getMapDetail(String orderId) {
        OrderVo vo = getDetail(orderId);
        if (vo == null) {
            return new HashMap<>(1);
        }
        Map<String, Object> map = BeanUtil.beanToMap(vo);
        // modify注释掉调用二期接口代码
//        Map<String, Map<Object, String>> enumMap = flowOrderService.createEnumMap();
//        String prefix = "Str";
//        Iterator<Map.Entry<String, Map<Object, String>>> iterator = enumMap.entrySet().iterator();
//        while (iterator.hasNext()) {
//            Map.Entry<String, Map<Object, String>> entry = iterator.next();
//            if (map.containsKey(entry.getKey())) {
//                String enumValue = entry.getValue().get(map.get(entry.getKey()));
//                map.put(entry.getKey() + prefix, enumValue);
//            }
//        }
//        OrderSurveyPO surveyPO = flowOrderService.getOrderSurvey(orderId);
        //添加勘测数据
        OrderSurvey orderSurvey = orderSurveyDao.selectByOrderId(orderId, 1);

        if (orderSurvey == null) {
            orderSurvey = new OrderSurvey();
        }
        map.put(LambdaUtils.getFieldName(OrderSurveyPO::getSurveyDate), DateUtil.format(orderSurvey.getSurveyDate(), DatePattern.NORM_DATE_PATTERN));
        return map;
    }

    /**
     * 查询订单详情
     *
     * @param orderId
     * @return
     */
    public OrderVo getDetail(String orderId) {
        OrderVo order = orderDao.selectDetailById(orderId);
        if (order == null) {
            return null;
        }
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.INSTALL_CHANGE.getType(), orderId);
        order.setInstallChangeFiles(files);

        List<UploadFile> attachFiles = uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.ORDER.getType(), orderId);
        order.setAttachFiles(attachFiles);

        if (StrUtil.isNotBlank(order.getConnectPerson())) {
            InfoConfig infoConfig = infoConfigDao.selectByPrimaryKey(order.getConnectPerson());
            if (infoConfig != null) {
                order.setConnectPersonName(infoConfig.getName());
            }
        }
        return order;
    }

    public Order get(String orderId) {
        return orderDao.selectById(orderId);
    }

    /**
     * 订单附件上传
     *
     * @param orderId
     * @param fileNames
     * @param fileUrls
     */
    public void uploadFile(String orderId, List<String> fileNames, List<String> fileUrls) {
        List<UploadFile> files = new ArrayList<>();
        for (int i = 0; i < fileUrls.size(); i++) {
            UploadFile file = new UploadFile();
            file.setUrl(fileUrls.get(i));
            file.setName(fileNames.get(i));
            file.setThirdPartyId(orderId);
            file.setType(UploadFileType.ORDER.getType());
            files.add(file);
        }
        uploadFileDao.batchInsert(files);
    }

    /**
     * 查询附件
     *
     * @param orderId
     * @return
     */
    public List<UploadFile> getFiles(String orderId) {
        return uploadFileDao.selectValidByTypeAndThirdPartyId(UploadFileType.ORDER.getType(), orderId);
    }

    public List<OrderVo> selectByCamsPhone(String camsPhone) {
        return orderDao.selectByCamsPhone(camsPhone);
    }

    public int countByCamsPhone(String camsPhone) {
        return orderDao.countByCamsPhone(camsPhone);
    }

    public List<OrderVo> selectOwnOrder(OrderBo condition) {
        return orderDao.selectOwnOrder(condition);
    }

    public OrderVo selectOwnOrderById(String orderId) {
        Assert.notEmpty(orderId, WiosConstant.EMPTY_ORDER_ID);

        OrderBo condition = new OrderBo();
        condition.setId(orderId);
        List<OrderVo> list = selectOwnOrder(condition);
        if (list != null && list.size() > 0) {
            OrderVo order = list.get(0);
            List<Dictionary> dic = dicService.selectValidByType("timing", OldTimingConfig.RESERVE_EXCEPTION.getType() + "");
            if (!dic.isEmpty()) {
                setShowReserveExpBtn(order, dic);
            }
            return order;
        }
        return null;
    }

    public void notifyRegionPM(String msg, String orderId, Integer regionId, int type, AlarmType alarmType) {
        if (StringUtils.isEmpty(orderId) || regionId == null) {
            return;
        }
        List<CamsAccountVo> camsAccountList = camsAccountService.getProjectManager(regionId, alarmType);
        for (CamsAccountVo account : camsAccountList) {
            systemInfoDao.insertForOrder(msg, account.getAccountId(), orderId, type);
        }
    }

    public void notifyRegionPM(String msg, String orderId, int type, AlarmType alarmType) {
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            return;
        }
        notifyRegionPM(msg, orderId, Integer.valueOf(order.getRegionId()), type, alarmType);
    }

    /**
     * @return void
     * @Description 管理员删除订单功能
     * @Param [orderId]
     * @author liwenxiang
     * @date 2021/7/28 10:31
     */
    @Transactional(rollbackFor = Exception.class)
    public void deleteOrder(String orderId) {
        Assert.notEmpty(orderId, WiosConstant.EMPTY_ORDER_ID);
        Order order = orderDao.selectById(orderId);
        if (ObjectUtils.isEmpty(order)) {
            throw new ServiceException(WiosConstant.EMPTY_ORDER_TYPE);
        }
        //备份数据到t_order_delete，删除t_order表数据
        OrderDelete orderDelete = new OrderDelete();
        BeanUtils.copyProperties(order, orderDelete);
        orderDeleteDao.insertSelective(orderDelete);
        orderDao.delete(order.getId());
        //备份数据到t_order_install_delete，删除t_order_install表数据
        OrderInstall orderInstall = orderInstallDao.selectByOrderId(order.getId());
        if (!ObjectUtils.isEmpty(orderInstall)) {
            OrderInstallDelete orderInstallDelete = new OrderInstallDelete();
            BeanUtils.copyProperties(orderInstall, orderInstallDelete);
            orderInstallDeleteDao.insertSelective(orderInstallDelete);
            orderInstallDao.deleteById(orderInstall.getId());
        }
        //备份数据到t_order_install_item_delete，删除t_order_install_item表数据
        List<OrderInstallItem> orderInstallItemList = orderInstallItemDao.selectByOrderId(order.getId());
        if (!CollectionUtils.isEmpty(orderInstallItemList)) {
            List<OrderInstallItemDelete> orderInstallItemDeletes = new ArrayList<>();
            for (OrderInstallItem ord : orderInstallItemList) {
                OrderInstallItemDelete orderInstallItemDelete = new OrderInstallItemDelete();
                BeanUtils.copyProperties(ord, orderInstallItemDelete);
                orderInstallItemDeletes.add(orderInstallItemDelete);
                orderInstallItemDao.deleteById(ord.getId());
            }
            orderInstallItemDeleteDao.batchInsert(orderInstallItemDeletes);
        }
        //备份数据到t_order_survey_delete，删除t_order_survey表数据
        List<OrderSurvey> orderSurveyList = orderSurveyDao.selectListByOrderId(order.getId());
        if (!CollectionUtils.isEmpty(orderSurveyList)) {
            List<OrderSurveyDelete> orderSurveyDeleteList = new ArrayList<>();
            for (OrderSurvey orderSurvey : orderSurveyList) {
                OrderSurveyDelete orderSurveyDelete = new OrderSurveyDelete();
                BeanUtils.copyProperties(orderSurvey, orderSurveyDelete);
                orderSurveyDeleteList.add(orderSurveyDelete);
                orderSurveyDao.deleteById(orderSurvey.getId());
            }
            orderSurveyDeleteDao.batchInsert(orderSurveyDeleteList);
        }
        //备份数据到t_user_pay_bill_delete，删除t_user_pay_bill表数据
        UserPayBill userPayBill = new UserPayBill();
        userPayBill.setRefId(order.getId());
        List<UserPayBill> list = userPayBillDao.list(userPayBill);
        if (!CollectionUtils.isEmpty(list)) {
            for (UserPayBill payBill : list) {
                UserPayBillDelete userPayBillDelete = new UserPayBillDelete();
                BeanUtils.copyProperties(payBill, userPayBillDelete);
                userPayBillDeleteDao.add(userPayBillDelete);
                userPayBillDao.deleteById(payBill.getId());
            }
        }
        //备份数据到t_system_info_delete，删除t_system_info表数据
        List<SystemInfo> systemInfoList = systemInfoDao.selectListByOrderId(order.getId());
        if (!CollectionUtils.isEmpty(systemInfoList)) {
            List<SystemInfoDelete> systemInfoDeleteList = new ArrayList<>();
            for (SystemInfo systemInfo : systemInfoList) {
                SystemInfoDelete systemInfoDelete = new SystemInfoDelete();
                BeanUtils.copyProperties(systemInfo, systemInfoDelete);
                systemInfoDeleteList.add(systemInfoDelete);
            }
            systemInfoDao.deleteByOrderId(order.getId());
            systemInfoDeleteDao.batchInsert(systemInfoDeleteList);
        }
        //备份数据到t_operate_log_delete，删除t_operate_log表数据
        OperateLogBo condition = new OperateLogBo();
        condition.setThirdPartyId(order.getId());
        List<OperateLog> operateLogList = operateLogDao.selectByExample(condition);
        if (!CollectionUtils.isEmpty(operateLogList)) {
            List<OperateLogDelete> systemInfoDeleteList = new ArrayList<>();
            for (OperateLog operateLog : operateLogList) {
                OperateLogDelete operateLogDelete = new OperateLogDelete();
                BeanUtils.copyProperties(operateLog, operateLogDelete);
                systemInfoDeleteList.add(operateLogDelete);
            }
            operateLogDao.deleteByOrderId(order.getId());
            operateLogDeleteDao.batchInsert(systemInfoDeleteList);
        }

        //将t_settlement_detail中is_del设为1（删除）
        settlementDetailDao.deleteByOrderId(order.getId());
        //将t_settlement_detail_supplier设为1（删除）
        settlementDetailSupplierDao.deleteByOrderId(order.getId());

    }

    public void reserveOverFollowTimeNotice() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -1);
        Date date = cal.getTime();
        SimpleDateFormat startSdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        SimpleDateFormat endSdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");

        //查询预约勘测超时订单
        List<Order> surveyOrderList = orderDao.selectSurveyPeriodOrders(startSdf.format(date), endSdf.format(date));
        List<InfoConfig> mailList = infoConfigDao.selectValidNameType(AlarmType.NEXT_SURVEY.getType(), null);
        for (Order order : surveyOrderList) {
            systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单等待预约勘测，请及时跟进", 0, order.getId(),
                    0, order.getStatus(), AlarmType.NEXT_SURVEY.getType());
            systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单等待预约勘测，请及时跟进", Integer.parseInt(order.getCreateAccount()), order.getId(),
                    0, order.getStatus(), AlarmType.NEXT_SURVEY.getType());
            List<CamsAccountVo> camsAccounts = camsAccountService.getProjectManager(Integer.valueOf(order.getRegionId()), AlarmType.NEXT_SURVEY);
            camsAccounts.stream().forEach(item -> {
                systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单等待预约勘测，请及时跟进", item.getAccountId(), order.getId(),
                        0, order.getStatus(), AlarmType.NEXT_SURVEY.getType());
                if (!StringUtils.isEmpty(item.getUserEmail())) {
                    pushService.sendEmailAsync(item.getUserEmail(), "订单消息通知",
                            "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单等待预约勘测，请及时跟进</p><p>本邮件是系统自动发送，无需直接回复</p>");
                }
            });

            //抄送指定邮箱（配置）
            mailList.stream().forEach(item -> {
                if (!StringUtils.isEmpty(item.getName())) {
                    pushService.sendEmailAsync(item.getName(), "订单消息通知",
                            "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单等待预约勘测，请及时跟进</p><p>本邮件是系统自动发送，无需直接回复</p>");
                }
            });
        }

        //查询预约安装超时订单
        mailList = infoConfigDao.selectValidNameType(AlarmType.NEXT_INSTALL.getType(), null);
        List<Order> installOrderList = orderDao.selectInstallPeriodOrders(startSdf.format(date), endSdf.format(date));
        for (Order order : installOrderList) {
            systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单等待预约安装，请及时跟进", 0, order.getId(),
                    0, order.getStatus(), AlarmType.NEXT_INSTALL.getType());
            systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单等待预约安装，请及时跟进", Integer.parseInt(order.getCreateAccount()), order.getId(),
                    0, order.getStatus(), AlarmType.NEXT_INSTALL.getType());

            List<CamsAccountVo> camsAccounts = camsAccountService.getProjectManager(Integer.valueOf(order.getRegionId()), AlarmType.NEXT_INSTALL);
            camsAccounts.stream().forEach(item -> {
                systemInfoDao.insertForOrder("订单号为" + order.getId() + "的订单等待预约安装，请及时跟进", item.getAccountId(), order.getId(),
                        0, order.getStatus(), AlarmType.NEXT_INSTALL.getType());
                if (!StringUtils.isEmpty(item.getUserEmail())) {
                    pushService.sendEmailAsync(item.getUserEmail(), "订单消息通知",
                            "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单等待预约安装，请及时跟进</p><p>本邮件是系统自动发送，无需直接回复</p>");
                }
            });

            //抄送指定邮箱（配置）
            mailList.stream().forEach(item -> {
                if (!StringUtils.isEmpty(item.getName())) {
                    pushService.sendEmailAsync(item.getName(), "订单消息通知",
                            "<p>您好：</p>" + "<p>订单号为" + order.getId() + "的订单等待预约安装，请及时跟进</p><p>本邮件是系统自动发送，无需直接回复</p>");
                }
            });
        }
    }

    /**
     * 手动第一次同步订单到sap
     */
    public ResponseMessage2<SapStatisticsBO> pushOrder(String orderId) {
        if (StringUtils.isEmpty(orderId)) {
            return ResponseMessage2.Failed("订单id不能为空");
        }
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            return ResponseMessage2.Failed("订单不存在");
        }
        if (OrderStatus.arePushSap(order.getStatus())) {
            return sapRemoteService.pushOrder(order, null);
        } else {
            return ResponseMessage2.Failed("同步失败，订单状态不对");
        }
    }


    /**
     * 只有个安装服务商=A的时候才传增项和高附加值材料  =B/C的时候只传安装服务费和墙盒
     */
    private final String PROVIDER_TYPE_A = "A";

    /**
     * sap同步增项
     */
    public ResponseMessage2<SapStatisticsBO> pushAddNewItems(String orderId) {
        if (StringUtils.isEmpty(orderId)) {
            return ResponseMessage2.Failed("订单id不能为空");
        }
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            return ResponseMessage2.Failed("订单不存在");
        }
        Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
        if (ObjectUtils.isEmpty(supplier)) {
            return ResponseMessage2.Failed("获取supplier失败");
        }
        if (!supplier.getLevel().equals(PROVIDER_TYPE_A)) {
            return ResponseMessage2.Success();
        }
        return sapRemoteService.pushAddNewItems(order, null, null);
    }


    /**
     * 手动第一次同步订单到sap---新增
     */
    public void pushNewOrder(String orderId, String newzbrandid, Integer contractId, String sapCode) {
        if (StringUtils.isEmpty(orderId)) {
            throw new ServiceException("订单id不能为空");
        }
        //修改订单安装的墙盒信息
        if (contractId != null && !StringUtils.isEmpty(sapCode)) {
            orderDao.updateInstallWallboxItem(orderId, newzbrandid);
        }
        //修改合同编号
        if (contractId != null) {
            Order orderForUpdate = new Order();
            orderForUpdate.setId(orderId);
            orderForUpdate.setContractId(contractId);
            orderMapper.updateDynamic(orderForUpdate);
        }

        Order order = orderDao.selectById(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        if (order.getStatus() == OrderStatus.INSTALL_EXAMINE_SUCCESS.getType() || order.getStatus() == OrderStatus.RETURN_VISIT.getType()) {
            ResponseMessage2<SapStatisticsBO> sapStatisticsBOResponseMessage2 = sapRemoteService.pushOrder(order, sapCode);
            if (sapStatisticsBOResponseMessage2.getStatus() == -1) {
                logger.error("sap订单状态推送失败,订单号：{}", order.getId());
                throw new ServiceException(sapStatisticsBOResponseMessage2.getMessage());
            }
        } else {
            throw new ServiceException("同步失败，订单状态不对");
        }
    }

    public List<ContractIdInfo> showContracts() {
        return orderMapper.selectContractEnum();
    }
}
