package com.boco.nbd.wios.manage.service.impl;

import com.bestvike.linq.Linq;
import com.boco.nbd.cams.core.config.RedisClient;
import com.boco.nbd.wios.flow.entity.vo.OrderInstallItemVO;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.UploadFileType;
import com.boco.nbd.wios.manage.entity.common.bo.UploadFile;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Objects;

/**
 * 勘察订单service
 *
 * @author xgl
 * @version [版本号, 2020年9月17日]
 */
@Service
public class OrderSurveyService {
    @Autowired
    private OrderSurveyDao orderSurveyDao;

    @Autowired
    private UploadFileDao uploadFileDao;

    @Autowired
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private OperateLogDao operateLogDao;

    @Autowired
    private OemContractService oemContractService;

    @Autowired
    private RedisClient redisClient;

    /**
     * 新增
     *
     * @param orderSurvey
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(OrderSurveyBo orderSurvey) {
        String lockId = "addOrderSurvey:" + orderSurvey.getOrderId() + "-" + orderSurvey.getType();
        String requestId = StringUtilsEx.getUUID();
        if (redisClient.lock(lockId, requestId, 60)) {
            try {
                OrderSurvey os = orderSurveyDao.selectByOrderId(orderSurvey.getOrderId(), orderSurvey.getType());
                if (os != null) {
                    throw new ServiceException("订单已存在");
                }
                List<OrderInstallItem> itemList = orderSurvey.getItemFees();
                if (itemList != null && !itemList.isEmpty()) {
                    itemList.stream().forEach(item -> {
                        item.setItemType(2);// 增项
                        if (orderSurvey.getType().intValue() == 1) {
                            item.setType(1);// 勘察订单
                        } else {
                            item.setType(3);// 变更安装订单
                        }
                        item.setOrderId(orderSurvey.getOrderId());
                    });
                    orderInstallItemDao.batchInsert(itemList);
                }
                orderSurveyDao.insert(orderSurvey);
                UploadFileType uploadFileType = orderSurvey.getType().intValue() == 1 ? UploadFileType.SURVEY_ORDER
                        : UploadFileType.INSTALL_CHANGE_ORDER;
                uploadFileDao.batchSave(orderSurvey.getFiles(), uploadFileType, orderSurvey.getId() + "");
                if (orderSurvey.getType() != null && orderSurvey.getType().intValue() == 1) {
                    // 勘测订单修改订单的墙盒型号和墙盒设备型号
                    Order order = new Order();
                    order.setSurveyWallboxModel(orderSurvey.getSurveyWallboxModel());
                    order.setSurveyItemModel(orderSurvey.getSurveyItemModel());
                    orderDao.updateById(orderSurvey.getOrderId(), order);
                }

                if (orderSurvey.getType() != null && orderSurvey.getType().intValue() == 2) {
                    // 安装变更
                    orderService.installChange(orderSurvey.getOrderId(), null, null);
                }
            } finally {
                redisClient.releaseLock(lockId, requestId);
            }
        } else {
            throw new ServiceException("系统忙，请稍后");
        }

    }

    /**
     * 修改
     *
     * @param id
     * @param orderSurvey
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(int id, OrderSurveyBo orderSurvey) {
        orderSurvey.setType(null);
        List<OrderInstallItem> itemList = orderSurvey.getItemFees();
        OrderSurvey os = orderSurveyDao.selectById(id);
        if (os == null) {
            throw new ServiceException("无此勘察单");
        }
        if (itemList != null && !itemList.isEmpty()) {
            itemList.stream().forEach(item -> {
                item.setItemType(2);// 增项
                if (os.getType().intValue() == 1) {
                    item.setType(1);// 勘察订单
                } else {
                    item.setType(3);// 变更安装订单
                }
                item.setOrderId(os.getOrderId());
            });
            orderInstallItemDao.delete(os.getOrderId(), os.getType().intValue() == 1 ? 1 : 3);
            orderInstallItemDao.batchInsert(itemList);
        }
        orderSurveyDao.update(id, orderSurvey);
        UploadFileType uploadFileType = os.getType().intValue() == 1 ? UploadFileType.SURVEY_ORDER : UploadFileType.INSTALL_CHANGE_ORDER;
        uploadFileDao.deleteByTypeAndThirdPartyId(uploadFileType.getType(), id + "");
        // 添加文件
        uploadFileDao.batchSave(orderSurvey.getFiles(), uploadFileType, id + "");

        if (os.getType() != null && os.getType().intValue() == 1) {
            // 勘测订单修改订单的墙盒型号和墙盒设备型号
            Order order = new Order();
            order.setSurveyWallboxModel(orderSurvey.getSurveyWallboxModel());
            order.setSurveyItemModel(orderSurvey.getSurveyItemModel());
            orderDao.updateById(os.getOrderId(), order);
        }

        if (os.getType() != null && os.getType().intValue() == 2) {
            // 安装变更
            orderService.installChange(os.getOrderId(), null, null);
        }
    }

    /**
     * 根据订单id查询
     *
     * @param orderId
     * @return
     */
    public OrderSurveyVo getByOrderId(String orderId, Integer type) {
        OrderSurvey orderSurvey = orderSurveyDao.selectByOrderId(orderId, type);
        if (orderSurvey == null) {
            return null;
        }
        OrderSurveyVo result = new OrderSurveyVo();
        BeanUtils.copyProperties(orderSurvey, result);

        Order order = orderDao.selectById(orderId);
        if (order == null) {
            return null;
        }
        result.copyOrder(order);

        List<UploadFile> files = uploadFileDao.selectValidByTypeAndThirdPartyId(type.intValue() == 1 ? UploadFileType.SURVEY_ORDER.getType() : UploadFileType.INSTALL_CHANGE_ORDER.getType(), orderId);
        result.setFiles(files);
        List<OrderInstallItemVO> itemFeeList = oemContractService.getOrderDetailItems(orderId, type.intValue() == 1 ? 1 : 3, order.getOemId());
        result.setItemFees(Linq.of(itemFeeList).where(s -> WiosConstant.FEE_TYPE.equals(s.getItemType())).toList());
        result.setItemHighAdd(Linq.of(itemFeeList).where(s -> WiosConstant.HIGH_ADD_TYPE.equals(s.getItemType())).toList());
        //加入上次审核不通过原因描述
        OperateLog operateLog = operateLogDao.getNewestCheckFailByOrderId(orderId, "28");
        if (Objects.nonNull(operateLog)) {
            result.setCheckFailDesc(operateLog.getDesc());
        } else {
            result.setCheckFailDesc("");
        }

        return result;
    }
}
