package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.DispatchConfig;
import com.boco.nbd.wios.manage.entity.bo.Region;
import com.boco.nbd.wios.manage.entity.bo.RegionDTO;
import com.boco.nbd.wios.manage.entity.bo.RegionVo;
import com.boco.nbd.wios.manage.mapper.def.RegionDao;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 区域管理service
 *
 * @author xgl
 * @version [版本号, 2020年9月21日]
 */
@Service
public class RegionService {
    @Autowired
    private RegionDao regionDao;

    public void add(Region region) {
        regionDao.insert(region);
    }

    public Region getById(int id) {
        return regionDao.selectById(id);
    }

    public List<Region> getByParentId(int parentId) {
        return regionDao.selectByParentId(parentId);
    }

    /**
     * 查询列表并组装数据
     *
     * @param status
     * @return
     */
    public List<RegionVo> selectAll(Integer status) {
        return selectAll(status, new ArrayList<>());
    }

    /**
     * 查询列表并组装数据
     *
     * @param status
     * @return
     */
    public List<RegionVo> selectAll(Integer status, List<Integer> selectedIds) {
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        List<Integer> regionIdList = null;
        if (sessionInfo != null) {
            String regionIds = sessionInfo.getAttribute().get("regionIds");
            if (!StringUtils.isEmpty(regionIds)) {
                regionIdList = new ArrayList<>();
                String[] arr = regionIds.split(CamsConstant.COMMA);
                for (String rId : arr) {
                    regionIdList.add(Integer.valueOf(rId));
                }
            }
        }
        List<Region> list = regionDao.selectAll(status, regionIdList);
        //modify 2022-11-11优化一下这部分效率 sql中 order by id_tree
        List<RegionVo> returnList = new ArrayList<>();
        for(Region region:list){
            if(CollUtil.isEmpty(returnList)){
                RegionVo regionVo = new RegionVo();
                BeanUtils.copyProperties(region, regionVo);
                returnList.add(regionVo);
                continue;
            }
            Integer parentId = region.getParentId();
            RegionVo regionVo = getRegionVo(returnList, parentId);
            if (regionVo == null) {
                regionVo = new RegionVo();
                BeanUtils.copyProperties(region, regionVo);
                returnList.add(regionVo);
                continue;
            }
            RegionVo regionVoNew = new RegionVo();
            if (selectedIds.contains(region.getId().intValue())) {
                regionVoNew.setChecked(true);
            }
            BeanUtils.copyProperties(region, regionVoNew);
            List<RegionVo> children = regionVo.getChildren();
            if (children == null) {
                children = new ArrayList<>();
            }
            children.add(regionVoNew);
            regionVo.setChildren(children);
        }
        return returnList;
    }

    public List<RegionVo> convertRegions(List<Region> list, Map<Integer, DispatchConfig> dispatchConfigMap){
        List<RegionDTO> dtoList = new ArrayList<>(8);
        for (Region item : list) {
            RegionDTO dto = new RegionDTO();
            BeanUtils.copyProperties(item, dto);
            String[] idTreeArr = item.getIdTree().split(CamsConstant.UNDER_LINE);
            if(idTreeArr.length > 4){
                continue;
            }
            dto.setIdLevelCount(idTreeArr.length);

            DispatchConfig dispatchConfig = dispatchConfigMap.get(item.getId());
            if(dispatchConfig != null){
                dto.setDispatchConfigId(dispatchConfig.getId());
                dto.setDispatchNum(dispatchConfig.getDispatchNum());
            }

            dtoList.add(dto);
        }
        // 排序
        dtoList = dtoList.stream().sorted(Comparator.comparing(RegionDTO::getIdLevelCount)).collect(Collectors.toList());

        List<RegionVo> returnList = new ArrayList<>();
        for (RegionDTO region : dtoList) {
            Integer parentId = region.getParentId();
            if (!returnList.isEmpty()) {
                RegionVo regionVo = getRegionVo(returnList, parentId);
                if (regionVo == null) {
                    regionVo = new RegionVo();
                    BeanUtils.copyProperties(region, regionVo);
                    returnList.add(regionVo);
                    continue;
                }
                RegionVo regionVoNew = new RegionVo();
                BeanUtils.copyProperties(region, regionVoNew);
                List<RegionVo> children = regionVo.getChildren();
                if (children == null) {
                    children = new ArrayList<>();
                }
                children.add(regionVoNew);
                regionVo.setChildren(children);
            } else {
                RegionVo regionVo = new RegionVo();
                BeanUtils.copyProperties(region, regionVo);
                returnList.add(regionVo);
            }
        }
        return returnList;
    }

    /**
     * 递归获取区域
     *
     * @param returnList
     * @param id
     * @return
     */
    public RegionVo getRegionVo(List<RegionVo> returnList, int id) {
        for (RegionVo regionVo : returnList) {
            if (regionVo.getId() == id) {
                return regionVo;
            }
            List<RegionVo> children = regionVo.getChildren();
            if (children != null) {
                RegionVo regionVo2 = getRegionVo(children, id);
                if (regionVo2 != null) {
                    return regionVo2;
                }
            }
        }
        return null;
    }

    public void update(int id, Region region) {
        if (region.getStatus() != null && region.getStatus().intValue() == 2) {
            List<Region> childRegionList = regionDao.selectByParentId(id);
            if (!childRegionList.isEmpty()) {
                throw new ServiceException("该区域存在下级，不可删除");
            }
        }
        regionDao.update(id, region);
    }
}
