package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.wios.manage.entity.bo.ReportOrderBo;
import com.boco.nbd.wios.manage.entity.bo.ReportOrderDTO;
import com.boco.nbd.wios.manage.entity.bo.ReportOrderResponseBo;
import com.boco.nbd.wios.manage.entity.bo.ReportOrderResponseDTO;
import com.boco.nbd.wios.manage.mapper.def.ReportDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 数据报表
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-27 14:08:14
 */
@Service
public class ReportService {

    @Autowired
    private ReportDao reportDao;

    /**
     * 安装进度等数据统计
     * @param reportOrderBo
     * @return
     */
    public List<ReportOrderDTO> installData(ReportOrderBo reportOrderBo){
        return reportDao.countOrderGroupBySupplier(reportOrderBo);
    }

    /**
     * 电桩安装报表数据统计
     * @param reportOrderBo
     * @return
     */
    public List<ReportOrderDTO> installDataTable(ReportOrderBo reportOrderBo){
        return reportDao.countOrderGroupBySupplierAndRegion(reportOrderBo);
    }

    /**
     * 安装效率统计
     * @param reportOrderBo
     * @return
     */
    public List<ReportOrderDTO> installEfficiency(ReportOrderBo reportOrderBo){
        List<ReportOrderDTO> list = reportDao.countOrderGroupBySupplier(reportOrderBo);
        List<ReportOrderDTO> timeList = reportDao.calcOrderFinishedTime(reportOrderBo);

        for(ReportOrderDTO reportOrderDTO : list){
            reportOrderDTO.setAvgInstallTime(0.0);
            reportOrderDTO.setFinishRatio(new BigDecimal(reportOrderDTO.getCountFinished())
                .divide(new BigDecimal(reportOrderDTO.getCountTotal()), 2, RoundingMode.HALF_UP).multiply(new BigDecimal(100))
                .doubleValue());
            for(ReportOrderDTO timeItme : timeList){
                if (timeItme != null && reportOrderDTO.getSupplierId().equals(timeItme.getSupplierId())) {
                    reportOrderDTO.setAvgInstallTime(timeItme.getAvgInstallTime());
                }
            }
        }
        return list;
    }


    /**
     * @Description 接单时效统计(管理员后台)
     * @Param [reportOrderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:53
     * @return java.util.List<com.starcharge.wios.dto.ReportOrderResponseDTO>
     */
    public List<ReportOrderResponseDTO> orderResponseToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        return reportDao.orderResponseToAdmin(reportOrderResponseBo);
    }

    public List<ReportOrderResponseDTO> surveyResponseToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        return reportDao.surveyResponseToAdmin(reportOrderResponseBo);
    }

    public List<ReportOrderResponseDTO> installResponseToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        return reportDao.installResponseToAdmin(reportOrderResponseBo);
    }

    public List<ReportOrderResponseDTO> installEfficientToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        return reportDao.installEfficientToAdmin(reportOrderResponseBo);
    }

    public List<ReportOrderResponseDTO> reportPassToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        return reportDao.reportPassToAdmin(reportOrderResponseBo);
    }



    public List<ReportOrderResponseDTO> orderResponseToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException {
       /* String beginMonth=reportOrderResponseBo.getDispatchBeginTime();
        String endMonth=reportOrderResponseBo.getDispatchEndTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        Calendar cal = Calendar.getInstance();
        cal.setTime(sdf.parse(beginMonth));
        ArrayList<ReportOrderResponseDTO> reportOrderResponseDTO = new ArrayList<>();
        List<ReportOrderResponseDTO> reportOrderResponseDTOS = reportDao.orderResponseToSupplier(reportOrderResponseBo);
        Map<String, BigDecimal> reportOrderResponseMap = reportOrderResponseDTOS.stream().collect(Collectors.toMap(ReportOrderResponseDTO::getMonth, ReportOrderResponseDTO::getOrderResponse, (v1, v2) -> v2));
        for (long d = cal.getTimeInMillis(); d <= sdf.parse(endMonth).getTime(); d = getMonthPlaus(cal)) {
            ReportOrderResponseDTO reportOrderResponse = new ReportOrderResponseDTO();
            reportOrderResponse.setMonth(sdf.format(d));
            if (reportOrderResponseMap.get(sdf.format(d))!=null) {
                reportOrderResponse.setOrderResponse(reportOrderResponseMap.get(sdf.format(d)));
            }else{
                reportOrderResponse.setOrderResponse(new BigDecimal(0.00));
            }
        }*/
        return getTotalResult(reportOrderResponseBo,1);
    }

    public List<ReportOrderResponseDTO> surveyResponseToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException{
        return getTotalResult(reportOrderResponseBo,2);
    }

    public List<ReportOrderResponseDTO> installResponseToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException{
        return getTotalResult(reportOrderResponseBo,3);
    }

    public List<ReportOrderResponseDTO> installEfficientToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException{
        return getTotalResult(reportOrderResponseBo,4);
    }

    public List<ReportOrderResponseDTO> reportPassToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException{
        return getTotalResult(reportOrderResponseBo,5);
    }

    public List<ReportOrderResponseDTO> getTotalResult(ReportOrderResponseBo reportOrderResponseBo,Integer reportFlag) throws ParseException {
        String beginMonth=reportOrderResponseBo.getDispatchBeginTime();
        String endMonth=reportOrderResponseBo.getDispatchEndTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        Calendar cal = Calendar.getInstance();
        cal.setTime(sdf.parse(beginMonth));
        ArrayList<ReportOrderResponseDTO> reportOrderResponseDTO = new ArrayList<>();
        List<ReportOrderResponseDTO> reportOrderResponseDTOs = new ArrayList<>();
        if (reportFlag==1) {
            reportOrderResponseDTOs=reportDao.orderResponseToSupplier(reportOrderResponseBo);
        }else if (reportFlag==2) {
            reportOrderResponseDTOs=reportDao.surveyResponseToSupplier(reportOrderResponseBo);
        }else if (reportFlag==3) {
            reportOrderResponseDTOs=reportDao.installResponseToSupplier(reportOrderResponseBo);
        }else if (reportFlag==4) {
            reportOrderResponseDTOs=reportDao.installEfficientToSupplier(reportOrderResponseBo);
        }else if (reportFlag==5) {
            reportOrderResponseDTOs=reportDao.reportPassToSupplier(reportOrderResponseBo);
        }
        Map<String, BigDecimal> reportOrderResponseMap = reportOrderResponseDTOs.stream().collect(Collectors.toMap(ReportOrderResponseDTO::getMonth, ReportOrderResponseDTO::getOrderResponse, (v1, v2) -> v2));
        for (long d = cal.getTimeInMillis(); d <= sdf.parse(endMonth).getTime(); d = getMonthPlaus(cal)) {
            ReportOrderResponseDTO reportOrderResponse = new ReportOrderResponseDTO();
            reportOrderResponse.setMonth(sdf.format(d));
            if (reportOrderResponseMap.get(sdf.format(d))!=null) {
                reportOrderResponse.setOrderResponse(reportOrderResponseMap.get(sdf.format(d)));
            }else{
                reportOrderResponse.setOrderResponse(new BigDecimal(0.00));
            }
            reportOrderResponseDTO.add(reportOrderResponse);
        }
        return reportOrderResponseDTO;
    }

    public long getMonthPlaus(Calendar c) {
        c.set(Calendar.MONTH, c.get(Calendar.MONTH) + 1);
        return c.getTimeInMillis();
    }
}
