package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.lang.Assert;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.mapper.def.AccountRoleMapper;
import com.boco.nbd.wios.manage.mapper.def.RoleMapper;
import com.boco.nbd.wios.manage.mapper.def.RoleMapperEx;
import com.boco.nbd.wios.manage.mapper.def.RoleMenuMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 角色Service
 *
 * @author lilin
 * @version [版本号, 2020年5月20日]
 */
@Service
public class RoleService {
    /**
     * 角色Mapper
     */
    @Resource
    private RoleMapper roleMapper;

    /**
     * 菜单service
     */
    @Resource
    private MenuService menuService;

    /**
     * 角色扩展Mapper
     */
    @Resource
    private RoleMapperEx roleMapperEx;

    /**
     * 角色菜单Mapper
     */
    @Resource
    private RoleMenuMapper roleMenuMapper;

    @Resource
    private AccountRoleMapper accountRoleMapper;

    /**
     * 查询角色列表
     *
     * @param role 查询条件
     * @return 角色列表
     */
    public List<Role> queryList(RoleBo role) {
        RoleCriteria roleCriteria = new RoleCriteria();
        RoleCriteria.Criteria criteria = roleCriteria.createCriteria();
        if (role.getStatus() != null)
            criteria.andStatusEqualTo(role.getStatus());
        // 去除超级管理员
        criteria.andIdNotEqualTo(1);
        if (role.getScene() != null && role.getScene().intValue() == 1) {
            criteria.andIdNotEqualTo(2);
            criteria.andIdNotEqualTo(3);
            criteria.andIdNotEqualTo(4);
        }

        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null) {
            pageLimit.setLimited(true);
        }
        Role camsPMRole = null;
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser != null) {
            int accountId = Integer.valueOf(currentUser.getUserId()).intValue();
            String roleIdActual = currentUser.getAttribute().get("roleIdActual");
            if (StringUtils.isNotEmpty(roleIdActual)) {
                // cams管理员
                if (Integer.valueOf(roleIdActual).intValue() == 6) {
                    // cams区域经理
                    camsPMRole = roleMapper.selectByPrimaryKey(5);
                }
            }
            if (accountId != 1) {
                criteria.andCreateAccountIdEqualTo(accountId);
            }
        }

        List<Role> roleList = roleMapper.selectByExample(roleCriteria);
        if (camsPMRole != null) {
            roleList.add(camsPMRole);
        }
        if (pageLimit != null) {
            return pageLimit.limitList(roleList);
        } else
            return roleList;
    }

    /**
     * 新增角色
     *
     * @param role 角色信息
     * @return 新增成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int addRole(RoleBo role) {
        Assert.notEmpty(role.getName(), "角色名称不能为空");
        role.setCreateTime(new Date());
        // 正常
        role.setStatus(1);
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser != null) {
            role.setCreateAccountId(Integer.valueOf(currentUser.getUserId()));

//            List<Role> roleList = roleMapperEx.findRoleListByCreate(Integer.valueOf(currentUser.getUserId()));
//            roleList = roleList.stream().filter(r->r.getName().equals(role.getName())).collect(Collectors.toList());
//            if(!roleList.isEmpty()) throw new ServiceException("名称不能重复");
        }
        int result = roleMapper.insertSelective(role);

        if (StringUtils.isNotBlank(role.getMenuIds())) {
            // 补全上级菜单
            {
                String menuIds = role.getMenuIds();
                String[] menuArr = menuIds.split(CamsConstant.COMMA);
                List<String> menuList = new ArrayList<>(Arrays.asList(menuArr));
                for (String menu : menuArr) {
                    while (true) {
                        if (menu.length() > 2) {
                            menu = menu.substring(0, menu.length() - 2);
                            if (!menuList.contains(menu)) {
                                menuList.add(menu);
                            }
                        } else {
                            break;
                        }
                    }
                }
                role.setMenuIds(org.springframework.util.StringUtils.collectionToCommaDelimitedString(menuList));
            }
        }

        batchInsertRoleMenu(role.getId(), role.getMenuIds());
        return result;
    }

    /**
     * 修改角色
     *
     * @param role 角色信息
     * @return 修改成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int updateRole(RoleBo role) {
        Assert.notNull(role.getId(), WiosConstant.EMPTY_ID);

        if (StringUtils.isNotBlank(role.getMenuIds())) {
            // 补全上级菜单
            {
                String menuIds = role.getMenuIds();
                String[] menuArr = menuIds.split(CamsConstant.COMMA);
                List<String> menuList = new ArrayList<>(Arrays.asList(menuArr));
                for (String menu : menuArr) {
                    while (true) {
                        if (menu.length() > 2) {
                            menu = menu.substring(0, menu.length() - 2);
                            if (!menuList.contains(menu)) {
                                menuList.add(menu);
                            }
                        } else {
                            break;
                        }
                    }
                }
                role.setMenuIds(org.springframework.util.StringUtils.collectionToCommaDelimitedString(menuList));
            }

            RoleMenuCriteria criteria = new RoleMenuCriteria();
            criteria.createCriteria().andRoleIdEqualTo(role.getId());
            roleMenuMapper.deleteByExample(criteria);
            batchInsertRoleMenu(role.getId(), role.getMenuIds());
        }
        if (!StringUtils.isEmpty(role.getName()) || !StringUtils.isEmpty(role.getRemark())) {
            return roleMapper.updateByPrimaryKeySelective(role);
        }
        return 1;
    }

    /**
     * 批量新增权限菜单
     *
     * @param roleId  角色id
     * @param menuIds 菜单id
     * @return 新增成功的条数
     */
    private int batchInsertRoleMenu(int roleId, String menuIds) {
        if (StringUtils.isBlank(menuIds)) {
            return 0;
        }
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        final Integer createAccountId = currentUser == null ? null : Integer.valueOf(currentUser.getUserId());
        Date now = new Date();

        return roleMapperEx.batchInsertRoleMenu(Stream.of(menuIds.split(CamsConstant.COMMA)).map(menuId -> {
            RoleMenu roleMenu = new RoleMenu();
            roleMenu.setRoleId(roleId);
            roleMenu.setMenuId(menuId);
            roleMenu.setCreateAccountId(createAccountId);
            roleMenu.setCreateTime(now);
            return roleMenu;
        }).collect(Collectors.toList()));
    }

    /**
     * 删除角色
     *
     * @param id 角色id
     * @return 删除成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int deleteRole(Integer id) {
        Assert.notNull(id, WiosConstant.EMPTY_ID);
        if (id == 1 || id == 2 || id == 3 || id == 4 || id == 5 || id == 6) {
            throw new ServiceException("内置角色不能被删除！");
        }

        AccountRoleCriteria example = new AccountRoleCriteria();
        example.createCriteria().andRoleIdEqualTo(id);
        List<AccountRole> arList = accountRoleMapper.selectByExample(example);
        if (!arList.isEmpty()) {
            throw new ServiceException("该角色已绑定账户，不可删除");
        }

        RoleMenuCriteria criteria = new RoleMenuCriteria();
        criteria.createCriteria().andRoleIdEqualTo(id);
        roleMenuMapper.deleteByExample(criteria);

        return roleMapper.deleteByPrimaryKey(id);
    }

    /**
     * 查询角色详细信息
     *
     * @param id 角色id
     * @return 角色详细信息
     */
    // public RoleDetailVo getRoleDetail(Integer id) {
    // Role role = null;
    // if (id != null) {
    // role = roleMapper.selectByPrimaryKey(id);
    // }
    // RoleDetailVo roleDetailVo = new RoleDetailVo();
    //
    // if (role == null) {
    // role = new Role();
    // } else {
    // BeanUtils.copyProperties(role, roleDetailVo);
    // }
    //
    // List<Menu> menuList = menuService.findRoleMenuList(id);
    // roleDetailVo.setMenuTreeList(menuService.convertMenuList(menuList, 1));
    //
    // return roleDetailVo;
    // }

    /**
     * 查询角色菜单权限
     *
     * @param id 角色id
     * @return 角色菜单权限
     */
    public List<MenuTreeVo> getRoleMenuList(Integer id) {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            return new ArrayList<>(1);
        }
        int currentAccountRoleId = Integer.valueOf(currentUser.getAttribute("roleIdActual"));
        List<Menu> currentAccountMenuList = menuService.findRoleMenuList(currentAccountRoleId);
        List<Menu> menuList = menuService.findRoleMenuList(id);

        for (Menu caMenu : currentAccountMenuList) {
            List<Menu> hasList = menuList.stream().filter(m -> m.getId().equals(caMenu.getId())).collect(Collectors.toList());
            if (hasList.isEmpty()) {
                caMenu.setStatus(null);
            } else {
                caMenu.setStatus(1);
            }
        }
        // TODO 判断如果不是管理员权限 去除角色管理的菜单
        {
            if (currentAccountRoleId != 1) {
                currentAccountMenuList =
                        currentAccountMenuList.stream().filter(menu -> !menu.getId().startsWith("0205")).collect(Collectors.toList());
            }
        }
        List<AbstractTreeNode> returnList = menuService.convertMenuList(currentAccountMenuList, 1);
        List<MenuTreeVo> returnValue = new ArrayList<>();
        for (AbstractTreeNode node : returnList) {
            MenuTreeVo vo = (MenuTreeVo) node;
            // vo.setChecked(judge(vo.getChildren()));
            returnValue.add(vo);
        }
        return returnValue;
    }

    public boolean judge(List<MenuTreeVo> children) {
        if (children == null) {
            return true;
        }
        boolean checked = true;
        for (MenuTreeVo vo : children) {
            if (!vo.isChecked()) {
                checked = false;
                break;
            } else {
                if (!judge(vo.getChildren())) {
                    return false;
                }
            }
        }
        return checked;
    }

    /**
     * 查询左侧菜单列表
     *
     * @param accountId
     * @return
     */
    public List<AbstractTreeNode> getMenuList(Integer accountId) {
        List<Menu> menuList = menuService.findAccountMenuList(accountId);
        return menuService.convertMenuList(menuList, 0);
    }

    /**
     * 根据账户id查询角色列表
     *
     * @param accountId 账户id
     * @return 角色列表
     */
    public List<Role> findAccountRoleList(String accountId) {
        return roleMapperEx.findAccountRoleList(accountId);
    }

    /**
     * 根据账号id查询角色和小程序权限
     *
     * @param accountId
     * @return
     */
    public List<RoleBo> findMinaRoleMenu(Integer accountId) {
        return roleMapperEx.findMinaRoleMenu(accountId);
    }

    /**
     * 获取某个页面配置的按钮列表
     *
     * @param menuId
     * @return
     */
    public List<Menu> getPageBtnList(String menuId, String accountId) {
        if (StringUtils.isEmpty(accountId)) {
            SessionInfo currentUser = ProcessUtil.getUserInfo();
            if (currentUser == null) {
                throw new ServiceException("获取用户信息失败，请重新登陆");
            }
            accountId = currentUser.getUserId();
        }
        List<Role> roleList = roleMapperEx.findAccountRoleList(accountId);
        List<Menu> menuList = new ArrayList<>();
        for (Role role : roleList) {
            List<String> ids = menuList.stream().map(item -> item.getId()).collect(Collectors.toList());
            List<Menu> list = menuService.findBtnList(role.getId(), menuId);
            list.stream().forEach(item -> {
                if (!ids.contains(item.getId())) {
                    menuList.add(item);
                }
            });
        }
        return menuList;
    }

    public List<Menu> getPageBtnList(String menuId) {
        return getPageBtnList(menuId, null);
    }

    /**
     * 角色启用/禁用
     *
     * @param roleId
     * @param status
     * @return
     */
    public void updateStatus(Integer roleId, Integer status) {
        Role record = new Role();
        record.setId(roleId);
        record.setStatus(status);
        roleMapper.updateByPrimaryKeySelective(record);
    }

}
