package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.cams.core.constant.MessageConstant;
import com.boco.nbd.wios.flow.entity.SecretBindDTO;
import com.boco.nbd.wios.flow.entity.SecretPhone;
import com.boco.nbd.wios.flow.util.PhoneNumProtectionUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.mapper.def.SecretPhoneDao;
import com.ihidea.core.support.exception.ServiceException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * @author haica
 */
@Service
public class SecretPhoneService {

    @Autowired
    private SecretPhoneDao secretPhoneDao;

    private static final String NO_AVAILABLE_NUMBER = "isv.NO_AVAILABLE_NUMBER";

    /**
     * 绑定虚拟号码
     *
     * @param phone
     * @return
     * @throws Exception
     */
    public SecretBindDTO bind(String phone) throws Exception {
        if (StringUtils.isEmpty(phone)) {
            throw new ServiceException(MessageConstant.MISSING_PARAM);
        }
        SecretPhone sp = secretPhoneDao.selectByPhone(phone);
        if (sp != null && sp.getStatus() != null && sp.getStatus().intValue() == 1) {
            throw new ServiceException("该号码已绑定虚拟号码");
        }
        SecretBindDTO result = PhoneNumProtectionUtil.bindAxn(phone);
        String code = result.getCode();
        if (!StringUtils.isEmpty(code) && NO_AVAILABLE_NUMBER.equals(code)) {
            // 购买号码
            PhoneNumProtectionUtil.buySecretNo();
            // 等一会儿
            Thread.sleep(1000);
            // 再次绑定
            result = PhoneNumProtectionUtil.bindAxn(phone);
            String code2 = result.getCode();
            if (!StringUtils.isEmpty(code2) && !WiosConstant.OK.equals(code2)) {
                throw new ServiceException("再次绑定失败，错误码：" + code2);
            }
        } else if (!StringUtils.isEmpty(code) && !WiosConstant.OK.equals(code) && !NO_AVAILABLE_NUMBER.equals(code)) {
            throw new ServiceException("绑定失败，错误码：" + code);
        }

        SecretPhone secretPhone = new SecretPhone();
        secretPhone.setPhone(phone);
        secretPhone.setPoolKey(PhoneNumProtectionUtil.ALI_POOLKEY);
        secretPhone.setExpiredTime(result.getExpiredDate());
        secretPhone.setSecretNo(result.getSecretNo());
        secretPhone.setSubsId(result.getSubsId());
        secretPhoneDao.insert(secretPhone);

        return result;
    }

    @Async
    public SecretBindDTO bindAsync(String phone) throws Exception {
        return bind(phone);
    }

    /**
     * 解绑虚拟号码
     *
     * @param phone
     * @throws Exception
     */
    public void unbind(String phone) throws Exception {
        if (StringUtils.isEmpty(phone)) {
            throw new ServiceException(MessageConstant.MISSING_PARAM);
        }
        SecretPhone sp = secretPhoneDao.selectByPhone(phone);
        if (sp == null) {
            throw new ServiceException("号码不存在");
        }
        PhoneNumProtectionUtil.unbindSubscription(sp.getSubsId(), sp.getSecretNo());

        // SecretPhone secretPhone = new SecretPhone();
        // secretPhone.setPhone(phone);
        // secretPhone.setStatus(0);
        // secretPhoneDao.updateDynamic(secretPhone);
        secretPhoneDao.delete(phone);
    }

    public SecretPhone get(String phone) {
        return secretPhoneDao.selectByPhone(phone);
    }

    public void releaseExpired() throws Exception {
        List<SecretPhone> list = secretPhoneDao.selectExpired();
        for (SecretPhone sp : list) {
            PhoneNumProtectionUtil.unbindSubscription(sp.getSubsId(), sp.getSecretNo());
            Thread.sleep(500);
            PhoneNumProtectionUtil.releaseSecretNo(sp.getSecretNo());
            secretPhoneDao.delete(sp.getPhone());
        }
    }
}
