package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.OemType;
import com.boco.nbd.wios.manage.entity.settlement.po.Settlement;
import com.boco.nbd.wios.manage.entity.settlement.po.SettlementDetail;
import com.boco.nbd.wios.manage.entity.settlement.po.SettlementDetailSupplier;
import com.boco.nbd.wios.manage.entity.settlement.vo.SettlementVo;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.boco.nbd.wios.manage.util.NumberUtil;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.DateUtilsEx;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 结算单service
 *
 * @author xgl
 * @version [版本号, 2020年9月24日]
 */
@Service
public class SettlementService {

    private Logger log = LoggerFactory.getLogger(getClass());

    @Autowired
    private SettlementDao settlementDao;

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private OrderInstallDao orderInstallDao;

    @Autowired
    private OemContractDao oemContractDao;

    @Autowired
    private OemContractItemDao oemContractItemDao;

    @Autowired
    private OemContractPriceDao oemContractPriceDao;

    @Autowired
    private RegionDao regionDao;

    @Autowired
    private SettlementDetailDao settlementDetailDao;

    @Autowired
    private SupplierContractDao supplierContractDao;

    @Autowired
    private SupplierContractPriceDao supplierContractPriceDao;

    @Autowired
    private SupplierContractPackPriceDao supplierContractPackPriceDao;

    @Autowired
    private SupplierContractItemDao supplierContractItemDao;

    @Autowired
    private SettlementDetailSupplierDao settlementDetailSupplierDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    private ExpandPriceDao expandPriceDao;

    @Autowired
    private DictionaryService dictionaryService;

    @Resource
    private RegionMapper regionMapper;

    /**
     * 添加结算
     *
     * @param settlement
     */
    public void add(Settlement settlement) {
        settlementDao.insert(settlement);
    }

    /**
     * 修改结算
     *
     * @param id
     * @param settlement
     */
    public void update(int id, Settlement settlement) {
        settlementDao.update(id, settlement);
    }

    public List<SettlementVo> getList(Settlement condition) {
        List<SettlementVo> list = settlementDao.selectByExample(condition);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        list.stream().forEach(item -> {
            if (item.getStartDate() != null && item.getEndDate() != null) {
                item.setPeriod(sdf.format(item.getStartDate()) + "~" + sdf.format(item.getEndDate()));
            }
        });
        return list;
    }

    public List<SettlementDetail> getDetailList(String settlementId) {
        return settlementDetailDao.selectBySettlementId(settlementId);
    }

    public List<SettlementDetailSupplier> getDetailListForSupplier(String settlementId) {
        return settlementDetailSupplierDao.selectBySettlementId(settlementId);
    }

    public BigDecimal getOemTotalPrice(String settlementId) {
        List<SettlementDetail> list = settlementDetailDao.selectBySettlementId(settlementId);
        return list.stream().map(SettlementDetail::getTotalFee).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    public BigDecimal getSupplierTotalPrice(String settlementId) {
        List<SettlementDetailSupplier> list = settlementDetailSupplierDao.selectBySettlementId(settlementId);
        return list.stream().map(SettlementDetailSupplier::getTotalFee).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    /**
     * 生成主机厂账单详情
     *
     * @param startDate
     * @param endDate
     */
    public void generateOemDetail(Date startDate, Date endDate) {
        SimpleDateFormat startSdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        SimpleDateFormat endSdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
        // 查询时间段内的所有订单
        List<OrderVo> orderList = orderDao.selectPeriodOrders(startSdf.format(startDate), endSdf.format(endDate));
        log.info("[generateOemDetail]订单ids={}",
                StringUtils.collectionToCommaDelimitedString(orderList.stream().map(OrderVo::getId).collect(Collectors.toList())));
        List<Dictionary> dictionaryList = dictionaryService.selectValidByType("wallbox", null);
        Map<String, Integer> wallboxMap = dictionaryList.stream().collect(Collectors.toMap(Dictionary::getValue, Dictionary::getIsSettle, (k1, k2) -> k2));
        for (OrderVo order : orderList) {
            if (order.getContractId() == null) {
                continue;
            }
            OemContract contract = oemContractDao.selectById(order.getContractId());
            // 查询合同关联的区域安装价格
            List<OemContractPrice> prices = oemContractPriceDao.selectPureByContractId(contract.getId());
            // 查询安装订单
            List<OrderInstallVo> orderInstallList = orderInstallDao.selectByOrderIds(order.getId());
            if (orderInstallList.isEmpty()) {
                log.error("[generateOemDetail]未找到安装订单,orderId={}", order.getId());
                continue;
            }
            OrderInstallVo orderInstall = orderInstallList.get(0);
            SettlementDetail detail = new SettlementDetail();
            String wallboxItemId = orderInstall.getWallboxItemId();
            // 查询设备价格
            List<OemContractItemVo> contractItemList = oemContractItemDao.selectByContractIdAndItemId(contract.getId(), wallboxItemId);
            if (!contractItemList.isEmpty()) {
                OemContractItemVo contractItem = contractItemList.get(0);

                BigDecimal price;
                BigDecimal ratePrice;
                //根据字典配置判断墙盒费用是否计算在内
                if ((wallboxMap.get(order.getWallboxModel()) != null && wallboxMap.get(order.getWallboxModel()) == 0)) {
                    price = BigDecimal.ZERO;
                    ratePrice = BigDecimal.ZERO;
                } else {
                    // 单价（不含税）
                    price = contractItem.getPrice();
                    // 单价（含税）
                    ratePrice = contractItem.getRatePrice();
                }
                detail.setOrderId(orderInstall.getOrderId());
                detail.setUserName(orderInstall.getUserName());
                detail.setUserPhone(orderInstall.getUserPhone());
                detail.setDeviceFee(price);
                detail.setDeviceFeeWithTax(ratePrice);
                detail.setInstallFinishTime(orderInstall.getInstallFinishTime());

                // 查询区域价格 没有的话 继续向上一级查找
                Integer regionId = orderInstall.getRegionId();
                Region region = regionDao.selectById(regionId);
                String idTree = region.getIdTree();
                String[] idTreeArr = idTree.split(CamsConstant.UNDER_LINE);
                OemContractPrice contractPrice = null;
                for (int i = idTreeArr.length - 1; i >= 0; i--) {
                    final int iFinal = i;
                    List<OemContractPrice> filterResult =
                            prices.stream().filter(item -> item.getRegionId().equals(idTreeArr[iFinal]))
                                    .collect(Collectors.toList());
                    if (!filterResult.isEmpty()) {
                        contractPrice = filterResult.get(0);
                        break;
                    }
                }
                if (contractPrice != null) {
                    detail.setInstallServiceFee(contractPrice.getInstallationPrice().add(contractPrice.getSurveyPrice()));
                    detail
                            .setInstallServiceFeeWithTax(contractPrice.getInstallationRatePrice().add(contractPrice.getSurveyRatePrice()));
                } else {
                    log.error("[generateOemDetail]未找到contractPrice,orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
                }
                detail.setTotalFee(
                        detail.getDeviceFee().add(detail.getInstallServiceFee() == null ? BigDecimal.ZERO : detail.getInstallServiceFee()));
                detail.setTotalFeeWithTax(detail.getDeviceFeeWithTax()
                        .add(detail.getInstallServiceFeeWithTax() == null ? BigDecimal.ZERO : detail.getInstallServiceFeeWithTax()));
                settlementDetailDao.insert(detail);
            } else {
                log.error("[generateOemDetail]未找到设备价格,orderId={}，wallboxNo={}", order.getId(), wallboxItemId);
            }
        }
    }

    public void generateSupplierDetail(Date startDate, Date endDate) {
        SimpleDateFormat startSdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        SimpleDateFormat endSdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
        // 查询时间段内的所有订单
        List<OrderVo> orderList = orderDao.selectPeriodOrders(startSdf.format(startDate), endSdf.format(endDate));
        log.info("[generateSupplierDetail]订单ids={}",
                StringUtils.collectionToCommaDelimitedString(orderList.stream().map(OrderVo::getId).collect(Collectors.toList())));
        for (OrderVo order : orderList) {
            Integer supplierContractId = order.getSupplierContractId();
            if (supplierContractId == null) {
                continue;
            }
            if (supplierContractId.intValue() == 31) {
                log.info("========");
            }
            SupplierContract contract = supplierContractDao.selectById(supplierContractId);
            if (contract == null) {
                continue;
            }
            // 查询合同关联的区域安装价格
            List<SupplierContractPrice> prices = supplierContractPriceDao.selectPureByContractId(contract.getId());
            // 查询合同关联的区域套包安装价格
            List<SupplierContractPackPrice> packPrices = supplierContractPackPriceDao.selectPureByContractId(contract.getId());
            // 查询安装订单
            List<OrderInstallVo> orderInstallList = orderInstallDao.selectByOrderIds(order.getId());
            if (orderInstallList.isEmpty()) {
                log.error("[generateSupplierDetail]未找到安装订单,orderId={}", order.getId());
                continue;
            }
            OrderInstallVo orderInstall = orderInstallList.get(0);
            SettlementDetailSupplier detail = new SettlementDetailSupplier();
            detail.setOrderId(orderInstall.getOrderId());
            detail.setUserName(orderInstall.getUserName());
            detail.setUserPhone(orderInstall.getUserPhone());
            detail.setFinishTime(orderInstall.getInstallFinishTime());
            // 查询区域价格 没有的话 继续向上一级查找

            Integer regionId = orderInstall.getRegionId();
            Region region = regionDao.selectById(regionId);
            String idTree = region.getIdTree();
            String[] idTreeArr = idTree.split(CamsConstant.UNDER_LINE);
            SupplierContractPrice contractPrice = null;
            for (int i = idTreeArr.length - 1; i >= 0; i--) {
                final int iFinal = i;
                List<SupplierContractPrice> filterResult =
                        prices.stream().filter(item -> item.getRegionId().toString().equals(idTreeArr[iFinal]))
                                .collect(Collectors.toList());
                if (!filterResult.isEmpty()) {
                    contractPrice = filterResult.get(0);
                    break;
                }
            }
            if (contractPrice == null) {
                SupplierContractPackPrice contractPackPrice = null;
                for (int i = idTreeArr.length - 1; i >= 0; i--) {
                    final int iFinal = i;
                    List<SupplierContractPackPrice> filterResult =
                            packPrices.stream().filter(item -> item.getRegionId().toString().equals(idTreeArr[iFinal]))
                                    .collect(Collectors.toList());
                    if (!filterResult.isEmpty()) {
                        contractPackPrice = filterResult.get(0);
                        break;
                    }
                }
                if (contractPackPrice != null) {
                    log.info("[generateSupplierDetail]套包安装价格orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
                    // 根据线缆长度判断套包价格
                    String realCableMeter = orderInstall.getRealCableMeter();
                    Double cableLength = contractPackPrice.getCableLength();
                    if (Double.valueOf(realCableMeter).compareTo(cableLength) <= 0) {
                        detail.setInstallFee(contractPackPrice.getPrice());
                        detail.setInstallFeeWithTax(contractPackPrice.getPriceWithTax());
                    } else {
                        detail.setInstallFee(contractPackPrice.getPriceTwo());
                        detail.setInstallFeeWithTax(contractPackPrice.getPriceWithTaxTwo());
                    }
                } else {
                    log.error("[generateSupplierDetail]未找到安装价格orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
                }
            } else {
                log.info("[generateSupplierDetail]常规安装价格orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
                detail.setSurveyFee(contractPrice.getSurveyPrice());
                detail.setSurveyFeeWithTax(contractPrice.getSurveyRatePrice());
                detail.setInstallFee(contractPrice.getInstallationPrice());
                detail.setInstallFeeWithTax(contractPrice.getInstallationRatePrice());
            }
            List<SupplierContractItem> items = supplierContractItemDao.selectPureByContractId(contract.getId());
            Map<String, SupplierContractItem> sciMap =
                    items.stream().collect(Collectors.toMap(SupplierContractItem::getItemId, supplierContractItem -> supplierContractItem));
            // 查询安装报告物料
            List<OrderInstallItemVo> orderInstallItemList = orderInstallItemDao.selectByOrderIdAndType(order.getId(), 2);
            BigDecimal totalItemPrice = BigDecimal.ZERO;
            BigDecimal totalItemRatePrice = BigDecimal.ZERO;
            for (OrderInstallItemVo oii : orderInstallItemList) {
                String itemId = oii.getItemId();
                SupplierContractItem sci = sciMap.get(itemId);
                if (sci != null) {
                    totalItemPrice = totalItemPrice.add(sci.getPrice().multiply(oii.getQunatity()));
                    totalItemRatePrice = totalItemRatePrice.add(sci.getRatePrice().multiply(oii.getQunatity()));
                }
            }

            detail.setItemFee(totalItemPrice);
            detail.setItemFeeWithTax(totalItemRatePrice);
            // 计算报装费用
            if (orderInstall.getElectricExpand() != null && orderInstall.getElectricExpand().intValue() == 1
                    && orderInstall.getExpandMethod() != null && orderInstall.getExpandMethod().intValue() == 1) {
                List<ExpandPrice> expandPrice = expandPriceDao.selectByRegionId(regionId);
                if (!expandPrice.isEmpty()) {
                    detail.setExpandFee(expandPrice.get(0).getPrice() == null ? BigDecimal.ZERO : expandPrice.get(0).getPrice());
                }
            }
            detail.setTotalFee(detail.getSurveyFee().add(detail.getInstallFee()).add(detail.getItemFee()).add(detail.getExpandFee()));
            detail.setTotalFeeWithTax(detail.getSurveyFeeWithTax().add(detail.getInstallFeeWithTax()).add(detail.getItemFeeWithTax())
                    .add(detail.getExpandFee()));
            settlementDetailSupplierDao.insert(detail);

        }
    }

    /**
     * 生成主机厂结算单
     *
     * @param startDate
     * @param endDate
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void generateOemSettlement(Date startDate, Date endDate) {
        SimpleDateFormat startSdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        SimpleDateFormat endSdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
        OemBo condition = new OemBo();
        condition.setHasContract(2);
        List<OemVo> oemList = oemDao.getList(condition);
        for (OemVo oem : oemList) {
            Settlement settlement = new Settlement();
            List<SettlementDetail> detailList =
                    settlementDetailDao.selectByOemId(oem.getId(), startSdf.format(startDate), endSdf.format(endDate));
            BigDecimal totalDeviceFee = detailList.stream().map(SettlementDetail::getDeviceFee).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalDeviceFeeWithTax =
                    detailList.stream().map(SettlementDetail::getDeviceFeeWithTax).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalInstallServiceFee =
                    detailList.stream().map(SettlementDetail::getInstallServiceFee).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalInstallServiceFeeWithTax =
                    detailList.stream().map(SettlementDetail::getInstallServiceFeeWithTax).reduce(BigDecimal.ZERO, BigDecimal::add);
            settlement.setTotalPrice(totalDeviceFee.add(totalInstallServiceFee));
            settlement.setTotalPriceWithTax(totalDeviceFeeWithTax.add(totalInstallServiceFeeWithTax));
            settlement.setTotalTax(settlement.getTotalPriceWithTax().subtract(settlement.getTotalPrice()));
            String settlementId = NumberUtil.getRandomOrderId();
            settlement.setSettlementId(settlementId);
            settlement.setTargetId(oem.getId());
            settlement.setType(1);
            settlement.setStartDate(startDate);
            settlement.setEndDate(endDate);
            settlement.setCreateTime(new Date());
            settlementDao.insert(settlement);

            List<Integer> idList = detailList.stream().map(SettlementDetail::getId).collect(Collectors.toList());
            if (!idList.isEmpty()) {
                settlementDetailDao.batchUpdateSettlementId(idList, settlementId);
            }
        }
    }

    /**
     * 生成服务商结算单
     *
     * @param startDate
     * @param endDate
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void generateSupplierSettlement(Date startDate, Date endDate) {
        SimpleDateFormat startSdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
        SimpleDateFormat endSdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
        SupplierBo condition = new SupplierBo();
        // condition.setStatus(1);// 启用的
        condition.setHasContract(2);
        List<SupplierVo> supplierList = supplierDao.getList(condition);
        for (SupplierVo supplier : supplierList) {
            Settlement settlement = new Settlement();
            List<SettlementDetailSupplier> detailList =
                    settlementDetailSupplierDao.selectBySupplierId(Integer.valueOf(supplier.getId()), startSdf.format(startDate),
                            endSdf.format(endDate));
            BigDecimal totalSurveyFee =
                    detailList.stream().map(SettlementDetailSupplier::getSurveyFee).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalSurveyFeeWithTax =
                    detailList.stream().map(SettlementDetailSupplier::getSurveyFeeWithTax).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalInstallFee =
                    detailList.stream().map(SettlementDetailSupplier::getInstallFee).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalInstallFeeWithTax =
                    detailList.stream().map(SettlementDetailSupplier::getInstallFeeWithTax).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalItemFee =
                    detailList.stream().map(SettlementDetailSupplier::getItemFee).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalItemFeeWithTax =
                    detailList.stream().map(SettlementDetailSupplier::getItemFeeWithTax).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalExpandFee =
                    detailList.stream().map(SettlementDetailSupplier::getExpandFee).reduce(BigDecimal.ZERO, BigDecimal::add);

            settlement.setTotalPrice(totalSurveyFee.add(totalInstallFee).add(totalItemFee).add(totalExpandFee));
            settlement.setTotalPriceWithTax(totalSurveyFeeWithTax.add(totalInstallFeeWithTax).add(totalItemFeeWithTax).add(totalExpandFee));
            settlement.setTotalTax(settlement.getTotalPriceWithTax().subtract(settlement.getTotalPrice()));
            String settlementId = NumberUtil.getRandomOrderId();
            settlement.setSettlementId(settlementId);
            settlement.setTargetId(Integer.valueOf(supplier.getId()));
            settlement.setType(2);
            settlement.setStartDate(startDate);
            settlement.setEndDate(endDate);
            settlement.setCreateTime(new Date());
            settlementDao.insert(settlement);

            List<Integer> idList = detailList.stream().map(SettlementDetailSupplier::getId).collect(Collectors.toList());
            if (!idList.isEmpty()) {
                settlementDetailSupplierDao.batchUpdateSettlementId(idList, settlementId);
            }
        }
    }

    /**
     * @return void
     * @Description 手动生成主机厂结算单
     * @Param [startDate, endDate, oemIds]
     * @author liwenxiang
     * @date 2021/8/16 10:54
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void generateOemSettlementById(String startDate, String endDate, String oemIds) throws ParseException {
        //检验主机厂在该时间段是否生成过主机厂结算单
        List<String> oemIdList = Arrays.asList(oemIds.split(CamsConstant.COMMA));
        List<Settlement> settlementList = settlementDao.getSettlementByCondition(startDate, endDate, oemIdList, OemType.OEM.getType());
        if (CollectionUtils.isNotEmpty(settlementList)) {
            throw new ServiceException("主机厂结算单生成时间段重叠");
        }
        for (String oemId : oemIdList) {
            Oem oem = oemDao.selectById(Integer.valueOf(oemId));
            //校验主机厂是否存在
            if (ObjectUtils.isEmpty(oem)) {
                throw new ServiceException("主机厂不存在");
            }
            Settlement settlement = new Settlement();
            List<SettlementDetail> detailList =
                    settlementDetailDao.selectByOemId(Integer.valueOf(oemId).intValue(), startDate + " 00:00:00", endDate + " 23:59:59");
            BigDecimal totalDeviceFee = detailList.stream().map(SettlementDetail::getDeviceFee).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalDeviceFeeWithTax =
                    detailList.stream().map(SettlementDetail::getDeviceFeeWithTax).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalInstallServiceFee =
                    detailList.stream().map(SettlementDetail::getInstallServiceFee).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalInstallServiceFeeWithTax =
                    detailList.stream().map(SettlementDetail::getInstallServiceFeeWithTax).reduce(BigDecimal.ZERO, BigDecimal::add);
            settlement.setTotalPrice(totalDeviceFee.add(totalInstallServiceFee));
            settlement.setTotalPriceWithTax(totalDeviceFeeWithTax.add(totalInstallServiceFeeWithTax));
            settlement.setTotalTax(settlement.getTotalPriceWithTax().subtract(settlement.getTotalPrice()));
            String settlementId = NumberUtil.getRandomOrderId();
            settlement.setSettlementId(settlementId);
            settlement.setTargetId(Integer.valueOf(oemId));
            settlement.setType(1);
            settlement.setStartDate(DateUtilsEx.formatToDate(startDate, "yyyy-MM-dd"));
            settlement.setEndDate(DateUtilsEx.formatToDate(endDate, "yyyy-MM-dd"));
            settlement.setCreateTime(new Date());
            settlementDao.insert(settlement);
            List<Integer> idList = detailList.stream().map(SettlementDetail::getId).collect(Collectors.toList());
            if (!idList.isEmpty()) {
                settlementDetailDao.batchUpdateSettlementId(idList, settlementId);
            }
        }

    }

    public SettlementDetail getSettlementDetail(Order order, Dictionary dictionary) {
        OemContract contract = oemContractDao.selectById(order.getContractId());
        // 查询合同关联的区域安装价格
        List<OemContractPrice> prices = oemContractPriceDao.selectPureByContractId(contract.getId());
        // 查询安装订单
        List<OrderInstallVo> orderInstallList = orderInstallDao.selectByOrderIds(order.getId());
        if (orderInstallList.isEmpty()) {
            log.error("[generateOemDetail]未找到安装订单,orderId={}", order.getId());
            return null;
        }
        OrderInstallVo orderInstall = orderInstallList.get(0);
        SettlementDetail detail = new SettlementDetail();
        String wallboxItemId = orderInstall.getWallboxItemId();
        BigDecimal price = new BigDecimal(-1);
        BigDecimal ratePrice = BigDecimal.ZERO;
        if (dictionary != null && dictionary.getIsSettle() == 0) {
            price = BigDecimal.ZERO;
            ratePrice = price;
        } else {
            // 查询设备价格
            List<OemContractItemVo> contractItemList = oemContractItemDao.selectByContractIdAndItemId(contract.getId(), wallboxItemId);
            if (!contractItemList.isEmpty()) {
                OemContractItemVo contractItem = contractItemList.get(0);
                price = contractItem.getPrice();
                ratePrice = contractItem.getRatePrice();
            } else {
                log.error("[generateOemDetail]未找到设备价格,orderId={}，wallboxNo={}", order.getId(), wallboxItemId);
            }
        }

        detail.setOrderId(orderInstall.getOrderId());
        detail.setUserName(orderInstall.getUserName());
        detail.setUserPhone(orderInstall.getUserPhone());
        detail.setDeviceFee(price);
        detail.setDeviceFeeWithTax(ratePrice);
        detail.setInstallFinishTime(orderInstall.getInstallFinishTime());
        // 查询区域价格 没有的话 继续向上一级查找
        Integer regionId = regionMapper.selectRegionByOrderId(order.getId());
        Region region = regionDao.selectById(regionId);
        String idTree = region.getIdTree();
        String[] idTreeArr = idTree.split(CamsConstant.UNDER_LINE);
        OemContractPrice contractPrice = null;
        for (int i = idTreeArr.length - 1; i >= 0; i--) {
            final int iFinal = i;
            List<OemContractPrice> filterResult =
                    prices.stream().filter(item -> item.getRegionId().equals(idTreeArr[iFinal]))
                            .collect(Collectors.toList());
            if (!filterResult.isEmpty()) {
                contractPrice = filterResult.get(0);
                break;
            }
        }
        if (contractPrice != null) {
            detail.setInstallServiceFee(contractPrice.getInstallationPrice().add(contractPrice.getSurveyPrice()));
            detail.setInstallServiceFeeWithTax(contractPrice.getInstallationRatePrice().add(contractPrice.getSurveyRatePrice()));
        } else {
            log.error("[generateOemDetail]未找到contractPrice,orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
        }
        detail.setTotalFee(
                detail.getDeviceFee().add(detail.getInstallServiceFee() == null ? BigDecimal.ZERO : detail.getInstallServiceFee()));
        detail.setTotalFeeWithTax(detail.getDeviceFeeWithTax()
                .add(detail.getInstallServiceFeeWithTax() == null ? BigDecimal.ZERO : detail.getInstallServiceFeeWithTax()));
        return detail;
    }

    public SettlementDetailSupplier getSettlementDetailSupplier(Order order, SupplierContract contract) {
        // 查询合同关联的区域安装价格
        List<SupplierContractPrice> prices = supplierContractPriceDao.selectPureByContractId(contract.getId());
        // 查询合同关联的区域套包安装价格
        List<SupplierContractPackPrice> packPrices = supplierContractPackPriceDao.selectPureByContractId(contract.getId());
        // 查询安装订单
        List<OrderInstallVo> orderInstallList = orderInstallDao.selectByOrderIds(order.getId());
        if (orderInstallList.isEmpty()) {
            log.error("[generateSupplierDetail]未找到安装订单,orderId={}", order.getId());
            return null;
        }
        OrderInstallVo orderInstall = orderInstallList.get(0);
        SettlementDetailSupplier detail = new SettlementDetailSupplier();
        detail.setOrderId(orderInstall.getOrderId());
        detail.setUserName(orderInstall.getUserName());
        detail.setUserPhone(orderInstall.getUserPhone());
        detail.setFinishTime(orderInstall.getInstallFinishTime());
        // 查询区域价格 没有的话 继续向上一级查找
        Integer regionId = regionMapper.selectRegionByOrderId(order.getId());
        Region region = regionDao.selectById(regionId);
        String idTree = region.getIdTree();
        String[] idTreeArr = idTree.split(CamsConstant.UNDER_LINE);
        SupplierContractPrice contractPrice = null;
        for (int i = idTreeArr.length - 1; i >= 0; i--) {
            final int iFinal = i;
            List<SupplierContractPrice> filterResult =
                    prices.stream().filter(item -> item.getRegionId().toString().equals(idTreeArr[iFinal]))
                            .collect(Collectors.toList());
            if (!filterResult.isEmpty()) {
                contractPrice = filterResult.get(0);
                break;
            }
        }
        if (contractPrice == null) {
            SupplierContractPackPrice contractPackPrice = null;
            for (int i = idTreeArr.length - 1; i >= 0; i--) {
                final int iFinal = i;
                List<SupplierContractPackPrice> filterResult =
                        packPrices.stream().filter(item -> item.getRegionId().toString().equals(idTreeArr[iFinal]))
                                .collect(Collectors.toList());
                if (!filterResult.isEmpty()) {
                    contractPackPrice = filterResult.get(0);
                    break;
                }
            }
            if (contractPackPrice != null) {
                log.info("[generateSupplierDetail]套包安装价格orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
                // 根据线缆长度判断套包价格
                String realCableMeter = orderInstall.getRealCableMeter();
                Double cableLength = contractPackPrice.getCableLength();
                if (Double.valueOf(realCableMeter).compareTo(cableLength) <= 0) {
                    detail.setInstallFee(contractPackPrice.getPrice());
                    detail.setInstallFeeWithTax(contractPackPrice.getPriceWithTax());
                } else {
                    detail.setInstallFee(contractPackPrice.getPriceTwo());
                    detail.setInstallFeeWithTax(contractPackPrice.getPriceWithTaxTwo());
                }
            } else {
                log.error("[generateSupplierDetail]未找到安装价格orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
            }
        } else {
            log.info("[generateSupplierDetail]常规安装价格orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
            detail.setSurveyFee(contractPrice.getSurveyPrice());
            detail.setSurveyFeeWithTax(contractPrice.getSurveyRatePrice());
            detail.setInstallFee(contractPrice.getInstallationPrice());
            detail.setInstallFeeWithTax(contractPrice.getInstallationRatePrice());
        }
        List<SupplierContractItem> items = supplierContractItemDao.selectPureByContractId(contract.getId());
        Map<String, SupplierContractItem> sciMap =
                items.stream().collect(Collectors.toMap(SupplierContractItem::getItemId, supplierContractItem -> supplierContractItem));
        // 查询安装报告物料
        List<OrderInstallItemVo> orderInstallItemList = orderInstallItemDao.selectByOrderIdAndType(order.getId(), 2);
        BigDecimal totalItemPrice = BigDecimal.ZERO;
        BigDecimal totalItemRatePrice = BigDecimal.ZERO;
        for (OrderInstallItemVo oii : orderInstallItemList) {
            String itemId = oii.getItemId();
            SupplierContractItem sci = sciMap.get(itemId);
            if (sci != null) {
                totalItemPrice = totalItemPrice.add(sci.getPrice().multiply(oii.getQunatity()));
                totalItemRatePrice = totalItemRatePrice.add(sci.getRatePrice().multiply(oii.getQunatity()));
            }
        }

        detail.setItemFee(totalItemPrice);
        detail.setItemFeeWithTax(totalItemRatePrice);
        // 计算报装费用
        if (orderInstall.getElectricExpand() != null && orderInstall.getElectricExpand().intValue() == 1
                && orderInstall.getExpandMethod() != null && orderInstall.getExpandMethod().intValue() == 1) {
            List<ExpandPrice> expandPrice = expandPriceDao.selectByRegionId(regionId);
            if (!expandPrice.isEmpty()) {
                detail.setExpandFee(expandPrice.get(0).getPrice() == null ? BigDecimal.ZERO : expandPrice.get(0).getPrice());
            }
        }
        detail.setTotalFee(detail.getSurveyFee().add(detail.getInstallFee()).add(detail.getItemFee()).add(detail.getExpandFee()));
        detail.setTotalFeeWithTax(detail.getSurveyFeeWithTax().add(detail.getInstallFeeWithTax()).add(detail.getItemFeeWithTax())
                .add(detail.getExpandFee()));
        return detail;
    }
}
