/**
 * Project Name: warehouse
 * File Name: TransferSingleServiceImpl
 * Package Name: com.starcharge.warehouse.service.single
 * Date: 2020/10/21 18:53
 * Copyright (c) 2020,All Rights Reserved.
 */
package com.boco.nbd.wios.manage.service.impl;

import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.enums.WareHouseOrderEnum;
import com.boco.nbd.wios.manage.mapper.extdb.*;
import com.boco.nbd.wios.manage.service.ISingleService;
import com.boco.nbd.wios.manage.util.CommonUtils;
import com.ihidea.core.support.exception.ServiceException;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 调拨单实现类
 * @author lvpeng
 * @date 2020/10/21 18:53
 */
@Component
public class TransferSingleServiceImpl implements ISingleService {

    @Autowired
    MaterialsService materialsService;

    @Resource
    SingleMapper singleMapper;

    @Autowired
    SingleMapperEx singleMapperEx;

    @Autowired
    WarehouseService warehouseService;

    @Resource
    WarehouseMaterialsMapper warehouseMaterialsMapper;

    @Resource
    MaterialsMapperEx materialsMapperEx;

    @Resource
    WarehouseMapper warehouseMapper;

    @Override
    public Integer saveSingle(SingleSaveQuery singleSaveQuery) {
        // 判断调出仓库及调入仓库是否都存在，不存在则提示
        if(StringUtils.isBlank(singleSaveQuery.getTransferIn()) && StringUtils.isBlank(singleSaveQuery.getTransferOut())){
            throw new ServiceException("11001",StringUtils.EMPTY);
        }

        if(singleSaveQuery.getTransferIn().equals(singleSaveQuery.getTransferOut())){
            throw  new ServiceException("调出仓库和调入仓库不可相同");
        }

        //Warehouse warehouse = warehouseService.qryWarehouseById(singleSaveQuery.getTransferIn());
        Warehouse warehouse = warehouseMapper.selectByPrimaryKey(singleSaveQuery.getTransferIn());
        if(warehouse == null){
            throw new ServiceException("仓库不存在");
        }
        //warehouse = warehouseService.qryWarehouseById(singleSaveQuery.getTransferOut());
        warehouse = warehouseMapper.selectByPrimaryKey(singleSaveQuery.getTransferOut());
        if(warehouse == null){
            throw new ServiceException("仓库不存在");
        }

        String singleId = singleSaveQuery.getAppkey() + CommonUtils.singleId();
        // 实列化单据对象
        Single single = new Single();
        single.setId(singleId);
        single.setAppkey(singleSaveQuery.getAppkey());
        single.setType(singleSaveQuery.getType());
        single.setReceiver(singleSaveQuery.getReceiver());
        single.setShippingAddress(singleSaveQuery.getShippingAddress());
        single.setContactNumber(singleSaveQuery.getContactNumber());
        single.setStatus(9);
        single.setInfo(singleSaveQuery.getInfo());
        single.setTransferIn(singleSaveQuery.getTransferIn());
        single.setTransferOut(singleSaveQuery.getTransferOut());
        single.setCreateTime(new Date());
        single.setCreateAccount(singleSaveQuery.getAccountId());
        single.setModifyAccount(singleSaveQuery.getAccountId());
        single.setModifyTime(new Date());

        List<SingleMaterialsEx> singleMaterialsList = new ArrayList<>();
        // 执行具体业务逻辑
        singleSaveQuery.getWarehouseMaterialsQueryList().forEach(warehouseMaterialsQuery -> {

            if(warehouseMaterialsQuery.getNumber().compareTo(BigDecimal.ZERO) <= 0){
                throw new ServiceException("物料数量必须大于0");
            }
            if(warehouseMaterialsQuery.getNumber().intValue() > 999999999) {
                throw new ServiceException("物料数量值过大");
            }
            // 获取仓库及物料的数量
            List<WarehouseMaterials> warehouseMaterialsList = materialsService.getWarehouseMaterials(singleSaveQuery.getTransferOut(),warehouseMaterialsQuery.getMaterialsId());
            // 如果该仓库下物料不存在则报错
            if(warehouseMaterialsList.size() <= 0){
                throw new ServiceException("materials_11001",StringUtils.EMPTY);
            }
            WarehouseMaterials warehouseMaterials = warehouseMaterialsList.get(0);
            // 判断调出仓库数量是否足够
            if((warehouseMaterials.getRemainingNumber() == null ? BigDecimal.ZERO : warehouseMaterials.getRemainingNumber()).compareTo(warehouseMaterialsQuery.getNumber()) < 0){
                //throw new ServiceException("materials_11001",StringUtils.EMPTY);
                throw new ServiceException("仓库数量不够,无法调出!");
            }
            // 实列化单据与物料关系
            SingleMaterialsEx singleMaterialsEx = new SingleMaterialsEx();
            singleMaterialsEx.setMaterialsId(warehouseMaterialsQuery.getMaterialsId());
            singleMaterialsEx.setSignalId(singleId);
            singleMaterialsEx.setQuantity(warehouseMaterialsQuery.getNumber());
            singleMaterialsEx.setTransferIn(singleSaveQuery.getTransferIn());
            singleMaterialsEx.setTransferOut(singleSaveQuery.getTransferOut());
            singleMaterialsList.add(singleMaterialsEx);

        });
        // 调出/调入仓库数量进行相减
        updateMaterialsNumber(singleMaterialsList);
        // 单据数据及单据物料数据进行新增
        singleMapper.insert(single);
        singleMapperEx.saveSingleMaterials(singleMaterialsList);
        return 1;
    }


    /**
     * 更新调入及调出物料数据
     * 计算逻辑
     *  调出仓库：总数量相减，已用数量不管，剩余数量相减
     *  调入仓库：总数量相加，已用数量不管，剩余数量相加
     */
    public void updateMaterialsNumber(List<SingleMaterialsEx> singleMaterialsExList){
        singleMaterialsExList.forEach(singleMaterialsEx -> {
            if(StringUtils.isNotBlank(singleMaterialsEx.getTransferOut())){
                // 调出仓库不为空执行数量逻辑处理
                materialsMapperEx.updateWarehouseMaterials(singleMaterialsEx.getTransferOut(),singleMaterialsEx.getMaterialsId(),singleMaterialsEx.getQuantity(), WareHouseOrderEnum.TRANSFER_SINGLE.getType());
            }
            if(StringUtils.isNotBlank(singleMaterialsEx.getTransferIn())){
                WarehouseMaterialsCriteria warehouseMaterialsCriteria = new WarehouseMaterialsCriteria();
                WarehouseMaterialsCriteria.Criteria criteria = warehouseMaterialsCriteria.createCriteria();
                criteria.andMaterialsIdEqualTo(singleMaterialsEx.getMaterialsId());
                criteria.andWarehouseIdEqualTo(singleMaterialsEx.getTransferIn());
                List<WarehouseMaterials> warehouseMaterialsList = warehouseMaterialsMapper.selectByExample(warehouseMaterialsCriteria);
                if(warehouseMaterialsList.size() > 0){
                    WarehouseMaterials warehouseMaterials = warehouseMaterialsList.get(0);
                    BigDecimal totalNum = (warehouseMaterials.getRemainingNumber() == null ? BigDecimal.ZERO : warehouseMaterials.getRemainingNumber()).add(singleMaterialsEx.getQuantity());
                    if(totalNum.intValue() > 999999999){
                        throw new ServiceException("物料数量值过大");
                    }
                    warehouseMaterials.setTotalNumber(totalNum);
                    warehouseMaterials.setRemainingNumber((warehouseMaterials.getRemainingNumber() == null ? BigDecimal.ZERO : warehouseMaterials.getRemainingNumber()).add(singleMaterialsEx.getQuantity()));
                    warehouseMaterialsMapper.updateByPrimaryKey(warehouseMaterials);
                }else {
                    // 调入仓库不为空执行数量逻辑处理
                    WarehouseMaterials warehouseMaterials = new WarehouseMaterials();
                    warehouseMaterials.setMaterialsId(singleMaterialsEx.getMaterialsId());
                    warehouseMaterials.setWarehouseId(singleMaterialsEx.getTransferIn());
                    warehouseMaterials.setTotalNumber(singleMaterialsEx.getQuantity());
                    warehouseMaterials.setHasNumber(BigDecimal.ZERO);
                    warehouseMaterials.setRemainingNumber(singleMaterialsEx.getQuantity());
                    materialsMapperEx.insertWarehouseMaterials(warehouseMaterials);
                }
            }
        });
    }
}
