package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.EnumUtil;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.dto.SingleSaveQuery;
import com.boco.nbd.wios.manage.entity.dto.*;
import com.boco.nbd.wios.manage.enums.WareHouseOrderEnum;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.boco.nbd.wios.manage.mapper.extdb.SingleMapperEx;
import com.boco.nbd.wios.manage.mapper.extdb.WarehouseLevenMapper;
import com.boco.nbd.wios.manage.mapper.extdb.WarehouseMapper;
import com.boco.nbd.wios.manage.mapper.extdb.WarehouseMapperEx;
import com.boco.nbd.wios.manage.service.ISingleService;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.SpringContextLoader;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import com.ihidea.core.util.JSONUtilsEx;
import com.ihidea.core.util.StringUtilsEx;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 仓库service
 *
 * @author
 */
@Service
public class WarehouseService {

    @Value("${appkey.warehouse}")
    private String appkey;


    @Autowired
    private RegionDao regionDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private OemContractItemDao ociDao;

    @Autowired
    private SupplierContractItemDao sciDao;

    @Autowired
    private WarehouseMapperEx warehouseMapperEx;

    @Autowired
    private WarehouseMapper warehouseMapper;

    @Autowired
    SingleMapperEx singleMapperEx;


    @Autowired
    MaterialsService materialsService;

    @Autowired
    WarehouseLevenMapper warehouseLevenMapper;

    private Logger logger = LoggerFactory.getLogger(getClass());


    /**
     * 新增/修改仓库等级
     *
     * @param warehouseLevenSaveQuery
     * @return
     */
    public void saveWarehouseLevel(WarehouseLevenSaveQuery warehouseLevenSaveQuery) {
        if (StringUtils.isEmpty(warehouseLevenSaveQuery.getAccountId())) {
            SessionInfo sessionInfo = ProcessUtil.getUserInfo();
            if (sessionInfo != null) {
                warehouseLevenSaveQuery.setAccountId(sessionInfo.getUserId());
            }
        }
        warehouseLevenSaveQuery.setAppkey(appkey);


        // 查询父级下同名是否存在
        if (warehouseLevenSaveQuery.getId() == null && warehouseMapperEx.getWarehouseLevenByNameCount(warehouseLevenSaveQuery.getParentId(), warehouseLevenSaveQuery.getName(), null) > 0) {
            throw new ServiceException("11003", org.apache.commons.lang3.StringUtils.EMPTY);
        }
        if (warehouseLevenSaveQuery.getId() != null && warehouseMapperEx.getWarehouseLevenByNameCount(null, warehouseLevenSaveQuery.getName(), warehouseLevenSaveQuery.getId()) > 0) {
            throw new ServiceException("11003", org.apache.commons.lang3.StringUtils.EMPTY);
        }
        // 查询父级下目前的等级，获取新增的等级ID
        int lastId = warehouseLevenSaveQuery.getId() != null ? warehouseLevenSaveQuery.getId() : warehouseMapperEx.getWarehouseLevenParentLastId(warehouseLevenSaveQuery.getParentId());
        // 获取等级的全路径code及名称
        Map<String, String> map = getWarehouseLevenPath(warehouseLevenSaveQuery.getParentId(), String.valueOf(lastId), warehouseLevenSaveQuery.getName());
        String pathCode = map.get(LambdaUtils.getFieldName(WarehouseLeven::getPathCode));
        String pathName = map.get(LambdaUtils.getFieldName(WarehouseLeven::getPathName));
        // 判断排序是否存在，如果不存在则获取当前数据的排序
        Integer seq = warehouseLevenSaveQuery.getSeq() != null ? warehouseLevenSaveQuery.getSeq() : warehouseMapperEx.getWarehouseLevenParentSeq(warehouseLevenSaveQuery.getParentId());
        // 进行新增或修改操作
        WarehouseLeven warehouseLeven = new WarehouseLeven();
        warehouseLeven.setId(lastId);
        warehouseLeven.setName(warehouseLevenSaveQuery.getName());
        warehouseLeven.setAppkey(warehouseLevenSaveQuery.getAppkey());
        warehouseLeven.setSeq(seq);
        warehouseLeven.setParentId(warehouseLevenSaveQuery.getParentId());
        warehouseLeven.setPathCode(pathCode);
        warehouseLeven.setPathName(pathName);
        warehouseLeven.setStatus(1);
        warehouseLeven.setCreateAccount(warehouseLevenSaveQuery.getAccountId());
        warehouseLeven.setCreateTime(new Date());
        warehouseLeven.setModifyAccount(warehouseLevenSaveQuery.getAccountId());
        warehouseLeven.setModifyTime(new Date());
        warehouseLeven.setMsg(warehouseLevenSaveQuery.getMsg());
        warehouseMapperEx.insertWarehouseLeven(warehouseLeven);
    }

    /**
     * 获取全路径
     *
     * @param parentId
     * @param pathCode
     * @param pathName
     * @return
     */
    private Map<String, String> getWarehouseLevenPath(Integer parentId, String pathCode, String pathName) {
        Map<String, String> resultMap = new HashMap<>(1);
        Map<String, Object> mapData = warehouseMapperEx.qryWarehouseLevenById(parentId);
        if (mapData != null) {
            Integer id = Integer.parseInt(String.valueOf(mapData.get("id")));
            String name = String.valueOf(mapData.get("name"));
            if (id == 0) {
                pathCode = id + "/" + pathCode;
                pathName = name + "/" + pathName;
                resultMap.put(LambdaUtils.getFieldName(WarehouseLeven::getPathCode), pathCode);
                resultMap.put(LambdaUtils.getFieldName(WarehouseLeven::getPathName), pathName);
                return resultMap;
            }
            Integer _parentId = Integer.parseInt(String.valueOf(mapData.get("parentId")));
            pathCode = id + "/" + pathCode;
            pathName = name + "/" + pathName;
            return getWarehouseLevenPath(_parentId, pathCode, pathName);
        } else {
            resultMap.put(LambdaUtils.getFieldName(WarehouseLeven::getPathCode), pathCode);
            resultMap.put(LambdaUtils.getFieldName(WarehouseLeven::getPathName), pathName);
            return resultMap;
        }
    }

    /**
     * 新增修改仓库
     *
     * @param warehouseSaveQuery
     * @return
     */
    public void saveWarehouse(WarehouseSaveQuery warehouseSaveQuery) {
        if (StringUtils.isEmpty(warehouseSaveQuery.getAccountId())) {
            SessionInfo sessionInfo = ProcessUtil.getUserInfo();
            if (sessionInfo != null) {
                warehouseSaveQuery.setAccountId(sessionInfo.getUserId());
            }
        }
        warehouseSaveQuery.setAppkey(appkey);


        if (warehouseSaveQuery.getName().length() > 20) {
            throw new ServiceException("仓库名称不能大于20字");
        }
        // 判断仓库名称是否存在
        WarehouseCriteria warehouseCriteria = new WarehouseCriteria();
        WarehouseCriteria.Criteria criteria = warehouseCriteria.createCriteria();
        criteria.andNameEqualTo(warehouseSaveQuery.getName());
        if (org.apache.commons.lang3.StringUtils.isNotBlank(warehouseSaveQuery.getId())) {
            criteria.andIdNotEqualTo(warehouseSaveQuery.getId());
        }
        List<Warehouse> warehouseList = warehouseMapper.selectByExample(warehouseCriteria);
        if (CollUtil.isNotEmpty(warehouseList)) {
            throw new ServiceException("仓库名称已存在");
        }

        Warehouse warehouse = new Warehouse();
        BeanUtils.copyProperties(warehouseSaveQuery, warehouse);
        if (org.apache.commons.lang3.StringUtils.isBlank(warehouse.getId())) {
            warehouse.setId(StringUtilsEx.getUUID());
        }
        warehouse.setCreateAccount(warehouseSaveQuery.getAccountId());
        warehouse.setCreateTime(new Date());
        warehouse.setModifyAccount(warehouseSaveQuery.getAccountId());
        warehouse.setModifyTime(new Date());
        warehouse.setStatus(1);
        warehouseMapperEx.insertWarehouse(warehouse);
    }


    /**
     * 查询仓库
     *
     * @param warehouseQuery2
     * @return
     */
    public List<WarehouseDTO2> qryWarehouseDetailList(WarehouseQuery2 warehouseQuery2) {
        SessionInfo user = ProcessUtil.getUserInfo();
        if (user != null) {

            String installSupplierId = user.getAttribute().get(LambdaUtils.getFieldName(Order::getInstallSupplierId));
            warehouseQuery2.setServiceProvider(installSupplierId);
        }

        if (!StringUtils.isEmpty(warehouseQuery2.getServiceProviderName())) {
            SupplierBo condition = new SupplierBo();
            condition.setName(warehouseQuery2.getServiceProviderName());
            List<SupplierVo> supplierList = supplierDao.getList(condition);
            if (!supplierList.isEmpty()) {
                List<String> supplierIdList = supplierList.stream().map(SupplierVo::getId).collect(Collectors.toList());
                warehouseQuery2.setServiceProvider(StringUtils.collectionToCommaDelimitedString(supplierIdList));
            } else {
                return new ArrayList<>();
            }
        }

        List<WarehouseDTO2> returnList = new ArrayList<>();
        List<WarehouseDTO> result = qryWarehouseList(warehouseQuery2);
        List<Region> regionList = regionDao.selectAll(null);
        Map<Integer, Region> regionMap = regionList.stream().collect(Collectors.toMap(Region::getId, region -> region));
        List<Supplier> supplierList = supplierDao.selectAll();
        Map<String, String> supplierMap = supplierList.stream().collect(Collectors.toMap(Supplier::getId, Supplier::getName));
        result.stream().forEach(item -> {
            WarehouseDTO2 dto = new WarehouseDTO2();
            BeanUtils.copyProperties(item, dto);
            Region region = regionMap.get(Integer.valueOf(dto.getArea()));
            if (region != null) {
                dto.setAreaName(region.getName());
                dto.setAreaPathCode(region.getIdTree().replaceAll(CamsConstant.UNDER_LINE, "\\/"));
            }
            dto.setProviderName(supplierMap.get(dto.getServiceProvider()));
            returnList.add(dto);
        });
        return returnList;
    }

    public List<WarehouseDTO> qryWarehouseList(WarehouseQuery warehouseQuery) {
        warehouseQuery.setAppkey(appkey);
        logger.info("qryWarehouseList请求数据:{}", JSONUtilsEx.serialize(warehouseQuery));
        System.out.println("分页"+warehouseQuery.getPage());
       if (warehouseQuery.getIsWarehousePage()==1){
           PageLimitHolderFilter.setContext(warehouseQuery.getPage(), warehouseQuery.getPagecount(), 0);
       }
        List<String> serviceProviders = new ArrayList<>(1);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(warehouseQuery.getServiceProvider())) {
            serviceProviders = Arrays.asList(warehouseQuery.getServiceProvider().split(CamsConstant.COMMA));
        }
        List<WarehouseEx> warehouseList = warehouseMapperEx.qryWarehouseList(warehouseQuery.getId(), warehouseQuery.getAppkey(), warehouseQuery.getWarehouseLevenId(),
                warehouseQuery.getName(), warehouseQuery.getIsEnable(), serviceProviders, warehouseQuery.getArea(),
                warehouseQuery.getWarehouseAttribution(),warehouseQuery.getCreateAccount(),warehouseQuery.getSuperiorWarehouseId());


        BaseResponse<List<WarehouseDTO>> response;

        if (warehouseList == null) {
            response = new BaseResponse<>(new ArrayList<>());
        } else {
            List<WarehouseDTO> result = new ArrayList<>();
            for (WarehouseEx warehouseEx : warehouseList) {
                //获取安装服务商的名字
                Supplier supplier=supplierDao.selectById(Integer.valueOf(warehouseEx.getServiceProvider()));
                if (supplier != null){
                    warehouseEx.setServiceProviderName(supplier.getName());
                }
                //获取上级仓库的名字
                WarehouseEx warehouseEx1=warehouseMapperEx.wareHouseBySuperiorWarehouseId(warehouseEx.getSuperiorWarehouseId());
                if (warehouseEx1 !=null){
                    warehouseEx.setSuperiorWarehouseName(warehouseEx1.getName());
                }
                WarehouseDTO warehouseDTO = JSONUtilsEx.deserialize(JSONUtilsEx.serialize(warehouseEx), WarehouseDTO.class);
                if (warehouseEx.getParentId() == null) {
                    warehouseDTO.setIsDel(0);
                } else {
                    warehouseDTO.setIsDel(1);
                }
                result.add(warehouseDTO);
            }
            response = new BaseResponse<>(result);
        }


        if (response.success()) {
            PageLimitHandle(response.getPageLimit());
            return response.getData();
        } else {
            logger.error("qryWarehouseList调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }

    /**
     * 查询仓库等级
     *
     * @param name
     * @param serviceProvider
     * @param parentId
     * @return
     */
    public List<WarehouseLevelDTO2> qryWarehouseLevel(String serviceProvider, Integer parentId, String name) {
        BaseResponse<List<WarehouseLevenDTO>> response = qryWarehouseLeven(null, appkey, serviceProvider, parentId, name);
        if (response.success()) {
            List<WarehouseLevenDTO> warehouseLevelList = response.getData();

            // 根据条件查询是补全上一级等级

            Set<String> pathCodeSet = new HashSet<>();
            if (!StringUtils.isEmpty(name)) {
                for (WarehouseLevenDTO dto : warehouseLevelList) {
                    String pathCode = dto.getPathCode();
                    String[] pathCodeArr = pathCode.split("\\/");
                    for (int i = 0; i < pathCodeArr.length - 1; i++) {
                        pathCodeSet.add(pathCodeArr[i]);
                    }
                }
                Iterator<String> it = pathCodeSet.iterator();
                while (it.hasNext()) {
                    String pathCode = it.next();
                    BaseResponse<List<WarehouseLevenDTO>> response2 = qryWarehouseLeven(Integer.valueOf(pathCode), appkey, null, null, null);
                    if (response2.success()) {
                        if (!response2.getData().isEmpty()) {
                            warehouseLevelList.add(response2.getData().get(0));
                        }
                    }
                }
                // 从小到大排序（总仓要放第一个）
                warehouseLevelList =
                        warehouseLevelList.stream().sorted(Comparator.comparing(WarehouseLevenDTO::getId)).collect(Collectors.toList());
            }


            List<WarehouseLevelDTO2> returnList = new ArrayList<>();
            for (WarehouseLevenDTO dto : warehouseLevelList) {
                String pId = dto.getParentId();
                if (!StringUtils.isEmpty(pId)) {
                    WarehouseLevelDTO2 dto2 = getLevelVo(returnList, pId);
                    if (dto2 == null) {
                        continue;
                    }
                    WarehouseLevelDTO2 dto2New = new WarehouseLevelDTO2();
                    BeanUtils.copyProperties(dto, dto2New);
                    List<WarehouseLevelDTO2> children = dto2.getChildren();
                    if (children == null) {
                        children = new ArrayList<>();
                    }
                    children.add(dto2New);
                    dto2.setChildren(children);
                } else {
                    WarehouseLevelDTO2 dto2 = new WarehouseLevelDTO2();
                    BeanUtils.copyProperties(dto, dto2);
                    returnList.add(dto2);
                }
            }
            return returnList;
        } else {
            logger.error("qryWarehouseLevel调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }

    public WarehouseLevelDTO2 getLevelVo(List<WarehouseLevelDTO2> returnList, String id) {
        for (WarehouseLevelDTO2 dto : returnList) {
            if (dto.getId().equals(id)) {
                return dto;
            }
            List<WarehouseLevelDTO2> children = dto.getChildren();
            if (children != null) {
                WarehouseLevelDTO2 dto2 = getLevelVo(children, id);
                if (dto2 != null) {
                    return dto2;
                }
            }
        }
        return null;
    }


    private BaseResponse<List<WarehouseLevenDTO>> qryWarehouseLeven(Integer id, String appkey, String serviceProvider, Integer parentId, String name) {
        List<WarehouseLeven> warehouseLevensList = warehouseMapperEx.qryWarehouseLeven(id, appkey, serviceProvider, parentId, name);
        if (warehouseLevensList == null) {
            return new BaseResponse<>(null);
        }
        List<WarehouseLevenDTO> result = new ArrayList<>(1);
        warehouseLevensList.forEach(warehouseLeven -> {
            WarehouseLevenDTO warehouseLevenDTO = JSONUtilsEx.deserialize(JSONUtilsEx.serialize(warehouseLeven), WarehouseLevenDTO.class);
            if (warehouseLeven.getParentId() == null) {
                warehouseLevenDTO.setIsDel(0);
            } else {
                warehouseLevenDTO.setIsDel(1);
            }
            result.add(warehouseLevenDTO);
        });
        return new BaseResponse<>(result);
    }

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    public WarehouseLevenDTO qryLevelDetail(Integer id) {
        BaseResponse<List<WarehouseLevenDTO>> response = qryWarehouseLeven(id, appkey, null, null, null);


        if (response.success()) {
            List<WarehouseLevenDTO> list = response.getData();
            return list.isEmpty() ? null : list.get(0);
        } else {
            logger.error("qryWarehouseLeven调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }

    /**
     * 新增调拨单
     *
     * @param singleSaveQuery
     * @return
     */
    public void saveSingle(SingleSaveQuery2 singleSaveQuery) {
        if (StringUtils.isEmpty(singleSaveQuery.getAccountId())) {
            SessionInfo sessionInfo = ProcessUtil.getUserInfo();
            if (sessionInfo != null) {
                String accountName = sessionInfo.getUserName();
                String oemId = sessionInfo.getAttribute().get("oemId");
                String installSupplierId = sessionInfo.getAttribute().get("installSupplierId");
                if (!StringUtils.isEmpty(oemId)) {
                    Oem oem = oemDao.selectById(Integer.valueOf(oemId));
                    if (oem != null) {
                        accountName = oem.getName();
                    }
                }
                if (!StringUtils.isEmpty(installSupplierId)) {
                    Supplier supplier = supplierDao.selectById(Integer.valueOf(installSupplierId));
                    if (supplier != null) {
                        accountName = supplier.getName();
                    }
                }
                singleSaveQuery.setAccountId(accountName);
            }
        }
        List<WarehouseMaterialsQuery> list = WarehouseMaterialsQuery.jsonHandle(singleSaveQuery.getWarehouseMaterialsQueryJson());
        singleSaveQuery.setWarehouseMaterialsQueryList(list);
        singleSaveQuery.setAppkey(appkey);
        SingleSaveQuery query = new SingleSaveQuery();
        BeanUtils.copyProperties(singleSaveQuery, query);
        getCameraControl(singleSaveQuery.getType()).saveSingle(singleSaveQuery);
    }

    /**
     * 获取控制的service
     *
     * @param type
     * @return
     */
    private ISingleService getCameraControl(Integer type) {
        WareHouseOrderEnum orderEnum = EnumUtil.likeValueOf(WareHouseOrderEnum.class, type);
        if (orderEnum == null) {
            throw new ServiceException("未知的单据类型");
        }
        Class<?> implClz;
        switch (orderEnum) {
            case TRANSFER_SINGLE:
                implClz = TransferSingleServiceImpl.class;
                break;
            case WAREHOUSING_SINGLE:
                implClz = WarehousingSingleServiceImpl.class;
                break;
            case WITHDRAWAL_SINGLE:
                implClz = WithdrawalSingleServiceImpl.class;
                break;
            case OUT_SINGLE:
                implClz = OutStockSingleServiceImpl.class;
                break;
            default:
                throw new ServiceException("未知的单据类型");
        }
        return (ISingleService) SpringContextLoader.getBean(implClz);
    }


    /**
     * 查询调拨单
     *
     * @param singleQuery
     * @return
     */
    public List<SingleDTO> qrySingal(SingleQuery singleQuery) {
        SessionInfo user = ProcessUtil.getUserInfo();
        if (user != null) {
            String supplierId = user.getAttribute().get("installSupplierId");
            if (!StringUtils.isEmpty(supplierId)) {
                singleQuery.setServiceProvider(supplierId);
            }
        }
        singleQuery.setAppkey(appkey);
        BaseResponse<List<SingleDTO>> response = new BaseResponse<>(null);
        PageLimitHolderFilter.setContext(singleQuery.getPage(), singleQuery.getPagecount(), 0);
        List<String> serviceProviders = new ArrayList<>(1);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(singleQuery.getServiceProvider())) {
            serviceProviders = Arrays.asList(singleQuery.getServiceProvider().split(",", -1));
        }
        List<SingleQo> singleQoList = singleMapperEx.qrySingle(singleQuery.getAppkey(), singleQuery.getId(), singleQuery.getType(), singleQuery.getTransferIn(), singleQuery.getTransferOut(), singleQuery.getMaterialsCode(), singleQuery.getShippingAddress(), singleQuery.getContactNumber(), singleQuery.getReceiver(), serviceProviders);
        List<SingleDTO> result = new ArrayList<>(1);
        for (SingleQo singleQo : singleQoList) {
            result.add(JSONUtilsEx.deserialize(JSONUtilsEx.serialize(singleQo), SingleDTO.class));
        }
        response = new BaseResponse<>(result);
        if (response.success()) {
            PageLimitHandle(response.getPageLimit());
        } else {
            logger.error("qrySingal调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();
    }

    /**
     * pageLimit 处理代码合并
     *
     * @param pageLimit
     */
    private void PageLimitHandle(PageLimit pageLimit) {
        if (pageLimit != null) {
            PageLimitHolderFilter.setContext(pageLimit.getCurrentPageNo(), pageLimit.getPageLength(), pageLimit.getTotalCount());
        }
        PageLimit pageLimit1 = PageLimitHolderFilter.getContext();
        if (pageLimit1 != null) {
            pageLimit1.setLimited(true);
        }
    }

    /**
     * 查询单据物料
     *
     * @param singleId
     * @param page
     * @param pagecount
     * @return
     */
    public List<SingleMaterialsDTO> qrySingleMaterialsBySingleId(String singleId, Integer page, Integer pagecount) {
        BaseResponse<List<SingleMaterialsDTO>> response = new BaseResponse<>(null);
        if (page != null || pagecount != null) {
            PageLimitHolderFilter.setContext(page, pagecount, 0);
        }

        List<SingleMaterialsQo> singleMaterialsQoList = singleMapperEx.qrySingleMaterialsBySingleId(singleId);
        List<SingleMaterialsDTO> result = new ArrayList<>(1);
        for (SingleMaterialsQo singleMaterialsQo : singleMaterialsQoList) {
            result.add(JSONUtilsEx.deserialize(JSONUtilsEx.serialize(singleMaterialsQo), SingleMaterialsDTO.class));
        }
        response = new BaseResponse<>(result);


        if (response.success()) {
            PageLimitHandle(response.getPageLimit());
        } else {
            logger.error("qrySingleMaterialsBySingleId调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();
    }

    /**
     * 新增物料
     *
     * @param materialsListQueryJson
     * @return
     */
    public void saveMaterials(String materialsListQueryJson) {
        MaterialsListQuery materialsListQuery = new MaterialsListQuery();
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null) {
            materialsListQuery.setAccountId(sessionInfo.getUserId());
        }
        materialsListQuery.setAppkey(appkey);
        List<MaterialsSaveQuery> list = MaterialsSaveQuery.jsonHandle(materialsListQueryJson);
        materialsListQuery.setMaterialsSaveQueryList(list);
        materialsService.saveMaterials(materialsListQuery.getAppkey(), materialsListQuery.getMaterialsSaveQueryList(), materialsListQuery.getAccountId());
        BaseResponse<Void> response = new BaseResponse<>(null);
        if (!response.success()) {
            logger.error("saveMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }

    /**
     * 新增物料
     *
     * @return
     */
    public void saveMaterials(MaterialsSaveQuery materialsSaveQuery) {
        MaterialsListQuery materialsListQuery = new MaterialsListQuery();
        materialsListQuery.setAccountId("sap");
        materialsListQuery.setAppkey(appkey);
        List<MaterialsSaveQuery> list = new ArrayList<>();
        list.add(materialsSaveQuery);
        materialsListQuery.setMaterialsSaveQueryList(list);
        materialsService.saveMaterials(materialsListQuery.getAppkey(), materialsListQuery.getMaterialsSaveQueryList(), materialsListQuery.getAccountId());
    }

    /**
     * 更新单个物料
     *
     * @param materialsSaveQuery
     */
    public void updateMaterial(MaterialsSaveQuery materialsSaveQuery) {
        MaterialsListQuery materialsListQuery = new MaterialsListQuery();
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null) {
            materialsListQuery.setAccountId(sessionInfo.getUserId());
        }
        materialsListQuery.setAppkey(appkey);
        List<MaterialsSaveQuery> list = new ArrayList<>();
        list.add(materialsSaveQuery);
        materialsListQuery.setMaterialsSaveQueryList(list);
        materialsService.saveMaterials(materialsListQuery.getAppkey(), materialsListQuery.getMaterialsSaveQueryList(), materialsListQuery.getAccountId());
    }

    /**
     * 查询物料
     *
     * @param materialsQuery
     * @return
     */
    public List<MaterialsDTO> qryMaterials(MaterialsQuery materialsQuery) {

        materialsQuery.setAppkey(appkey);
        BaseResponse<List<MaterialsDTO>> response = new BaseResponse<>(null);

        // 判断是否有分页参数
        if (materialsQuery.getPage() != null || materialsQuery.getPagecount() != null) {
            PageLimitHolderFilter.setContext(materialsQuery.getPage(), materialsQuery.getPagecount(), 0);
        }
        List<MaterialsEx> materialsExList = materialsService.qryMaterialsList(materialsQuery);
        List<MaterialsDTO> result = new ArrayList<>(1);
        materialsExList.forEach(materialsEx -> {
            result.add(JSONUtilsEx.deserialize(JSONUtilsEx.serialize(materialsEx), MaterialsDTO.class));
        });
        response = new BaseResponse<>(result);
        if (response.success()) {
            PageLimitHandle(response.getPageLimit());
        } else {
            logger.error("qryMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();

    }

    /**
     * 删除物料
     *
     * @param materialsId
     * @return
     */
    public void delMaterials(String materialsId) {
        List<MaterialsDTO> dtoList = getStockMaterialById(null, materialsId);
        if (!dtoList.isEmpty()) {
            throw new ServiceException("物料已入库，无法删除");
        }
        List<OemContractItem> ociList = ociDao.selectByItemId(materialsId);
        if (ociList.isEmpty()) {
            List<SupplierContractItem> sciList = sciDao.selectByItemId(materialsId);
            if (!sciList.isEmpty()) {
                throw new ServiceException("物料已被合同使用，无法删除");
            }
        } else {
            throw new ServiceException("物料已被合同使用，无法删除");
        }
        materialsService.delMaterials(materialsId);
    }

    public List<MaterialsDTO> qryMaterialsCount(String materialsName, String materialsCode, String regularModel, Integer type) {

        BaseResponse<List<MaterialsDTO>> response = new BaseResponse<>(null);
        List<MaterialsEx> materialsExList = materialsService.qryMaterialsCount(appkey, materialsName, materialsCode, regularModel, type);
        List<MaterialsDTO> result = new ArrayList<>(1);
        materialsExList.forEach(materialsEx -> {
            result.add(JSONUtilsEx.deserialize(JSONUtilsEx.serialize(materialsEx), MaterialsDTO.class));
        });
        response = new BaseResponse<>(result);


        if (response.success()) {
            return response.getData();
        } else {
            logger.error("qryMaterialsCount调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }

    /**
     * 根据id删除仓库
     *
     * @param id
     */
    public void delWarehouseById(String id) {
        //仓库有子仓库则无法删除
        int countId=warehouseMapper.selectBySuperiorWarehouseId(id);
        if (countId>0){
            throw new ServiceException("仓库下有子仓库，无法删除");
        }
        List<MaterialsDTO> materialList = getStockMaterialByWId(id);
        if (!materialList.isEmpty()) {
            BigDecimal count = materialList.stream().filter(m -> m.getRemainingNumber() != null).map(MaterialsDTO::getRemainingNumber)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            if (count.compareTo(BigDecimal.ZERO) > 0) {
                throw new ServiceException("仓库下已有库存，无法删除");
            }
        }
        Warehouse warehouse = warehouseMapper.selectByPrimaryKey(id);
        if (warehouse == null) {
            throw new ServiceException("11004", org.apache.commons.lang3.StringUtils.EMPTY);
        }
        // 判断仓库是否存在物料
        List<WarehouseMaterials> warehouseMaterialsList = materialsService.getWarehouseMaterials(id, null);
        if (warehouseMaterialsList == null) {
            throw new ServiceException("11004", org.apache.commons.lang3.StringUtils.EMPTY);
        }
        warehouse.setStatus(0);
        warehouseMapper.updateByPrimaryKey(warehouse);

        BaseResponse<Void> response = new BaseResponse<>(null);


        if (response.success()) {
        } else {
            logger.error("delWarehouseById调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }

    /**
     * 根据id删除仓库等级
     *
     * @param id
     */
    public void delWarehouseLevelById(Integer id) {
        // 查询数据，如果父节点是空则提示不能删除
        WarehouseLeven warehouseLeven = warehouseLevenMapper.selectByPrimaryKey(id);
        if (warehouseLeven == null || warehouseLeven.getParentId() == null) {
            throw new ServiceException("11004", org.apache.commons.lang3.StringUtils.EMPTY);
        }
        // 如果仓库等级下有下级不能删除
        List<WarehouseLeven> warehouseLevenList = warehouseMapperEx.qryWarehouseLeven(null, appkey, null, id, null);

        if (CollUtil.isNotEmpty(warehouseLevenList)) {
            throw new ServiceException("11005", org.apache.commons.lang3.StringUtils.EMPTY);
        }
        // 如果仓库等级下有仓库不能删除
        WarehouseQuery warehouseQuery = new WarehouseQuery();
        warehouseQuery.setAppkey(appkey);
        warehouseQuery.setWarehouseLevenId(id.toString());
        List<String> serviceProviders = new ArrayList<>(1);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(warehouseQuery.getServiceProvider())) {
            serviceProviders = Arrays.asList(warehouseQuery.getServiceProvider().split(CamsConstant.COMMA));
        }
        List<WarehouseEx> warehouseList = warehouseMapperEx.qryWarehouseList(warehouseQuery.getId(), warehouseQuery.getAppkey(), warehouseQuery.getWarehouseLevenId(), warehouseQuery.getName(), warehouseQuery.getIsEnable(), serviceProviders, warehouseQuery.getArea(), warehouseQuery.getWarehouseAttribution(),warehouseQuery.getCreateAccount(),warehouseQuery.getSuperiorWarehouseId());


        if (CollUtil.isNotEmpty(warehouseList)) {
            throw new ServiceException("11006", org.apache.commons.lang3.StringUtils.EMPTY);
        }
        warehouseLeven.setStatus(0);
        warehouseLevenMapper.updateByPrimaryKey(warehouseLeven);
    }

    /**
     * 组装查询数据
     *
     * @param id
     * @return
     */
    public List<WarehouseSelectorData> combineSelectorData(Integer id, Integer type) {
        SessionInfo user = ProcessUtil.getUserInfo();
        String installSupplierId = null;
        if (user != null) {
            installSupplierId = user.getAttribute().get("installSupplierId");
        }
        BaseResponse<List<WarehouseLevenDTO>> levelResponse = null;
        if (id == null) {
            levelResponse = qryWarehouseLeven2(0, appkey, null, null, null);
        } else {
            levelResponse = qryWarehouseLeven2(null, appkey, null, id, null);
        }
        WarehouseQuery warehouseQuery = new WarehouseQuery();
        warehouseQuery.setWarehouseLevenId(id + "");
        warehouseQuery.setAppkey(appkey);
        warehouseQuery.setIsEnable(1);
        warehouseQuery.setPage(1);
        warehouseQuery.setPagecount(1000);
        if (!StringUtils.isEmpty(installSupplierId)) {
            warehouseQuery.setServiceProvider(installSupplierId);
        }
        PageLimitHolderFilter.setContext(warehouseQuery.getPage(), warehouseQuery.getPagecount(), 0);
        List<String> serviceProviders = new ArrayList<>(1);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(warehouseQuery.getServiceProvider())) {
            serviceProviders = Arrays.asList(warehouseQuery.getServiceProvider().split(CamsConstant.COMMA));
        }
        List<WarehouseEx> warehouseList = warehouseMapperEx.qryWarehouseList(warehouseQuery.getId(), warehouseQuery.getAppkey(), warehouseQuery.getWarehouseLevenId(), warehouseQuery.getName(), warehouseQuery.getIsEnable(), serviceProviders, warehouseQuery.getArea(), warehouseQuery.getWarehouseAttribution(),warehouseQuery.getCreateAccount(),warehouseQuery.getSuperiorWarehouseId());

        BaseResponse<List<WarehouseDTO>> warehouseResponse = new BaseResponse<>(new ArrayList<>());
        List<WarehouseDTO> result = new ArrayList<>();
        for (WarehouseEx warehouseEx : warehouseList) {
            WarehouseDTO warehouseDTO = JSONUtilsEx.deserialize(JSONUtilsEx.serialize(warehouseEx), WarehouseDTO.class);
            if (warehouseEx.getParentId() == null) {
                warehouseDTO.setIsDel(0);
            } else {
                warehouseDTO.setIsDel(1);
            }
            result.add(warehouseDTO);
        }
        warehouseResponse = new BaseResponse<>(result);


        List<WarehouseSelectorData> returnList = new ArrayList<>();

        levelResponse.getData().stream().forEach(item -> {
            WarehouseSelectorData data = new WarehouseSelectorData();
            data.setName(item.getName());
            data.setId(item.getId());
            data.setLevelOr(true);
            if (item.getId().equals("0") || type == null) {
                returnList.add(data);
            }
        });

        warehouseResponse.getData().stream().forEach(item -> {
            WarehouseSelectorData data = new WarehouseSelectorData();
            data.setName(item.getName());
            data.setLevelOr(false);
            data.setId(item.getId());
            returnList.add(data);
        });
        return returnList;
    }

    private BaseResponse<List<WarehouseLevenDTO>> qryWarehouseLeven2(Integer id, String appkey, String serviceProvider, Integer parentId, String name) {
        List<WarehouseLeven> warehouseLevensList = warehouseMapperEx.qryWarehouseLeven(id, appkey, serviceProvider, parentId, name);
        if (warehouseLevensList == null) {
            return new BaseResponse<>(null);
        }
        List<WarehouseLevenDTO> result = new ArrayList<>(1);
        warehouseLevensList.forEach(warehouseLeven -> {
            WarehouseLevenDTO warehouseLevenDTO = JSONUtilsEx.deserialize(JSONUtilsEx.serialize(warehouseLeven), WarehouseLevenDTO.class);
            if (warehouseLeven.getParentId() == null) {
                warehouseLevenDTO.setIsDel(0);
            } else {
                warehouseLevenDTO.setIsDel(1);
            }
            result.add(warehouseLevenDTO);
        });
        return new BaseResponse<>(result);
    }

    /**
     * 启用/禁用
     *
     * @param id
     * @param enable
     */
    public void updateEnable(String id, Integer enable) {
        if (enable != null && enable.intValue() == 0) {
            List<MaterialsDTO> materialList = getStockMaterialByWId(id);
            if (!materialList.isEmpty()) {
                BigDecimal count = materialList.stream().filter(m -> m.getRemainingNumber() != null).map(MaterialsDTO::getRemainingNumber)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                if (count.compareTo(BigDecimal.ZERO) > 0) {
                    throw new ServiceException("仓库下已有库存，无法禁用");
                }
            }
        }

        warehouseMapperEx.updateWarehouseisEnabled(id, enable);
    }

    /**
     * 根据物料id查询详情
     *
     * @param id
     * @return
     */
    public MaterialsDTO getMaterialById(String id) {
        MaterialsQuery materialsQuery = new MaterialsQuery();
        materialsQuery.setId(id);
        materialsQuery.setPage(1);
        materialsQuery.setPagecount(1);
        List<MaterialsDTO> list = qryMaterials(materialsQuery);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据物料id及仓库id查询库存详情
     *
     * @param id
     * @return
     */
    public List<MaterialsDTO> getStockMaterialById(String warehouseId, String id) {
        MaterialsQuery materialsQuery = new MaterialsQuery();
        materialsQuery.setId(id);
        if (!StringUtils.isEmpty(warehouseId)) {
            materialsQuery.setWarehouseId(warehouseId);
        }
        materialsQuery.setPage(1);
        materialsQuery.setPagecount(1000);
        materialsQuery.setIsBindWareHouse(1);
        return qryStockMaterials(materialsQuery);
    }

    /**
     * 根据仓库id查询物料库存
     *
     * @param warehouseId
     * @return
     */
    public List<MaterialsDTO> getStockMaterialByWId(String warehouseId) {
        MaterialsQuery materialsQuery = new MaterialsQuery();
        materialsQuery.setWarehouseId(warehouseId);
        materialsQuery.setPage(1);
        materialsQuery.setPagecount(1000);
        materialsQuery.setIsBindWareHouse(1);
        return qryStockMaterials(materialsQuery);
    }

    /**
     * 查询库存物料
     *
     * @param materialsQuery
     * @return
     */
    public List<MaterialsDTO> qryStockMaterials(MaterialsQuery materialsQuery) {
        SessionInfo user = ProcessUtil.getUserInfo();
        if (user != null && StringUtils.isEmpty(materialsQuery.getWarehouseId())) {
            String supplierId = user.getAttribute().get("installSupplierId");
            if (!StringUtils.isEmpty(supplierId)) {
                WarehouseQuery warehouseQuery = new WarehouseQuery();
                warehouseQuery.setPage(1);
                warehouseQuery.setPagecount(1000);
                warehouseQuery.setServiceProvider(supplierId);
                warehouseQuery.setIsEnable(1);
                List<WarehouseDTO> warehouseList = qryWarehouseList(warehouseQuery);
                List<String> warehouseIds = warehouseList.stream().map(WarehouseDTO::getId).collect(Collectors.toList());
                if (!warehouseIds.isEmpty()) {
                    materialsQuery.setWarehouseId(StringUtils.collectionToCommaDelimitedString(warehouseIds));
                } else {
                    return new ArrayList<>();
                }
            }
        }
        materialsQuery.setAppkey(appkey);
        logger.info("qryStockMaterials请求数据:{}", JSONUtilsEx.serialize(materialsQuery));
        BaseResponse<List<MaterialsDTO>> response = new BaseResponse<>(null);
        // 判断是否有分页参数
        if (materialsQuery.getPage() != null || materialsQuery.getPagecount() != null) {
            PageLimitHolderFilter.setContext(materialsQuery.getPage(), materialsQuery.getPagecount(), 0);
        }
        List<MaterialsEx> materialsExList = materialsService.qryMaterialsStockList(materialsQuery);
        List<MaterialsDTO> result = new ArrayList<>(1);
        materialsExList.forEach(materialsEx -> result.add(JSONUtilsEx.deserialize(JSONUtilsEx.serialize(materialsEx), MaterialsDTO.class)));

        response = new BaseResponse<>(result);


        if (response.success()) {
            PageLimitHandle(response.getPageLimit());
        } else {
            logger.error("qryStockMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();
    }
}
