package com.boco.nbd.wios.score.controller;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.PageData;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.common.ResponseStatus;
import com.boco.nbd.wios.flow.entity.po.OrderPO;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.score.entity.po.ScoreConfigPO;
import com.boco.nbd.wios.score.entity.po.ScoreSupplierPO;
import com.boco.nbd.wios.score.entity.po.SupplierConfigPO;
import com.boco.nbd.wios.score.entity.qo.ScoreItemQo;
import com.boco.nbd.wios.score.entity.qo.SupplierConfigQo;
import com.boco.nbd.wios.score.entity.qo.SupplierScoreQo;
import com.boco.nbd.wios.score.entity.vo.ScoreConfigVO;
import com.boco.nbd.wios.score.entity.vo.ScoreSupplierItemVO;
import com.boco.nbd.wios.score.entity.vo.ScoreSupplierVO;
import com.boco.nbd.wios.score.entity.vo.SupplierConfigVO;
import com.boco.nbd.wios.score.service.IScoreSupplierService;
import com.ihidea.core.support.session.SessionInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author:cao hai
 * @date:2022/10/11 14:16
 * @version:V1.0
 * @description:评分管理
 * @modify:
 */
@RestController
@Slf4j
@RequestMapping("/scoreManager")
@Api(tags = "评分管理")
public class ScoreManagerController {

    @Autowired
    private IScoreSupplierService scoreSupplierService;

    @ApiOperation("保存服务商配置")
    @PostMapping("/saveSupplierConfig")
    public ResponseMessage2<Boolean> saveSupplierConfig(@RequestBody SupplierConfigVO vo) {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            ResponseMessage2.Failed(ProcessUtil.RE_LOGIN);
            return ResponseMessage2.Failed(ResponseStatus.ForbiddenAccess.getIndex(), ProcessUtil.RE_LOGIN);
        }
        vo.setUserName(currentUser.getUserName());
        try {
            return scoreSupplierService.saveSupplierConfig(vo);
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }

    @ApiOperation("获取服务商配置(服务商账号只能看自己,supplierId为空的时候后端判断用户权限，不为空不处理)")
    @PostMapping("/getSupplierConfig")
    public ResponseMessage2<PageData<SupplierConfigPO>> getSupplierConfig(@RequestBody SupplierConfigQo qo) {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            ResponseMessage2.Failed(ProcessUtil.RE_LOGIN);
            return ResponseMessage2.Failed(ResponseStatus.ForbiddenAccess.getIndex(), ProcessUtil.RE_LOGIN);
        }
        try {
            String installSupplierId = LambdaUtils.getFieldName(OrderPO::getInstallSupplierId);
            if (qo.getSupplierId() == null && StringUtils.isEmpty(currentUser.getAttribute().get(installSupplierId))) {
                qo.setSupplierId(Convert.toLong(currentUser.getAttribute().get(installSupplierId)));
            }
            return ResponseMessage2.Success2(scoreSupplierService.getSupplierConfig(qo));
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }

    @ApiOperation("删除服务商配置")
    @GetMapping("/deleteSupplierConfig")
    public ResponseMessage2<Boolean> deleteSupplierConfig(@RequestParam(value = "id") Long id) {
        try {
            scoreSupplierService.deleteSupplierConfig(id);
            return ResponseMessage2.Success();
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }

    @ApiOperation("服务商配置明细")
    @GetMapping("/getSupplierConfigDetail")
    public ResponseMessage2<SupplierConfigVO> getSupplierConfigDetail(@RequestParam(value = "id") Long id) {
        try {
            return scoreSupplierService.getSupplierConfigDetail(id);
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }

    @ApiOperation("获取评分项配置")
    @PostMapping("/getScoreConfig")
    public ResponseMessage2<List<ScoreConfigPO>> getScoreConfig() {
        try {
            return ResponseMessage2.Success2(scoreSupplierService.getScoreConfig());
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }


    @ApiOperation("保存评分项配置")
    @PostMapping("/saveScoreConfig")
    public ResponseMessage2<Boolean> saveScoreConfig(@RequestBody List<ScoreConfigVO> items) {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            ResponseMessage2.Failed(ProcessUtil.RE_LOGIN);
            return ResponseMessage2.Failed(ResponseStatus.ForbiddenAccess.getIndex(), ProcessUtil.RE_LOGIN);
        }
        try {
            scoreSupplierService.saveScoreConfig(items, currentUser.getUserName());
            return ResponseMessage2.Success();
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }

    @ApiOperation("判断是否服务商账号")
    @PostMapping("/hasSupplierRight")
    public ResponseMessage2<Boolean> hasSupplierRight() {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            ResponseMessage2.Failed(ProcessUtil.RE_LOGIN);
            return ResponseMessage2.Failed(ResponseStatus.ForbiddenAccess.getIndex(), ProcessUtil.RE_LOGIN);
        }
        String installSupplierId = LambdaUtils.getFieldName(OrderPO::getInstallSupplierId);
        return ResponseMessage2.Success2(StrUtil.isNotEmpty(currentUser.getAttribute().get(installSupplierId)));
    }


    @ApiOperation("获取服务商评价(服务商账号只能看自己,supplierId为空的时候后端判断用户权限，不为空不处理)")
    @PostMapping("/getScoreSupplier")
    public ResponseMessage2<PageData<ScoreSupplierPO>> getScoreSupplier(@RequestBody SupplierScoreQo qo) {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            ResponseMessage2.Failed(ProcessUtil.RE_LOGIN);
            return ResponseMessage2.Failed(ResponseStatus.ForbiddenAccess.getIndex(), ProcessUtil.RE_LOGIN);
        }
        try {
            String installSupplierId = LambdaUtils.getFieldName(OrderPO::getInstallSupplierId);
            if (qo.getSupplierId() == null && StringUtils.isEmpty(currentUser.getAttribute().get(installSupplierId))) {
                qo.setSupplierId(Convert.toLong(currentUser.getAttribute().get(installSupplierId)));
            }
            qo.setLoginUserId(currentUser.getUserId());
            return ResponseMessage2.Success2(scoreSupplierService.getScoreSupplier(qo));
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }

    @ApiOperation("获取服务商评价明细(合并查询和评分，后端判断当前用户是否拥有每项评分权限)")
    @GetMapping("/getScoreSupplierDetail")
    public ResponseMessage2<ScoreSupplierVO> getScoreSupplierDetail(@RequestParam(value = "id") Long id) {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            ResponseMessage2.Failed(ProcessUtil.RE_LOGIN);
            return ResponseMessage2.Failed(ResponseStatus.ForbiddenAccess.getIndex(), ProcessUtil.RE_LOGIN);
        }
        try {
            ScoreItemQo qo = new ScoreItemQo();
            qo.setId(id);
            qo.setLoginUserId(currentUser.getUserId());
            return scoreSupplierService.getScoreSupplierDetail(qo);
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }

    @ApiOperation("保存评分==>(接口参数为界面调整内容)")
    @PostMapping("/saveScoreSupplierItems")
    public ResponseMessage2<Boolean> saveScoreSupplierItems(@RequestBody List<ScoreSupplierItemVO> items) {
        SessionInfo currentUser = ProcessUtil.getUserInfo();
        if (currentUser == null) {
            ResponseMessage2.Failed(ProcessUtil.RE_LOGIN);
            return ResponseMessage2.Failed(ResponseStatus.ForbiddenAccess.getIndex(), ProcessUtil.RE_LOGIN);
        }
        try {
            scoreSupplierService.saveScoreSupplierItems(items, currentUser.getUserName());
            return ResponseMessage2.Success();
        } catch (Exception ex) {
            return ResponseMessage2.Failed(ex.getMessage());
        }
    }


}
