package com.boco.nbd.wios.score.entity.po;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.persistence.Column;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.io.Serializable;
import java.util.Date;

/**
 * @author:cao hai
 * @date:2022/10/10 17:34
 * @version:V1.0
 * @description:ScoreSupplierPO
 * @modify:
 */
@Data
@Table(name = "t_score_supplier_v2")
public class ScoreSupplierPO implements Serializable {

    @Id
    @Column(name = "id")
    @ApiModelProperty(value = "id")
    private Long id;

    @Column(name = "supplier_config_id")
    @ApiModelProperty(value = "安装服务配置表Id")
    private Long supplierConfigId;

    @Column(name = "supplier_id")
    @ApiModelProperty(value = "服务商Id")
    private Long supplierId;

    @Column(name = "supplier_name")
    @ApiModelProperty(value = "服务商")
    private String supplierName;

    @Column(name = "supplier_code")
    @ApiModelProperty(value = "服务商编码")
    private String supplierCode;

    @Column(name = "contact_name")
    @ApiModelProperty(value = "联系人姓名")
    private String contactName;

    @Column(name = "contact_phone")
    @ApiModelProperty(value = "联系人电话")
    private String contactPhone;

    @Column(name = "region_id")
    @ApiModelProperty(value = "区域id")
    private Long regionId;

    @Column(name = "region_name")
    @ApiModelProperty(value = "区域名称")
    private String regionName;

    @Column(name = "month")
    @ApiModelProperty(value = "评价月份")
    private String month;

    @Column(name = "final_score")
    @ApiModelProperty(value = "综合评分")
    private Float finalScore;

    @Column(name = "status")
    @ApiModelProperty(value = "状态 预留字段")
    private Integer status;

    @Column(name = "create_account")
    @ApiModelProperty(value = "创建人")
    private String createAccount;

    @Column(name = "create_time")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    @Column(name = "modify_account")
    @ApiModelProperty(value = "修改人")
    private String modifyAccount;

    @Column(name = "modify_time")
    @ApiModelProperty(value = "修改时间")
    private Date modifyTime;

    @Transient
    @ApiModelProperty(value = "拥有操作权限用户", hidden = true)
    private String appraisalUser;

    @Transient
    @ApiModelProperty(value = "是否拥有操作权限", hidden = true)
    private Boolean hasRight;

    /**
     * 对象转换
     *
     * @param config
     * @return
     */
    public static ScoreSupplierPO toPo(SupplierConfigPO config) {
        ScoreSupplierPO po = new ScoreSupplierPO();
        BeanUtil.copyProperties(config, po);
        po.setId(null);
        po.setSupplierConfigId(config.getId());
        return po;
    }

    /**
     * 获取月份格式数据
     *
     * @param time
     * @return
     */
    public static String getMonthDate(Date time) {
        return DateUtil.format(time, WiosConstant.CHINESE_MONTH_DATE_PATTERN);
    }
}
