package com.boco.nbd.wios.score.entity.vo;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.cams.core.enums.YesOrNoEnum;
import com.boco.nbd.wios.score.entity.po.SupplierConfigPO;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.List;

/**
 * @author:cao hai
 * @date:2022/10/11 8:57
 * @version:V1.0
 * @description:服务商配置对象
 * @modify:
 */
@Data
public class SupplierConfigVO {

    /**
     * 评分项共计6项目
     */
    private static final int SCORE_CONFIG_SIZE = 6;

    @ApiModelProperty(value = "主键")
    private Long id;

    @ApiModelProperty(value = "服务商Id")
    private Long supplierId;

    @ApiModelProperty(value = "服务商")
    private String supplierName;

    @ApiModelProperty(value = "服务商编码")
    private String supplierCode;

    @ApiModelProperty(value = "联系人姓名")
    private String contactName;

    @ApiModelProperty(value = "联系人电话")
    private String contactPhone;

    @ApiModelProperty(value = "区域id")
    private Long regionId;

    @ApiModelProperty(value = "区域名称")
    private String regionName;

    @ApiModelProperty(value = "评分账号设置")
    private List<AppraisalDepartmentVO> scoreConfig;

    @ApiModelProperty(value = "操作人", hidden = true)
    private String userName;

    /**
     * 对象转换
     *
     * @param vo
     * @return
     */
    public static SupplierConfigPO toPo(SupplierConfigVO vo) {
        if (vo.getSupplierId() == null) {
            throw new FlowException(LambdaUtils.getFieldApiModelProperty(SupplierConfigVO::getSupplierId) + "为空");
        }
        if (vo.getRegionId() == null) {
            throw new FlowException(LambdaUtils.getFieldApiModelProperty(SupplierConfigVO::getRegionId) + "为空");
        }
        if (CollUtil.isEmpty(vo.getScoreConfig())) {
            throw new FlowException(LambdaUtils.getFieldApiModelProperty(SupplierConfigVO::getScoreConfig) + "为空");
        }
        if (vo.getScoreConfig().size() != SCORE_CONFIG_SIZE) {
            throw new FlowException(LambdaUtils.getFieldApiModelProperty(SupplierConfigVO::getScoreConfig) + "配置未完成");
        }
        SupplierConfigPO po = new SupplierConfigPO();
        BeanUtil.copyProperties(vo, po);
        po.setStatus(YesOrNoEnum.YES.getKey());
        return po;
    }
}
