package com.boco.nbd.wios.score.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.boco.nbd.cams.core.util.LambdaUtils;
import com.boco.nbd.framework.common.PageData;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.framework.persistence.BaseDaoService3;
import com.boco.nbd.cams.core.enums.YesOrNoEnum;
import com.boco.nbd.wios.score.entity.bo.ScoreItemBO;
import com.boco.nbd.wios.score.entity.bo.ScoreSupplierItemBO;
import com.boco.nbd.wios.score.entity.po.*;
import com.boco.nbd.wios.score.entity.qo.ScoreItemQo;
import com.boco.nbd.wios.score.entity.qo.ScoreSupplierQo;
import com.boco.nbd.wios.score.entity.qo.SupplierConfigQo;
import com.boco.nbd.wios.score.entity.qo.SupplierScoreQo;
import com.boco.nbd.wios.score.entity.vo.*;
import com.boco.nbd.wios.score.mapper.def.*;
import com.boco.nbd.wios.score.service.IScoreSupplierService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * @author:cao hai
 * @date:2022/10/11 8:38
 * @version:V1.0
 * @description:TODO ScoreSupplierServiceImpl
 * @modify:
 */
@Service
@Slf4j
public class ScoreSupplierServiceImpl extends BaseDaoService3<ScoreSupplierPO, ScoreSupplierV2Mapper> implements IScoreSupplierService {

    @Resource
    private SupplierConfigMapper supplierConfigMapper;

    @Resource
    private SupplierUserConfigMapper supplierUserConfigMapper;

    @Resource
    private ScoreConfigV2Mapper scoreConfigV2Mapper;

    @Resource
    private ScoreSupplierItemMapper scoreSupplierItemMapper;

    /**
     * 保存服务商配置
     *
     * @param vo
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseMessage2<Boolean> saveSupplierConfig(SupplierConfigVO vo) {
        SupplierConfigPO po = SupplierConfigVO.toPo(vo);
        if (po.getId() == null) {
            SupplierConfigPO existPo = getExistSupplierConfig(po);
            if (existPo != null) {
                return ResponseMessage2.Failed(vo.getSupplierName() + "," + vo.getRegionName() + ",片区" + "已配置！");
            }
            po.setCreateAccount(vo.getUserName());
            po.setCreateTime(new Date());
            supplierConfigMapper.insertSelective(po);
            //获取主键
            existPo = getExistSupplierConfig(po);
            if (existPo == null) {
                return ResponseMessage2.Failed(vo.getSupplierName() + "入库异常！");
            }
            po.setId(existPo.getId());
        } else {
            po.setModifyAccount(vo.getUserName());
            po.setModifyTime(new Date());
            supplierConfigMapper.updateByPrimaryKeySelective(po);
        }
        po.initScoreSupplierUser(vo.getScoreConfig());
        deleteSupplierUser(po.getId());
        po.getScoreSupplierUser().stream().forEach(s -> supplierUserConfigMapper.insertSelective(s));
        return ResponseMessage2.Success();
    }

    /**
     * 根据supplierConfigId删除配置用户
     *
     * @param supplierConfigId
     * @return
     */
    private int deleteSupplierUser(Long supplierConfigId) {
        Example example = new Example(SupplierUserConfigPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(SupplierUserConfigPO::getSupplierConfigId), supplierConfigId);
        return supplierUserConfigMapper.deleteByExample(example);
    }

    /**
     * 通过supplierConfigId获取配置用户
     *
     * @param supplierConfigId
     * @return
     */
    private List<SupplierUserConfigPO> getSupplierUser(Long supplierConfigId) {
        Example example = new Example(SupplierUserConfigPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(SupplierUserConfigPO::getSupplierConfigId), supplierConfigId);
        return supplierUserConfigMapper.selectByExample(example);
    }

    /**
     * 根据服务商 状态查询数据
     *
     * @param po
     * @return
     */
    private SupplierConfigPO getExistSupplierConfig(SupplierConfigPO po) {
        Example example = new Example(SupplierConfigPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(SupplierConfigPO::getSupplierId), po.getSupplierId());
        criteria.andEqualTo(LambdaUtils.getFieldName(SupplierConfigPO::getStatus), po.getStatus());
        criteria.andEqualTo(LambdaUtils.getFieldName(SupplierConfigPO::getRegionId), po.getRegionId());
        List<SupplierConfigPO> list = supplierConfigMapper.selectByExample(example);
        if (CollUtil.isNotEmpty(list)) {
            return list.get(0);
        }
        return null;
    }

    /**
     * 获取服务商配置
     *
     * @param qo
     * @return
     */
    @Override
    public PageData<SupplierConfigPO> getSupplierConfig(SupplierConfigQo qo) {
        Page<SupplierConfigPO> page = PageHelper.startPage(qo.getPageIndex(), qo.getPageSize());
        Example example = new Example(SupplierConfigPO.class);
        Example.Criteria criteria = example.createCriteria();
        if (qo.getSupplierId() != null) {
            criteria.andEqualTo(LambdaUtils.getFieldName(SupplierConfigPO::getSupplierId), qo.getSupplierId());
        }
        criteria.andEqualTo(LambdaUtils.getFieldName(SupplierConfigPO::getStatus), YesOrNoEnum.YES.getKey());
        supplierConfigMapper.selectByExample(example);
        return PageData.create2(page.getTotal(), page.getResult());
    }

    /**
     * 删除服务商配置
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteSupplierConfig(Long id) {
        SupplierConfigPO po = new SupplierConfigPO();
        po.setId(id);
        po.setStatus(YesOrNoEnum.NO.getKey());
        deleteSupplierUser(po.getId());
        return supplierConfigMapper.updateByPrimaryKeySelective(po);
    }

    /**
     * 服务商配置明细
     *
     * @param id
     * @return
     */
    @Override
    public ResponseMessage2<SupplierConfigVO> getSupplierConfigDetail(Long id) {
        SupplierConfigPO po = supplierConfigMapper.selectByPrimaryKey(id);
        if (po == null) {
            return ResponseMessage2.Failed("数据缺失!");
        }
        SupplierConfigVO vo = new SupplierConfigVO();
        BeanUtil.copyProperties(po, vo);
        List<SupplierUserConfigPO> users = getSupplierUser(po.getId());
        List<AppraisalDepartmentVO> scoreConfig = new ArrayList<>(users.size());
        users.stream().forEach(s -> {
            AppraisalDepartmentVO departmentVO = new AppraisalDepartmentVO();
            departmentVO.setAppraisalDepartmentId(s.getAppraisalDepartmentId());
            departmentVO.setAppraisalUser(s.getAppraisalUser());
            scoreConfig.add(departmentVO);
        });
        vo.setScoreConfig(scoreConfig);
        return ResponseMessage2.Success2(vo);
    }

    /**
     * 获取评分项配置
     *
     * @return
     */
    @Override
    public List<ScoreConfigPO> getScoreConfig() {
        Example example = new Example(ScoreConfigPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreConfigPO::getStatus), YesOrNoEnum.YES.getKey());
        return scoreConfigV2Mapper.selectByExample(example);
    }

    /**
     * 保存评分项配置
     *
     * @param items
     * @param userName
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveScoreConfig(List<ScoreConfigVO> items, String userName) {
        scoreConfigV2Mapper.truncateData();
        items.stream().forEach(s -> {
            ScoreConfigPO po = ScoreConfigVO.toPo(s);
            po.setCreateAccount(userName);
            po.setCreateTime(new Date());
            scoreConfigV2Mapper.insertSelective(po);
        });
    }

    /**
     * 获取服务商评价数据
     *
     * @param qo
     * @return
     */
    @Override
    public PageData<ScoreSupplierPO> getScoreSupplier(SupplierScoreQo qo) {
        Page<ScoreSupplierPO> page = PageHelper.startPage(qo.getPageIndex(), qo.getPageSize());
        this.mapper.queryScoreSupplier(qo);
        List<ScoreSupplierPO> list = page.getResult();
        list.stream().forEach(s -> s.setHasRight(SupplierUserConfigPO.hasRight(s.getAppraisalUser(), qo.getLoginUserId())));
        return PageData.create2(page.getTotal(), list);
    }

    /**
     * 根据月份获取所有数据
     *
     * @param month
     * @return
     */
    @Override
    public List<ScoreSupplierPO> getAllScoreByMonth(String month) {
        Example example = new Example(ScoreSupplierPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierPO::getMonth), month);
        return this.mapper.selectByExample(example);
    }

    /**
     * 保存安装服务评分
     *
     * @param po
     * @return
     */
    @Override
    public int saveScoreSupplier(ScoreSupplierPO po) {
        deleteSameScoreSupplier(po);
        return this.mapper.insertSelective(po);
    }

    /**
     * 根据供应商和月份获取已存在评分
     *
     * @param po
     * @return
     */
    @Override
    public ScoreSupplierPO getExistsScoreSupplier(ScoreSupplierPO po) {
        Example example = new Example(ScoreSupplierPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierPO::getSupplierId), po.getSupplierId());
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierPO::getMonth), po.getMonth());
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierPO::getRegionId), po.getRegionId());
        List<ScoreSupplierPO> items = this.mapper.selectByExample(example);
        if (CollUtil.isNotEmpty(items)) {
            return items.get(0);
        }
        return null;
    }

    /**
     * 保存安装服务评分明细
     *
     * @param items
     * @param userName
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveScoreSupplierItems(List<ScoreSupplierItemVO> items, String userName) {
        if (CollUtil.isEmpty(items)) {
            return;
        }
        boolean insert = (items.get(0).getId() == null);
        ScoreSupplierPO scoreSupplierPO = new ScoreSupplierPO();
        scoreSupplierPO.setId(items.get(0).getScoreSupplierId());
        if (insert) {
            deleteScoreSupplierItems(scoreSupplierPO.getId());
        }
        for (ScoreSupplierItemVO item : items) {
            ScoreSupplierItemPO po = ScoreSupplierItemPO.toPo(item);
            po.setModifyAccount(userName);
            if (po.getFinalScore() == null) {
                po.setFinalScore(0F);
            }
            if (insert) {
                scoreSupplierItemMapper.insertSelective(po);
            } else {
                scoreSupplierItemMapper.updateByPrimaryKeySelective(po);
            }
        }
        //更新总分
        List<ScoreItemBO> scoreItems = this.mapper.queryScoreItem(items.get(0).getScoreSupplierId());
        scoreSupplierPO.setFinalScore(ScoreItemBO.getFinalScore(scoreItems));
        scoreSupplierPO.setModifyTime(new Date());
        scoreSupplierPO.setModifyAccount(userName);
        this.mapper.updateByPrimaryKeySelective(scoreSupplierPO);
    }

    /**
     * 获取服务商打分明细
     *
     * @param qo
     * @return
     */
    @Override
    public ResponseMessage2<ScoreSupplierVO> getScoreSupplierDetail(ScoreItemQo qo) {
        ScoreSupplierPO po = this.mapper.selectByPrimaryKey(qo.getId());
        if (po == null) {
            return ResponseMessage2.Failed("打分表不存在");
        }
        ScoreSupplierVO vo = new ScoreSupplierVO();
        BeanUtil.copyProperties(po, vo);
        List<ScoreSupplierItemBO> items = getScoreSupplierItem(po);
        if(CollUtil.isEmpty(items)){
            return ResponseMessage2.Failed("评分项不存在");
        }
        HashMap<String, Boolean> map = new HashMap<>(8);
        items.stream().forEach(s -> {
            //单项最高分
            s.setMaxScore(0f);
            if (s.getTotalScore() != null && s.getWeight() != null) {
                s.setMaxScore(s.getTotalScore() * s.getWeight() / 100);
            }
            //存在非自动计算配置成算法方式 s.getAppraisalId() != null
            if (s.getAppraisalDepartmentId() == 0) {
                s.setHasRight(false);
            } else {
                if (!map.containsKey(s.getAppraisalUser())) {
                    map.put(s.getAppraisalUser(), SupplierUserConfigPO.hasRight(s.getAppraisalUser(), qo.getLoginUserId()));
                }
                s.setHasRight(map.get(s.getAppraisalUser()));
            }
        });
        vo.setItems(items);
        return ResponseMessage2.Success2(vo);
    }

    /**
     * 安装服务评分明细
     *
     * @param po
     * @return
     */
    private List<ScoreSupplierItemBO> getScoreSupplierItem(ScoreSupplierPO po) {
        ScoreSupplierQo qo = new ScoreSupplierQo();
        qo.setScoreSupplierId(po.getId());
        qo.setSupplierConfigId(po.getSupplierConfigId());
        return this.mapper.getScoreSupplierItem(qo);
    }

    /**
     * 删除已存在的评分
     *
     * @param scoreSupplierId
     * @return
     */
    private int deleteScoreSupplierItems(Long scoreSupplierId) {
        Example example = new Example(ScoreSupplierItemPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierItemPO::getScoreSupplierId), scoreSupplierId);
        return scoreSupplierItemMapper.deleteByExample(example);
    }

    /**
     * 删除供应商当前月份已存在评分表
     *
     * @param po
     * @return
     */
    private int deleteSameScoreSupplier(ScoreSupplierPO po) {
        Example example = new Example(ScoreSupplierPO.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierPO::getSupplierId), po.getSupplierId());
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierPO::getMonth), po.getMonth());
        criteria.andEqualTo(LambdaUtils.getFieldName(ScoreSupplierPO::getRegionId), po.getRegionId());
        return this.mapper.deleteByExample(example);
    }
}
